"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInteractionByTypeAndId = exports.getInteractions = void 0;
const callDB = require("./call");
const interaction_model_1 = require("../../../models/interaction.model");
const log4js = require("log4js");
let logger = log4js.getLogger('ServerScripts');
function getInteractions(vcc, limit, params) {
    return new Promise(function (resolve, reject) {
        let promises = [];
        logger.info('[HistoricalData::getInteractions] Searched types: ' + params.types.length);
        if (params.types && params.types.length > 0) {
            if (params.types.includes('CALL')) {
                logger.info('[HistoricalData::getInteractions]  Added interactions search "Call" for vcc ' + vcc);
                promises.push(callDB.getInteractionsRefactor(vcc, limit, params));
            }
        }
        let res = new interaction_model_1.InteractionView;
        res.totalRecords = 0;
        res.totalTime = 0;
        res.roundedTime = 0;
        res.interactions = [];
        logger.info('[HistoricalData::getInteractions] Request the added promises.');
        Promise.all(promises).then(results => {
            logger.info('[HistoricalData::getInteractions] Request OK');
            for (let interactionView of results) {
                res.totalRecords += interactionView['totalRecords'];
                res.totalTime += interactionView['totalTime'];
                res.roundedTime += interactionView['roundedTime'];
                for (let i of interactionView['interactions']) {
                    res.interactions.push(i);
                }
            }
            resolve(res);
        })
            .catch(error => {
            logger.error('[HistoricalData::getInteractions] Error during search in HistoricalDara: ' + error);
            reject(error);
        });
    });
}
exports.getInteractions = getInteractions;
function getInteractionByTypeAndId(vcc, type, id, loadContent = false) {
    switch (type) {
        case 'CALL': return callDB.getInteractionById(vcc, id, loadContent);
        default:
            return new Promise((resolve, reject) => {
                reject('Interaction Type not valid or not found: ' + type);
            });
    }
}
exports.getInteractionByTypeAndId = getInteractionByTypeAndId;
exports.default = {
    getInteractionByTypeAndId,
    getInteractions
};
