"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetHistoricAgents = exports.BulkAgent = exports.GetAgentQualificationAverage = exports.GetAgentsFromEvaluation = exports.GetAgents = exports.AgentMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function AgentMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AgentMSSql = AgentMSSql;
function GetAgents(vcc, agentIds) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AgentIds', sql.VarChar(50), agentIds)
        .execute('GetAgents')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAgents = GetAgents;
function GetAgentsFromEvaluation(vcc, userId, campaigns, searchType) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('UserId', sql.VarChar(50), userId)
        .input('Campaigns', sql.VarChar(sql.MAX), campaigns)
        .input('SearchType', sql.VarChar(50), searchType)
        .execute('GetAgentsFromEvaluation')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAgentsFromEvaluation = GetAgentsFromEvaluation;
function GetAgentQualificationAverage(instance, idAgent) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('IdAgent', sql.VarChar(50), idAgent)
        .execute('GetAgentQualificationAverage')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAgentQualificationAverage = GetAgentQualificationAverage;
function BulkAgent(vcc, agents) {
    let table = SetAgentsTemporaryTable(vcc, agents);
    return BulkAgentTransaction(vcc, table);
}
exports.BulkAgent = BulkAgent;
function SetAgentsTemporaryTable(vcc, data) {
    let table = new sql.Table('#AgentTemp');
    table.create = true;
    table.columns.add('VCC', sql.VarChar(50), { nullable: true });
    table.columns.add('AgentId', sql.VarChar(50), { nullable: true });
    table.columns.add('Name', sql.VarChar(250), { nullable: true });
    data.map(d => {
        table.rows.add(vcc, d.userId, d.userName);
    });
    return table;
}
function BulkAgentTransaction(vcc, temporaryTable) {
    let transaction = new sql.Transaction(sqlConn);
    let request;
    return transaction.begin()
        .then(result => {
        logger.info('[AgentMSSql::BulkAgentTransaction] Creating transaction');
        return new sql.Request(transaction);
    })
        .then(result => {
        request = result;
        return request.bulk(temporaryTable)
            .then((rowCount) => {
            logger.info('[AgentMSSql::BulkAgentTransaction] Inserted ' + rowCount + ' records to temporary Table');
            return true;
        })
            .catch((err) => {
            logger.error('[AgentMSSql::BulkAgentTransaction] Error on bulk: ' + err.message);
            throw err;
        });
    })
        .then(result => {
        return request
            .execute('BulkAgent')
            .then((sqlResult) => {
            logger.info('[AgentMSSql::BulkAgentTransaction] Succed Execution');
            let results = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
            let promiseArray = [];
            promiseArray.push(Promise.resolve(results && results.length > 0 ? results[0] : null));
            promiseArray.push(transaction.commit());
            return Promise.all(promiseArray);
        })
            .catch((err) => {
            logger.error('[AgentMSSql::BulkAgentTransaction] Error on: ' + err.message);
            return Promise.reject(err);
        });
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[AgentMSSql::BulkAgentTransaction] Parsing error: ' + err.message);
        transaction.rollback();
        return Promise.reject(err);
    });
}
function GetHistoricAgents(vcc, excludesWorkGroupSup) {
    return new sql.Request(sqlConn)
        .input('Vcc', sql.VarChar(50), vcc)
        .input('ExcludesWorkGroupSup', sql.Bit, excludesWorkGroupSup)
        .execute('GetHistoricAgents')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetHistoricAgents = GetHistoricAgents;
