"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BulkAsyncInteractionTemp = exports.AsyncInteractionMSSql = void 0;
const log4js = require("log4js");
const moment = require("moment");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function AsyncInteractionMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AsyncInteractionMSSql = AsyncInteractionMSSql;
function BulkAsyncInteractionTemp(vcc, processId, asyncInteractionArray, userId) {
    let logLabel = `[AsyncInteractionMSSql::BulkAsyncInteractionTemp] [vcc::processId => ${vcc}::${processId}]: `;
    let tableAsyncInteraction = new sql.Table('#AsyncInteractionTemp');
    tableAsyncInteraction.create = true;
    let tableProcessAsyncInteraction = new sql.Table('#ProcessAsyncInteractionTemp');
    tableProcessAsyncInteraction.create = true;
    tableAsyncInteraction.columns.add('VCC', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('ID', sql.VarChar(250), { nullable: true });
    tableAsyncInteraction.columns.add('Folder', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('ExportedContent', sql.VarChar(250), { nullable: true });
    tableAsyncInteraction.columns.add('Channel', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('Campaign', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('StartDate', sql.DateTime, { nullable: true });
    tableAsyncInteraction.columns.add('EndDate', sql.DateTime, { nullable: true });
    tableAsyncInteraction.columns.add('Account', sql.VarChar(250), { nullable: true });
    tableAsyncInteraction.columns.add('Initiative', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('ContactName', sql.VarChar(300), { nullable: true });
    tableAsyncInteraction.columns.add('FirstTakedAgent', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('LastTakedAgent', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('IsTaken', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('TotalMessages', sql.Int, { nullable: true });
    tableAsyncInteraction.columns.add('DurationTime', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('AttentionTime', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('DispositionCode', sql.VarChar(sql.MAX), { nullable: true });
    tableAsyncInteraction.columns.add('DispositionTreePath', sql.VarChar(sql.MAX), { nullable: true });
    tableAsyncInteraction.columns.add('RawChar', sql.Int, { nullable: true });
    tableAsyncInteraction.columns.add('CreatedDate', sql.DateTime, { nullable: true });
    tableAsyncInteraction.columns.add('CreatedByUserId', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('LastModifiedDate', sql.DateTime, { nullable: true });
    tableAsyncInteraction.columns.add('LastModifiedByUserId', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('ContactId', sql.VarChar(50), { nullable: true });
    tableAsyncInteraction.columns.add('ContactAddress', sql.VarChar(300), { nullable: true });
    tableAsyncInteraction.columns.add('NeedI6Update', sql.Bit, { nullable: true });
    tableAsyncInteraction.columns.add('Ender', sql.VarChar(300), { nullable: true });
    tableAsyncInteraction.columns.add('Ticket', sql.VarChar(300), { nullable: true });
    tableProcessAsyncInteraction.columns.add('VCC', sql.VarChar(50), { nullable: true });
    tableProcessAsyncInteraction.columns.add('ProcessId', sql.VarChar(50), { nullable: true });
    tableProcessAsyncInteraction.columns.add('InteractionType', sql.VarChar(50), { nullable: true });
    tableProcessAsyncInteraction.columns.add('Campaign', sql.VarChar(50), { nullable: true });
    tableProcessAsyncInteraction.columns.add('InteractionId', sql.VarChar(250), { nullable: true });
    tableProcessAsyncInteraction.columns.add('EndDate', sql.DateTime, { nullable: true });
    tableProcessAsyncInteraction.columns.add('FormattedEndDate', sql.VarChar(100), { nullable: true });
    tableProcessAsyncInteraction.columns.add('CreatedDate', sql.DateTime, { nullable: true });
    tableProcessAsyncInteraction.columns.add('CreatedByUserId', sql.VarChar(50), { nullable: true });
    tableProcessAsyncInteraction.columns.add('LastModifiedDate', sql.DateTime, { nullable: true });
    tableProcessAsyncInteraction.columns.add('LastModifiedByUserId', sql.VarChar(50), { nullable: true });
    asyncInteractionArray.forEach(ai => {
        tableAsyncInteraction.rows.add(vcc, ai.id ? ai.id : null, ai.folder ? ai.folder : null, ai.exportedContent ? ai.exportedContent : null, ai.channel ? ai.channel : null, ai.campaign ? ai.campaign : null, ai.startDate ? (typeof ai.startDate === 'string' ? moment(ai.startDate).toDate() : null) : null, ai.endDate ? (typeof ai.endDate === 'string' ? moment(ai.endDate).toDate() : null) : null, ai.account ? ai.account : null, ai.initiative ? ai.initiative : null, ai.contactName ? ai.contactName : null, ai.firstTakedAgent ? ai.firstTakedAgent : null, ai.lastTakedAgent ? ai.lastTakedAgent : null, ai.isTaken ? ai.isTaken : null, ai.totalMessages ? ai.totalMessages : null, ai.durationTime ? ai.durationTime : null, ai.attentionTime ? ai.attentionTime : null, ai.dispositionCode ? ai.dispositionCode : null, ai.dispositionTreePath ? ai.dispositionTreePath : null, ai.rawChar ? ai.rawChar : null, moment().utc().toDate(), userId, moment().utc().toDate(), userId, ai.contactId ? ai.contactId : null, ai.contactAddress ? ai.contactAddress : null, ai.needI6Update ? ai.needI6Update : false, ai.ender || null, ai.ticket || null);
        tableProcessAsyncInteraction.rows.add(vcc, processId ? processId : null, ai.channel ? ai.channel : null, ai.campaign ? ai.campaign : null, ai.id ? ai.id : null, ai.endDate ? (typeof ai.endDate === 'string' ? moment(ai.endDate).toDate() : null) : null, ai.folder ? ai.folder : null, moment().utc().toDate(), userId, moment().utc().toDate(), userId);
    });
    logger.info(`${logLabel}Temporary tables filled (AsyncInteractionTemp, ProcessAsyncInteractionTemp)`);
    let genericResult = {};
    let transaction = new sql.Transaction(sqlConn);
    let request;
    return transaction.begin()
        .then(_ => {
        logger.info(`${logLabel}Creating transaction`);
        return new sql.Request(transaction);
    })
        .then((result) => {
        request = result;
        return request.bulk(tableAsyncInteraction)
            .then((rowCount) => {
            logger.info(`${logLabel}Inserted ${rowCount} records to #AsyncInteractionTemp temporary Table`);
            return true;
        })
            .catch((err) => {
            logger.error(`${logLabel}Bulk failed: ${err.message}`);
            throw err;
        });
    })
        .then((result) => {
        return request.bulk(tableProcessAsyncInteraction)
            .then((rowCount) => {
            logger.info(`${logLabel}Inserted ${rowCount} records to #ProcessAsyncInteractionTemp temporary Table`);
            return true;
        })
            .catch((err) => {
            logger.error(`${logLabel}Bulk failed: ${err.message}`);
            throw err;
        });
    })
        .then(result => {
        return request.execute('BulkAsyncInteraction')
            .then((recordset) => {
            logger.info(`${logLabel}BulkAsyncInteraction OK`);
            return Promise.resolve(true);
        })
            .catch((err) => {
            logger.error(`${logLabel}BulkAsyncInteraction failed: ${err.message}`);
            return Promise.reject(err);
        });
    })
        .then((result) => {
        logger.info(`${logLabel}Committing transaction`);
        return transaction.commit();
    })
        .then((result) => {
        genericResult.status = true;
        genericResult.description = 'OK';
        return Promise.resolve(genericResult);
    })
        .catch((err) => {
        transaction.rollback();
        logger.error(`${logLabel}Bulk error failed: ${err.message}`);
        genericResult.status = false;
        genericResult.description = 'SERVER_ERROR_UNKNOWN_MESSAGE';
        genericResult.error = typeof err === 'string' ? err : err.message;
        return Promise.resolve(genericResult);
    });
}
exports.BulkAsyncInteractionTemp = BulkAsyncInteractionTemp;
