"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveEvaluationReviewCommitment = exports.GetEvaluationReviewCommitments = exports.SaveEvaluationReview = exports.GetEvaluationReview = exports.EvaluationReviewMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function EvaluationReviewMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.EvaluationReviewMSSql = EvaluationReviewMSSql;
function GetEvaluationReview(vcc, evaluationPlanId, interactionId, supervisorId, agentId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .input('InteractionId', sql.VarChar(250), interactionId)
        .input('SupervisorId', sql.VarChar(50), supervisorId)
        .input('AgentId', sql.VarChar(50), agentId)
        .execute('GetEvaluationReview')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let evaluationReview = (recordset && recordset.length > 0 ? recordset[0] : null);
        let evaluationReviewCommitments = (sqlResult.recordsets.length > 1 ? sqlResult.recordsets[1] : []);
        return Promise.resolve({
            review: evaluationReview,
            commitments: evaluationReviewCommitments
        });
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetEvaluationReview = GetEvaluationReview;
function SaveEvaluationReview(vcc, evaluationReview, saveUserId) {
    let saveEvaluationReviewResult = false;
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationReview.evaluationPlanId)
        .input('InteractionId', sql.VarChar(250), evaluationReview.interactionId)
        .input('SupervisorId', sql.VarChar(50), evaluationReview.supervisorId)
        .input('Review', sql.VarChar(500), evaluationReview.review)
        .input('SaveUserId', sql.VarChar(50), saveUserId)
        .execute('SaveEvaluationReview')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        saveEvaluationReviewResult = output && output.id ? true : false;
        return Promise.resolve(saveEvaluationReviewResult);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveEvaluationReview = SaveEvaluationReview;
function GetEvaluationReviewCommitments(vcc, evaluationPlanId, interactionId, supervisorId, agentId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .input('InteractionId', sql.VarChar(250), interactionId)
        .input('SupervisorId', sql.VarChar(50), supervisorId)
        .input('AgentId', sql.VarChar(50), agentId)
        .execute('GetEvaluationReviewCommitments')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetEvaluationReviewCommitments = GetEvaluationReviewCommitments;
function SaveEvaluationReviewCommitment(vcc, evaluationReviewCommitment, saveUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Id', sql.VarChar(50), evaluationReviewCommitment.id)
        .input('LargeInteractionId', sql.VarChar(255), evaluationReviewCommitment.largeInteractionId)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationReviewCommitment.evaluationPlanId)
        .input('InteractionId', sql.VarChar(250), evaluationReviewCommitment.interactionId)
        .input('SupervisorId', sql.VarChar(50), evaluationReviewCommitment.supervisorId)
        .input('AgentId', sql.VarChar(50), evaluationReviewCommitment.agentId)
        .input('Commitment', sql.VarChar(250), evaluationReviewCommitment.commitment)
        .input('SaveUserId', sql.VarChar(50), saveUserId)
        .execute('SaveEvaluationReviewCommitment')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveEvaluationReviewCommitment = SaveEvaluationReviewCommitment;
