"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BulkInteractionEvaluationPlan = exports.GetInteractionsEvaluationPlan = exports.DeleteEvaluationPlanInteractionByTypeAndId = exports.GetEvaluationPlanInteractionByTypeAndId = exports.InteractionEvaluationPlanMSSql = void 0;
const moment = require("moment");
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function InteractionEvaluationPlanMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.InteractionEvaluationPlanMSSql = InteractionEvaluationPlanMSSql;
function GetEvaluationPlanInteractionByTypeAndId(vcc, evaluationPlanId, interactionType, interactionId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .input('InteractionType', sql.VarChar(50), interactionType)
        .input('InteractionId', sql.VarChar(250), interactionId)
        .execute('GetEvaluationPlanInteractionByTypeAndId')
        .then((sqlResult) => {
        logger.debug('[EvaluationPlanMSSql::GetEvaluationPlanInteractionByTypeAndId] EvaluationPlan::VCC => ' + evaluationPlanId + '::' + vcc);
        let recordset = sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null;
        let interactionEvaluationPlan = recordset?.length ? recordset[0] : null;
        return Promise.resolve(interactionEvaluationPlan);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Error executing: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[EvaluationPlanMSSql::GetEvaluationPlanByName] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetEvaluationPlanInteractionByTypeAndId = GetEvaluationPlanInteractionByTypeAndId;
function DeleteEvaluationPlanInteractionByTypeAndId(vcc, evaluationPlanId, interactionType, interactionId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .input('InteractionType', sql.VarChar(50), interactionType)
        .input('InteractionId', sql.VarChar(250), interactionId)
        .execute('DeleteEvaluationPlanInteractionByTypeAndId')
        .then((sqlResult) => {
        logger.debug('[EvaluationPlanMSSql::DeleteEvaluationPlanInteractionByTypeAndId] EvaluationPlan::VCC => ' + evaluationPlanId + '::' + vcc);
        let recordset = sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null;
        let output = recordset?.length ? recordset[0] : null;
        return Promise.resolve(output?.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Error executing: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[EvaluationPlanMSSql::DeleteEvaluationPlanInteractionByTypeAndId] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteEvaluationPlanInteractionByTypeAndId = DeleteEvaluationPlanInteractionByTypeAndId;
function GetInteractionsEvaluationPlan(vcc, evaluationPlanId) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), vcc)
        .input('IdEvaluationPlan', sql.VarChar(50), evaluationPlanId)
        .execute('GetInteractionsEvaluationPlan')
        .then((sqlResult) => {
        let recordset = sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null;
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInteractionsEvaluationPlan = GetInteractionsEvaluationPlan;
function BulkInteractionEvaluationPlan(vcc, evaluationPlan) {
    let table = SetInteractionEvaluationPlanTemporaryTable(vcc, evaluationPlan);
    return BulkInteractionEvaluationPlanTransaction(vcc, table);
}
exports.BulkInteractionEvaluationPlan = BulkInteractionEvaluationPlan;
function SetInteractionEvaluationPlanTemporaryTable(vcc, evaluationPlan) {
    let table = new sql.Table('#InteractionEvaluationPlanTemp');
    table.create = true;
    let relativeFileNameArray = [];
    let relativeFileName;
    table.columns.add('VCC', sql.VarChar(50), { nullable: true });
    table.columns.add('IdEvaluationPlan', sql.VarChar(50), { nullable: true });
    table.columns.add('IdAgent', sql.VarChar(sql.MAX), { nullable: true });
    table.columns.add('AgentName', sql.VarChar(sql.MAX), { nullable: true });
    table.columns.add('IdCampaign', sql.VarChar(sql.MAX), { nullable: true });
    table.columns.add('IdInteraction', sql.VarChar(250), { nullable: true });
    table.columns.add('InteractionSection', sql.Int, { nullable: true });
    table.columns.add('InteractionPart', sql.Int, { nullable: true });
    table.columns.add('InteractionFolder', sql.VarChar(50), { nullable: true });
    table.columns.add('InteractionType', sql.VarChar(50), { nullable: true });
    table.columns.add('TotalMessages', sql.Int, { nullable: true });
    table.columns.add('InteractionDateStart', sql.DateTime, { nullable: true });
    table.columns.add('InteractionDateEnd', sql.DateTime, { nullable: true });
    table.columns.add('InteractionDuration', sql.Int, { nullable: true });
    table.columns.add('InteractionOrigin', sql.VarChar(50), { nullable: true });
    table.columns.add('InteractionManagementResult', sql.VarChar(sql.MAX), { nullable: true });
    table.columns.add('RelativePath', sql.VarChar(500), { nullable: true });
    table.columns.add('FileName', sql.VarChar(200), { nullable: true });
    table.columns.add('InteractionContactId', sql.VarChar(100), { nullable: true });
    table.columns.add('InteractionFormat', sql.VarChar(50), { nullable: true });
    table.columns.add('CreatedByUserId', sql.VarChar(50), { nullable: true });
    table.columns.add('InteractionContactAddress', sql.VarChar(300), { nullable: true });
    table.columns.add('InteractionContactName', sql.VarChar(300), { nullable: true });
    table.columns.add('RelativeFileName', sql.VarChar(1000), { nullable: true });
    table.columns.add('Ender', sql.VarChar(300), { nullable: true });
    table.columns.add('Ticket', sql.VarChar(300), { nullable: true });
    logger.info('[EvaluationPlanMSSql::SetInteractionEvaluationPlanTemporaryTable] Records to temporary table InteractionEvaluationPlanTemp: ' + evaluationPlan.channelInteractions.length);
    evaluationPlan.channelInteractions.map(i => {
        if (i.ID)
            i.id = i.ID;
        if (i.Direction)
            i.Type = i.Direction;
        if (i.ID && i.Direction) {
            i.Part = 0;
            i.Section = 0;
            i.Channel = 'CALL';
        }
        let managementResult = null;
        if (i.Description) {
            managementResult = '/';
            if (i.Category1)
                managementResult += i.Category1 + '/';
            if (i.Category2)
                managementResult += i.Category2 + '/';
            if (i.Category3)
                managementResult += i.Category3 + '/';
            managementResult += i.Description;
            managementResult = managementResult.split('//').join('/');
        }
        else {
            managementResult = i.dispositionTreePath || i.dispositionCode || null;
        }
        if (['allegro', 'i-ftp', 'i-sftp', 'i-s3'].includes(evaluationPlan.environmentType)) {
            relativeFileNameArray = i.RelativeFileName?.split('/');
            if (relativeFileNameArray?.length > 0) {
                relativeFileName = relativeFileNameArray[1] + '/' + relativeFileNameArray[2] + '/' + relativeFileNameArray[3] + '/' + relativeFileNameArray[5].replace('.mp3', '').replace('.wav', '').replace('.ogg', '');
            }
        }
        else if (['ftp', 'sftp', 's3'].includes(evaluationPlan.environmentType)) {
            relativeFileName = i.RelativeFileName;
        }
        table.rows.add(vcc, evaluationPlan.id, i.Agent ? i.Agent : (i.firstTakedAgent || i.lastTakedAgent || null), i.AgentName ? i.AgentName.trim().replace('  ', ' ') : null, i.Campaign ? i.Campaign : i.campaign || null, i.InteractionId ? i.InteractionId : i.id || null, (i.Section || i.Section === 0) ? i.Section : null, (i.Part || i.Part === 0) ? i.Part : null, i.folder ? i.folder : null, i.Channel ? i.Channel : i.channel || null, i.totalMessages ? i.totalMessages : null, i.StartDate ? moment.utc(i.StartDate).toDate() : moment.utc(i.startDate).toDate() || null, i.EndDate ? moment.utc(i.EndDate).toDate() : moment.utc(i.endDate).toDate() || null, i.DurationTime ? i.DurationTime : i.durationTime || null, i.Type ? (i.Type === 'E' ? 'IN' : 'OUT') : i.initiative || null, managementResult, i.RelativePath ? i.RelativePath : null, i.FileName ? i.FileName : null, i.Number ? i.Number : i.contactId || i.contactName, i.Format ? i.Format : i.format || null, evaluationPlan.createdByUserId, i.contactAddress ? i.contactAddress : i.ContactAddress, i.contactName ? i.contactName : i.ContactName, relativeFileName ? relativeFileName : '', i.ender || null, i.ticket || null);
    });
    return table;
}
function BulkInteractionEvaluationPlanTransaction(vcc, temporaryTable) {
    let transaction = new sql.Transaction(sqlConn);
    let request;
    return transaction.begin()
        .then(result => {
        logger.info('[EvaluationPlanMSSql::BulkInteractionEvaluationPlanTransaction] Creating transaction');
        return new sql.Request(transaction);
    })
        .then(result => {
        request = result;
        return request.bulk(temporaryTable)
            .then((rowCount) => {
            logger.info('[EvaluationPlanMSSql::BulkInteractionEvaluationPlanTransaction] Inserted ' + rowCount + ' records to temporary Table');
            return true;
        })
            .catch((err) => {
            logger.error('[EvaluationPlanMSSql::BulkInteractionEvaluationPlanTransaction] Error on bulk: ' + err.message);
            throw err;
        });
    })
        .then(result => {
        return request
            .execute('BulkInteractionEvaluationPlan')
            .then((sqlResult) => {
            logger.info('[EvaluationPlanMSSql::BulkInteractionEvaluationPlanTransaction] Succed Execution');
            let results = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
            let promiseArray = [];
            promiseArray.push(Promise.resolve(results && results.length > 0 ? results[0] : null));
            promiseArray.push(transaction.commit());
            return Promise.all(promiseArray);
        })
            .catch((err) => {
            logger.error('[EvaluationPlanMSSql::BulkInteractionEvaluationPlanTransaction] Error on: ' + err.message);
            return Promise.reject(err);
        });
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[EvaluationPlanMSSql::BulkInteractionEvaluationPlanTransaction] Parsing error: ' + err.message);
        transaction.rollback();
        return Promise.reject(err);
    });
}
