"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveUnderlined = exports.DeleteUtteranceEntity = exports.DeleteUtterance = exports.DeleteMultipleUtterances = exports.SaveUtterance = exports.SaveUtteranceEntity = exports.GetUtteranceEntities = exports.GetUtterancesByInteraction = exports.GetUtterancesByIntentId = exports.GetUtterances = exports.GetUtterance = exports.UtteranceMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function UtteranceMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.UtteranceMSSql = UtteranceMSSql;
function CatchHandler(err) {
    let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
    logger.error(errorMsg);
    return Promise.reject(err);
}
function GetUtterance(vcc, appId, intentName, text) {
    logger.info('[UtteranceMSSql::GetUtterance] getting vcc:', vcc, 'text:', text);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentName', sql.VarChar(50), intentName)
        .input('Text', sql.VarChar(50), text)
        .execute('GetUtterance')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let utterance = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(utterance);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[UtteranceMSSql::GetUtterance] getting errorMsg:', errorMsg, 'of GetUtterance');
        return Promise.reject(err);
    });
}
exports.GetUtterance = GetUtterance;
function GetUtterances(vcc, appId, intent, searchCriteria, searchExpression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('Intent', sql.VarChar(50), intent)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(50), searchExpression)
        .execute('GetUtterances')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(CatchHandler);
}
exports.GetUtterances = GetUtterances;
function GetUtterancesByIntentId(vcc, intentId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('IntentId', sql.VarChar(50), intentId)
        .execute('GetUtterancesByIntentId')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('GetUtterancesByIntentId: ', recordset);
        return Promise.resolve(recordset);
    }).catch(CatchHandler);
}
exports.GetUtterancesByIntentId = GetUtterancesByIntentId;
function GetUtterancesByInteraction(vcc, audioId, section, part) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AudioId', sql.VarChar(50), audioId)
        .input('Section', sql.Numeric(5, 0), section)
        .input('Part', sql.Numeric(5, 0), part)
        .execute('GetUtterancesByInteraction')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('GetUtterancesByInteraction: ', recordset);
        return Promise.resolve(recordset);
    }).catch(CatchHandler);
}
exports.GetUtterancesByInteraction = GetUtterancesByInteraction;
function GetUtteranceEntities(vcc, utteranceId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('UtteranceId', sql.VarChar(50), utteranceId)
        .execute('GetUtteranceEntities')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(CatchHandler);
}
exports.GetUtteranceEntities = GetUtteranceEntities;
function SaveUtteranceEntity(vcc, utteranceId, entityName, start, end, startTokenIndex, endTokenIndex) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('UtteranceId', sql.VarChar(50), utteranceId)
        .input('EntityName', sql.VarChar(50), entityName)
        .input('Start', sql.Int, start)
        .input('End', sql.Int, end)
        .input('EndTokenIndex', sql.Int, endTokenIndex)
        .input('StartTokenIndex', sql.Int, startTokenIndex)
        .execute('SaveUtteranceEntity')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[UtteranceMSSql::SaveUtteranceEntity] Output: ', output);
        return Promise.resolve(output && output.result === 1);
    })
        .catch(CatchHandler);
}
exports.SaveUtteranceEntity = SaveUtteranceEntity;
function SaveUtterance(vcc, appId, intentId, intentName, id, text, userId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .input('IntentName', sql.VarChar(50), intentName)
        .input('Id', sql.VarChar(50), id)
        .input('Text', sql.VarChar(500), text)
        .input('UserId', sql.VarChar(50), userId)
        .execute('SaveUtterance')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('SaveUtterance - recordset: ', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('SaveUtterance - output: ', output);
        if (output) {
            return Promise.resolve(output.utteranceId);
        }
        return Promise.resolve(null);
    })
        .catch(CatchHandler);
}
exports.SaveUtterance = SaveUtterance;
function DeleteMultipleUtterances(vcc, ids, deleteByIntent = true) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Ids', sql.VarChar(sql.MAX), ids)
        .input('DeleteByIntent', sql.Bit, deleteByIntent)
        .execute('DeleteMultipleUtterances')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('DeleteMultipleUtterances - recordset: ', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('DeleteMultipleUtterances - output: ', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(CatchHandler);
}
exports.DeleteMultipleUtterances = DeleteMultipleUtterances;
function DeleteUtterance(vcc, appId, intentId, utteranceId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .input('Id', sql.VarChar(50), utteranceId)
        .execute('DeleteUtterance')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('DeleteUtterance - recordset: ', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('DeleteUtterance - output: ', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(CatchHandler);
}
exports.DeleteUtterance = DeleteUtterance;
function DeleteUtteranceEntity(vcc, utteranceId, entityName, start, end, startTokenIndex, endTokenIndex) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('UtteranceId', sql.VarChar(50), utteranceId)
        .input('EntityName', sql.VarChar(50), entityName)
        .input('Start', sql.Int, start)
        .input('End', sql.Int, end)
        .input('EndTokenIndex', sql.Int, endTokenIndex)
        .input('StartTokenIndex', sql.Int, startTokenIndex)
        .execute('DeleteUtteranceEntity')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch(CatchHandler);
}
exports.DeleteUtteranceEntity = DeleteUtteranceEntity;
function SaveUnderlined(vcc, appId, intentId, audioId, section, part, speaker, anchorOffset, focusOffset, currentTime, endCurrentTime, utteranceId, textContent, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .input('AudioId', sql.VarChar(50), audioId)
        .input('Section', sql.Numeric(5, 0), section)
        .input('Part', sql.Numeric(5, 0), part)
        .input('Speaker', sql.VarChar(3), speaker)
        .input('AnchorOffset', sql.Numeric(5, 0), anchorOffset)
        .input('FocusOffset', sql.Numeric(5, 0), focusOffset)
        .input('CurrentTime', sql.Float, currentTime)
        .input('EndCurrentTime', sql.Float, endCurrentTime)
        .input('UtteranceId', sql.VarChar(50), utteranceId)
        .input('TextContent', sql.VarChar(500), textContent)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveUnderlined')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('SaveUnderlined - recordset: ', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('SaveUnderlined - output: ', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(CatchHandler);
}
exports.SaveUnderlined = SaveUnderlined;
