"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAllRunningRasaModelFromTest = exports.GetRunningRasaModels = exports.DeleteRunningRasaModelFromTest = exports.DeleteRunningRasaModel = exports.GetRunningRasaModelFromTest = exports.GetRunningRasaModel = exports.SaveRunningRasaModelFromTest = exports.SaveRunningRasaModel = exports.GetAllReadyCognitiveAbilityStorages = exports.GetReadyCognitiveAbilities = exports.SaveReadyCognitiveAbility = exports.GetReadyCognitiveInteractions = exports.SaveReadyCognitiveInteraction = exports.DeleteCognitiveInteraction = exports.SaveCognitiveInteraction = exports.GetCognitiveInteraction = exports.CognitiveRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
let logger = log4js.getLogger('Redis');
let redisClient;
function CognitiveRedis(app) {
    redisClient = app.get('redisClient');
}
exports.CognitiveRedis = CognitiveRedis;
function GetCognitiveInteraction(key) {
    let storage = 'Cognitive';
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let interaction;
        try {
            interaction = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return interaction;
    })
        .catch((err) => {
        logger.error('[GetInteractionCognitive] getting err value:', err, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetCognitiveInteraction = GetCognitiveInteraction;
function SaveCognitiveInteraction(key, interaction) {
    let storage = 'Cognitive';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[SaveInteractionCognitive] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveCognitiveInteraction = SaveCognitiveInteraction;
function DeleteCognitiveInteraction(key) {
    let storage = 'Cognitive';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[DeleteCognitiveInteraction] getting error:', err, 'key:', key);
        return Promise.resolve(false);
    });
}
exports.DeleteCognitiveInteraction = DeleteCognitiveInteraction;
function SaveReadyCognitiveInteraction(key, interaction, executionCode) {
    let storage = 'Cognitive';
    storage += ':';
    storage += executionCode;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then((result) => {
        return redisClient.expireAsync(key + ':' + storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[SaveReadyCognitiveInteraction] getting error:', err, 'with vcc:', interaction.vcc, 'processId:', interaction.processId, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveReadyCognitiveInteraction = SaveReadyCognitiveInteraction;
function GetReadyCognitiveInteractions(executionCode) {
    let hash = 'Cognitive:';
    hash += executionCode;
    return redisClient.hgetallAsync(hash)
        .then((result) => {
        let interactions = [];
        try {
            let interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (let key in interactionsToSendResults) {
                    interactions.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[GetReadyCognitiveInteractions] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactions;
    })
        .catch((err) => {
        logger.error('[GetReadyCognitiveInteractions] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetReadyCognitiveInteractions = GetReadyCognitiveInteractions;
function SaveReadyCognitiveAbility(key, interaction, executionCode) {
    let storage = 'AbilityCognitive';
    storage += ':';
    storage += executionCode;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then((result) => {
        return redisClient.expireAsync(key + ':' + storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[SaveReadyAbilityCognitive] getting error:', err, 'with vcc:', interaction.vcc, 'processId:', interaction.processId, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveReadyCognitiveAbility = SaveReadyCognitiveAbility;
function GetReadyCognitiveAbilities(executionCode) {
    let hash = 'AbilityCognitive:';
    hash += executionCode;
    return redisClient.hgetallAsync(hash)
        .then((result) => {
        let interactions = [];
        try {
            let interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (let key in interactionsToSendResults) {
                    interactions.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[GetReadyAbilitiesCognitive] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactions;
    })
        .catch((err) => {
        logger.error('[GetReadyAbilitiesCognitive] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetReadyCognitiveAbilities = GetReadyCognitiveAbilities;
function GetAllReadyCognitiveAbilityStorages(executionCode) {
    let config = redis_config_1.REDIS_OPTIONS;
    let pattern = `${config.prefix}AbilityCognitive:*`;
    return redisClient.keysAsync([pattern])
        .then((result) => {
        if (result?.length > 1)
            logger.info(`[TipificationRedis::GetAllReadyCognitiveAbilityStorages] Getting ${result.length} storages for execution ${executionCode}`);
        let executionCodes = [];
        result.forEach(r => {
            executionCodes.push(r.replace(`${config.prefix}AbilityCognitive:`, ''));
        });
        executionCodes = executionCodes.sort();
        return executionCodes;
    })
        .catch((err) => {
        logger.error('[TipificationRedis::GetAllReadyCognitiveAbilityStorages] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetAllReadyCognitiveAbilityStorages = GetAllReadyCognitiveAbilityStorages;
function SaveRunningRasaModel(key, runningModel) {
    let storage = 'RasaModel';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(runningModel))
        .then((result) => {
        return redisClient.expireAsync(key + ':' + storage, 60 * 60);
    }).then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        logger.error('[cognitive.redis::SaveRunningRasaModel] nluId:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveRunningRasaModel = SaveRunningRasaModel;
function SaveRunningRasaModelFromTest(key, runningModel) {
    let storage = 'RasaModelFromTest';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(runningModel))
        .then((result) => {
        return redisClient.expireAsync(key + ':' + storage, 60 * 15);
    }).then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        logger.error('[cognitive.redis::SaveRunningRasaModelFromTest] nluId:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveRunningRasaModelFromTest = SaveRunningRasaModelFromTest;
function GetRunningRasaModel(key) {
    let storage = 'RasaModel';
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let interactions = [];
        try {
            let nlu = (typeof result === 'string') ? JSON.parse(result) : result;
            return Promise.resolve(nlu);
        }
        catch (err) {
            logger.error('[cognitive.redis:GetRunningRasaModel] getting err value:', result);
            return Promise.resolve(null);
        }
    }).catch((err) => {
        logger.error('[cognitive.redis:GetRunningRasaModel] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetRunningRasaModel = GetRunningRasaModel;
function GetRunningRasaModelFromTest(key) {
    let storage = 'RasaModelFromTest';
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let interactions = [];
        try {
            let nlu = (typeof result === 'string') ? JSON.parse(result) : result;
            return Promise.resolve(nlu);
        }
        catch (err) {
            logger.error('[cognitive.redis:GetRunningRasaModelFromTest] getting err value:', result);
            return Promise.resolve(null);
        }
    }).catch((err) => {
        logger.error('[cognitive.redis:GetRunningRasaModelFromTest] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetRunningRasaModelFromTest = GetRunningRasaModelFromTest;
function DeleteRunningRasaModel(key) {
    let storage = 'RasaModel';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        logger.error('[cognitive.redis::DeleteRunningRasaModel] getting error: ', err, 'key: ', key);
        return Promise.resolve(false);
    });
}
exports.DeleteRunningRasaModel = DeleteRunningRasaModel;
function DeleteRunningRasaModelFromTest(key) {
    let storage = 'RasaModelFromTest';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        logger.error('[cognitive.redis::DeleteRunningRasaModelFromTest] getting error: ', err, 'key: ', key);
        return Promise.resolve(false);
    });
}
exports.DeleteRunningRasaModelFromTest = DeleteRunningRasaModelFromTest;
function GetRunningRasaModels() {
    let storage = 'RasaModel';
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let runningRasaModelsList = [];
        try {
            runningRasaModelsList = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            logger.error('[cognitive.redis::GetRunningRasaModels] getting err value:', result);
            return Promise.resolve(null);
        }
        return Promise.resolve(runningRasaModelsList);
    }).catch((err) => {
        logger.error('[cognitive.redis::GetRunningRasaModels] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetRunningRasaModels = GetRunningRasaModels;
function GetAllRunningRasaModelFromTest() {
    let storage = 'RasaModelFromTest';
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let runningRasaModelsList = [];
        try {
            runningRasaModelsList = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            logger.error('[cognitive.redis::GetRunningRasaModelsFromTest] getting err value:', result);
            return Promise.resolve(null);
        }
        return Promise.resolve(runningRasaModelsList);
    }).catch((err) => {
        logger.error('[cognitive.redis::GetRunningRasaModels] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetAllRunningRasaModelFromTest = GetAllRunningRasaModelFromTest;
