"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeletEvaluationStatus = exports.GetEvaluationsStatus = exports.GetEvaluationStatus = exports.SaveEvaluationStatus = exports.EvaluationRedis = void 0;
const moment = require("moment");
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
let logger = log4js.getLogger('Redis');
let redisClient;
function EvaluationRedis(app) {
    redisClient = app.get('redisClient');
}
exports.EvaluationRedis = EvaluationRedis;
function SaveEvaluationStatus(vcc, evaluationToUpdate) {
    let storage = vcc + ':EvaluationStatus';
    let key = '';
    key += evaluationToUpdate.evaluationPlanId;
    key += ':';
    key += evaluationToUpdate.interactionId;
    key += ':';
    key += evaluationToUpdate.auditorId;
    evaluationToUpdate.unlockDate = moment().utc().add(30, 'minutes').toDate();
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(evaluationToUpdate))
        .then((result) => {
        return redisClient.expireAsync(storage, 60 * 30);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[EvaluationPlanAbilityRedis::SaveEvaluationPlanAbility] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveEvaluationStatus = SaveEvaluationStatus;
function GetEvaluationStatus(vcc, evaluationToGet) {
    let storage = vcc + ':EvaluationStatus';
    let evaluations = [];
    let key = '';
    key += evaluationToGet.evaluationPlanId;
    key += ':';
    key += evaluationToGet.interactionId;
    key += ':';
    key += evaluationToGet.auditorId;
    if (!key)
        return Promise.resolve(null);
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let evaluation;
        try {
            evaluation = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(evaluation);
    })
        .catch((err) => {
        logger.error('[EvaluationPlanAbilityRedis::GetEvaluationStatus] getting err value:', err, 'hash:', storage, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetEvaluationStatus = GetEvaluationStatus;
function GetEvaluationsStatus(vcc) {
    let storage = vcc + ':EvaluationStatus';
    let evaluations = [];
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let promiseArray = [];
        try {
            let evaluationsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && evaluationsToSendResults) {
                Object.keys(evaluationsToSendResults).map(k => evaluations.push(JSON.parse(evaluationsToSendResults[k])));
                Object.keys(evaluationsToSendResults).forEach(k => {
                    let evaluation = JSON.parse(evaluationsToSendResults[k]);
                    let unlockDate = moment(evaluation.unlockDate);
                    evaluation.unlockDate = moment(evaluation.unlockDate).toDate();
                    if (unlockDate.isSameOrBefore(moment().utc()))
                        promiseArray.push(DeletEvaluationStatus(vcc, evaluation));
                    else
                        evaluations.push(evaluation);
                });
            }
        }
        catch (err) {
            logger.error('[EvaluationPlanAbilityRedis::GetEvaluationsStatus] getting err value:', result);
            return Promise.resolve(null);
        }
        if (promiseArray?.length)
            return Promise.all(promiseArray);
        else
            return Promise.all([true]);
    })
        .then(_ => {
        return Promise.resolve(evaluations);
    })
        .catch((err) => {
        logger.error('[EvaluationPlanAbilityRedis::GetEvaluationsStatus] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetEvaluationsStatus = GetEvaluationsStatus;
function DeletEvaluationStatus(vcc, evaluationToUpdate) {
    let storage = vcc + ':EvaluationStatus';
    let key = '';
    key += evaluationToUpdate.evaluationPlanId;
    key += ':';
    key += evaluationToUpdate.interactionId;
    key += ':';
    key += evaluationToUpdate.auditorId;
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[EvaluationRedis::DeletEvaluationStatus] getting error:', err, 'with key:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.DeletEvaluationStatus = DeletEvaluationStatus;
