"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteCsvProcess = exports.SaveCsvProcess = exports.GetCsvProcess = exports.GetAllReadyProcessInteraccionStorages = exports.GetReadyProcessInteractions = exports.SaveReadyProcessInteraction = exports.SaveProcessEnvironment = exports.GetProcessEnvironment = exports.DeleteProcessInteraction = exports.SaveProcessInteraction = exports.GetProcessInteraction = exports.GetReadyI6UpdateInteractions = exports.SaveReadyI6UpdateInteraction = exports.ExistsProcessI6Update = exports.DeleteProcessI6Update = exports.SaveProcessI6Update = exports.GetProcessI6Update = exports.ProcessRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
let logger = log4js.getLogger('Redis');
let redisClient;
function ProcessRedis(app) {
    redisClient = app.get('redisClient');
}
exports.ProcessRedis = ProcessRedis;
function GetProcessI6Update(storage, key) {
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let interaction;
        try {
            interaction = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return interaction;
    })
        .catch((err) => {
        logger.error('[ProcessRedis::GetProcessI6Update] getting err value:', err, 'hash:', storage, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetProcessI6Update = GetProcessI6Update;
function SaveProcessI6Update(storage, key, processI6Interaction) {
    if (!key || !storage)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(processI6Interaction))
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[ProcessRedis::SaveProcessI6Update] getting error:', err, 'with key:', key);
        return Promise.resolve(false);
    });
}
exports.SaveProcessI6Update = SaveProcessI6Update;
function DeleteProcessI6Update(storage, key) {
    if (!key || !storage)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[ProcessRedis::DeleteProcessI6Update] getting error:', err, 'with key:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.DeleteProcessI6Update = DeleteProcessI6Update;
function ExistsProcessI6Update(storage) {
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let processI6ToUpdate = [];
        try {
            processI6ToUpdate = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            logger.error('[ProcessRedis::ExistsProcessI6Update] getting err value:', result);
            return Promise.resolve(null);
        }
        return Promise.resolve(processI6ToUpdate ? true : false);
    })
        .catch((err) => {
        logger.error('[ProcessRedis::ExistsProcessI6Update] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.ExistsProcessI6Update = ExistsProcessI6Update;
function SaveReadyI6UpdateInteraction(executionCode, key, processI6Interaction) {
    let storage = 'I6Update:';
    storage += executionCode;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(processI6Interaction))
        .then(_ => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[ProcessRedis::SaveReadyI6UpdateInteraction] getting error:', err, 'with vcc:', processI6Interaction.vcc, 'processId:', processI6Interaction.processId, 'processInteraction:', processI6Interaction, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveReadyI6UpdateInteraction = SaveReadyI6UpdateInteraction;
function GetReadyI6UpdateInteractions(executionCode) {
    let storage = 'I6Update:';
    storage += executionCode;
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let interactionProcessToSendList = [];
        try {
            let interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (let key in interactionsToSendResults) {
                    interactionProcessToSendList.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[ProcessRedis::GetReadyProcGetReadyI6UpdateInteractionsessInteractions] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactionProcessToSendList;
    })
        .catch((err) => {
        logger.error('[ProcessRedis::GetReadyI6UpdateInteractions] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetReadyI6UpdateInteractions = GetReadyI6UpdateInteractions;
function GetProcessInteraction(storage, key) {
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let interaction;
        try {
            interaction = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return interaction;
    })
        .catch((err) => {
        logger.error('[GetInteraction] getting err value:', err, 'hash:', storage, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetProcessInteraction = GetProcessInteraction;
function SaveProcessInteraction(storage, key, processInteraction) {
    if (!key || !storage)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(processInteraction))
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[SaveProcessInteraction] getting error:', err, 'with key:', key);
        return Promise.resolve(false);
    });
}
exports.SaveProcessInteraction = SaveProcessInteraction;
function DeleteProcessInteraction(storage, key) {
    if (!key || !storage)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[DeleteProcessInteraction] getting error:', err, 'with key:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.DeleteProcessInteraction = DeleteProcessInteraction;
function GetProcessEnvironment(key) {
    let storage = 'ProcessEnvironment';
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let environment = null;
        try {
            environment = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(environment);
    })
        .catch((err) => {
        logger.error('[process.redis::GetProcessEnvironment] getting err value:', err, 'hash:', storage, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetProcessEnvironment = GetProcessEnvironment;
function SaveProcessEnvironment(key, environment) {
    let storage = 'ProcessEnvironment';
    if (!key || !storage)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(environment))
        .then(_ => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[process.redis::SaveProcessEnvironment] getting error:', err, 'with key:', key);
        return Promise.resolve(false);
    });
}
exports.SaveProcessEnvironment = SaveProcessEnvironment;
function SaveReadyProcessInteraction(executionCode, key, processInteraction) {
    let storage = 'Audios';
    storage += ':';
    storage += executionCode;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(processInteraction))
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[SaveReadyInteractionProcess] getting error:', err, 'with vcc:', processInteraction.vcc, 'processId:', processInteraction.processId, 'processInteraction:', processInteraction, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveReadyProcessInteraction = SaveReadyProcessInteraction;
function GetReadyProcessInteractions(executionCode) {
    let storage = 'Audios:';
    storage += executionCode;
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let interactionProcessToSendList = [];
        try {
            let interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (let key in interactionsToSendResults) {
                    interactionProcessToSendList.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[GetReadyProcessInteractions] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactionProcessToSendList;
    })
        .catch((err) => {
        logger.error('[GetReadyProcessInteractions] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetReadyProcessInteractions = GetReadyProcessInteractions;
function GetAllReadyProcessInteraccionStorages(executionCode) {
    let config = redis_config_1.REDIS_OPTIONS;
    let pattern = `${config.prefix}Audios:*`;
    return redisClient.keysAsync([pattern])
        .then((result) => {
        if (result && result.length > 1)
            logger.info(`[ProcessRedis::GetReadyProcessInteractions] Getting ${result.length} storages for execution ${executionCode}`);
        let executionCodes = [];
        result.map(r => {
            executionCodes.push(r.replace(`${config.prefix}Audios:`, ''));
        });
        executionCodes = executionCodes.sort();
        return executionCodes;
    })
        .catch((err) => {
        logger.error('[ProcessRedis::GetReadyProcessInteractions] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetAllReadyProcessInteraccionStorages = GetAllReadyProcessInteraccionStorages;
function GetCsvProcess(storage, key) {
    return redisClient.hgetAsync(storage + ":CsvProcess", key)
        .then((result) => {
        let interactions;
        try {
            interactions = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return interactions;
    })
        .catch((err) => {
        logger.error('[GetCsvProcess] getting err value:', err, 'hash:', storage, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetCsvProcess = GetCsvProcess;
function SaveCsvProcess(storage, key, records) {
    if (!key || !storage)
        return Promise.resolve(false);
    let body = '{"timeInit": "' + new Date() + '","data":' + records + '}';
    return redisClient.hsetAsync(storage + ":CsvProcess", key, body)
        .then((result) => {
        return redisClient.expireAsync(storage + ":CsvProcess", redis_config_1.REDIS_DATA_PERSISTENCE_ONE_HOUR);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[SaveCsvProcess] getting error:', err, 'with key:', key);
        return Promise.resolve(false);
    });
}
exports.SaveCsvProcess = SaveCsvProcess;
function DeleteCsvProcess(storage, key) {
    if (!key || !storage)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage + ":CsvProcess", key)
        .then((result) => {
        return redisClient.expireAsync(storage + ":CsvProcess", redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[DeleteCsvProcess] getting error:', err, 'with key:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.DeleteCsvProcess = DeleteCsvProcess;
