"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetRole = exports.DeleteRole = exports.SaveRole = exports.RoleRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
let logger = log4js.getLogger('Redis');
let redisClient;
function RoleRedis(app) {
    redisClient = app.get('redisClient');
}
exports.RoleRedis = RoleRedis;
function SaveRole(vcc, role) {
    if (!role || !role.id)
        throw 'CANNOT_SAVE_WITH_NULL_ID';
    return redisClient.hsetAsync(vcc + ':Role', role.id, JSON.stringify(role))
        .then((result) => {
        return redisClient.expireAsync(vcc + ':Role', redis_config_1.REDIS_DATA_PERSISTENCE);
    }).then((result) => {
        return redisClient.hgetallAsync(vcc + ':User');
    }).then((users) => {
        for (var userId in users) {
            try {
                users[userId] = JSON.parse(users[userId]);
            }
            catch (err) {
                users[userId] = {};
            }
            if (users[userId].role === role.id || JSON.stringify(users[userId]) === JSON.stringify({})) {
                redisClient.hdel(vcc + ':User', userId);
            }
        }
        return Promise.resolve(true);
    }).catch((err) => {
        let errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveRole = SaveRole;
function DeleteRole(vcc, role) {
    if (!role || !role.id)
        throw 'CANNOT_DELETE_WITH_NULL_ID';
    return redisClient.hdelAsync(vcc + ':Role', role.id)
        .then((result) => {
        return redisClient.hgetallAsync(vcc + ':User');
    }).then((users) => {
        for (var userId in users) {
            try {
                users[userId] = JSON.parse(users[userId]);
            }
            catch (err) {
                users[userId] = {};
            }
            if (users[userId].role === role.id || JSON.stringify(users[userId]) === JSON.stringify({})) {
                redisClient.hdel(vcc + ':User', userId);
            }
        }
        return Promise.resolve(true);
    }).catch((err) => {
        let errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteRole = DeleteRole;
function GetRole(vcc, roleId) {
    if (!roleId)
        throw 'CANNOT_GET_WITH_NULL_ID';
    return redisClient.hgetAsync(vcc + ':Role', roleId)
        .then((result) => {
        let role;
        try {
            role = JSON.parse(result);
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(role);
    }).catch((err) => {
        let errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetRole = GetRole;
