"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetSession = exports.DeleteSession = exports.SaveSession = exports.SessionRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
let logger = log4js.getLogger('Redis');
let redisClient;
function SessionRedis(app) {
    redisClient = app.get('redisClient');
}
exports.SessionRedis = SessionRedis;
function SaveSession(vcc, userId, session) {
    if (!userId)
        throw 'CANNOT_SAVE_WITH_NULL_ID';
    return redisClient.hsetAsync(vcc + ':Session', userId, session)
        .then((_) => {
        return redisClient.expireAsync(vcc + ':Session', redis_config_1.REDIS_SESSION_PERSISTENCE);
    }).then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        let errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveSession = SaveSession;
function DeleteSession(vcc, userId) {
    if (!userId)
        throw 'CANNOT_DELETE_WITH_NULL_ID';
    return redisClient.hdelAsync(vcc + ':Session', userId)
        .then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        let errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteSession = DeleteSession;
function GetSession(vcc, userId) {
    if (!userId)
        throw 'CANNOT_GET_WITH_NULL_ID';
    return redisClient.hgetAsync(vcc + ':Session', userId)
        .then((session) => {
        return Promise.resolve(session);
    }).catch((err) => {
        let errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetSession = GetSession;
