"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiUserImplementation = void 0;
const log4js = require("log4js");
const sy = require("systeminformation");
const promises_shared_1 = require("../shared/promises.shared");
const md5 = require("md5");
const mssql_service_1 = require("../db/mssql.service");
const platform_service_1 = require("../services/platform.service");
let logger = log4js.getLogger('Task');
let icModule = require('intermediate-component-sa');
class ApiUserImplementation {
    GetSystemInformation(guid) {
        let systemInformationData = {
            installationId: guid,
            macAddress: '',
            publicIp: ''
        };
        return promises_shared_1.DummyPromise()
            .then((result) => {
            logger.info('[ApiUserImplementation::GetSystemInformation] guid:', guid);
            let sys = sy.networkInterfaces();
            return sys;
        }).then((result) => {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_NETWORK_INTERFACE_INFORMATION';
            if (result && result.length) {
                logger.info('[ApiUserImplementation::GetSystemInformation] guid:', guid, 'getting result[0].mac:', result[0].mac);
                let myNetworkInterface = null;
                myNetworkInterface = result.filter(r => r.operstate === 'up')[0];
                systemInformationData.macAddress = myNetworkInterface.mac;
                systemInformationData.publicIp = myNetworkInterface.ip4;
            }
            logger.info('[ApiUserImplementation::GetSystemInformation] guid:', guid, 'getting systemInformationData:', systemInformationData);
            return Promise.resolve(systemInformationData);
        }).catch((err) => {
            logger.error('[ApiUserImplementation::GetSystemInformation] guid:', guid, 'getting error:', err);
            return Promise.resolve(err);
        });
    }
    GenerateMD5Content(data) {
        let result = '';
        if (data) {
            result += data.installationId;
            result += data.macAddress;
            logger.info('[ApiUserImplementation::GenerateMD5Content] getting result:', result);
            result = md5(result);
        }
        return result;
    }
    GetApiUsers(vcc) {
        let systemInformationData = null;
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[ApiUserImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            let body = { vcc: vcc };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_API_USERS_GET', body);
        }).then(result => {
            logger.info('[ApiUserImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'from external server');
            return Promise.resolve(result);
        }).catch((err) => {
            logger.error('[ApiUserImplementation::GetPlatformAvailableLicenses] Getting error:', err);
            return Promise.resolve(null);
        });
    }
    SaveApiUser(vcc, userApi, pin, userSpeech) {
        let systemInformationData = null;
        let installationId;
        let pin_md5;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            if (!result.installationId) {
                throw 'INSTALLATIONID_MISSING';
            }
            installationId = result.installationId;
            pin_md5 = this.GenerateUserPinMD5(vcc, userApi, pin, installationId);
            let body = {
                installationId: installationId,
                vcc: vcc,
                userApi: userApi,
                pin: pin_md5,
                userSpeech: userSpeech
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_API_USER_SAVE', body);
        }).then(result => {
            logger.info('[ApiUserImplementation:SaveApiUser] Getting result:', result, 'from external server');
            if (result.status) {
                return Promise.resolve(pin_md5);
            }
            else {
                return Promise.resolve(null);
            }
        }).catch((err) => {
            logger.error('[ApiUserImplementation::SaveApiUser] Getting error:', err);
            return Promise.resolve(null);
        });
    }
    CreateApiUser(vcc, userApi, pin, userSpeech) {
        let systemInformationData = null;
        let installationId;
        let pin_md5;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            if (!result.installationId) {
                throw 'INSTALLATIONID_MISSING';
            }
            installationId = result.installationId;
            pin_md5 = this.GenerateUserPinMD5(vcc, userApi, pin, installationId);
            let body = {
                installationId: installationId,
                vcc: vcc,
                userApi: userApi,
                pin: pin_md5,
                userSpeech: userSpeech
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_API_USER_CREATE', body);
        }).then(result => {
            logger.info('[ApiUserImplementation:SaveApiUser] Getting result:', result, 'from external server');
            if (result.status) {
                return Promise.resolve(pin_md5);
            }
            else {
                return Promise.resolve(null);
            }
        }).catch((err) => {
            logger.error('[ApiUserImplementation::SaveApiUser] Getting error:', err);
            return Promise.resolve(null);
        });
    }
    DeleteApiUser(vcc, user) {
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[ApiUserImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then((result) => {
            let body = {
                vcc: vcc,
                user: user
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_API_USER_DELETE', body);
        }).then(result => {
            logger.info('[ApiUserImplementation:SaveApiUser] Getting result:', result, 'from external server');
            return Promise.resolve(result.status);
        }).catch((err) => {
            logger.error('[ApiUserImplementation::SaveApiUser] Getting error:', err);
            return Promise.resolve(null);
        });
    }
    DeleteApiUsers(vcc, user) {
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[ApiUserImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then((result) => {
            let body = {
                vcc: vcc,
                users: user
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_API_USERS_DELETE', body);
        }).then(result => {
            logger.info('[ApiUserImplementation:SaveApiUser] Getting result:', result, 'from external server');
            return Promise.resolve(result.status);
        }).catch((err) => {
            logger.error('[ApiUserImplementation::SaveApiUser] Getting error:', err);
            return Promise.resolve(null);
        });
    }
    GenerateUserPinMD5(vcc, user, pin, installationId) {
        let result = null;
        result = user;
        result += '+';
        result += pin;
        result += '@';
        result += vcc;
        result += ':';
        result += installationId;
        logger.info('[ApiUserImplementation::GenerateUserPinMD5] Getting result:', result);
        result = md5(result);
        return result;
    }
    ValidateUserPin(vcc, user, pin) {
        let systemInformationData = null;
        let installationId;
        let apiUser = '';
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return mssql_service_1.MSSql.GetApiUser(vcc, user);
        }).then((result) => {
            if (result) {
                apiUser = result.user;
            }
            else {
                throw 'API_USER_NOT_FOUND';
            }
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[ApiUserImplementation::ValidateUserPin] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let userPinMD5 = this.GenerateUserPinMD5(vcc, user, pin, systemInformationData.installationId);
            logger.info('[ApiUserImplementation::ValidateUserPin] Getting userPinMD5:', userPinMD5);
            let body = {
                platformHash: plaformHash,
                vcc: vcc,
                apiUser: apiUser,
                user: user,
                pin: userPinMD5,
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_API_USER_AUTH', body);
        }).then(result => {
            logger.info('[ApiUserImplementation::ValidateUserPin] Getting result:', result, 'from external server');
            return Promise.resolve(result);
        }).catch((err) => {
            logger.error('[ApiUserImplementation::ValidateUserPin] Getting error:', err);
            return Promise.resolve(null);
        });
    }
}
exports.ApiUserImplementation = ApiUserImplementation;
