"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BillingBatchStatusImplementation = void 0;
const billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
const log4js = require("log4js");
const promises_shared_1 = require("../shared/promises.shared");
let logger = log4js.getLogger('Task');
class BillingBatchStatusImplementation {
    Execute(executionCode) {
        let executeIndex = 1;
        let executeResult = true;
        let genericResult = {};
        return promises_shared_1.DummyPromise()
            .then((result) => {
            logger.debug('[BillingBatchStatusImplementation::Execute] Requesting data');
            let billing = new billing_batch_implementation_1.BillingBatchImplementation();
            return billing.GetCurrentBatches();
        })
            .then(result => {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = {
                        allowedBatches: this.ProcessResult(result.data['allowedBatches']),
                        cancelledBatches: this.ProcessResult(result.data['cancelledBatches']),
                    };
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'SERVER_ERROR_CURRENT_BATCHES_REQUEST_FAILED';
                    logger.warn('[BillingBatchStatusImplementation::Execute] executionCode:', executionCode, 'Fail getting 02', result);
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'SERVER_ERROR_CURRENT_BATCHES_REQUEST_FAILED';
                logger.warn('[BillingBatchStatusImplementation::Execute] executionCode:', executionCode, 'Fail getting 02', result);
            }
            return Promise.resolve(genericResult);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[BillingBatchStatusImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            genericResult.status = false;
            genericResult.description = errorMessage;
            return Promise.resolve(genericResult);
        });
    }
    ProcessResult(batches) {
        let allowedBatches = {};
        allowedBatches['transcription'] = batches.filter(b => b.SystemService === 'TRANSCRIPTION');
        allowedBatches['semanticAnalysis'] = batches.filter(b => b.SystemService === 'SEMANTIC_ANALYSIS');
        allowedBatches['cognitiveAnalysis'] = batches.filter(b => b.SystemService === 'COGNITIVE_ANALYSIS');
        return allowedBatches;
    }
}
exports.BillingBatchStatusImplementation = BillingBatchStatusImplementation;
