"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BillingImplementation = void 0;
const moment = require("moment");
const log4js = require("log4js");
const sy = require("systeminformation");
const promises_shared_1 = require("../shared/promises.shared");
const md5 = require("md5");
const mssql_service_1 = require("../db/mssql.service");
const account_model_1 = require("../models/account.model");
const redis_service_1 = require("../db/redis.service");
const platform_model_1 = require("../models/platform.model");
const platform_service_1 = require("../services/platform.service");
let icModule = require('intermediate-component-sa');
let logger = log4js.getLogger('Task');
class BillingImplementation {
    SavePlatformVersion() {
        let genericResult = {};
        let version = null;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[BillingBatchImplementation::SavePlatformVersion] Getting result after GetVersion');
            if (result && result.installationId) {
                version = result;
                let body = {
                    buildVersion: version.buildVersion
                };
                return icModule.InconcertRequest(version.installationId, 'IC_PARAM_URL_PLATFORM_VERSION_UPDATE', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = result.data;
                    return mssql_service_1.MSSql.SaveVersionRegistration(version.installationId, version.buildVersion);
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'ERROR_FAILED_SEARCH';
                    return Promise.resolve(false);
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
                return Promise.resolve(false);
            }
        })
            .then(result => {
            return Promise.resolve(genericResult);
        })
            .catch((err) => {
            logger.error('[BillingImplementation::SavePlatformVersion] Getting error:', err);
            return Promise.reject(err);
        });
    }
    AddInsertTranscriptionDetailToMainFile(interaction, batchType, batchTypeId) {
        let entityId = '';
        if (interaction.interactionType === 'CALL') {
            entityId = interaction.id + '_' + interaction.section + '_' + interaction.part;
        }
        else {
            entityId = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + interaction.id;
        }
        let line = '';
        line += 'NEW';
        line += '::';
        line += interaction.vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'TRANSCRIPTION';
        line += '::';
        line += interaction.processId;
        line += '::';
        line += entityId;
        line += '::';
        line += interaction.interactionType;
        line += '::';
        line += moment(interaction.startDate).utc().format('YYYY-MM-DD HH:mm:ss');
        line += '::';
        line += moment(interaction.endDate).utc().format('YYYY-MM-DD HH:mm:ss');
        line += '::';
        line += interaction.durationTime;
        line += '::';
        line += interaction.durationLinguisticTime;
        icModule.InconcertAddDataToGeneralFile(line, 'new');
    }
    AddUpdateTranscriptionDetailToMainFile(interaction, target, batchType, batchTypeId) {
        let entityId = '';
        if (interaction.interactionType === 'CALL') {
            entityId = interaction.id + '_' + interaction.section + '_' + interaction.part;
        }
        else {
            entityId = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + interaction.id;
        }
        let line = '';
        line += 'UPDATE';
        line += '::';
        line += interaction.vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'TRANSCRIPTION';
        line += '::';
        line += interaction.processId;
        line += '::';
        line += entityId;
        switch (target) {
            case 'ValidDurationTime':
                line += '::';
                line += 'ValidDurationTime';
                line += '::';
                line += interaction.durationLinguisticTime;
                break;
            case 'SpeechmaticsApi':
                line += '::';
                line += 'SpeechmaticsApi';
                line += '::';
                line += interaction.speechmaticsApi;
                break;
            case 'IdathaApi':
                line += '::';
                line += 'IdathaApi';
                line += '::';
                line += interaction.idathaApi;
                break;
            case 'ValidChar':
                line += '::';
                line += 'ValidChar';
                line += '::';
                line += interaction.validChar;
                break;
            case 'ValidWords':
                line += '::';
                line += 'ValidWords';
                line += '::';
                line += interaction.validWords;
                break;
            case 'TotalPhrases':
                line += '::';
                line += 'TotalPhrases';
                line += '::';
                line += interaction.totalPhrases;
                break;
            case 'CharsPerPhrase':
                line += '::';
                line += 'CharsPerPhrase';
                line += '::';
                line += interaction.charsPerPhrase;
                break;
        }
        icModule.InconcertAddDataToGeneralFile(line, 'update');
    }
    AddCloseTranscriptionDetailToMainFile(interaction, batchType, batchTypeId, resultCode, resultDescription) {
        let entityId = '';
        if (interaction.interactionType === 'CALL') {
            entityId = interaction.id + '_' + interaction.section + '_' + interaction.part;
        }
        else {
            entityId = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + interaction.id;
        }
        let line = '';
        line += 'CLOSE';
        line += '::';
        line += interaction.vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'TRANSCRIPTION';
        line += '::';
        line += interaction.processId;
        line += '::';
        line += entityId;
        line += '::';
        line += resultCode;
        line += '::';
        line += resultDescription;
        icModule.InconcertAddDataToGeneralFile(line, resultCode ? 'success' : 'failed');
    }
    AddInsertSemanticAnalysisDetailToMainFile(vcc, batchType, batchTypeId, processId, queryId) {
        let line = '';
        line += 'NEW';
        line += '::';
        line += vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'SEMANTIC_ANALYSIS';
        line += '::';
        line += processId;
        line += '::';
        line += queryId;
        logger.info('[BillingImplementation::AddInsertSemanticAnalysisDetailToMainFile] Adding new Semantic Analysis to MainFile');
        icModule.InconcertAddDataToGeneralFile(line, 'new');
    }
    AddUpdateSemanticAnalysisDetailToMainFile(vcc, processId, queryId, batchType, batchTypeId, updateTarget, updateValue) {
        let line = '';
        line += 'UPDATE';
        line += '::';
        line += vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'SEMANTIC_ANALYSIS';
        line += '::';
        line += processId;
        line += '::';
        line += queryId;
        switch (updateTarget) {
            case 'TotalValidRequest':
                line += '::';
                line += 'TotalValidRequest';
                line += '::';
                line += updateValue;
                break;
            case 'TotalInvalidRequest':
                line += '::';
                line += 'TotalInvalidRequest';
                line += '::';
                line += updateValue;
                break;
            case 'TotalResults':
                line += '::';
                line += 'TotalResults';
                line += '::';
                line += updateValue;
                break;
            case 'TotalPages':
                line += '::';
                line += 'TotalPages';
                line += '::';
                line += updateValue;
                break;
            case 'TotalItemsPerPage':
                line += '::';
                line += 'TotalItemsPerPage';
                line += '::';
                line += updateValue;
                break;
        }
        icModule.InconcertAddDataToGeneralFile(line, 'update');
    }
    AddCloseSemanticAnalysisDetailToMainFile(vcc, processId, queryId, batchType, batchTypeId, resultCode, resultDescription) {
        let line = '';
        line += 'CLOSE';
        line += '::';
        line += vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'SEMANTIC_ANALYSIS';
        line += '::';
        line += processId;
        line += '::';
        line += queryId;
        line += '::';
        line += resultCode;
        line += '::';
        line += resultDescription;
        icModule.InconcertAddDataToGeneralFile(line, resultCode ? 'success' : 'failed');
    }
    AddInsertCognitiveAnalysisDetailToMainFile(vcc, batchType, batchTypeId, processId, nluModel) {
        let line = '';
        line += 'NEW';
        line += '::';
        line += vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'COGNITIVE_ANALYSIS';
        line += '::';
        line += processId;
        line += '::';
        line += nluModel;
        logger.info('[BillingImplementation::AddInsertCognitiveAnalysisDetailToMainFile] Adding new Cognitive Analysis to MainFile');
        icModule.InconcertAddDataToGeneralFile(line, 'new');
    }
    AddUpdateCognitiveAnalysisDetailToMainFile(vcc, processId, nluModel, updateTarget, updateValue, batchType, batchTypeId) {
        let line = '';
        line += 'UPDATE';
        line += '::';
        line += vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'COGNITIVE_ANALYSIS';
        line += '::';
        line += processId;
        line += '::';
        line += nluModel;
        switch (updateTarget) {
            case 'TotalValidRequest':
                line += '::';
                line += 'TotalValidRequest';
                line += '::';
                line += 1;
                break;
            case 'TotalInvalidRequest':
                line += '::';
                line += 'TotalInvalidRequest';
                line += '::';
                line += 1;
                break;
            case 'IsValid':
                line += '::';
                line += 'IsValid';
                line += '::';
                line += updateValue ? 1 : 0;
                break;
            case 'ApiId':
                line += '::';
                line += 'ApiId';
                line += '::';
                line += updateValue;
                break;
            case 'IsDemo':
                line += '::';
                line += 'IsDemo';
                line += '::';
                line += updateValue ? 1 : 0;
                break;
            default:
                line += '::';
                line += updateTarget;
                line += '::';
                line += updateValue ? 1 : 0;
                break;
        }
        icModule.InconcertAddDataToGeneralFile(line, 'update');
    }
    AddCloseCognitiveAnalysisDetailToMainFile(vcc, processId, nluModel, batchType, batchTypeId, resultCode, resultDescription) {
        let line = '';
        line += 'CLOSE';
        line += '::';
        line += vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'COGNITIVE_ANALYSIS';
        line += '::';
        line += processId;
        line += '::';
        line += nluModel;
        line += '::';
        line += resultCode;
        line += '::';
        line += resultDescription;
        icModule.InconcertAddDataToGeneralFile(line, resultCode ? 'success' : 'failed');
    }
    SplitGeneralFileToSegmentedFiles() {
        icModule.InconcertSplitGeneralFileData();
    }
    SegmentedFilesUpload(installationId) {
        return icModule.InconcertSegmentedFileUpload(installationId)
            .then(result => {
            logger.info('[BillingImplementation::SegmentedFilesUpload] Getting result:', result);
            return Promise.resolve(true);
        })
            .catch(err => {
            logger.error('[BillingImplementation::SegmentedFilesUpload] Getting error:', err);
            return Promise.resolve(null);
        });
    }
    GetAccountsSubAccounts(vcc, updated = false) {
        let storage = '';
        let key = '';
        let installationId = '';
        let isForced = updated;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            if (result && result.installationId) {
                installationId = result.installationId;
            }
            else {
                throw 'NO_INSTALLATION_ID_PARAMETER';
            }
            storage = vcc;
            storage += ':billing:';
            storage += installationId;
            key = 'platformStatus';
            if (isForced === false) {
                return redis_service_1.Redis.GetPlatformStatusData(storage, key);
            }
            else {
                return redis_service_1.Redis.RemovePlatformStatusData(storage, key);
            }
        })
            .then((result) => {
            let accountList = null;
            if (result && result.accountList && result.subAccountList) {
                accountList = new account_model_1.AccountList(result.accountList, result.subAccountList);
            }
            return Promise.resolve(accountList);
        })
            .then((result) => {
            if (result === null) {
                return this.GetPlatformStatus(vcc);
            }
            return result;
        })
            .then((result) => {
            if (result && isForced) {
                return redis_service_1.Redis.SavePlatformStatusData(storage, key, result);
            }
            return Promise.resolve(result);
        })
            .then(result => {
            if (typeof result == 'boolean') {
                if (result) {
                    return this.GetAccountsSubAccounts(vcc);
                }
                else {
                    throw 'FAILED_SAVE_IN_REDI_PLATFORM_STATUS';
                }
            }
            else {
                return Promise.resolve(result);
            }
        })
            .catch((err) => {
            logger.error('[BillingImplementation::GetAccountsSubAccounts] Getting error:', err, 'vcc:', vcc);
            return Promise.resolve(null);
        });
    }
    GetPlatformAccounts(vcc) {
        let storage = '';
        let key = '';
        let installationId = '';
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            if (result && result.installationId) {
                installationId = result.installationId;
            }
            else {
                throw 'NO_INSTALLATION_ID_PARAMETER';
            }
            storage = vcc;
            storage += ':billing:';
            storage += installationId;
            key = 'platformStatus';
            return redis_service_1.Redis.GetPlatformStatusData(storage, key);
        }).then((result) => {
            logger.info('[BillingImplementation::GetPlatformAccounts] Getting result:', result, 'after external api/account/get_platform_accounts request');
            let platformAccount = [];
            if (result && result.platformAccounts) {
                platformAccount = result.platformAccounts;
            }
            return Promise.resolve(platformAccount);
        }).catch((err) => {
            logger.error('[BillingImplementation::GetPlatformAccounts] Getting error:', err, 'vcc:', vcc);
            return Promise.resolve(null);
        });
    }
    ActivatePlatform(account, token, user) {
        let systemInformationData = null;
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[BillingImplementation::ActivatePlatform] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let platformHash = '';
            if (systemInformationData) {
                platformHash = this.GenerateMD5Content(systemInformationData);
            }
            let body = {
                account: account,
                token: token,
                user: user,
                data: result,
                platformHash: platformHash
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_PLATFORM_ACTIVATE', body);
        }).then(result => {
            logger.info('[BillingImplementation::ActivatePlatform] Getting result:', result, 'from external server');
            return Promise.resolve(result);
        }).catch((err) => {
            logger.error('[BillingImplementation::ActivatePlatform] Getting error:', err);
            return Promise.resolve(null);
        });
    }
    GetPlatformStatus(vcc) {
        let storage = '';
        let key = '';
        let installationId = '';
        let mustSaveRedis = false;
        let platformStatusData = null;
        let systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[BillingImplementation::GetPlatformStatus] Getting result:', result, 'after GetVersion');
            if (result && result.installationId) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then((result) => {
            systemInformationData = result;
            storage = vcc;
            storage += ':billing:';
            storage += installationId;
            key = 'platformStatus';
            return redis_service_1.Redis.GetPlatformStatusData(storage, key);
        })
            .then((result) => {
            if (result
                && result.platformStatus && result.platformStatus.macAddress && result.platformStatus.macAddress === systemInformationData.macAddress
                && result.accountList && result.accountList.length
                && result.subAccountList && result.subAccountList.length) {
                platformStatusData = new platform_model_1.PlatformSatusData();
                platformStatusData = result;
                return platformStatusData;
            }
            else {
                mustSaveRedis = true;
                let body = systemInformationData;
                return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_PLATFORM_GET_STATUS', body);
            }
        })
            .then((result) => {
            if (result && result.status === false)
                throw result.error;
            if (result && result.status && result.data && result.data.platformStatus && result.data.accountList && result.data.subAccountList) {
                platformStatusData = result.data;
            }
            if (mustSaveRedis && platformStatusData && platformStatusData.platformStatus && platformStatusData.accountList && platformStatusData.subAccountList) {
                return redis_service_1.Redis.SavePlatformStatusData(storage, key, platformStatusData);
            }
            return true;
        })
            .then((result) => {
            return Promise.resolve(platformStatusData);
        })
            .catch((err) => {
            logger.error('[BillingImplementation::GetPlatformStatus] Getting error:', err);
            return Promise.reject(err);
        });
    }
    GetPlatformAvailableLicenses() {
        let systemInformationData = null;
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[BillingImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                macAddress: systemInformationData.macAddress,
                baseboardSerialNumber: systemInformationData.baseboardSerialNumber
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_PLATFORM_AVAILABLE_LICENSE', body);
        }).then(result => {
            logger.info('[BillingImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'from external server');
            let products = [];
            if (result && result.data) {
                products = result.data;
            }
            return Promise.resolve(products);
        }).catch((err) => {
            logger.error('[BillingImplementation::GetPlatformAvailableLicenses] Getting error:', err);
            return Promise.resolve(null);
        });
    }
    GetSystemInformation(guid) {
        let systemInformationData = {
            installationId: guid,
            macAddress: '',
            publicIp: ''
        };
        return promises_shared_1.DummyPromise()
            .then((result) => {
            logger.info('[BillingImplementation::GetSystemInformation] guid:', guid, 'getting result serial:', result.serial);
            let sys = sy.networkInterfaces();
            return sys;
        }).then((result) => {
            if (result && result.length) {
                logger.info('[BillingImplementation::GetSystemInformation] guid:', guid, 'getting result[0].mac:', result[0].mac);
                let myNetworkInterface = null;
                myNetworkInterface = result.filter(r => r.operstate === 'up')[0];
                systemInformationData.macAddress = myNetworkInterface.mac;
                systemInformationData.publicIp = myNetworkInterface.ip4;
            }
            return Promise.resolve(systemInformationData);
        }).catch((err) => {
            logger.error('[BillingImplementation::GetSystemInformation] guid:', guid, 'getting error:', err);
            return Promise.resolve(false);
        });
    }
    GenerateLicenseFileData(guid) {
        let installationId = '';
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            if (result && result.installationId) {
                installationId = result.installationId;
            }
            else {
                throw 'NO_INSTALLATION_ID_PARAMETER';
            }
            return this.GetSystemInformation(installationId);
        }).then((result) => {
            logger.info('[BillingImplementation::GenerateLicenseFileData] guid:', guid, 'getting result:', result, 'after this.GetSystemInformation');
            return Promise.resolve(result);
        }).then((result) => {
            let licenseFileContent = '';
            if (result) {
                licenseFileContent = this.GenerateMD5Content(result);
                logger.info('[BillingImplementation::GenerateLicenseFileData] getting licenseFileContent:', licenseFileContent);
            }
            return Promise.resolve(licenseFileContent);
        }).catch((err) => {
            logger.error('[BillingImplementation::GenerateLicenseFileData] guid:', guid, 'getting error:', err);
            return Promise.resolve('ERROR');
        });
    }
    GenerateMD5Content(data) {
        let result = '';
        if (data) {
            result += data.installationId;
            result += data.macAddress;
            result = md5(result);
        }
        return result;
    }
    CreateVCC(vcc, productVCC) {
        let installationId = '';
        let systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[BillingImplementation::GetPlatformStatus] Getting result:', result, 'after GetVersion');
            if (result && result.installationId) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let products = [];
            if (productVCC.products && productVCC.products.length) {
                products = productVCC.products.map((x) => {
                    return x.product;
                });
            }
            let account = {};
            account.vcc = vcc;
            account.accountId = productVCC.accountId;
            account.subAccountId = productVCC.subAccountId;
            account.asignableBalance = productVCC.asignableBalance;
            account.installationId = installationId;
            account.transcriptionRatePerHour = productVCC.transcriptionRatePerHour;
            account.transcriptionRatePerSecond = productVCC.transcriptionRatePerSecond;
            account.semanticAnalysisRatePerTransaction = productVCC.semanticAnalysisRatePerTransaction;
            account.cognitiveAnalysisRatePerTransaction = productVCC.cognitiveAnalysisRatePerTransaction;
            let body = {
                vcc: vcc,
                platformHash: plaformHash,
                account: productVCC.accountId,
                subAccount: productVCC.subAccountId,
                products: products,
                accountVcc: account
            };
            logger.info('[BillingImplementation::CreateVCC] Getting body:', JSON.stringify(body), 'before sendig request to api/vcc/create_vcc');
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_VCC_CREATE', body);
        })
            .then((result) => {
            let rpta = {
                status: result.status,
                description: result.description ? result.description : null,
                error: result.error ? result.error : null
            };
            return Promise.resolve(rpta);
        })
            .catch((err) => {
            logger.error('[BillingImplementation::GetPlatformStatus] Getting error:', err);
            return Promise.resolve(false);
        });
    }
    UpdateVCC(vcc, productVCC) {
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            if (result && result.installationId) {
                let products = [];
                if (productVCC.products && productVCC.products.length) {
                    products = productVCC.products.map((x) => {
                        return x.product;
                    });
                }
                let body = {
                    vcc: vcc,
                    products: products
                };
                logger.info('[BillingImplementation::UpdateVCC] Getting body:', JSON.stringify(body), 'before sendig request to api/vcc/UpdateVCC');
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_VCC_UPDATE', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then((result) => {
            let rpta;
            if (result) {
                rpta = result.status;
            }
            else {
                rpta = false;
            }
            return Promise.resolve(rpta);
        })
            .catch((err) => {
            logger.error('[BillingImplementation::UpdateVCC] Getting error:', err);
            return Promise.resolve(false);
        });
    }
    GetVCCStatusDetail(vcc) {
        let installationId = '';
        let vccStatusData = null;
        let systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[BillingImplementation::GetVCCStatusDetail] Getting result:', result, 'after GetVersion');
            if (result && result.installationId) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let body = {
                vcc,
                platformHash: plaformHash,
                installationId: installationId
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_VCC_GET_STATUS', body);
        }).then(result => {
            if (result.data) {
                vccStatusData = result.data;
            }
            else {
                throw 'PLATFORM_IS_NOT_ACTIVATED';
            }
            return Promise.resolve(vccStatusData);
        }).catch((err) => {
            logger.error('[BillingImplementation::GetVCCStatusDetail] Getting error:', err);
            return Promise.resolve(err);
        });
    }
    GetVCCStatus(vcc) {
        let installationId = '';
        let vccStatusData = null;
        let systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[BillingImplementation::GetVCCStatus] Getting result:', result, 'after GetVersion');
            if (result && result.installationId) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let body = {
                vcc: vcc,
                platformHash: plaformHash,
                installationId: installationId
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_VCC_GET_STATUS', body);
        }).then(result => {
            if (result.data) {
                vccStatusData = result.data;
            }
            else {
                throw 'PLATFORM_IS_NOT_ACTIVATED';
            }
            return Promise.resolve(vccStatusData);
        }).catch((err) => {
            logger.error('[BillingImplementation::GetPlatformStatus] Getting error:', err);
            return Promise.resolve(false);
        });
    }
    GetVCCRates(vcc, processId) {
        let installationId = '';
        let systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            if (result && result.installationId) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                vcc: vcc,
                processId: processId
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_VCC_GET_RATES', body);
        }).then((result) => {
            logger.info('[BillingImplementation::GetVCCRates] Getting result:', result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE', ' from external server');
            let vccRates = null;
            if (result && result.data) {
                vccRates = result.data;
            }
            return Promise.resolve(vccRates);
        }).catch((err) => {
            logger.error('[BillingImplementation::GetVCCRates] Getting error:', err, 'vcc:', vcc);
            return Promise.resolve(null);
        });
    }
    GetVCCBalance(vcc) {
        let installationId = '';
        let systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            if (result && result.installationId) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let body = {
                platformHash: plaformHash,
                installationId: installationId,
                vcc: vcc
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_VCC_GET_BALANCE', body);
        }).then((result) => {
            logger.info('[BillingImplementation::GetVCCBalance] Getting result:', result.length, 'from external server');
            let vccBalance = null;
            if (result && result.data) {
                vccBalance = result.data;
            }
            return Promise.resolve(vccBalance);
        }).catch((err) => {
            console.log(err);
            logger.error('[BillingImplementation::GetVCCBalance] Getting error:', err, 'vcc:', vcc);
            return Promise.resolve(null);
        });
    }
    GetInstallationData(installationId) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            let body = {
                installationId: installationId
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_PLATFORM_INSTALLATION_DATA', body);
        }).then((result) => {
            logger.info('[BillingImplementation::GetVCCBalance] Getting result:', result.length, 'from external server');
            let response = [];
            if (result.status && result.data) {
                response = result.data;
            }
            return Promise.resolve(response);
        }).catch((err) => {
            console.log(err);
            logger.error('[BillingImplementation::GetVCCBalance] Getting error:', err);
            return Promise.resolve(null);
        });
    }
    GetExternalLibrary() {
        let icModule = null;
        icModule = require('intermediate-component-sa');
        return icModule;
    }
    SaveVCCBalance(vcc, account, subAccount, transcriptionOld, transcriptionNew, transcriptonDetail, cognitiveOld, cognitiveNew, cognitiveDetail, semanticOld, semanticNew, semanticDetail, freeOld, freeNew, freeDetail) {
        let installationId = '';
        let systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            if (result && result.installationId) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let data = [];
            data.push({ target: 'transcription', total: transcriptionNew, balance: transcriptionOld, operation: transcriptonDetail });
            data.push({ target: 'semantic_analysis', total: semanticNew, balance: semanticOld, operation: semanticDetail });
            data.push({ target: 'cognitive_analysis', total: cognitiveNew, balance: cognitiveOld, operation: cognitiveDetail });
            data.push({ target: 'free', total: freeNew, balance: freeOld, operation: freeDetail });
            let body = {
                platformHash: plaformHash,
                installationId: installationId,
                vcc: vcc,
                account: account,
                subAccount: subAccount,
                data: data
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_VCC_BALANCE_ALLOCATION', body);
        }).then((result) => {
            logger.info('[BillingImplementation::SaveVCCBalance] Getting result:', result, 'from external server');
            return Promise.resolve(result.status);
        }).catch((err) => {
            logger.error('[BillingImplementation::SaveVCCBalance] Getting error:', err, 'vcc:', vcc);
            return Promise.resolve(null);
        });
    }
}
exports.BillingImplementation = BillingImplementation;
