"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetCognitiveService = exports.CognitiveAnalysisImplementation = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const moment = require("moment");
const config_1 = require("../config/config");
const promises_shared_1 = require("../shared/promises.shared");
const file_manager_1 = require("../shared/file-manager");
const billing_implementation_1 = require("./billing.implementation");
const luis_service_implementation_1 = require("./luis-service.implementation");
const rasa_service_implementation_1 = require("./rasa-service.implementation");
const billing_batch_model_1 = require("../models/billing-batch.model");
const audio_service_1 = require("../services/audio.service");
const rasa_api_service_implementation_1 = require("./rasa-api-service.implementation");
let logger = log4js.getLogger('Task');
let cognitiveService;
const CURRENT_SYSTEM_SERVICE = 'COGNITIVE_ANALYSIS';
const CURRENT_IMPLEMENTATION = 'CognitiveAnalysisImplementation';
let useRasaApi = config_1.USE_RASA_API;
class CognitiveAnalysisImplementation {
    constructor() {
        this.errorData = new billing_batch_model_1.BatchError();
    }
    Execute(executionCode, rateRecursive, availableBatch, cancelledBatch) {
        let executeIndex = 1;
        let ratio = config_1.RATIO_COGNITIVE_ANALYSIS;
        let alowedRatio = 0;
        let notAlowedRatio = 0;
        let executeResult = true;
        return promises_shared_1.SleepPromise(2)
            .then(result => {
            return redis_service_1.Redis.LRange('CognitiveAnalysis', 0, ratio - 1);
        })
            .then((result) => {
            let promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            logger.info('[CognitiveAnalysisImplementation::Execute] CognitiveAnalysis lrange result: ' + result);
            result.map(r => {
                let arrQueue = r.split(':');
                if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[7].toLowerCase())) {
                    alowedRatio++;
                }
                else {
                    notAlowedRatio++;
                }
            });
            while (ratio >= executeIndex) {
                promiseArray.push(this.CognitiveAnalysisExecute(executionCode, availableBatch, cancelledBatch));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then((results) => {
            if (results.filter(item => item === false).length > notAlowedRatio) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('CognitiveAnalysis', 0, ratio - 1);
        }).then((result) => {
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return executeResult;
        }).then(result => {
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::Execute] getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    CognitiveAnalysisExecute(executionCode, availableBatch, cancelledBatch) {
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let type = '';
        let typeId = '';
        let currentQueueId = '';
        let interactionCognitive;
        let queueProgressStorage = '';
        let filePath = '';
        return redis_service_1.Redis.RemoveQueueL('CognitiveAnalysis')
            .then((result) => {
            let arrQueue = result.split(':');
            let promiseArray = [];
            if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[8].toLowerCase())) {
                promiseArray.push(Promise.resolve('ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('CognitiveAnalysisExecute', result));
                promiseArray.push(Promise.resolve(result));
            }
            else {
                promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('CognitiveAnalysis', result));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[8], 'COGNITIVE_ANALYSIS', 'PAUSED'));
            }
            return Promise.all(promiseArray);
        })
            .then((result) => {
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            let queueId = result[2];
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[CognitiveQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                let item = currentQueueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    type = item[7] ? item[7] : null;
                    typeId = item[8] ? item[8] : null;
                    if (type && typeId)
                        queueProgressStorage = vcc + ':' + processId + ':' + type + ':' + typeId;
                    if (type && !typeId)
                        queueProgressStorage = vcc + ':' + processId + ':' + type;
                    logger.info('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', queueProgressStorage);
                    return redis_service_1.Redis.GetCognitiveInteraction(currentQueueId);
                }
                else {
                    logger.error('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then((result) => {
            if (!result)
                logger.warn('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error: NOT_FOUND_INTERACTION_COGNITIVE');
            if (result) {
                if (cancelledBatch.find(b => b.VCC === result.vcc && b.BatchTypeId.toLowerCase() === (result.studyId ? result.studyId : result.evaluationPlanId).toLowerCase())) {
                    result.status = -2;
                    result.statusDescription = 'REMOVED_BY_CANCELLED';
                    result.retry = config_1.RETRY_COGNITIVE_ANALYSIS;
                    result.errorFunction = 'CognitiveAnalysisExecute';
                    result.errorMessage = 'REMOVED_BY_CANCELLED';
                    result.errorDetail = 'REMOVED_BY_CANCELLED';
                }
            }
            interactionCognitive = result;
            return file_manager_1.CreateFolder(config_1.COGNITIVE_ANALYSIS_FOLDER + '/' + vcc);
        }).then((result) => {
            if (!result)
                throw 'ERROR_CREATE_COGNITIVE_ANALYSIS_FOLDER';
            if (interactionCognitive && interactionCognitive.statusDescription === 'pendingPhrases') {
                logger.info('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'pendingPhrases');
                return this.GetPhrasesFromIdathaJson(interactionCognitive, currentQueueId, executionCode);
            }
            else if (interactionCognitive && interactionCognitive.statusDescription === 'pendingNLU') {
                logger.info('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'pendingNLU');
                return this.GetCognitive(interactionCognitive, currentQueueId, executionCode);
            }
            return Promise.resolve(null);
        }).then((resultInteraction) => {
            if (interactionCognitive)
                return redis_service_1.Redis.SaveCognitiveInteraction(currentQueueId, interactionCognitive);
            return Promise.resolve(null);
        }).then((result) => {
            let promiseArray = [];
            if (result && interactionCognitive?.status === 6 && interactionCognitive?.statusDescription === 'ready') {
                logger.info('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'READY');
                this.CloseCognitiveModel(currentQueueId, true, 'SUCCED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
                if (type === 'tipification') {
                    interactionCognitive.statusDescription = 'pendingAnalysis';
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('TipificationAnalysis', currentQueueId));
                    promiseArray.push(redis_service_1.Redis.SaveTipificationInteraction(currentQueueId, interactionCognitive));
                }
            }
            else if (result && interactionCognitive?.status === 6 && interactionCognitive?.statusDescription === 'pendingNLU' && interactionCognitive.retry < config_1.RETRY_COGNITIVE_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.CreateQueueL('CognitiveAnalysis', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else if (result && interactionCognitive && interactionCognitive.status === -2 && interactionCognitive.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                this.CloseCognitiveModel(currentQueueId, false, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else if (result && interactionCognitive && interactionCognitive.status === -6 && interactionCognitive.statusDescription === 'ERROR_FILE_JSON') {
                this.CloseCognitiveModel(currentQueueId, false, 'JSON_PROBLEM');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else if (interactionCognitive && interactionCognitive.retry >= config_1.RETRY_COGNITIVE_ANALYSIS) {
                logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                this.CloseCognitiveModel(currentQueueId, false, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else if (currentQueueId && !interactionCognitive) {
                logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                this.CloseCognitiveModel(currentQueueId, false, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else {
                logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.all(promiseArray);
        }).then((result) => {
            let promiseArray = [];
            if (interactionCognitive
                && interactionCognitive.status === 6
                && interactionCognitive.statusDescription === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteCognitiveInteraction(currentQueueId));
                if (['study', 'template'].includes(type)) {
                    logger.debug('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for study:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyCognitiveInteraction(currentQueueId, interactionCognitive, executionCode));
                }
                else if (['ability', 'tipification'].includes(type)) {
                    logger.debug('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for evaluationplan:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyCognitiveAbility(currentQueueId, interactionCognitive, executionCode));
                }
                else {
                    logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TYPE_IS_NOT_FOUND', type);
                }
            }
            else if (interactionCognitive && interactionCognitive.retry >= config_1.RETRY_COGNITIVE_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteCognitiveInteraction(currentQueueId));
                if (['study', 'template'].includes(type)) {
                    logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for study:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyCognitiveInteraction(currentQueueId, interactionCognitive, executionCode));
                }
                else if (['ability', 'tipification'].includes(type)) {
                    logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for evaluationplan:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyCognitiveAbility(currentQueueId, interactionCognitive, executionCode));
                }
                else {
                    logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TYPE_IS_NOT_FOUND', type);
                }
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, typeId, ['study', 'template'].includes(type) ? typeId : null, CURRENT_IMPLEMENTATION, interactionCognitive.id, interactionCognitive.nluId, this.errorData.errorFunction, this.errorData.errorMessage, this.errorData.errorDetail));
            }
            return Promise.all(promiseArray);
        }).then((result) => {
            return Promise.resolve(true);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    GetPhrasesFromIdathaJson(interaction, currentQueueId, executionCode) {
        let existsDocument = false;
        let totalPhrases = 0;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return GetCognitiveService(interaction.vcc, null);
        })
            .then(result => {
            cognitiveService = result;
            return this.GetIdathaJson(interaction, currentQueueId, executionCode);
        })
            .then((result) => {
            if (!result)
                throw 'ERROR_NOT_FOUND_FILE_JSON';
            if (result?.existsDocument) {
                existsDocument = result.existsDocument;
                totalPhrases = result.phrases.length;
                delete (result.existsDocument);
                return Promise.resolve(null);
            }
            let transcription = this.SortJson(result);
            let phrases = this.SplitTranscriptionContentInPhrases(cognitiveService.GetCognitiveEngineCharsPerPhrase(), transcription);
            logger.info('[CognitiveAnalysisImplementation::GetPhrasesFromIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting phrases:', phrases.length);
            return Promise.resolve(phrases);
        }).then(result => {
            if (existsDocument)
                return Promise.resolve(null);
            let phrases;
            if (result && result.length) {
                totalPhrases = result.length;
                let regex = /\.(mp3|wav|ogg|json)$/;
                let jsonFileName = interaction.fileName.replace(regex, '.json');
                let path = config_1.COGNITIVE_ANALYSIS_FOLDER;
                path += '/';
                path += interaction.vcc;
                path += '/';
                path += jsonFileName;
                phrases = { phrases: result };
                let stringPhrases = JSON.stringify(phrases);
                return file_manager_1.WriteFileContent(path, stringPhrases, 'utf-8');
            }
            return Promise.resolve(false);
        }).then(result => {
            if (result || existsDocument) {
                interaction['status'] = 6;
                interaction['totalPhrases'] = interaction['totalPhrases'] ? interaction['totalPhrases'] : totalPhrases;
                interaction['statusDescription'] = 'pendingNLU';
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
                this.errorData.CleanErrorData();
            }
            else {
                logger.warn('[CognitiveAnalysisImplementation::GetPhrasesFromIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', 'NOT_FOUND_PHASES');
                interaction['status'] = -6;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = 'GetPhrasesFromIdathaJson';
                interaction['errorMessage'] = 'NOT_FOUND_PHASES';
                this.errorData.SetErrorData('GetPhrasesFromIdathaJson', interaction['errorMessage'], interaction['errorMessage']);
            }
            return Promise.resolve(interaction);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::GetPhrasesFromIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            interaction['status'] = -6;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('GetPhrasesFromIdathaJson', errorMessage, err);
            return Promise.resolve(interaction);
        });
    }
    GetIdathaJson(interaction, currentQueueId, executionCode) {
        let existsDocument = false;
        let regex = /\.(mp3|wav|ogg|json)$/;
        let jsonFileName = interaction.fileName.replace(regex, '.json');
        return promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[CognitiveAnalysisImplementation::GetIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'searching on cognitive folder.');
            let path = config_1.COGNITIVE_ANALYSIS_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then((result) => {
            if (!result) {
                logger.info('[CognitiveAnalysisImplementation::GetIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'searching on semantic folder.');
                let path = config_1.SEMANCTIC_ANALYSIS_FOLDER;
                path += '/';
                path += interaction.vcc;
                path += '/';
                path += jsonFileName;
                return file_manager_1.ReadFileContent(path, 'utf-8', true);
            }
            else
                existsDocument = true;
            return Promise.resolve(result);
        })
            .then((result) => {
            if (!result) {
                logger.info('[CognitiveAnalysisImplementation::GetIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'searching on idatha server.');
                let path = config_1.SEMANCTIC_ANALYSIS_FOLDER;
                path += '/';
                path += interaction.vcc;
                path += '/';
                path += jsonFileName;
                return audio_service_1.GetIdathaDocument(interaction.vcc, interaction, path)
                    .then(res => {
                    if (res) {
                        let path = config_1.SEMANCTIC_ANALYSIS_FOLDER;
                        path += '/';
                        path += interaction.vcc;
                        path += '/';
                        path += jsonFileName;
                        logger.info('[CognitiveAnalysisImplementation::GetIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'searching json:', path);
                        return file_manager_1.ReadFileContent(path, 'utf-8');
                    }
                    else {
                        logger.info('[CognitiveAnalysisImplementation::GetIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'not found on idatha server.');
                        throw 'ERROR_FILE_JSON';
                    }
                });
            }
            return Promise.resolve(result);
        })
            .then((result) => {
            let jsonContent = null;
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
            }
            catch (ex) {
                jsonContent = null;
                this.errorData.SetErrorData('GetIdathaJson', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            jsonContent.existsDocument = existsDocument;
            return Promise.resolve(jsonContent);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::GetIdathaJson] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            throw err;
        });
    }
    GetCognitive(interaction, currentQueueId, executionCode) {
        let item = currentQueueId.split(':');
        let type = item[7] ? item[7] : null;
        let path = '';
        let isLast = false;
        let azure = {};
        let phrase = '';
        let index = 0;
        let currentNlu = {};
        let currentSubscriptionKey = {};
        let apiIdLabel = null;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return GetCognitiveService(interaction.vcc, null);
        })
            .then(result => {
            cognitiveService = result;
            apiIdLabel = cognitiveService.GetCognitiveEngineName() === 'LUIS' ? 'LuisApi' : 'RasaApi';
            let regex = /\.(mp3|wav|ogg|json)$/;
            let jsonFileName = interaction.fileName.replace(regex, '.json');
            path = config_1.COGNITIVE_ANALYSIS_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return this.GetNluResults(interaction, path, currentQueueId, executionCode);
        })
            .then(result => {
            if (!result || !result.status)
                throw 'ERROR_SAVE_DOCUMENT';
            isLast = result.isLast;
            if (isLast && type !== 'tipification')
                return this.EvaluateCognitive(path, interaction, executionCode, currentQueueId);
            return Promise.resolve(null);
        })
            .then((result) => {
            if (isLast) {
                interaction.succedItems = result ? result : [];
                if (!result)
                    logger.warn('[CognitiveAnalysisImplementation::GetCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', 'NOT_FOUND_SCORE');
            }
            return Promise.resolve(true);
        })
            .then(result => {
            if (result) {
                interaction['status'] = 6;
                interaction['statusDescription'] = isLast ? 'ready' : 'pendingNLU';
                interaction['totalReadPhrases'] = interaction['totalReadPhrases'] ? interaction['totalReadPhrases'] + 1 : 1;
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
                this.errorData.CleanErrorData();
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            if (errorMessage === 'NOT_FOUND_PHRASE') {
                logger.warn('[CognitiveAnalysisImplementation::GetCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                interaction['status'] = 6;
                interaction['statusDescription'] = 'ready';
                interaction['totalReadPhrases'] = interaction['totalReadPhrases'] ? interaction['totalReadPhrases'] + 1 : 1;
                interaction['retry'] = 0;
            }
            else if (errorMessage === 'OUT_OF_VOLUME_QUOTA') {
                logger.error('[CognitiveAnalysisImplementation::GetCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                interaction['status'] = -6;
                interaction['retry'] = config_1.RETRY_COGNITIVE_ANALYSIS;
            }
            else {
                logger.error('[CognitiveAnalysisImplementation::GetCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                interaction['status'] = -6;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            }
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('GetCognitive', errorMessage, err);
            return Promise.resolve(interaction);
        });
    }
    GetNluResults(interaction, path, currentQueueId, executionCode) {
        let index = 0;
        let isLast = false;
        let alreadyEvaluated = false;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then(result => {
            let jsonContent;
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
            }
            catch (ex) {
                jsonContent = null;
                this.errorData.SetErrorData('GetNluResultsFromCognitiveEngine', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return Promise.resolve(jsonContent);
        })
            .then(result => {
            if (!result)
                throw 'ERROR_NOT_FOUND_FILE_JSON';
            if (result.phrases)
                alreadyEvaluated = !result.phrases.map(p => p[interaction.nluId] ? true : false).includes(false) ? true : false;
            index = interaction.totalReadPhrases ? interaction.totalReadPhrases : 0;
            isLast = interaction.totalPhrases - 1 <= index ? true : false;
            return this.GetNluResultsFromCognitiveEngine(interaction, path, result, currentQueueId, executionCode);
        })
            .then(result => {
            return Promise.resolve({ status: result ? true : false, isLast });
        })
            .catch(err => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.info(`[CognitiveAnalysisImplementation::GetNluResults] executionCode: ${executionCode} getting queueId: ${currentQueueId} getting error: ${errorMessage}`);
            return Promise.reject(err);
        });
    }
    GetNluResultsFromCognitiveEngine(interaction, path, content, currentQueueId, executionCode) {
        let phrase = '';
        let index = 0;
        let currentNlu = {};
        let currentSubscriptionKey = {};
        let apiIdLabel = cognitiveService.GetCognitiveEngineName() === 'LUIS' ? 'LuisApi' : 'RasaApi';
        return promises_shared_1.DummyPromise()
            .then(_ => {
            index = interaction.totalReadPhrases ? interaction.totalReadPhrases : 0;
            phrase = content.phrases[index]?.content ? content.phrases[index].content : null;
            if (!phrase)
                throw 'NOT_FOUND_PHRASE';
            return Promise.resolve(true);
        })
            .then(result => {
            return this.GetNlu(interaction.vcc, interaction.nluId, currentQueueId);
        })
            .then(result => {
            if (!result)
                throw 'NLU_MODEL_NOT_FOUND';
            currentNlu = result;
            if (apiIdLabel === 'LuisApi') {
                return this.GetAzureSubscriptionKey(interaction.vcc, currentNlu.subscriptionKey);
            }
            else {
                return Promise.resolve(null);
            }
        })
            .then((result) => {
            if (apiIdLabel === 'LuisApi') {
                currentSubscriptionKey = result;
                if (!currentSubscriptionKey)
                    throw 'SUSCRIPTION_KEY_NOT_FOUND';
                this.UpdateCognitiveModel(currentQueueId, apiIdLabel, currentSubscriptionKey.subscriptionKey);
                this.UpdateCognitiveModel(currentQueueId, 'IsDemo', currentSubscriptionKey.isDemo);
            }
            else {
                currentSubscriptionKey.subscriptionRegion = null;
                currentSubscriptionKey.subscriptionKey = null;
                this.UpdateCognitiveModel(currentQueueId, apiIdLabel, config_1.RASA_IP + ':' + config_1.RASA_PORT);
                this.UpdateCognitiveModel(currentQueueId, 'IsDemo', false);
            }
            return this.ModelQuery(cognitiveService, currentSubscriptionKey.subscriptionRegion, currentSubscriptionKey.subscriptionKey, interaction.appId, phrase, interaction.nluId, executionCode, currentQueueId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_COGNITIVE_REQUEST_RESULT_NOT_FOUND';
            if (result && result.error)
                throw result.error;
            return this.SaveDocumentCognitive(index, path, result, interaction.nluId, executionCode, currentQueueId);
        })
            .catch(err => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::GetNluResultsFromCognitiveEngine] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.reject(err);
        });
    }
    GetNlu(vcc, name, queue) {
        logger.info('[CognitiveAnalysisImplementation::GetNLU] getting vcc:', vcc, 'name:', name);
        let cacheUpdateNeeded = false;
        let arrQueue = queue.split(':');
        let nluRedisKey = vcc + ":" + arrQueue[7] + ":" + arrQueue[8] + ":" + name;
        return redis_service_1.Redis.GetNlu(nluRedisKey)
            .then(result => {
            if (result) {
                return Promise.resolve(result);
            }
            else {
                cacheUpdateNeeded = true;
                return mssql_service_1.MSSql.GetNLU(vcc, name);
            }
        })
            .then(result => {
            if (result) {
                if (cacheUpdateNeeded)
                    redis_service_1.Redis.SaveNLU(nluRedisKey, result);
                this.errorData.CleanErrorData();
                return Promise.resolve(result);
            }
            else {
                throw 'ERROR_NLU_MODEL_NOT_FOUND';
            }
        })
            .catch((err) => {
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CognitiveAnalysisImplementation::GetNLU] getting errorMsg:', errorMsg);
            this.errorData.SetErrorData('GetNLU', errorMsg, err);
            return Promise.resolve(null);
        });
    }
    GetAzureSubscriptionKey(vcc, azureSubscriptionKey) {
        logger.info('[CognitiveAnalysisImplementation::GetAzureSubscriptionKey] getting vcc: ', vcc, ' azureSubscriptionKey: ', azureSubscriptionKey);
        let cacheUpdateNeeded = false;
        let azureSubscriptionKeyRedisKey = vcc + ":" + azureSubscriptionKey;
        return redis_service_1.Redis.GetSubscriptionKey(azureSubscriptionKeyRedisKey)
            .then(result => {
            if (result) {
                return Promise.resolve(result);
            }
            else {
                cacheUpdateNeeded = true;
                return mssql_service_1.MSSql.GetAzureSubscriptionKeyByKey(vcc, azureSubscriptionKey);
            }
        })
            .then(result => {
            if (result) {
                if (cacheUpdateNeeded)
                    redis_service_1.Redis.SaveSubscriptionKey(azureSubscriptionKeyRedisKey, result);
                this.errorData.CleanErrorData();
                return Promise.resolve(result);
            }
            else {
                throw 'ERROR_AZURE_SUBSCRIPTIONKEY_NOT_FOUND';
            }
        })
            .catch((err) => {
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CognitiveAnalysisImplementation::GetAzureSubscriptionKey] getting errorMsg:', errorMsg);
            this.errorData.SetErrorData('GetAzureSubscriptionKey', errorMsg, err);
            return Promise.resolve(null);
        });
    }
    SortJson(jsonobj) {
        let outlength;
        let inlength;
        let i;
        let audioInit;
        let obj = {};
        obj.transcription = [];
        obj.summarySpeechData = {};
        obj.confidence = {};
        obj.init = '';
        let json = jsonobj;
        audioInit = json.words[0].speaker;
        obj.init = audioInit;
        obj.nlp = {};
        outlength = json.out_sentences.length;
        inlength = json.in_sentences.length;
        if (outlength >= inlength) {
            for (i in json.out_sentences) {
                if (i < inlength && inlength > 0) {
                    if (audioInit === 'out' && inlength > 0) {
                        obj.transcription.push({
                            out: json.out_sentences[i].text,
                            in: json.in_sentences[i] === 'undefined' ? '' : json.in_sentences[i].text
                        });
                    }
                    else if (audioInit === 'in' && inlength > 0) {
                        obj.transcription.push({
                            in: json.in_sentences[i].text,
                            out: json.out_sentences[i].text
                        });
                    }
                }
                else {
                    if (audioInit === 'out' && inlength >= 0) {
                        obj.transcription.push({
                            out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text,
                            in: ''
                        });
                    }
                    else if (audioInit === 'in' && inlength >= 0) {
                        obj.transcription.push({
                            in: '',
                            out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text
                        });
                    }
                }
            }
        }
        else if (outlength <= inlength) {
            for (i in json.in_sentences) {
                if (i < outlength && outlength > 0) {
                    if (audioInit === 'out' && outlength > 0) {
                        obj.transcription.push({
                            out: json.out_sentences[i].text,
                            in: json.in_sentences[i].text
                        });
                    }
                    else if (audioInit === 'in' && outlength > 0) {
                        obj.transcription.push({
                            in: json.in_sentences[i].text,
                            out: json.out_sentences[i].text
                        });
                    }
                }
                else {
                    if (audioInit === 'out' && inlength >= 0) {
                        obj.transcription.push({
                            out: '',
                            in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                        });
                    }
                    else if (audioInit === 'in' && inlength >= 0) {
                        obj.transcription.push({
                            in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                            out: ''
                        });
                    }
                }
            }
        }
        obj.nlp.in = json.pnl_scores_in ? json.pnl_scores_in : 0;
        obj.nlp.out = json.pnl_scores_out ? json.pnl_scores_out : 0;
        var dataTotalIn = {
            duration: 0,
            silence: 0
        }, dataTotalOut = {
            duration: 0,
            silence: 0
        }, data = json.words, len = data.length, word, inDuration = 0, inSilence = 0, outDuration = 0, outSilence = 0;
        for (i = 0; i < len; i++) {
            word = data[i];
            if (word.speaker === 'in') {
                inDuration += word.duration;
                inSilence += word.silence;
            }
            else {
                outDuration += word.duration;
                outSilence += word.silence;
            }
        }
        dataTotalIn.duration = inDuration.toFixed(2) * 1;
        dataTotalIn.silence = inSilence.toFixed(2) * 1;
        dataTotalOut.duration = outDuration.toFixed(2) * 1;
        dataTotalOut.silence = outSilence.toFixed(2) * 1;
        obj.summarySpeechData = {
            dataTotalIn: dataTotalIn,
            dataTotalOut: dataTotalOut
        };
        obj.duration = json.duration;
        obj.words = json.words;
        obj.topics = json.topics;
        obj.confidence.in_confidence = json.in_confidence ? json.in_confidence : 0;
        obj.confidence.out_confidence = json.out_confidence ? json.out_confidence : 0;
        return obj;
    }
    SplitTranscriptionContentInPhrases(cognitiveEngineCharsPerPhrase, data) {
        let string_500 = [], phrases = '', len = data.transcription.length, item, phrasesResult = '', total = 0;
        string_500.push({ content: '', length: 0 });
        for (let i = 0; i < len; i++) {
            phrases = '';
            item = data.transcription[i];
            if (data.init === 'out') {
                phrases += item.out;
                phrases += ' ';
                phrases += item.in;
            }
            else {
                phrases += item.in;
                phrases += ' ';
                phrases += item.out;
            }
            total = phrasesResult.length + phrases.length;
            if (total > cognitiveEngineCharsPerPhrase) {
                if (phrases.length < cognitiveEngineCharsPerPhrase) {
                    string_500.push({ content: phrases, length: phrases.length });
                    phrasesResult = phrases;
                }
                else {
                    phrasesResult += phrases;
                    let phraseParts = phrasesResult.split('.');
                    phrasesResult = '';
                    phraseParts.map(pp => {
                        while (pp.length > 0) {
                            if ((phrasesResult.length + pp.length + 1) <= cognitiveEngineCharsPerPhrase) {
                                phrasesResult += pp + '.';
                                pp = '';
                            }
                            else {
                                let arrPp = pp.split(' ');
                                if ((phrasesResult.length + 1 + arrPp[0].length + 1) > cognitiveEngineCharsPerPhrase) {
                                    string_500.push({ content: phrasesResult, length: phrasesResult.length });
                                    phrasesResult = arrPp.shift();
                                }
                                else {
                                    phrasesResult += ' ' + arrPp.shift();
                                    if (pp.length === 0)
                                        phrasesResult += '.';
                                }
                                pp = arrPp.join(' ');
                            }
                            if (phrasesResult.length <= cognitiveEngineCharsPerPhrase) {
                                string_500[string_500.length - 1].content = phrasesResult;
                                string_500[string_500.length - 1].length = phrasesResult.length;
                                if (phrasesResult.length === cognitiveEngineCharsPerPhrase) {
                                    phrasesResult = '';
                                    string_500.push({ content: phrasesResult, length: phrasesResult.length });
                                }
                            }
                        }
                    });
                }
            }
            else {
                phrasesResult += phrases;
                string_500[string_500.length - 1].content += phrases;
                string_500[string_500.length - 1].length = phrasesResult.length;
            }
            phrases = '';
        }
        return string_500;
    }
    SaveDocumentCognitive(index, path, data, nluId, executionCode, currentQueueId) {
        return promises_shared_1.DummyPromise()
            .then(() => {
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        }).then((resultFile) => {
            let jsonContent;
            let result = data ? data : [];
            logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting path: ', path);
            logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting index: ', index);
            logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting nlu: ', nluId);
            logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting result: ', result);
            try {
                if (resultFile) {
                    jsonContent = JSON.parse(resultFile);
                    logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting jsonContent: ', jsonContent);
                    logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting phrases: ', jsonContent.phrases.length);
                    jsonContent['phrases'][index][nluId] = result;
                }
                else {
                    jsonContent = { phrases: [{}] };
                    logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'creating item on jsonContent');
                    jsonContent['phrases'][index][nluId] = result;
                }
            }
            catch (ex) {
                logger.error('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', ex);
                this.errorData.SetErrorData('SaveDocumentCognitive', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return file_manager_1.WriteFileContent(path, JSON.stringify(jsonContent), 'utf-8');
        }).then((result) => {
            this.errorData.CleanErrorData();
            return Promise.resolve(true);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            this.errorData.SetErrorData('SaveDocumentCognitive', errorMessage, error);
            return Promise.resolve(false);
        });
    }
    ModelQuery(cognitiveService, subscriptionRegion, subscriptionKey, appId, query, nluName, executionCode, currentQueueId) {
        return cognitiveService.ModelQuery(subscriptionRegion, subscriptionKey, appId, query, null, null, null)
            .then((response) => {
            logger.info('[CognitiveAnalysisImplementation::ModelQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Request finished OK');
            logger.info('[CognitiveAnalysisImplementation::ModelQuery] Response: ' + JSON.stringify(response));
            this.UpdateCognitiveModel(currentQueueId, 'TotalValidRequest', 1);
            this.errorData.CleanErrorData();
            return Promise.resolve({
                intents: response.intents,
                entities: response.entities ? response.entities : null
            });
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::ModelQuery] Error 1: ' + JSON.stringify(err));
            this.UpdateCognitiveModel(currentQueueId, 'TotalInvalidRequest', 1);
            let responseError = {};
            if (typeof errorMessage === 'string') {
                responseError['error'] = err;
                if (errorMessage.indexOf('Out of call volume quota') > -1)
                    responseError['error'] = 'OUT_OF_VOLUME_QUOTA';
            }
            else
                responseError['error'] = 'SERVER_ERROR_MODEL_QUERY_UNKNOWN_ERROR';
            this.errorData.SetErrorData('ModelQuery', responseError && responseError['error'] ? responseError['error'] : errorMessage, err);
            return Promise.resolve(responseError);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::ModelQuery] Error 2: ' + JSON.stringify(err));
            this.UpdateCognitiveModel(currentQueueId, 'TotalInvalidRequest', 1);
            this.errorData.SetErrorData('ModelQuery', errorMessage, err);
            return Promise.resolve(null);
        });
    }
    EvaluateCognitive(path, interaction, executionCode, currentQueueId) {
        let studyTemplateThemes = [];
        let evaluationPlanAbilities = [];
        return promises_shared_1.DummyPromise()
            .then(result => {
            if (['STUDY', 'TEMPLATE'].includes(interaction.batchTypeName))
                return redis_service_1.Redis.GetStudyTemplateThemes(interaction.vcc, interaction.studyId);
            else
                return redis_service_1.Redis.GetEvaluationPlanAbilities(interaction.vcc, interaction.evaluationPlanId);
        })
            .then(result => {
            if (['STUDY', 'TEMPLATE'].includes(interaction.batchTypeName))
                studyTemplateThemes = result;
            else
                evaluationPlanAbilities = result;
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then((resultFile) => {
            let jsonContent = null;
            try {
                if (resultFile) {
                    jsonContent = JSON.parse(resultFile);
                }
            }
            catch (ex) {
                logger.error('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', ex);
                this.errorData.SetErrorData('EvaluateCognitive', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return jsonContent;
        })
            .then((jsonContent) => {
            if (!jsonContent)
                throw 'ERROR_FILE_JSON';
            let phrases = jsonContent['phrases'];
            let succedItems = [];
            if (phrases && phrases.length) {
                phrases.forEach((phrase) => {
                    if (phrase[interaction.nluId] && phrase[interaction.nluId].intents && phrase[interaction.nluId].intents.length) {
                        if (['STUDY', 'TEMPLATE'].includes(interaction.batchTypeName)) {
                            if (!studyTemplateThemes.length)
                                logger.warn('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'studyTemplateThemes is empty, need items to get succedItems');
                            let templateThemes = interaction.templateThemes.split(',');
                            let succedTemplateThemeItems = [];
                            templateThemes.map(tt => {
                                let currentTemplateTheme = studyTemplateThemes.find(stt => stt.id === tt);
                                if (currentTemplateTheme) {
                                    let succedIntents = [];
                                    currentTemplateTheme.items.map(tti => {
                                        let intent = phrase[interaction.nluId].intents.find(i => i.intent === tti.id.split('|')[1]);
                                        if (!intent)
                                            intent = phrase[interaction.nluId].intents.find(i => i.intent === tti.query.split('|')[1]);
                                        if (intent) {
                                            if (intent.score >= tti.acceptancePercentage) {
                                                if (tti.entities && tti.entities.length > 0) {
                                                    let succeedEntities = [];
                                                    tti.entities.map(e => {
                                                        let entities = phrase[interaction.nluId].entities.filter(en => en.type === e.entityName && en.score >= e.acceptancePercentage);
                                                        if (entities && entities.length) {
                                                            succeedEntities.push(entities[0]);
                                                        }
                                                    });
                                                    if (succeedEntities.length > 0) {
                                                        intent.entities = succeedEntities;
                                                        succedIntents.push(intent);
                                                        succedTemplateThemeItems.push(tti.id);
                                                    }
                                                }
                                                else {
                                                    intent.entities = [];
                                                    succedIntents.push(intent);
                                                    succedTemplateThemeItems.push(tti.id);
                                                }
                                            }
                                        }
                                    });
                                    logger.info('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'succedIntents result:', succedIntents);
                                    if (succedIntents.length > 0) {
                                        if (succedItems.find(stt => stt.id === tt)) {
                                            let succeedTemplateTheme = succedItems.find(stt => stt.id === tt);
                                            succeedTemplateTheme.intents.map(i => {
                                                let succeedIntent = succedIntents.find(si => si.intent === i.id.split('|')[1]);
                                                if (succeedIntent.score > i.score) {
                                                    i = succeedIntent;
                                                }
                                            });
                                        }
                                        else {
                                            succedItems.push({
                                                templateThemeId: tt,
                                                templateThemeItems: succedTemplateThemeItems,
                                                intents: succedIntents
                                            });
                                        }
                                    }
                                }
                            });
                        }
                        else {
                            if (!evaluationPlanAbilities.length)
                                logger.warn('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'evaluationPlanAbilities is empty, need items to get succedItems');
                            let abilities = interaction.abilities.split(',');
                            abilities.map(a => {
                                let currentAbility = evaluationPlanAbilities.find(epa => epa.idAbility === a);
                                if (currentAbility) {
                                    let succedIntents = [];
                                    currentAbility.queries.map(q => {
                                        let intent = phrase[interaction.nluId].intents.find(i => i.intent === q.queryContent.split('|')[1]);
                                        if (!intent)
                                            intent = phrase[interaction.nluId].intents.find(i => i.intent === q.queryId);
                                        if (intent) {
                                            if (intent.score >= q.acceptancePercentage) {
                                                if (q.entities && q.entities.length > 0) {
                                                    let succeedEntities = [];
                                                    q.entities.map(e => {
                                                        let entities = phrase[interaction.nluId].entities.filter(en => en.type === e.entityName && en.score >= e.acceptancePercentage);
                                                        if (entities && entities.length) {
                                                            succeedEntities.push(entities[0]);
                                                        }
                                                    });
                                                    if (succeedEntities.length > 0) {
                                                        intent.entities = succeedEntities;
                                                        succedIntents.push(intent);
                                                    }
                                                }
                                                else {
                                                    intent.entities = [];
                                                    succedIntents.push(intent);
                                                }
                                            }
                                        }
                                    });
                                    logger.info('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'succedIntents result:', succedIntents);
                                    if (succedIntents.length > 0) {
                                        if (succedItems.find(sa => sa.abilityId === a)) {
                                            let succeedAbility = succedItems.find(sa => sa.abilityId === a);
                                            succeedAbility.intents.map(i => {
                                                let succeedIntent = succedIntents.find(si => si.intent === i.intent);
                                                if (succeedIntent.score > i.score) {
                                                    i = succeedIntent;
                                                }
                                            });
                                        }
                                        else {
                                            succedItems.push({
                                                abilityId: a,
                                                intents: succedIntents
                                            });
                                        }
                                    }
                                    else {
                                    }
                                }
                            });
                        }
                    }
                });
            }
            logger.info('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'succedItems result:', succedItems);
            this.errorData.CleanErrorData();
            return Promise.resolve(succedItems);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            this.errorData.SetErrorData('EvaluateCognitive', errorMessage, error);
            return Promise.resolve(null);
        });
    }
    UpdateCognitiveModel(queueId, target, value) {
        logger.info('[CognitiveAnalysisImplementation::UpdateCognitiveModel] Target::Value ' + target + '::' + value);
        let billing = new billing_implementation_1.BillingImplementation();
        let myBatchName = '';
        let myQueueIdArray = queueId.split(':');
        switch (myQueueIdArray[7]) {
            case 'study':
                myBatchName = 'STUDY';
                break;
            case 'template':
                myBatchName = 'TEMPLATE';
                break;
            case 'ability':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'tipification':
                myBatchName = 'EVALUATION_PLAN';
                break;
        }
        billing.AddUpdateCognitiveAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[6], target, value, myBatchName, myQueueIdArray[8]);
    }
    CloseCognitiveModel(queueId, resultCode, resultDescription) {
        let billing = new billing_implementation_1.BillingImplementation();
        let myBatchName = '';
        let myQueueIdArray = queueId.split(':');
        switch (myQueueIdArray[7]) {
            case 'study':
                myBatchName = 'STUDY';
                break;
            case 'template':
                myBatchName = 'TEMPLATE';
                break;
            case 'ability':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'tipification':
                myBatchName = 'EVALUATION_PLAN';
                break;
        }
        billing.AddCloseCognitiveAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[6], myBatchName, myQueueIdArray[8], resultCode ? 1 : 0, resultDescription);
    }
}
exports.CognitiveAnalysisImplementation = CognitiveAnalysisImplementation;
function GetCognitiveService(vcc, cognitiveEngine) {
    let cognitiveService;
    vcc = vcc ? vcc : null;
    cognitiveEngine = cognitiveEngine ? cognitiveEngine : null;
    return promises_shared_1.DummyPromise()
        .then(result => {
        if (vcc || cognitiveEngine) {
            if (vcc)
                return redis_service_1.Redis.GetVCC(vcc, vcc);
            if (cognitiveEngine)
                return Promise.resolve(true);
            throw 'SERVER_ERROR_MISSING_PARAMETERS';
        }
        else
            throw 'SERVER_ERROR_MISSING_PARAMETERS';
    }).then(result => {
        if (vcc) {
            cognitiveEngine = result && result.cognitiveEngine ? result.cognitiveEngine : null;
        }
        switch (cognitiveEngine) {
            case 'LUIS':
                cognitiveService = new luis_service_implementation_1.LUISServiceImplementation();
                break;
            case 'RASA':
                if (useRasaApi)
                    cognitiveService = new rasa_api_service_implementation_1.RASAAPIServiceImplementation();
                else
                    cognitiveService = new rasa_service_implementation_1.RASAServiceImplementation();
                break;
            case 'OLD_RASA_ENGINE':
                if (useRasaApi)
                    cognitiveService = new rasa_service_implementation_1.RASAServiceImplementation();
                else
                    cognitiveService = new rasa_api_service_implementation_1.RASAAPIServiceImplementation();
                break;
            case 'NEW_RASA_ENGINE':
                if (useRasaApi)
                    cognitiveService = new rasa_api_service_implementation_1.RASAAPIServiceImplementation();
                else
                    cognitiveService = new rasa_service_implementation_1.RASAServiceImplementation();
                break;
            default:
                cognitiveService = null;
                logger.warn('[cognitive-analysis.impl::GetCognitiveService] Undefined engine: ' + cognitiveEngine);
                break;
        }
        return Promise.resolve(cognitiveService);
    }).catch((error) => {
        let errorMessage = typeof error === 'string' ? error : 'ERROR';
        logger.error('[cognitive-analysis.impl::GetCognitiveService] Error: ' + error);
        return Promise.reject(errorMessage);
    });
}
exports.GetCognitiveService = GetCognitiveService;
