"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NluMigrator = void 0;
const redis_service_1 = require("../../db/redis.service");
const log4js = require("log4js");
const mssql_service_1 = require("../../db/mssql.service");
const promises_shared_1 = require("../../shared/promises.shared");
const helper_1 = require("./helper");
const uuid_1 = require("uuid");
const enum_1 = require("./enum");
const arrays_shared_1 = require("../../shared/arrays.shared");
const logger = log4js.getLogger('Task');
class NluMigrator {
    MigrateNlus(vcc, data, start = 0, end = 1) {
        logger.info('[CognitiveEngineMigrateImplementation::MigrateNlus] Migrate nlu for ', vcc, ' with arg :', { start, end });
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.LRange(`${enum_1.ROOT_KEY}:${vcc}:Nlus:Keys`, start, end - 1);
        })
            .then((nluKeys) => {
            logger.debug("[CognitiveEngineMigrateImplementation::MigrateNlus] Found nlu keys:", nluKeys);
            let promises = [];
            if (nluKeys && nluKeys.length > 0) {
                promises.push(Promise.resolve(true));
                promises.push(redis_service_1.Redis.GetCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Nlus`, arrays_shared_1.GetUniqueFromArray(nluKeys)));
            }
            else {
                promises.push(Promise.resolve(false));
            }
            return Promise.all(promises);
        })
            .then((results) => {
            if (!results[0]) {
                return false;
            }
            let nlus = results[1];
            if (nlus) {
                let nluList = [];
                for (let nluStr of nlus) {
                    let nlu = JSON.parse(nluStr);
                    if (nlu) {
                        nluList.push(nlu);
                    }
                }
                logger.info("[CognitiveEngineMigrateImplementation::MigrateNlus] Found nlus:", nluList.length);
                if (nluList.length > 0) {
                    return this.ProcessNlus(vcc, data, nluList);
                }
            }
            return false;
        })
            .then(_ => {
            let promises = [];
            if (end < data.totalNlu) {
                promises.push(this.MigrateNlus(vcc, data, start + enum_1.ITEMS_PER_PAGE, end + enum_1.ITEMS_PER_PAGE));
            }
            return Promise.all(promises);
        })
            .then((results) => {
            logger.info('[CognitiveEngineMigrateImplementation::MigrateNlus] After promises:', results);
            return !results.includes(false);
        })
            .catch(err => {
            logger.error('[CognitiveEngineMigrateImplementation::MigrateNlus] Errror on migrate nlus ', err);
            return false;
        });
    }
    ProcessNlus(vcc, data, nlus) {
        return promises_shared_1.DummyPromise().then(_ => {
            let promises = [];
            for (let nlu of nlus) {
                switch (nlu.status) {
                    case 0:
                        if (nlu.totalRetries < enum_1.TOTAL_RETRIES) {
                            promises.push(this.AddApp(vcc, data, nlu));
                        }
                        else {
                            promises.push(this.Cancel(vcc, 'Nlus', nlu.appId, nlu.originalName, nlu, 'nlu', enum_1.CANCELED_NLU));
                        }
                        break;
                    case 1:
                        promises.push(this.DeleteApp(vcc, data, nlu));
                        break;
                    default: break;
                }
            }
            return Promise.all(promises);
        }).then((results) => {
            if (results.length > 0) {
                let totalDeleted = results.filter(r => r == enum_1.SUCCESS_DELETE_NLU).length;
                let totalCreated = results.filter(r => r == enum_1.SUCCESS_ADD_NLU).length;
                let totalError = results.filter(r => r == enum_1.ERROR_ADD_NLU).length;
                let totalCanceled = results.filter(r => r == enum_1.CANCELED_NLU).length;
                return redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, {
                    totalDeletedNlu: totalDeleted + data.totalDeletedNlu,
                    totalCreatedNlu: totalCreated + data.totalCreatedNlu,
                    totalErrorNlu: totalError + data.totalErrorNlu,
                    totalCanceledNlu: data.totalCanceledNlu + totalCanceled
                }));
            }
            return false;
        });
    }
    AddApp(vcc, data, nlu) {
        logger.info('[CognitiveEngineMigrateImplementation::AddApp] Adding Nlu for ', vcc, ' into ', data.newEngine);
        let culture = enum_1.CULTURES[`${data.oldEngine}_${data.newEngine}`][nlu.culture];
        return promises_shared_1.DummyPromise().then(_ => {
            return helper_1.GetServiceInstance(data.newEngine);
        })
            .then((service) => {
            let nluToAdd = nlu;
            return service.AddApp(data.subscriptionRegion, data.subscriptionKey, Object.assign(nluToAdd, {
                culture, vcc
            }));
        })
            .then((newAppId) => {
            logger.info('[CognitiveEngineMigrateImplementation::AddApp] New AppId:', newAppId);
            if (newAppId) {
                let promises = [];
                promises.push(mssql_service_1.MSSql.UpdateNluAppId(vcc, nlu.appId, newAppId, culture, data.newEngine));
                promises.push(redis_service_1.Redis.SaveCognitiveEngineMigrationNewId(`${enum_1.ROOT_KEY}:${vcc}:AppIds`, nlu.appId, newAppId));
                promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'nlu', nlu.originalName, 8, enum_1.MIGRATED));
                return Promise.all(promises);
            }
            return [false];
        }).then((results) => {
            logger.info('[CognitiveEngineMigrateImplementation::AddApp] After update AppId:', results);
            if (!results.includes(false)) {
                return redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Nlus`, nlu.appId, Object.assign(nlu, {
                    status: 1,
                }));
            }
            return false;
        }).then((result) => {
            return result ? enum_1.SUCCESS_ADD_NLU : enum_1.ERROR_ADD_NLU;
        }).catch((err) => {
            logger.error(`[CognitiveEngineMigrateImplementation::AddUtterance] Unknown error on add nlu into ${data.newEngine}`, err);
            return this.ProcessErrors(vcc, data, "AddApp", nlu, err).then((results) => {
                return results.includes(enum_1.SUCCESS_ADD_NLU) ? enum_1.SUCCESS_ADD_NLU : enum_1.ERROR_ADD_NLU;
            });
        });
    }
    DeleteApp(vcc, data, nlu) {
        logger.info('[CognitiveEngineMigrateImplementation::DeleteApp] Deleting nlu for ', vcc, ' nlu is:', nlu);
        return promises_shared_1.DummyPromise().then(_ => {
            return helper_1.GetServiceInstance(data.oldEngine);
        }).then((service) => {
            if (service) {
                let nluToDelete = nlu;
                return service.DeleteApp(data.subscriptionRegion, data.subscriptionKey, Object.assign(nluToDelete, {
                    vcc: vcc,
                    name: nlu.originalName,
                }));
            }
            logger.error('[CognitiveEngineMigrateImplementation::DeleteApp] NOT_FOUND_OLD_COGNITIVE_ENGINE', service);
            return false;
        }).then((deleted) => {
            logger.error('[CognitiveEngineMigrateImplementation::DeleteApp] Deleted app in ', data.oldEngine, ' nlu is:', nlu);
            if (deleted) {
                return redis_service_1.Redis.DeleteCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Nlus`, nlu.appId);
            }
            return false;
        }).then((deletedFromRedis) => {
            return deletedFromRedis ? enum_1.SUCCESS_DELETE_NLU : enum_1.ERROR_DELETE_NLU;
        }).catch((err) => {
            logger.error(`[CognitiveEngineMigrateImplementation::AddUtterance] Unknown error on delete nlu from ${data.oldEngine}`, err);
            return this.ProcessErrors(vcc, data, "DeleteApp", nlu, err).then((results) => {
                return results.includes(enum_1.SUCCESS_DELETE_NLU) ? enum_1.SUCCESS_DELETE_NLU : enum_1.ERROR_DELETE_NLU;
            });
        });
    }
    ProcessErrors(vcc, data, method, model, err) {
        let promises = [];
        logger.error("[CognitiveEngineMigrateImplementation::ProcessErrors] Error:", err);
        if ([data.newEngine, data.oldEngine].includes("LUIS")) {
            switch (method) {
                case 'AddApp':
                    if (typeof err == 'object') {
                        if (err.statusCode == 400 && err.message.includes("already exists")) {
                            model.totalRetries += 1;
                            let uuid = uuid_1.v4();
                            let newNlu = model;
                            promises.push(this.AddApp(vcc, data, Object.assign(newNlu, {
                                name: `${model.originalName}-${uuid}`,
                            })));
                        }
                    }
                    break;
                case 'DeleteApp':
                    if (typeof err == 'object') {
                        if (err.statusCode == 400) {
                            promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Nlus`, model.appId, Object.assign(model, {
                                status: 3,
                                totalRetries: model.totalRetries + 1
                            })));
                            promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'nlu', model.orignalName, 1, 'AVAILABLE_NOT_REMOVED_SOURCE_ENGINE'));
                        }
                    }
                    break;
                default: break;
            }
        }
        if ([data.newEngine, data.oldEngine].includes("RASA")) {
            switch (method) {
                case 'DeleteApp':
                    if (err == "MODEL_NLU_NOT_FOUND") {
                        promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Nlus`, model.appId, Object.assign(model, {
                            status: 3
                        })));
                        promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'nlu', model.originalName, 1, 'AVAILABLE_NOT_REMOVED_SOURCE_ENGINE'));
                    }
                    break;
                case 'AddApp':
                    if (err == "MODEL_NLU_ALREADY_EXIST") {
                        promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Nlus`, model.appId, Object.assign(model, {
                            status: 3
                        })));
                        promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'nlu', model.originalName, 3, 'ERROR_NLU_EXISTS'));
                    }
                    break;
                default: break;
            }
        }
        return Promise.all(promises);
    }
    Cancel(vcc, key, redisId, dbId, model, modelName = 'nlu', returnCode) {
        return promises_shared_1.DummyPromise().then(_ => {
            let promises = [];
            promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:${key}`, redisId, Object.assign(model, {
                status: 4
            })));
            promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, modelName, dbId, 4, enum_1.CANCELED_MANY_ATTEMPTS));
            return Promise.all(promises);
        }).then(_ => {
            return returnCode;
        });
    }
}
exports.NluMigrator = NluMigrator;
