"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UtteranceMigrator = void 0;
const redis_service_1 = require("../../db/redis.service");
const log4js = require("log4js");
const mssql_service_1 = require("../../db/mssql.service");
const promises_shared_1 = require("../../shared/promises.shared");
const helper_1 = require("./helper");
const enum_1 = require("./enum");
const arrays_shared_1 = require("../../shared/arrays.shared");
const logger = log4js.getLogger('Task');
class UtteranceMigrator {
    MigrateUtterances(vcc, data, start, end) {
        logger.info('[CognitiveEngineMigrateImplementation::MigrateUtterances] Migratint utterances for ', vcc, ' into ', data.newEngine, ' from ', start, ' to ', end);
        return promises_shared_1.DummyPromise().then(_ => {
            return redis_service_1.Redis.LRange(`${enum_1.ROOT_KEY}:${vcc}:Utterances:Keys`, start, end - 1);
        }).then((keys) => {
            logger.info('[CognitiveEngineMigrateImplementation::MigrateUtterances] Found keys for utterances:', keys);
            if (keys && keys.length > 0) {
                return redis_service_1.Redis.GetCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Utterances`, arrays_shared_1.GetUniqueFromArray(keys));
            }
            return [];
        }).then((utterances) => {
            logger.info('[CognitiveEngineMigrateImplementation::MigrateUtterances] Found utterances:', utterances);
            if (utterances) {
                let utteranceList = [];
                for (let u of utterances) {
                    let utterance = JSON.parse(u);
                    if (utterance) {
                        if (utterance.rawEntityLabels) {
                            utterance.entityLabels = JSON.parse(utterance.rawEntityLabels);
                        }
                        utteranceList.push(utterance);
                    }
                }
                if (utteranceList.length > 0) {
                    return this.ProcessUtterances(vcc, data, utteranceList);
                }
            }
            return false;
        }).then(_ => {
            if (end < data.totalUtterance) {
                return this.MigrateUtterances(vcc, data, start + enum_1.ITEMS_PER_PAGE, end + enum_1.ITEMS_PER_PAGE);
            }
            return true;
        });
    }
    ProcessUtterances(vcc, data, utterances) {
        logger.info('[CognitiveEngineMigrateImplementation::AddUtterance] Process utterance ', utterances.length, ' into ', data.newEngine, ' for ', vcc);
        return promises_shared_1.DummyPromise().then(_ => {
            let promises = [];
            for (let utterance of utterances) {
                if (utterance.status != 3) {
                    promises.push(this.AddUtterance(vcc, data, utterance));
                }
            }
            return Promise.all(promises);
        }).then((results) => {
            logger.info('[CognitiveEngineMigrateImplementation::ProcessUtterances] Results for process utterances:', results);
            let totalCreated = results.filter(r => r == true).length;
            let totalError = results.filter(r => !r).length;
            return redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, {
                totalCreatedUtterance: totalCreated + data.totalCreatedUtterance,
                totalErrorUtterance: totalError + data.totalErrorUtterance
            }));
        });
    }
    AddUtterance(vcc, data, utterance) {
        logger.info('[CognitiveEngineMigrateImplementation::AddUtterance] Adding Utterance ', utterance.name, ' into ', data.newEngine);
        let AppId = null;
        let IntentId = null;
        let OriginalUtteranceId = null;
        return promises_shared_1.DummyPromise().then(_ => {
            let promises = [
                redis_service_1.Redis.GetCognitiveEngineMigrationNewId(`${enum_1.ROOT_KEY}:${vcc}:AppIds`, utterance.appId),
                redis_service_1.Redis.GetCognitiveEngineMigrationNewId(`${enum_1.ROOT_KEY}:${vcc}:IntentIds`, utterance.intentKey)
            ];
            return Promise.all(promises);
        })
            .then((ids) => {
            logger.info('[CognitiveEngineMigrateImplementation::AddUtterance] Found ids for utterance', utterance, ':', ids);
            if (ids && ids.length == 2) {
                AppId = ids[0];
                IntentId = ids[1];
                return helper_1.GetServiceInstance(data.newEngine);
            }
            return null;
        })
            .then((service) => {
            if (service && AppId && IntentId) {
                OriginalUtteranceId = utterance.id;
                return service.AddUtterance(data.subscriptionRegion, data.subscriptionKey, AppId, Object.assign(utterance, {
                    intent: utterance.intent
                }));
            }
            logger.error('[CognitiveEngineMigrateImplementation::AddUtterance] Error on get new app id and new intentId from redis ', utterance);
            return null;
        }).then((newUtteranceId) => {
            logger.info(`[CognitiveEngineMigrateImplementation::AddUtterance] New utterance id:`, newUtteranceId);
            if (newUtteranceId && newUtteranceId.ExampleId) {
                let promises = [];
                promises.push(Promise.resolve(newUtteranceId.ExampleId));
                promises.push(mssql_service_1.MSSql.UpdateUtteranceId(vcc, AppId, IntentId, OriginalUtteranceId, newUtteranceId.ExampleId));
                return Promise.all(promises);
            }
            logger.error(`[CognitiveEngineMigrateImplementation::AddUtterance] Error on add utterance into ${data.newEngine}:`, newUtteranceId, IntentId);
            return [null];
        })
            .then((results) => {
            let newUtteranceId = results[0];
            logger.info('[CognitiveEngineMigrateImplementation::AddIntent] Updated Utterance ID:', results);
            if (newUtteranceId) {
                let promises = [];
                promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Utterances`, utterance.key));
                return Promise.all(promises);
            }
            return [false];
        })
            .then((results) => {
            return !results.includes(false);
        })
            .catch((err) => {
            logger.error(`[CognitiveEngineMigrateImplementation::AddUtterance] Unknown error on add utterance into ${data.newEngine}`, err);
            let promises = [];
            promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Utterances`, utterance.key, Object.assign(utterance, {
                status: 3
            })));
            promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'utterance', OriginalUtteranceId, 3, 'WITH_ERROR'));
            return Promise.all(promises).then(() => {
                return false;
            });
        });
    }
}
exports.UtteranceMigrator = UtteranceMigrator;
