"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.I6UpdateImplementation = void 0;
const config_1 = require("../config/config");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const log4js = require("log4js");
const interaction_service_1 = require("../services/interaction.service");
const promises_shared_1 = require("../shared/promises.shared");
const arrays_shared_1 = require("../shared/arrays.shared");
const http_request_occ_1 = require("../shared/http-request-occ");
const i6_model_1 = require("../models/i6.model");
let logger = log4js.getLogger('Task');
let httpRequestOccCompatible = true;
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'I6UpdateImplementation';
let executionVccs = [];
let executionVCCQuotes = new i6_model_1.ExecutionVCCQuotes();
class I6UpdateImplementation {
    Execute(executionCode, rateRecursive) {
        let executeIndex = 1;
        let ratio = config_1.RATIO_I6_UPDATE;
        let executeResult = true;
        let keysList = null;
        let currentExecutionVccs = [];
        return redis_service_1.Redis.LRange('I6Update', 0, ratio - 1)
            .then((result) => {
            keysList = result;
            ratio = keysList?.length <= ratio ? keysList.length : 0;
            currentExecutionVccs = arrays_shared_1.GetUniqueFromArray(keysList.map(kl => {
                let items = kl.split(':');
                return items[0];
            }));
            let promiseArray = [];
            currentExecutionVccs.forEach(v => {
                if (!executionVccs.includes(v)) {
                    executionVccs.push(v);
                    if (httpRequestOccCompatible)
                        promiseArray.push(http_request_occ_1.GetRequestQuote(v));
                }
            });
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(result => {
            executionVCCQuotes.quotes = result.map((r, i) => {
                return new i6_model_1.VCCQuote(currentExecutionVccs[i], r.body.i6IRequestQuote.low.rateLimits.maxRequests);
            });
            let promiseArray = [];
            logger.info('[I6UpdateImplementation::Execute] executionCode:', executionCode, 'ratio:', ratio, 'getting Redis.LRange:', result);
            while (ratio >= executeIndex) {
                let currentKeyReference = keysList[executeIndex - 1].split(':');
                let requestsToUse = currentKeyReference[2] === 'CALL' ? 2 : 1;
                if (executionVCCQuotes.GetVCCFreeAvailableQuote(currentKeyReference[0])
                    >=
                        (executionVCCQuotes.GetVCCUsedQuote(currentKeyReference[0])
                            +
                                requestsToUse)) {
                    promiseArray.push(this.I6UpdateExecute(executionCode));
                    executionVCCQuotes.IncreaseVCCUsedQuote(currentKeyReference[0], requestsToUse);
                }
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((results) => {
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('I6Update', 0, ratio - 1);
        })
            .then((result) => {
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return executeResult;
        })
            .then(result => {
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[I6UpdateImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    I6UpdateExecute(executionCode) {
        let storage = '';
        let currentQueueId = '';
        let vcc = '';
        let processId = '';
        let batchType = '';
        let interactionToUpdate = null;
        return redis_service_1.Redis.RPopLPush('I6Update', 'I6UpdateExecute')
            .then((queueId) => {
            if (queueId?.length) {
                currentQueueId = queueId;
                logger.info('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId);
                let item = queueId.split(':');
                if (item?.length) {
                    vcc = item[0];
                    processId = item[1];
                    storage = `${vcc}:${processId}:i6update`;
                    logger.info('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessI6Update(storage, currentQueueId);
                }
                else {
                    logger.error('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            if (result) {
                logger.info('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1');
                interactionToUpdate = result;
                return this.UpdateProcessI6InteractionToUpdate(interactionToUpdate, executionCode, currentQueueId);
            }
            else {
                if (currentQueueId)
                    logger.warn('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return Promise.resolve(null);
            }
        })
            .then(result => {
            if (result) {
                interactionToUpdate = result;
                return redis_service_1.Redis.SaveProcessI6Update(storage, currentQueueId, interactionToUpdate);
            }
            return null;
        })
            .then((result) => {
            let promiseArray = [];
            if (result && interactionToUpdate?.status === 1) {
                logger.info('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('I6UpdateExecute', currentQueueId));
            }
            else if (interactionToUpdate?.retry >= config_1.RETRY_I6_UPDATE) {
                logger.error('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                promiseArray.push(this.CloseProcessI6InteractionToUpdate(interactionToUpdate, executionCode, currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('I6UpdateExecute', currentQueueId));
            }
            else if (currentQueueId && !interactionToUpdate) {
                logger.error('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('I6UpdateExecute', currentQueueId));
            }
            else {
                logger.error('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((result) => {
            let promiseArray = [];
            if (interactionToUpdate?.status === 1
                && interactionToUpdate?.statusDescription === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessI6Update(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyI6UpdateInteraction(executionCode, currentQueueId, interactionToUpdate));
            }
            else if (interactionToUpdate?.retry >= config_1.RETRY_I6_UPDATE) {
                interactionToUpdate.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessI6Update(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyI6UpdateInteraction(executionCode, currentQueueId, interactionToUpdate));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processId, processId, CURRENT_IMPLEMENTATION, interactionToUpdate.id, null, interactionToUpdate.errorFunction, interactionToUpdate.errorMessage, interactionToUpdate.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    }
    UpdateProcessI6InteractionToUpdate(interactionToUpdate, executionCode, currentQueueId) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (interactionToUpdate?.needI6Update) {
                return interaction_service_1.UpdateI6Interaction(interactionToUpdate, interactionToUpdate.processId);
            }
            else {
                return Promise.resolve(null);
            }
        })
            .then((result) => {
            if (!result)
                throw 'SERVER_ERROR_INTERACTION_I6_UPDATE_FAILED';
            if (interactionToUpdate) {
                interactionToUpdate.needI6Update = 0;
                interactionToUpdate['status'] = 1;
                interactionToUpdate['statusDescription'] = 'ready';
                interactionToUpdate['retry'] = 0;
                interactionToUpdate['errorMessage'] = null;
            }
            else {
                logger.error('[I6UpdateImplementation::UpdateProcessI6InteractionToUpdate] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interactionToUpdate['status'] = -1;
                interactionToUpdate['retry'] = interactionToUpdate['retry'] ? interactionToUpdate['retry'] + 1 : 1;
                interactionToUpdate['errorFunction'] = 'UpdateProcessI6InteractionToUpdate';
                interactionToUpdate['errorMessage'] = result;
                interactionToUpdate['errorDetail'] = result;
            }
            return Promise.resolve(interactionToUpdate);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[I6UpdateImplementation::UpdateProcessI6InteractionToUpdate] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interactionToUpdate['status'] = -1;
            interactionToUpdate['retry'] = interactionToUpdate['retry'] ? interactionToUpdate['retry'] + 1 : 1;
            interactionToUpdate['errorFunction'] = 'UpdateProcessI6InteractionToUpdate';
            interactionToUpdate['errorMessage'] = errorMessage;
            interactionToUpdate['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interactionToUpdate);
        });
    }
    CloseProcessI6InteractionToUpdate(interactionToUpdate, executionCode, currentQueueId) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (interactionToUpdate.interactionType === 'CALL')
                return interaction_service_1.CloseI6CallInteraction(interactionToUpdate);
            else
                return interaction_service_1.CloseI6TextInteraction(interactionToUpdate);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch(err => {
            logger.error('[I6UpdateImplementation::CloseProcessI6InteractionToUpdate] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', err);
            return Promise.resolve(false);
        });
    }
    AddProcessQueue(vcc, processId, storage) {
        let readyToCreateProcessQueue = false;
        let processQueue = `${vcc}:${processId}`;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.ExistsProcessI6Update(storage);
        })
            .then(result => {
            if (!result) {
                readyToCreateProcessQueue = true;
                return redis_service_1.Redis.DeleteQueue('Process', processQueue);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(_ => {
            if (readyToCreateProcessQueue) {
                logger.error('[I6UpdateImplementation::AddProcessQueue] Creating process queue');
                return redis_service_1.Redis.CreateQueueL('Process', processQueue);
            }
            else {
                return Promise.resolve(true);
            }
        });
    }
}
exports.I6UpdateImplementation = I6UpdateImplementation;
