"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReportImplementation = void 0;
const log4js = require("log4js");
const sy = require("systeminformation");
const promises_shared_1 = require("../shared/promises.shared");
const md5 = require("md5");
const platform_service_1 = require("../services/platform.service");
let logger = log4js.getLogger('Task');
let icModule = require('intermediate-component-sa');
class ReportImplementation {
    GetSystemInformation(guid) {
        let systemInformationData = {
            installationId: guid,
            macAddress: '',
            publicIp: ''
        };
        return promises_shared_1.DummyPromise()
            .then((result) => {
            logger.info('[ReportImplementation::GetSystemInformation] guid:', guid);
            let sys = sy.networkInterfaces();
            return sys;
        }).then((result) => {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_NETWORK_INTERFACE_INFORMATION';
            if (result && result.length) {
                logger.info('[ReportImplementation::GetSystemInformation] guid:', guid, 'getting result[0].mac:', result[0].mac);
                let myNetworkInterface = null;
                myNetworkInterface = result.filter(r => r.operstate === 'up')[0];
                systemInformationData.macAddress = myNetworkInterface.mac;
                systemInformationData.publicIp = myNetworkInterface.ip4;
            }
            logger.info('[ReportImplementation::GetSystemInformation] guid:', guid, 'getting systemInformationData:', systemInformationData);
            return Promise.resolve(systemInformationData);
        }).catch((err) => {
            logger.error('[ReportImplementation::GetSystemInformation] guid:', guid, 'getting error:', err);
            return Promise.resolve(false);
        });
    }
    GenerateMD5Content(data) {
        let result = '';
        if (data) {
            result += data.installationId;
            result += data.macAddress;
            logger.info('[ReportImplementation::GenerateMD5Content] getting result:', result);
            result = md5(result);
        }
        return result;
    }
    GetReportProceduraParameters(procedure) {
        let systemInformationData = null;
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[ReportImplementation::GetReportProceduraParameters] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            let body = { procedure: procedure };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_REPORT_GET_PARAMETERS', body);
        }).then(result => {
            logger.info('[ReportImplementation::GetReportProceduraParameters] Getting result:', result.length, 'from external server');
            if (result.status) {
                return Promise.resolve(result.data);
            }
            else {
                return Promise.resolve(null);
            }
        }).catch((err) => {
            logger.error('[ReportImplementation::GetReportProceduraParameters] Getting error:', err);
            return Promise.resolve(null);
        });
    }
    ReportExecution(procedureParameters, procedure, clientOffset) {
        let systemInformationData = null;
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[ReportImplementation::GetReportProceduraParameters] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                procedureParameters.push({
                    column: "InstallationId",
                    value: installationId
                });
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            if (!installationId)
                throw 'MISSING_PARAMETER_INSTALLATION_ID';
            let body = {
                procedure: procedure,
                clientOffset,
                procedureParameters: procedureParameters
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_REPORT_EXECUTE', body);
        }).then(result => {
            logger.info('[ReportImplementation::GetReportProceduraParameters] Getting result:', result.length, 'from external server');
            if (result.status) {
                return Promise.resolve(result.data);
            }
            else {
                return Promise.resolve(null);
            }
        }).catch((err) => {
            logger.error('[ReportImplementation::GetReportProceduraParameters] Getting error:', err);
            return Promise.resolve(null);
        });
    }
}
exports.ReportImplementation = ReportImplementation;
