"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TranscriptionsDownloadImplementation = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const config_1 = require("../config/config");
const redis_service_1 = require("../db/redis.service");
const request_shared_1 = require("../shared/request.shared");
const file_manager_1 = require("../shared/file-manager");
const moment = require("moment");
const billing_implementation_1 = require("./billing.implementation");
const process_implementation_1 = require("./process.implementation");
const nlu_service_1 = require("../services/nlu.service");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'TranscriptionsDownloadImplementation';
class TranscriptionsDownloadImplementation {
    Execute(executionCode, rateRecursive) {
        logger.info(`[TranscriptionsDownloadImplementation::Execute] executionCode: ${executionCode} executing wave #${rateRecursive}`);
        let executeIndex = 1;
        let ratio = config_1.RATIO_TRANSCRIPTION;
        let alowedRatio = 0;
        let notAlowedRatio = 0;
        let executeResult = true;
        let currentBlockPromise;
        return promises_shared_1.SleepPromise(config_1.SLEEP_TRANSCRIPTION)
            .then(_ => {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.LRange('TranscriptionsDownload', 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_02_LAUNCH_TRANSCRIPTIONS_DOWNLOAD_IMPL';
            let promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(this.TranscriptionDownloadExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((results) => {
            if (results.filter(item => item === false).length > notAlowedRatio)
                executeResult = false;
            return redis_service_1.Redis.LRange('TranscriptionsDownload', 0, ratio - 1);
        })
            .then((result) => {
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(result => {
            if (rateRecursive === config_1.RATE_RECURSIVE_TRANSCRIPTION)
                logger.info(`[TranscriptionsDownloadImplementation::Execute] executionCode: ${executionCode} Finishing execution wave #${rateRecursive}`);
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info('[TranscriptionsDownloadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting info:', errorMessage);
            else
                logger.error('[TranscriptionsDownloadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    TranscriptionDownloadExecute(executionCode) {
        let storage = '';
        let currentQueueId = '';
        let processInteraction;
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let interactionId = '';
        let interactionSection = 0;
        let interactionPart = 0;
        let sentimentNlu;
        let currentBlockPromise = null;
        return redis_service_1.Redis.RPopLPush('TranscriptionsDownload', 'TranscriptionsDownloadExecute')
            .then((queueId) => {
            currentBlockPromise = 'BLOCK_01_GETTING_INTERACTION_DATA_FROM_REDIS_STORAGE';
            currentQueueId = queueId;
            let item = currentQueueId.split(':');
            if (item?.length) {
                vcc = item[0];
                processId = item[1];
                interactionType = item[2];
                interactionId = item[3];
                interactionSection = Number(item[4]);
                interactionPart = Number(item[5]);
                storage = vcc + ':' + processId;
                logger.info('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
            }
            else {
                logger.error('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                return Promise.resolve(null);
            }
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_02_SETTING_INTERACTION_DATA';
            if (result) {
                processInteraction = result;
                logger.info('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                return Promise.resolve(true);
            }
            else {
                if (currentQueueId)
                    logger.warn('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return Promise.resolve(null);
            }
        })
            .then(_ => {
            return nlu_service_1.GetNLU(vcc, 'sentiment-' + processInteraction.language);
        })
            .then(result => {
            if (result)
                sentimentNlu = result;
        })
            .then(_ => {
            currentBlockPromise = 'BLOCK_03_CREATE_TRANSCRIPTION_FOLDER';
            if (processInteraction)
                return file_manager_1.CreateFolder(`${config_1.TRANSCRIPTION_FOLDER}/${processInteraction.vcc}`);
            else
                return Promise.resolve(null);
        })
            .then(_ => {
            currentBlockPromise = 'BLOCK_04_EXECUTE_MAIN_FUNCTION';
            if (processInteraction) {
                if (processInteraction.statusDescription === 'pendingTranscription') {
                    let validTime = Number(moment().format('x'));
                    let nextExecutionTime = Number(processInteraction.uploadedDate) + (config_1.SPEECHMATICS_CALLBACK_USAGE ? 0 : config_1.NEXT_EXECUTION_TIME_TRANSCRIPTION);
                    if (validTime > nextExecutionTime)
                        return this.DownloadTranscription(processInteraction, null, currentQueueId, executionCode);
                    else
                        logger.info('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_EXECUTION', nextExecutionTime, 'CURRENT_TIME', validTime);
                }
                else
                    logger.error(`[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode: ${executionCode} getting queueId: ${currentQueueId}, Received invalid status: ${processInteraction.statusDescription}`);
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_05_SAVING_UPDATED_DATA_ON_REDIS';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_06_SENDING_TO_NEXT_QUEUE';
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            if (result
                &&
                    processInteraction?.status === 3
                &&
                    ['pendingIdentifyAgent', 'pendingInvertChannels', 'pendingSentimentAnalysis', 'pendingAnalysis'].includes(processInteraction.statusDescription)) {
                logger.info('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                let process = new process_implementation_1.ProcessImplementation();
                if (processInteraction.interactionType === 'CALL') {
                    billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidDurationTime', processInteraction.batchTypeName, processInteraction.batchTypeId);
                    billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'SpeechmaticsApi', processInteraction.batchTypeName, processInteraction.batchTypeId);
                }
                switch (processInteraction.statusDescription) {
                    case 'pendingIdentifyAgent':
                    case 'pendingInvertChannels':
                        promiseArray.push(redis_service_1.Redis.CreateQueueL('DiarizationAnalysis', currentQueueId));
                        break;
                    case 'pendingSentimentAnalysis':
                        promiseArray.push(process.RunCurrentBatchesRasaModels(sentimentNlu.vcc, 'TRANSCRIPTION', processInteraction.processId, [sentimentNlu.name], executionCode, currentQueueId));
                        promiseArray.push(redis_service_1.Redis.CreateQueueL('SentimentAnalysis', currentQueueId));
                        break;
                    case 'pendingAnalysis':
                        promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticAnalysisUpload', currentQueueId));
                        break;
                    default:
                        break;
                }
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsDownloadExecute', currentQueueId));
            }
            else if (result
                &&
                    processInteraction?.status === -3
                &&
                    processInteraction?.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsDownloadExecute', currentQueueId));
            }
            else if (processInteraction?.retry >= config_1.RETRY_TRANSCRIPTION) {
                logger.error('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsDownloadExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_TRANSCRIPTION;
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                let interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsDownloadExecute', currentQueueId));
            }
            else
                logger.error('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_07_UPDATING_PROGRESS';
            let promiseArray = [];
            if (processInteraction?.retry >= config_1.RETRY_TRANSCRIPTION) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            currentBlockPromise = 'BLOCK_08_FINISHING';
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsDownloadImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    DownloadTranscription(interaction, content = null, currentQueueId, executionCode) {
        let retry = false;
        let canceled = false;
        let empty = false;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (content)
                return Promise.resolve(content);
            else
                return this.DownloadSpeechMatics(interaction, currentQueueId, executionCode);
        })
            .then((result) => {
            if (result?.statusDescription === 'jobInProgress') {
                retry = true;
                return Promise.resolve(null);
            }
            else if (result?.statusDescription === 'jobNotFound') {
                retry = false;
                canceled = true;
                return Promise.resolve(null);
            }
            else
                logger.warn(`[TranscriptionsDownloadImplementation::DownloadTranscription] executionCode: ${executionCode} getting queueId: ${currentQueueId} unknown speechmatics status: ${result?.statusDescription || null}`);
            let regex = /\.(mp3|wav|ogg)$/;
            let jsonFileName = interaction.fileName.replace(regex, '.json');
            result.speechmaticsApi = interaction.speechmaticsApi;
            let jsonContent = JSON.stringify(result);
            let targetLocation = `${config_1.TRANSCRIPTION_FOLDER}/${interaction.vcc}/${jsonFileName}`;
            return this.SaveTranscriptionJSONFile(targetLocation, jsonContent, currentQueueId, executionCode);
        })
            .then((result) => {
            let jobStatus = null;
            if (result) {
                try {
                    jobStatus = JSON.parse(result);
                    let validChar = 0;
                    if (jobStatus?.results?.length) {
                        jobStatus.results.forEach(r => {
                            if (r.alternatives?.length) {
                                validChar += r.alternatives[0].content ? r.alternatives[0].content.length + 1 : 0;
                            }
                        });
                    }
                    else {
                        empty = true;
                        jobStatus = null;
                    }
                    interaction['durationLinguisticTime'] = jobStatus?.job?.duration ? jobStatus.job.duration : null;
                    interaction['validChar'] = validChar;
                    interaction['validWords'] = jobStatus?.results ? jobStatus.results.length : 0;
                }
                catch (err) {
                    jobStatus = null;
                }
            }
            return Promise.resolve(jobStatus);
        })
            .then((result) => {
            if (result) {
                interaction['status'] = 3;
                if (interaction.channelchange && interaction.isDiariced !== true)
                    interaction['statusDescription'] = 'pendingInvertChannels';
                else
                    interaction['statusDescription'] = interaction.isDiariced === true ? 'pendingIdentifyAgent' : (config_1.USAGE_SENTIMENT_ANALYSIS ? 'pendingSentimentAnalysis' : 'pendingAnalysis');
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else if (retry) {
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = 'DownloadTranscription';
                interaction['errorMessage'] = 'JOB_IN_PROGRESS';
                interaction['errorDetail'] = 'JOB_IN_PROGRESS';
            }
            else if (empty) {
                interaction['status'] = -3;
                interaction['retry'] = config_1.RETRY_TRANSCRIPTION;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = 'DownloadSpeechMatics';
                interaction['errorFunction'] = 'DownloadTranscription';
                interaction['errorMessage'] = 'JSON_TRANSCRIPTION_CONTENT_IS_EMPTY';
                interaction['errorDetail'] = 'JSON_TRANSCRIPTION_CONTENT_IS_EMPTY';
            }
            else if (canceled) {
                interaction['status'] = -3;
                interaction['retry'] = config_1.RETRY_TRANSCRIPTION;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = 'DownloadTranscription';
                interaction['errorMessage'] = 'JOB_EXPIRED';
                interaction['errorDetail'] = 'JOB_EXPIRED';
            }
            else {
                logger.error('[TranscriptionsDownloadImplementation::DownloadTranscription] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'DOWNLOAD_ERROR');
                interaction['status'] = -3;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = 'DownloadTranscription';
                interaction['errorMessage'] = 'DOWNLOAD_ERROR';
                interaction['errorDetail'] = 'DOWNLOAD_ERROR';
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsDownloadImplementation::DownloadTranscription] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            interaction['status'] = -3;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorFunction'] = 'DownloadTranscription';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    DownloadSpeechMatics(interaction, currentQueueId, executionCode) {
        let options = {
            uri: `${interaction.speechmaticsProtocol}://${interaction.speechmaticsApi}/v2/jobs/${interaction.jobId}/transcript?format=json-v2`,
            retryProtocol: false,
            validateErrorResponse: true,
            timeout: 30000
        };
        if (interaction.speechmaticsToken)
            options['headers'] = {
                Authorization: `Bearer ${interaction.speechmaticsToken}`
            };
        let data;
        return request_shared_1.SimpleGetRequest(options)
            .then((result) => {
            data = result;
            return Promise.resolve(result);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            let pending = {};
            if (typeof errorMessage === 'string'
                &&
                    (errorMessage.indexOf('Job In Progress') > -1
                        ||
                            errorMessage.indexOf('Transcription not ready') > -1)) {
                logger.warn('[TranscriptionsDownloadImplementation::DownloadSpeechMatics] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                pending['statusDescription'] = 'jobInProgress';
            }
            else if (typeof errorMessage === 'string'
                &&
                    (errorMessage.indexOf('Job not found') > -1
                        ||
                            errorMessage.includes('Job rejected'))) {
                logger.warn('[TranscriptionsDownloadImplementation::DownloadSpeechMatics] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                pending['statusDescription'] = 'jobNotFound';
            }
            else {
                logger.error('[TranscriptionsDownloadImplementation::DownloadSpeechMatics] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            }
            data = pending?.statusDescription ? pending : null;
            return Promise.resolve(data);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsDownloadImplementation::DownloadSpeechMatics] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(null);
        });
    }
    SaveTranscriptionJSONFile(fileName, data, currentQueueId, executionCode) {
        return file_manager_1.WriteFileContent(fileName, data, 'utf-8', true)
            .then((result) => {
            return Promise.resolve(data);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[TranscriptionsDownloadImplementation::SaveTranscriptionJSONFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(null);
        });
    }
}
exports.TranscriptionsDownloadImplementation = TranscriptionsDownloadImplementation;
