"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TranscriptionsStandbyImplementation = void 0;
const config_1 = require("../config/config");
const redis_service_1 = require("../db/redis.service");
const log4js = require("log4js");
const promises_shared_1 = require("../shared/promises.shared");
const request_shared_1 = require("../shared/request.shared");
const moment = require("moment");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'TranscriptionsStandbyImplementation';
class TranscriptionsStandbyImplementation {
    Execute(executionCode, rateRecursive) {
        logger.info(`[TranscriptionsStandbyImplementation::Execute] executionCode: ${executionCode} executing wave #${rateRecursive}`);
        let executeIndex = 1;
        let ratio = config_1.RATIO_TRANSCRIPTION_STANDBY;
        let executeResult = true;
        let currentBlockPromise;
        return promises_shared_1.SleepPromise(config_1.SLEEP_TRANSCRIPTION)
            .then(_ => {
            currentBlockPromise = 'BLOCK_01_CHECK_IF_IS_EXECUTION_TIME';
            let available = false;
            let currentMoment = moment();
            if ((currentMoment.minute() % 5) === 1)
                available = true;
            if (!available)
                throw 'SERVER_INFO_WAITING_FOR_TIME_TO_EXECUTE';
            return Promise.resolve(true);
        })
            .then(_ => {
            currentBlockPromise = 'BLOCK_02_GETTING_SPEECHMATICS_JOBS';
            return this.GetCurrentJobs(executionCode);
        })
            .then(_ => {
            currentBlockPromise = 'BLOCK_03_GET_FROM_REDIS';
            return redis_service_1.Redis.LRange('TranscriptionsStandby', 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_04_LAUNCH_TRANSCRIPTIONS_STANDBY_IMPL';
            let promiseArray = [];
            ratio = result?.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(this.TranscriptionsStandbyExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((results) => {
            currentBlockPromise = 'BLOCK_05_GET_FROM_REDIS_AGAIN';
            if (results.filter(item => item === false).length)
                executeResult = false;
            return redis_service_1.Redis.LRange('TranscriptionsStandby', 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_06_EXECUTING_RECURSIVITY';
            if (result?.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_07_FINISHING';
            if (rateRecursive === config_1.RATE_RECURSIVE_TRANSCRIPTION_STANDBY)
                logger.info(`[TranscriptionsStandbyImplementation::Execute] executionCode: ${executionCode} Finishing execution wave #${rateRecursive}`);
            redis_service_1.Redis.Del(`CurrentJobs:${executionCode}`);
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage === 'SERVER_INFO_WAITING_FOR_TIME_TO_EXECUTE')
                logger.info('[TranscriptionsStandbyImplementation::Execute] executionCode:', executionCode, 'getting info:', errorMessage);
            else
                logger.error('[TranscriptionsStandbyImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            redis_service_1.Redis.Del(`CurrentJobs:${executionCode}`);
            return Promise.resolve(false);
        });
    }
    TranscriptionsStandbyExecute(executionCode) {
        let storage = '';
        let currentQueueId = '';
        let processInteraction;
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let interactionId = '';
        let interactionSection = 0;
        let interactionPart = 0;
        let currentBlockPromise = null;
        return redis_service_1.Redis.RPopLPush('TranscriptionsStandby', 'TranscriptionsStandbyExecute')
            .then((queueId) => {
            currentBlockPromise = 'BLOCK_01_GETTING_INTERACTION_DATA_FROM_REDIS_STORAGE';
            currentQueueId = queueId;
            let item = currentQueueId.split(':');
            if (item?.length) {
                vcc = item[0];
                processId = item[1];
                interactionType = item[2];
                interactionId = item[3];
                interactionSection = Number(item[4]);
                interactionPart = Number(item[5]);
                storage = vcc + ':' + processId;
                logger.info('[TranscriptionsStandbyImplementation::TranscriptionsStandbyExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
            }
            else {
                logger.error('[TranscriptionsStandbyImplementation::TranscriptionsStandbyExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                return Promise.resolve(null);
            }
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_02_SETTING_INTERACTION_DATA';
            if (result) {
                processInteraction = result;
                logger.info('[TranscriptionsStandbyImplementation::TranscriptionsStandbyExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                let currentTime = Number(moment().format('x'));
                let limitTime = Number(processInteraction.uploadedDate) + 1800;
                if (currentTime > limitTime) {
                    logger.info('[TranscriptionsStandbyImplementation::TranscriptionsStandbyExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Moving to Download queue for limitTime');
                    return Promise.resolve(true);
                }
                else
                    return this.CheckCurrentQueue(executionCode, currentQueueId, processInteraction);
            }
            else {
                if (currentQueueId)
                    logger.warn('[TranscriptionsStandbyImplementation::TranscriptionsStandbyExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return Promise.resolve(null);
            }
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_03_REINSERTING_TO_DOWNLOAD_QUEUE';
            if (result)
                return redis_service_1.Redis.DeleteInsertOnList('TranscriptionsStandbyExecute', 'TranscriptionsDownload', currentQueueId);
            return Promise.resolve(null);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_04_FINISHING';
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsStandbyImplementation::TranscriptionDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    GetCurrentJobs(executionCode) {
        let speechmaticsServers = config_1.SPEECHMATICS_SERVERS;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let promiseArray = [];
            speechmaticsServers.forEach(ss => promiseArray.push(this.GetCurrentJob(executionCode, ss.url, ss.token)));
            return Promise.all(promiseArray);
        })
            .then(result => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsStandbyImplementation::GetCurrentJobs] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    GetCurrentJob(executionCode, url, token) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let options = {
                uri: `http://${url}/v2/jobs/`,
                retryProtocol: false,
                validateErrorResponse: true,
                timeout: 30000
            };
            if (token)
                options['headers'] = {
                    Authorization: `Bearer ${token}`
                };
            return request_shared_1.SimpleGetRequest(options);
        })
            .then(result => {
            if (!result?.jobs?.length)
                throw 'SERVER_ERROR_SPEECHMATICS_JOBS_NOT_FOUND';
            return redis_service_1.Redis.SaveCurrentJob(executionCode, url, result);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsStandbyImplementation::GetCurrentJob] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    CheckCurrentQueue(executionCode, currentQueueId, processInteraction) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetCurrentJob(executionCode, processInteraction.speechmaticsApi);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_JOBS_STORAGE_NOT_FOUND';
            let jobs = result.jobs;
            if (jobs.find(j => j.id === processInteraction.jobId)) {
                let currentJob = jobs.find(j => j.id === processInteraction.jobId);
                if (currentJob.status === 'done') {
                    return Promise.resolve(true);
                }
                else {
                    logger.info(`[TranscriptionsStandbyImplementation::CheckCurrentQueue] executionCode: ${executionCode} queueId ${currentQueueId} status: ${currentJob.job_status}`);
                    return Promise.resolve(false);
                }
            }
            logger.warn(`[TranscriptionsStandbyImplementation::CheckCurrentQueue] executionCode: ${executionCode} interaction ${currentQueueId} not found on jobs (jobId ${processInteraction.jobId})`);
            return Promise.resolve(false);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsStandbyImplementation::CheckCurrentQueue] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
}
exports.TranscriptionsStandbyImplementation = TranscriptionsStandbyImplementation;
