"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TranscriptionsUploadImplementation = void 0;
const jwt = require("jsonwebtoken");
const moment = require("moment");
const log4js = require("log4js");
const fs = require("fs");
var FormData = require('form-data');
const mssql_service_1 = require("../db/mssql.service");
const config_1 = require("../config/config");
const redis_service_1 = require("../db/redis.service");
const file_manager_1 = require("../shared/file-manager");
const promises_shared_1 = require("../shared/promises.shared");
const billing_implementation_1 = require("./billing.implementation");
const io_request_1 = require("../io/io.request");
const io_model_1 = require("../models/io.model");
const events_1 = require("../io/events");
const request_shared_1 = require("../shared/request.shared");
const server_config_1 = require("../config/server.config");
const jwt_config_1 = require("../config/jwt.config");
const audio_transfer_implementation_1 = require("./audio-transfer.implementation");
const http = require('http');
const https = require('https');
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'TranscriptionsUploadImplementation';
class TranscriptionsUploadImplementation {
    constructor() {
        this.externalUserId = 'ext_user';
        this.externalUserVcc = 'system';
        this.availableSpeechmatics = config_1.SPEECHMATICS_SERVERS;
    }
    Execute(executionCode, rateRecursive, availableBatch, cancelledBatch) {
        logger.info(`[TranscriptionsUploadImplementation::Execute] executionCode: ${executionCode} executing wave #${rateRecursive}`);
        let executeIndex = 1;
        let ratio = config_1.RATIO_TRANSCRIPTION;
        let alowedRatio = 0;
        let notAlowedRatio = 0;
        let executeResult = true;
        let currentBlockPromise;
        return promises_shared_1.SleepPromise(config_1.SLEEP_TRANSCRIPTION)
            .then(_ => {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.GetLengthFromList('TranscriptionsUpload');
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_02_GET_SPEECHMATICS_FREE_STORAGE';
            if (!result)
                throw 'SERVER_INFO_REDIS_KEYS_FOR_TRANSCRIPTIONS_UPLOAD_NOT_FOUND';
            return this.GetSpeechmaticsFreeStorage(executionCode);
        })
            .then(_ => {
            currentBlockPromise = 'BLOCK_03_GET_QUEUE_ITEMS';
            if (!this.availableSpeechmatics.length)
                throw 'SERVER_INFO_NO_SPEECHMATICS_WITH_FREE_STORAGE';
            return redis_service_1.Redis.LRange('TranscriptionsUpload', 0, ratio - 1);
        })
            .then((result) => {
            let promiseArray = [];
            ratio = result?.length <= ratio ? result.length : 0;
            result.forEach(r => {
                let arrQueue = r.split(':');
                if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[1].toLowerCase())) {
                    alowedRatio++;
                }
                else {
                    notAlowedRatio++;
                }
            });
            while (ratio >= executeIndex) {
                promiseArray.push(this.TranscribeAudioExecute(executionCode, availableBatch, cancelledBatch, executeIndex));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((results) => {
            if (results.filter(item => item === false).length > notAlowedRatio)
                executeResult = false;
            return redis_service_1.Redis.LRange('TranscriptionsUpload', 0, ratio - 1);
        })
            .then((result) => {
            if (result?.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return Promise.resolve(executeResult);
        })
            .then(result => {
            if (rateRecursive === config_1.RATE_RECURSIVE_TRANSCRIPTION)
                logger.info(`[TranscriptionsUploadImplementation::Execute] executionCode: ${executionCode} Finishing execution wave #${rateRecursive}`);
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info('[TranscriptionsUploadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting info:', errorMessage);
            else
                logger.error('[TranscriptionsUploadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    TranscribeAudioExecute(executionCode, availableBatch, cancelledBatch, index) {
        let storage = '';
        let currentQueueId = '';
        let processInteraction;
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let interactionId = '';
        let interactionSection = 0;
        let interactionPart = 0;
        let currentBlockPromise = null;
        return redis_service_1.Redis.RemoveQueueL('TranscriptionsUpload')
            .then((queueId) => {
            currentBlockPromise = 'BLOCK_01_GETTING_INTERACTION_DATA_FROM_REDIS_STORAGE';
            currentQueueId = queueId;
            let item = currentQueueId.split(':');
            if (item?.length) {
                vcc = item[0];
                processId = item[1];
                interactionType = item[2];
                interactionId = item[3];
                interactionSection = Number(item[4]);
                interactionPart = Number(item[5]);
                storage = vcc + ':' + processId;
                logger.info('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
            }
            else {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                return Promise.resolve(null);
            }
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_02_SETTING_INTERACTION_DATA';
            if (result) {
                processInteraction = result;
                logger.info('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                return Promise.resolve(true);
            }
            else {
                if (currentQueueId)
                    logger.warn('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return Promise.resolve(null);
            }
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_03_VALIDATE_ALLOWABLE';
            let arrQueue = currentQueueId.split(':');
            let promiseArray = [];
            if (result) {
                if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === processInteraction.batchTypeId.toLowerCase())) {
                    promiseArray.push(Promise.resolve('ALLOWED'));
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('TranscriptionsUploadExecute', currentQueueId));
                    promiseArray.push(Promise.resolve(currentQueueId));
                }
                else {
                    promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                    promiseArray.push(redis_service_1.Redis.CreateQueueR('TranscriptionsUpload', currentQueueId));
                    promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[1], 'TRANSCRIPTION', 'PAUSED'));
                    promiseArray.push(io_request_1.SendIOEvents([
                        new io_model_1.IOEvent(io_model_1.VCC_EVENT, events_1.UPDATE_TRANSCRIPTION, null, arrQueue[0])
                    ]));
                }
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_04_EXECUTE_MAIN_FUNCTION';
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            if (processInteraction) {
                if (processInteraction.statusDescription === 'pendingUpload') {
                    if (processInteraction.interactionType === 'CALL') {
                        if (!processInteraction.speechmaticsApi
                            ||
                                processInteraction.status === -3) {
                            let speechmatics = this.GetSpeechmaticsApi(index - 1);
                            processInteraction.speechmaticsApi = speechmatics.url;
                            processInteraction.speechmaticsToken = speechmatics.token;
                            processInteraction.speechmaticsProtocol = speechmatics.protocol;
                        }
                        return this.UploadAudio(processInteraction, currentQueueId, executionCode);
                    }
                    else {
                        processInteraction.status = 3;
                        processInteraction.statusDescription = config_1.USAGE_SENTIMENT_ANALYSIS ? 'pendingSentimentAnalysis' : 'pendingAnalysis';
                    }
                }
                else
                    logger.error(`[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode: ${executionCode} getting queueId: ${currentQueueId}, Received invalid status: ${processInteraction.statusDescription}`);
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_05_SAVING_UPDATED_DATA_ON_REDIS';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_06_SENDING_TO_NEXT_QUEUE';
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            if (result
                &&
                    processInteraction?.status === 3
                &&
                    ['pendingTranscription', 'pendingSentimentAnalysis', 'pendingAnalysis'].includes(processInteraction?.statusDescription)
                &&
                    processInteraction?.retry < config_1.RETRY_TRANSCRIPTION) {
                if (processInteraction.interactionType === 'CALL' && config_1.SPEECHMATICS_CALLBACK_USAGE)
                    promiseArray.push(redis_service_1.Redis.InsertQueue('TranscriptionsStandby', currentQueueId));
                else
                    promiseArray.push(redis_service_1.Redis.InsertQueue('TranscriptionsDownload', currentQueueId, config_1.RATE_RECURSIVE_TRANSCRIPTION * config_1.RATIO_TRANSCRIPTION * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
            }
            else if (result
                &&
                    processInteraction?.status === -3
                &&
                    processInteraction?.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
            }
            else if (processInteraction?.status === -3
                &&
                    processInteraction?.errorMessage === 'UPLOAD_ERROR'
                &&
                    processInteraction?.retry >= 500) {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'UPLOAD_ERROR');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'UPLOAD_ERROR');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_TRANSCRIPTION;
            }
            else if (processInteraction?.retry >= config_1.RETRY_TRANSCRIPTION) {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_TRANSCRIPTION;
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                let interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart,
                    retry: config_1.RETRY_TRANSCRIPTION
                };
                let audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
                audioTransfer.CloseTranscriptionDetailWithMissingData(billing, interactionData, 'TranscriptionsUploadImplementation');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
            }
            else {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_07_UPDATING_PROGRESS';
            let promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_TRANSCRIPTION) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_08_FINISHING';
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    UploadAudio(interaction, currentQueueId, executionCode) {
        let fullPath = '';
        let dictionaryContent = [];
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return this.GetDictionaryContent(interaction, currentQueueId, executionCode);
        })
            .then(result => {
            dictionaryContent = result;
            return new Promise((resolve, reject) => {
                let baseFolder = (['ftp', 'sftp', 's3'].includes(interaction.environmentType)) ? config_1.PATH_FOLDER_PROCESS_AUDIO : config_1.PATH_FOLDER_TEMP_AUDIO;
                fullPath = `${baseFolder}/${interaction.vcc}/${interaction.fileName}`;
                logger.info('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'audio:', fullPath);
                fs.stat(fullPath, (err, stat) => {
                    resolve({ fullPath: fullPath, stat: stat });
                });
            });
        })
            .then((result) => {
            if (result && result.stat && result.stat.size > 0) {
                return file_manager_1.BufferFile(result.fullPath);
            }
            else {
                logger.error('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Audio file not found on fullPath:', fullPath);
                throw 'SERVER_ERROR_AUDIO_FILE_NOT_FOUND';
            }
        })
            .then((stream) => {
            if (stream) {
                logger.debug('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Process language:', interaction.language, 'READY_TO_UPLOAD');
                let trancriptionConfig = {
                    type: 'transcription',
                    transcription_config: {
                        diarization: 'channel',
                        channel_diarization_labels: ['Agent', 'Caller'],
                        language: interaction.language,
                        additional_vocab: dictionaryContent
                    }
                };
                if (config_1.SPEECHMATICS_CALLBACK_USAGE) {
                    trancriptionConfig.notification_config = [{
                            url: `${server_config_1.WEBHOOK_URL}/speechmatics/webhook/`,
                            contents: ['transcript'],
                            auth_headers: [
                                `Authorization: Bearer ${this.GetSystemUserToken()}`,
                                `speech: ${server_config_1.EXTERNAL_URL}/api/public/task/update_transcription_queue/${currentQueueId}/`
                            ]
                        }];
                }
                const formData = new FormData();
                formData.append('data_file', stream, interaction.fileName);
                formData.append('config', JSON.stringify(trancriptionConfig));
                let data = {
                    uri: `${interaction.speechmaticsProtocol}://${interaction.speechmaticsApi}/v2/jobs/`,
                    fileName: interaction.fileName,
                    validateErrorResponse: true,
                    timeout: 600000
                };
                if (interaction.speechmaticsToken)
                    data['headers'] = {
                        Authorization: `Bearer ${interaction.speechmaticsToken}`
                    };
                return request_shared_1.AttachedFilePostRequest(data, formData);
            }
            return Promise.resolve(null);
        })
            .then((jobStatus) => {
            let jobId = jobStatus && jobStatus.id ? jobStatus.id : null;
            if (jobId) {
                logger.info('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'UPLOADED');
                interaction['status'] = 3;
                interaction['statusDescription'] = 'pendingTranscription';
                interaction['retry'] = 0;
                interaction['jobId'] = jobId;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'JOBID_IS_NOT_FOUND');
                interaction['status'] = -3;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'UploadAudio';
                interaction['errorMessage'] = 'JOBID_IS_NOT_FOUND';
                interaction['errorDetail'] = 'JOBID_IS_NOT_FOUND';
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            if (typeof errorMessage === 'string'
                &&
                    errorMessage.includes('data_file is too small')) {
                interaction['isCancel'] = true;
                interaction['retry'] = config_1.RETRY_TRANSCRIPTION;
            }
            else
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['status'] = -3;
            interaction['errorFunction'] = 'UploadAudio';
            interaction['errorMessage'] = 'UPLOAD_ERROR';
            interaction['errorDetail'] = err.stack ? err.stack : errorMessage;
            return Promise.resolve(interaction);
        });
    }
    GetDictionaryContent(interaction, currentQueueId, executionCode) {
        let dictionaryWords = [];
        let dictionaryContent = [];
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetDictionaryWords(interaction.vcc, null, null, interaction.dictionaryId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GET_DICTIONARY_WORDS';
            dictionaryWords = result;
            let dictionaryWordIds = [];
            dictionaryWords.forEach(dw => dictionaryWordIds.push(dw.id));
            return mssql_service_1.MSSql.GetDictionaryWordSoundLikes(interaction.vcc, dictionaryWordIds.join('\',\''));
        })
            .then((result) => {
            logger.info('[TranscriptionsUploadImplementation::GetDictionaryContent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'found words:', dictionaryWords.length);
            dictionaryWords.forEach(dw => {
                dw.soundLikes = [];
                dw.soundLikes = result.filter(r => r.dictionaryWordId === dw.id);
            });
            dictionaryWords.forEach(dw => {
                if (dw.soundLikes && dw.soundLikes.length) {
                    let soundLikeWords = [];
                    dw.soundLikes.forEach(sl => soundLikeWords.push(sl.soundLike));
                    dictionaryContent.push({
                        content: dw.word,
                        sounds_like: soundLikeWords
                    });
                }
                else {
                    dictionaryContent.push(dw.word);
                }
            });
            return Promise.resolve(dictionaryContent);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsUploadImplementation::GetDictionaryContent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve([]);
        });
    }
    GetSpeechmaticsFreeStorage(executionCode) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let promiseArray = [];
            this.availableSpeechmatics.forEach(sa => {
                if (sa.ip) {
                    let getOptions = {
                        uri: `${sa.protocol}://${sa.ip}:8080/v1/management/freestorage`,
                        timeout: 30000
                    };
                    promiseArray.push(request_shared_1.SimpleGetRequest(getOptions));
                }
                else
                    promiseArray.push(Promise.resolve([true]));
            });
            return Promise.all(promiseArray);
        })
            .then(result => {
            result.forEach((r, i) => {
                if (typeof r !== 'boolean'
                    &&
                        r?.items?.length) {
                    let minimunStorage = (2 * 1024);
                    let freeStorage = r.items[0].bytes / (1024 * 1024);
                    this.availableSpeechmatics[i].available = freeStorage > minimunStorage;
                    logger.info(`[TranscriptionsUploadImplementation::GetSpeechmaticsFreeStorage] executionCode: ${executionCode} free storage for ${this.availableSpeechmatics[i].ip} => ${freeStorage}mb => ${this.availableSpeechmatics[i].available ? 'OK' : 'KO'}`);
                }
            });
            this.availableSpeechmatics = this.availableSpeechmatics.filter(as => as.available);
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsUploadImplementation::GetSpeechmaticsFreeStorage] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    GetSpeechmaticsApi(index) {
        let apiIndex = index % this.availableSpeechmatics.length;
        let server = this.availableSpeechmatics[apiIndex];
        return server;
    }
    GetSystemUserToken() {
        let token = jwt.sign({ userId: this.externalUserId, vcc: this.externalUserVcc }, jwt_config_1.JWT_SECRET, { expiresIn: jwt_config_1.JWT_EXPIRATION_TIME });
        return token;
    }
}
exports.TranscriptionsUploadImplementation = TranscriptionsUploadImplementation;
