"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IOServerInit = exports.SpeechIO = exports.IO = void 0;
const log4js = require("log4js");
const socket_io_1 = require("socket.io");
const redis_adapter_1 = require("@socket.io/redis-adapter");
const io_shared_1 = require("./io.shared");
const io_listeners_1 = require("./io.listeners");
let logger = log4js.getLogger('IO');
function IOServerInit(app, httpServer, httpsServer) {
    logger.info('Inicializado servidor de WebSocket');
    console.log('Inicializado servidor de WebSocket');
    try {
        exports.IO = new socket_io_1.Server(httpServer, {
            cors: {
                origin: "*",
                methods: ["GET", "POST"],
                credentials: true
            }
        });
        exports.IO.attach(httpsServer);
        const redisClient = app.get('redisClient');
        if (!redisClient) {
            throw 'REDIS_CLIENT_IS_REQUIRED';
        }
        exports.IO.adapter(redis_adapter_1.createAdapter(redisClient, redisClient.duplicate()));
        exports.SpeechIO = exports.IO.of("/speech");
        exports.SpeechIO.on("connection", onConnect);
        app.set('SpeechIO', exports.SpeechIO);
        io_shared_1.SocketShared(app);
        logger.info('Servidor de WebSocket inicializado');
        console.log('Servidor de WebSocket inicializado');
        return Promise.resolve(true);
    }
    catch (err) {
        if (exports.IO) {
            exports.IO.disconnectSockets(true);
        }
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error on init socket server');
        logger.error('No se pudo inicializar servidor de WebSocket: ' + errMsg);
        console.error('No se pudo inicializar servidor de WebSocket: ' + errMsg);
        return Promise.reject(err);
    }
}
exports.IOServerInit = IOServerInit;
function onConnect(socket) {
    let data = socket.handshake.query;
    socket.join([data.vcc, `${data.vcc}-${data.userId}`]);
    io_listeners_1.InitListeners(socket);
}
