"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAbilityQueries = exports.AbilityService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const arrays_shared_1 = require("../shared/arrays.shared");
const promises_shared_1 = require("../shared/promises.shared");
const utilities_1 = require("../shared/utilities");
let logger = log4js.getLogger('ServerScripts');
function AbilityService(app) {
    app.get('/api/ability/get_ability_detail/:abilityId', (req, res) => {
        logger.info('[AbilityService::get_ability_detail] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let id = req.params.abilityId || '';
        promises_shared_1.DummyPromise().then(_ => {
            return mssql_service_1.MSSql.GetAbilityDetail(vcc, id);
        }).
            then(items => {
            res.json({ status: items ? true : false, data: items });
        }).
            catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::get_abilitys] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/ability/get_abilitys/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[AbilityService::get_abilitys] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let executionType = req.query.executionType.toString() || 'BATCH';
        let disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetAbilitys(instance, search_criteria, search_expression, executionType, disabled);
        })
            .then(abilities => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: abilities });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::get_abilitys] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/create_ability', (req, res, next) => {
        logger.info('[AbilityService::create_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let wasCreated = false;
        let newAbility = (req.body || {});
        newAbility.createdDate = nowUtc;
        newAbility.createdByUserId = userId;
        newAbility.lastModifiedDate = nowUtc;
        newAbility.lastModifiedByUserId = userId;
        newAbility.title = newAbility.title.trim();
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetAbilityTitle(vcc, newAbility.title);
        })
            .then(result => {
            if (result)
                throw 'ABILITY_SERVER_ERROR_EXISTS';
            newAbility.id = utilities_1.guid();
            return SaveAbility(vcc, newAbility);
        })
            .then(result => {
            if (!result)
                throw 'ABILITY_SERVER_ERROR_ON_SAVE';
            wasCreated = true;
            if (newAbility?.queries?.length)
                return SaveAbilityQueries(vcc, newAbility, userId);
            else
                return Promise.resolve(true);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_ABILITY_QUERY_SAVE';
            return SaveScore(vcc, newAbility, userId);
        })
            .then(result => {
            if (!result)
                throw 'ABILITY_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(newAbility.id);
        })
            .then(resultSave => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'Abilidad save failed'), data: resultSave });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            if (wasCreated)
                mssql_service_1.MSSql.DeleteAbility(vcc, newAbility.id);
            logger.error('[AbilityService::create_ability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/ability/get_ability/:ability_id', (req, res, next) => {
        logger.info('[AbilityService::get_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let ability = {};
        let scores = [];
        let abilityId = req.params.ability_id;
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!abilityId)
                throw 'ABILITY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetAbility(vcc, abilityId);
        })
            .then(result => {
            if (!abilityId)
                throw 'ABILITY_SERVER_ERROR_MISSING_ID';
            ability = result || ability;
            return GetAbilityQueries(vcc, abilityId, true);
        })
            .then(result => {
            ability.queries = result;
            return mssql_service_1.MSSql.GetScores(vcc, abilityId);
        })
            .then(resultScores => {
            if (res.headersSent)
                return;
            scores = resultScores || scores;
            ability.scores = scores;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: ability });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/save_ability', (req, res, next) => {
        logger.info('[AbilityService::save_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newAbility = (req.body || {});
        newAbility.createdDate = null;
        newAbility.createdByUserId = null;
        newAbility.lastModifiedDate = nowUtc;
        newAbility.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetAbilityTitle(vcc, newAbility.title);
        }).then(result => {
            let abilityDesc = {};
            abilityDesc = result || abilityDesc;
            if (result && abilityDesc.id !== newAbility.id)
                throw 'ABILITY_SERVER_ERROR_EXISTS';
            return mssql_service_1.MSSql.SaveAbility(vcc, newAbility.id, newAbility.title, newAbility.detail, newAbility.type, newAbility.calificationType, newAbility.executionType, newAbility.binary, newAbility.notApply, newAbility.distribute, newAbility.isCritical, newAbility.acceptComments, newAbility.createdDate, newAbility.createdByUserId, newAbility.lastModifiedDate, newAbility.lastModifiedByUserId, newAbility.enabled, newAbility.analysisType, newAbility.language, newAbility.isNegative, newAbility.degradeScoreCard);
        })
            .then(result => {
            return mssql_service_1.MSSql.DeleteAbilityQueries(vcc, newAbility.id);
        })
            .then(result => {
            if (newAbility.queries && newAbility.queries.length > 0) {
                return SaveAbilityQueries(vcc, newAbility, userId);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(resultIdAbility => {
            return mssql_service_1.MSSql.DeleteScore(vcc, newAbility.id);
        }).then(resultDeleteScore => {
            return SaveScore(vcc, newAbility, userId);
        })
            .then(resultSave => {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'ABILITY_SERVER_ERROR_ON_SAVE'), data: resultSave });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::save_ability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/delete_ability', (req, res, next) => {
        logger.info('[AbilityService::delete_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let ability = (req.body || {});
        let abilityId = ability.id;
        logger.info(`[AbilityService::delete_ability] Deleting ability ${abilityId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!ability)
                throw 'ABILITY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteScore(vcc, abilityId);
        }).then(result => {
            return mssql_service_1.MSSql.DeleteAbility(vcc, abilityId);
        }).then(result => {
            if (!result)
                throw 'ABILITY_SERVER_ERROR_CANNOT_DELETE';
            return Promise.resolve(true);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'ABILITY_SERVER_ERROR_CANNOT_DELETE'), data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::delete_ability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/delete_abilitys', (req, res, next) => {
        logger.info('[abilityService::delete_abilitys] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let abilitys = req.body.abilitys || [];
        let ids = abilitys.map(item => item.id);
        logger.info(`[AbilityService::delete_abilitys] Deleting ability ${ids.join(',')} by the user ${res.locals.userId}`);
        let response;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetAbilitiesUsage(vcc, ids);
        })
            .then(resultNoDeleted => {
            let deleteData = ids;
            let noDelete = [];
            if (resultNoDeleted.length > 0) {
                deleteData = [];
                abilitys.forEach(item => {
                    if (!resultNoDeleted.find(noDeleted => noDeleted.idAbility === item.id)) {
                        deleteData.push(item.id);
                    }
                    else {
                        noDelete.push(item.title);
                    }
                });
            }
            return Promise.resolve({ noDelete, deleteData });
        })
            .then(resultDelete => {
            response = resultDelete;
            if (resultDelete.deleteData.length)
                return mssql_service_1.MSSql.DeleteAbilities(vcc, resultDelete.deleteData);
            return Promise.resolve(true);
        }).then(resultDelete => {
            if (res.headersSent)
                return;
            res.json({ status: resultDelete, description: 'SERVER_SUCCESS_MESSAGE', data: response });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[abilityService::delete_abilitys] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AbilityService = AbilityService;
function SaveAbility(vcc, ability) {
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveAbility(vcc, ability.id, ability.title, ability.detail, ability.type, ability.calificationType, ability.executionType, ability.binary, ability.notApply, ability.distribute, ability.isCritical, ability.acceptComments, ability.createdDate, ability.createdByUserId, ability.lastModifiedDate, ability.lastModifiedByUserId, ability.enabled, ability.analysisType, ability.language, ability.isNegative, ability.degradeScoreCard));
    return Promise.all(promiseArray)
        .then(results => {
        let falseResults = results.filter(res => res[0].result !== true && res[0].result !== 1);
        if (falseResults.length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function SaveAbilityQueries(vcc, ability, userId) {
    return promises_shared_1.DummyPromise()
        .then(result => {
        let promiseArray = [];
        if (ability && ability.queries && ability.queries.length > 0) {
            ability.queries.map(aq => {
                aq.id = utilities_1.guid();
                aq.abilityId = ability.id;
                promiseArray.push(SaveAbilityQuery(vcc, aq, userId));
            });
        }
        return Promise.all(promiseArray);
    })
        .then(result => {
        let saveResult = false;
        if (result.filter(r => { return (r !== true); }).length > 0) {
            saveResult = false;
        }
        else {
            saveResult = true;
        }
        return Promise.resolve(true);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[Abilityervice::SaveAbilityQueries] Error: ' + errorMsg);
        return Promise.resolve(false);
    });
}
function SaveAbilityQuery(vcc, abilityQuery, userId) {
    return promises_shared_1.DummyPromise()
        .then(result => {
        return mssql_service_1.MSSql.SaveAbilityQuery(vcc, abilityQuery.id, abilityQuery.abilityId, abilityQuery.queryId, abilityQuery.queryContent, abilityQuery.acceptancePercentage, abilityQuery.order, userId, userId);
    })
        .then(result => {
        let promiseArray = [];
        if (abilityQuery.entities && abilityQuery.entities.length > 0) {
            abilityQuery.entities.map(e => {
                e.abilityQueryId = abilityQuery.id;
                promiseArray.push(SaveAbilityQueryEntity(vcc, e, userId));
            });
        }
        return Promise.all(promiseArray);
    })
        .then(result => {
        let saveResult = false;
        if (result.filter(r => { return (r !== true); }).length > 0) {
            saveResult = false;
        }
        else {
            saveResult = true;
        }
        return Promise.resolve(true);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[Abilityervice::SaveAbilityQuery] Error: ' + errorMsg);
        return Promise.resolve(false);
    });
}
function SaveAbilityQueryEntity(vcc, abilityQueryEntity, userId) {
    return mssql_service_1.MSSql.SaveAbilityQueryEntity(vcc, abilityQueryEntity.abilityQueryId, abilityQueryEntity.entityId, abilityQueryEntity.entityName, abilityQueryEntity.acceptancePercentage, abilityQueryEntity.order, userId, userId);
}
function GetAbilityQueries(vcc, abilityId, getEntities = false, evaluationPlanId = null) {
    let abilityQueries = [];
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetAbilityQueries(vcc, abilityId, evaluationPlanId);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_ABILITY_QUERIES_NOT_FOUND';
        if (getEntities) {
            return GetAbilityQueryEntities(vcc, result, evaluationPlanId);
        }
        else {
            result.map(r => r.entities = []);
            return Promise.resolve(result);
        }
    })
        .then(result => {
        abilityQueries = result;
        return Promise.resolve(abilityQueries);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[AbilityService::GetAbilityQueries] Error: ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAbilityQueries = GetAbilityQueries;
function GetAbilityQueryEntities(vcc, abilityQueries, evaluationPlanId = null) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        let promiseArray = [];
        abilityQueries.map(aq => {
            promiseArray.push(mssql_service_1.MSSql.GetAbilityQueryEntities(vcc, aq.id, evaluationPlanId));
        });
        return Promise.all(promiseArray);
    })
        .then(result => {
        result.map((r, i) => {
            abilityQueries[i].entities = r;
        });
        return Promise.resolve(abilityQueries);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[Abilityervice::GetAbilityQueryEntities] Error: ' + errorMsg);
        return Promise.reject(err);
    });
}
function SaveScore(vcc, ability, userId) {
    let promiseArray = [];
    let hasTipifications = false;
    if (ability.scores?.length) {
        ability.scores.forEach(score => {
            promiseArray.push(mssql_service_1.MSSql.SaveScore(vcc, score.id, ability.id, score.description, score.value));
            if (score?.tipifications?.length)
                hasTipifications = true;
        });
    }
    return Promise.all(promiseArray)
        .then(_ => {
        return mssql_service_1.MSSql.GetScores(vcc, ability.id);
    })
        .then(result => {
        promiseArray = [];
        if (hasTipifications) {
            ability.scores.filter(score => score.tipifications).forEach(score => {
                score.tipifications = score.tipifications.filter(t => t);
                let savedScore = result.find(s => s.description === score.description && s.value === score.value);
                score.id = savedScore.id;
                let classifiedTipifications = arrays_shared_1.GetCompareItemsFromObjectArray(savedScore.tipifications, score.tipifications, 'id');
                score.tipifications.forEach(t => {
                    let savedTipification = savedScore.tipifications.find(st => st.id === t.id);
                    let classifiedTipificationItems = arrays_shared_1.GetCompareItemsFromObjectArray(savedTipification?.items ? savedTipification.items : [], t.items, 'id');
                    promiseArray.push(mssql_service_1.MSSql.SaveScoreTipification(vcc, score.id, t.id, t.description, t.descriptionDetail, t.order, userId));
                    promiseArray.push(mssql_service_1.MSSql.SaveScoreTipificationQuery(vcc, score.id, t.id, t.tipificationQuery.queryId, t.tipificationQuery.queryContent, t.tipificationQuery.acceptancePercentage, userId));
                    t.items.forEach(i => {
                        promiseArray.push(mssql_service_1.MSSql.SaveScoreTipificationItem(vcc, score.id, t.id, i.id, i.secondLevelItemId, i.description, i.order, userId));
                        if (t.isNew) {
                            promiseArray.push(mssql_service_1.MSSql.SaveTipificationItem(vcc, t.id, i.id, i.secondLevelItemId, i.description, i.order, userId));
                            promiseArray.push(mssql_service_1.MSSql.SaveTipificationQuery(vcc, t.id, t.tipificationQuery.queryId, t.tipificationQuery.queryContent, t.tipificationQuery.acceptancePercentage, userId));
                        }
                        ;
                    });
                    if (classifiedTipificationItems?.removed?.length)
                        classifiedTipificationItems.removed.forEach(i => promiseArray.push(mssql_service_1.MSSql.DeleteScoreTipificationItem(vcc, score.id, i.id)));
                });
                if (classifiedTipifications?.removed?.length)
                    classifiedTipifications.removed.forEach(i => promiseArray.push(mssql_service_1.MSSql.DeleteScoreTipification(vcc, score.id, i.id)));
            });
        }
        return Promise.all(promiseArray);
    })
        .then(results => {
        if (results.filter(result => { return (result !== true); }).length)
            return Promise.resolve(false);
        else
            return Promise.resolve(true);
    });
}
