"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiUserService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const api_user_implementation_1 = require("../implementation/api-user.implementation");
let logger = log4js.getLogger('ServerScripts');
function ApiUserService(app) {
    app.get('/api/api_user/get_api_users/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetApiUsers(vcc, search_criteria, search_expression);
        }).then(users => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: users });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/api_user/get_api_users_external', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(result => {
            let apiUser = new api_user_implementation_1.ApiUserImplementation();
            return apiUser.GetApiUsers(vcc);
        }).then(users => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: users });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/api_user/create_api_user', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let apiUserI = new api_user_implementation_1.ApiUserImplementation();
        let apiUser = (req.body || {});
        apiUser.isLocked = false;
        apiUser.createdDate = nowUtc;
        apiUser.createdByUserId = userId;
        apiUser.vcc = vcc;
        promises_shared_1.DummyPromise()
            .then((result) => {
            if (vcc && apiUser.user && apiUser.pin) {
                return apiUserI.CreateApiUser(vcc, apiUser.user, apiUser.pin.toString(), userId);
            }
            else {
                throw 'MISSING_PARAMETERS';
            }
        })
            .then(result => {
            if (!result)
                throw 'ERROR_SAVE_API_USER_SERVER_EXTERNAL';
            return mssql_service_1.MSSql.SaveApiUser(apiUser);
        })
            .then(resultSaveUser => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveUser, description: (resultSaveUser ? 'SERVER_SUCCESS_MESSAGE' : resultSaveUser), data: resultSaveUser });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/api_user/get_api_user/:user_id', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = req.params.user_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!userId)
                throw 'USER_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetApiUser(vcc, userId);
        }).then(resultUser => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultUser });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/api_user/get_current_user', (req, res, next) => {
        logger.info('[ApiUserService::get_current_user] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let genericResult = {};
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!userId)
                throw 'SERVER_ERROR_MISSING_PARAMETER';
            return mssql_service_1.MSSql.GetApiUser(vcc, userId);
        })
            .then(resultUser => {
            if (!resultUser)
                throw 'SERVER_ERROR_API_USER_NOT_FOUND';
            if (res.headersSent)
                return;
            genericResult.status = true;
            genericResult.description = 'OK';
            res.json(genericResult);
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[ApiUserService::get_current_user] Error al procesar \'' + req.path + '\': ' + errorMsg);
            genericResult.status = false;
            genericResult.description = clientMsg;
            genericResult.error = errorMsg;
            if (res.headersSent)
                return;
            res.json(genericResult);
        });
    });
    app.post('/api/api_user/save_api_user', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let apiUserI = new api_user_implementation_1.ApiUserImplementation();
        let apiUser = (req.body || {});
        apiUser.createdDate = new Date(apiUser.createdDate);
        apiUser.lastModifiedDate = nowUtc;
        apiUser.lastModifiedByUserId = userId;
        apiUser.vcc = vcc;
        promises_shared_1.DummyPromise()
            .then((result) => {
            if (vcc && apiUser.user && apiUser.pin) {
                return apiUserI.SaveApiUser(vcc, apiUser.user, apiUser.pin.toString(), userId);
            }
            else {
                throw 'MISSING_PARAMETERS';
            }
        })
            .then(result => {
            if (!result)
                throw 'ERROR_SAVE_API_USER_SERVER_EXTERNAL';
            return mssql_service_1.MSSql.SaveApiUser(apiUser);
        })
            .then(resultSaveUser => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveUser, description: (resultSaveUser ? 'SERVER_SUCCESS_MESSAGE' : resultSaveUser), data: resultSaveUser });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/api_user/delete_api_user', (req, res) => {
        logger.info('[ApiUserService::delete_api_user] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let user = req.body.user;
        let userId = user.user;
        logger.info(`[ApiUserService::delete_api_user] Deleting apiUser ${userId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!userId)
                throw 'USER_SERVER_ERROR_MISSING_ID';
            let apiUser = new api_user_implementation_1.ApiUserImplementation();
            return apiUser.DeleteApiUser(vcc, userId);
        })
            .then(resultDeleteUser => {
            if (!resultDeleteUser)
                throw 'FAILED_DELETE_API_USER_EXTERNAL_SERVER';
            return DeleteApiUser(vcc, userId);
        })
            .then(resultDeleteUser => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: '', data: resultDeleteUser });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/api_user/delete_api_users', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let users = req.body.users || [];
        let ids = users.map(x => x.user);
        logger.info(`[ApiUserService::delete_api_users] Deleting apiUser ${ids.join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            let usersApi = [];
            users.forEach(us => {
                usersApi.push(us.user);
            });
            let apiUser = new api_user_implementation_1.ApiUserImplementation();
            return apiUser.DeleteApiUsers(vcc, usersApi);
        })
            .then(result => {
            if (!result)
                throw 'FAILED_DELETE_API_USERS';
            let promiseArray = [];
            users.forEach((user) => {
                promiseArray.push(DeleteApiUser(vcc, user.user));
            });
            return Promise.all(promiseArray);
        }).then(resultDeleteUsers => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: '', data: resultDeleteUsers });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.ApiUserService = ApiUserService;
function DeleteApiUser(vcc, userId) {
    let user, reason = '';
    return promises_shared_1.DummyPromise()
        .then((result) => {
        if (!userId) {
            reason = 'MISSING_USER_ID_PARAMETER';
            throw reason;
        }
        return mssql_service_1.MSSql.GetApiUser(vcc, userId);
    }).then(result => {
        user = result;
        if (!result) {
            reason = 'ERROR_USER_NOT_FOUND';
            throw reason;
        }
        return mssql_service_1.MSSql.DeleteApiUser(vcc, userId);
    }).then((result) => {
        if (!result) {
            reason = 'ERROR_DELETING_API_USER';
            throw reason;
        }
        return Promise.resolve(result);
    }).catch((error) => {
        return SetError(userId, error);
    });
}
function SetError(userId, reason) {
    let details = null;
    if ((userId && userId.length) && (reason && reason.length)) {
        details = {};
        details.error = {
            userId: userId,
            reason: reason
        };
    }
    return Promise.resolve(details);
}
