"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuditorService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const role_service_1 = require("./role.service");
let logger = log4js.getLogger('ServerScripts');
function AuditorService(app) {
    app.get('/api/auditor/get_auditors/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[AuditorService::get_auditors] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        logger.info('[AuditorService::get_auditors] Received search_criteria: ' + search_criteria);
        logger.info('[AuditorService::get_auditors] Received search_expression: ' + search_expression);
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[AuditorService::get_auditors] Calling GetAuditors SP');
            return mssql_service_1.MSSql.GetAuditors(instance, search_criteria, search_expression);
        }).then(result => {
            if (res.headersSent)
                return;
            logger.info('[AuditorService::get_auditors] Received results from GetAuditors SP: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let groupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[AuditorService::get_auditors] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: groupMsg, error: errorMsg });
        });
    });
    app.get('/api/auditor/get_auditors_interactions/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('Se recibió p' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('Instance ' + instance);
            logger.info('sear. criterial  ' + search_criteria);
            logger.info('sear. expression  ' + search_expression);
            return mssql_service_1.MSSql.GetAuditorsInteractions(instance, search_criteria, search_expression);
        }).then(result => {
            logger.info('Datos prueba ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let groupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: groupMsg, error: errorMsg });
        });
    });
    app.get('/api/auditor/get_auditor/:auditor_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let auditorId = req.params.auditor_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!auditorId)
                throw 'AUDITOR_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetAuditor(instance, auditorId);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/auditor/get_auditor_evaluationplan_evaluation', (req, res, next) => {
        logger.info('[AuditorService::get_auditor_evaluationplan_evaluation] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let userId = res.locals.userId;
        let userRole = res.locals.user.role;
        let auditorId = null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return role_service_1.GetRoleFunctions(vcc, userId, userRole);
        })
            .then(result => {
            auditorId = req.body.auditorId ? req.body.auditorId : ((result?.data?.hasAdminFunctions || result?.data?.hasAuditorFunctions) ? userId : null);
            if (!auditorId)
                throw 'AUDITOR_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetAuditorEvaluationPlanEvaluation(vcc, auditorId, req.body.evaluationPlanId, req.body.interactionId, req.body.interactionType, req.body.efectivityPlanId);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AuditorService = AuditorService;
