"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveCategories = exports.SaveCategory = exports.GetCategory = exports.CategoryService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
let logger = log4js.getLogger('ServerScripts');
function CategoryService(app) {
    app.get('/api/category/get_categories/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetCategories(vcc, search_criteria, search_expression);
        }).then(categories => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: categories });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/category/get_category/:category_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let categoryId = req.params.category_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!categoryId)
                throw 'CATEGORY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetCategory(vcc, categoryId);
        }).then(resultCategory => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultCategory });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/category/create_category', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newCategory = (req.body || {});
        newCategory.createdDate = nowUtc;
        newCategory.createdByUserId = userId;
        newCategory.lastModifiedDate = nowUtc;
        newCategory.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetCategoryByName(vcc, newCategory.name);
        }).then(resultCategory => {
            if (resultCategory)
                throw 'CATEGORY_SERVER_ERROR_EXISTS';
            return SaveCategory(vcc, newCategory);
        }).then((resultSaveCategory) => {
            if (!resultSaveCategory)
                throw 'CATEGORY_SERVER_ERROR_ON_SAVE';
            return resultSaveCategory;
        }).then(resultSaveCategory => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveCategory ? 'SERVER_SUCCESS_MESSAGE' : 'CATEGORY_SERVER_ERROR_ON_SAVE'), data: resultSaveCategory });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/category/save_category', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newCategory = (req.body || {});
        newCategory.createdDate = null;
        newCategory.createdByUserId = null;
        newCategory.lastModifiedDate = nowUtc;
        newCategory.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(resultDeleteCategory => {
            if (!resultDeleteCategory)
                throw 'CATEGORY_SERVER_ERROR_ON_SAVE';
            return SaveCategory(vcc, newCategory);
        }).then(resultSaveCategory => {
            if (!resultSaveCategory)
                throw 'CATEGORY_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(resultSaveCategory => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveCategory, description: (resultSaveCategory ? 'SERVER_SUCCESS_MESSAGE' : 'CATEGORY_SERVER_ERROR_ON_SAVE'), data: resultSaveCategory });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/category/delete_category', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let category = req.body.category;
        let categoryId = category.name;
        logger.info(`[CategoryService::delete_category] Deleting category ${categoryId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!category)
                throw 'CATEGORY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteCategory(vcc, categoryId);
        }).then(resultDeleteCategory => {
            if (!resultDeleteCategory)
                throw 'CATEGORY_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(resultDeleteCategory => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteCategory, description: (resultDeleteCategory ? 'SERVER_SUCCESS_MESSAGE' : 'CATEGORY_SERVER_ERROR_ON_DELETE'), data: resultDeleteCategory });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/category/delete_categories', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let categories = req.body.categories || [];
        logger.info(`[CategoryService::delete_categories] Deleting categories ${categories.map(x => x.name).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!categories)
                throw 'CATEGORY_SERVER_ERROR_MISSING_ID';
            let promiseArray = [];
            categories.forEach(category => {
                promiseArray.push(mssql_service_1.MSSql.DeleteCategory(vcc, category.name));
            });
            return Promise.all(promiseArray);
        }).then(resultDeleteCategories => {
            let result;
            if (resultDeleteCategories.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Categorías eliminadas correctamente');
            }
            return result;
        }).then(resultDeleteCategories => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteCategories, description: (resultDeleteCategories ? 'SERVER_SUCCESS_MESSAGE' : 'CATEGORY_SERVER_ERROR_ON_DELETE'), data: resultDeleteCategories });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.CategoryService = CategoryService;
function GetCategory(vcc, name) {
    let cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(() => {
        return mssql_service_1.MSSql.GetCategory(vcc, name);
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetCategory: ' + errMsg);
    });
}
exports.GetCategory = GetCategory;
function SaveCategory(vcc, category) {
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveCategory(vcc, category.name, category.description, category.createdDate, category.createdByUserId, category.lastModifiedDate, category.lastModifiedByUserId));
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveCategory = SaveCategory;
function SaveCategories(vcc, category) {
    let promiseArray = [];
    if (category) {
        category.forEach(categories => {
            promiseArray.push(mssql_service_1.MSSql.SaveCategory(vcc, categories.name, categories.description, categories.createdDate, categories.createdByUserId, categories.lastModifiedDate, categories.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveCategories = SaveCategories;
