"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiarizationService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
let logger = log4js.getLogger('ServerScripts');
function DiarizationService(app) {
    let cognitiveService;
    app.post('/api/diarization/get_model_availability', (req, res, next) => {
        logger.info('[DiarizationService::get_model_availability] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let diarizationModels = [];
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
        })
            .then(result => {
            cognitiveService = result;
            return mssql_service_1.MSSql.GetNLUs(vcc, null, null, null, true);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_DIARIZATION_MODELS_NOT_FOUND';
            diarizationModels = result;
            let promiseArray = [];
            let timeOut = 0;
            diarizationModels.map((r, i) => {
                timeOut = i * 1000;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(() => {
                        return resolve(cognitiveService.GetUtterancesList(null, null, r.appId));
                    }, timeOut);
                }));
            });
            return Promise.all(promiseArray);
        })
            .catch(err => {
            logger.error('[DiarizationService::get_model_availability] Passing error: ', err);
            return Promise.resolve([null, null, null, null]);
        })
            .then(result => {
            result.map((r, i) => {
                diarizationModels[i].utterances = r;
            });
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: diarizationModels });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DiarizationService::get_model_availability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.DiarizationService = DiarizationService;
