"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DictionaryService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const dictionary_model_1 = require("../models/dictionary.model");
const arrays_shared_1 = require("../shared/arrays.shared");
let logger = log4js.getLogger('ServerScripts');
function DictionaryService(app) {
    app.post('/api/dictionary/get_dictionary', (req, res, next) => {
        logger.info('[DictionaryService::get_dictionary] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let dictionaryId = req.body.dictionaryId || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetDictionary(vcc, dictionaryId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::delete_word] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/get_dictionaries', (req, res, next) => {
        logger.info('[DictionaryService::get_dictionaries] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetDictionaries(vcc, null, null);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::get_dictionaries] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/word/get_dictionary_word', (req, res, next) => {
        logger.info('[DictionaryService::get_dictionary_word] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let dictionaryWord = new dictionary_model_1.DictionaryWord();
        dictionaryWord.id = req.body.dictionaryWordId || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetDictionaryWord(vcc, dictionaryWord.id);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_DICTIONARY_WORD_NOT_FOUND';
            dictionaryWord = result;
            return mssql_service_1.MSSql.GetDictionaryWordSoundLikes(vcc, dictionaryWord.id);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_DICTIONARY_WORD_SOUND_LIKES_NOT_FOUND';
            dictionaryWord.soundLikes = result;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: dictionaryWord });
        })
            .catch(err => {
            let environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::get_dictionary_word] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/word/get_dictionary_words', (req, res, next) => {
        logger.info('[DictionaryService::get_dictionary_words] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let dictionaryId = req.body.dictionaryId || '';
        let search_criteria = req.body.searchCriteria || '';
        let search_expression = req.body.searchExpression || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetDictionaryWords(vcc, search_criteria, search_expression, dictionaryId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::get_dictionary_words] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/word/create_word', (req, res, next) => {
        logger.info('[DictionaryService::create_word] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let dictionaryWord = (req.body || {});
        dictionaryWord.word = dictionaryWord.word.trim();
        promises_shared_1.DummyPromise()
            .then(result => {
            return SaveDictionaryWord(vcc, dictionaryWord, userId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_DICTIONARY_WORD_CREATE_FAILED'), data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::create_word] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/word/save_word', (req, res, next) => {
        logger.info('[DictionaryService::save_word] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let dictionaryWord = (req.body || {});
        dictionaryWord.word = dictionaryWord.word.trim();
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!dictionaryWord.soundLikes.length)
                return Promise.resolve([]);
            return mssql_service_1.MSSql.GetSoundLikesFromDictionaryWords(vcc, dictionaryWord.id, dictionaryWord.soundLikes.map(item => item.soundLike).join(','));
        })
            .then(result => {
            if (result.length)
                return Promise.resolve(result);
            return SaveDictionaryWord(vcc, dictionaryWord, userId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            if (result.length)
                res.json({ status: true, description: 'SERVER_ERROR_DICTIONARY_SOUND_LIKE_EXISTS', data: result });
            else
                res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_DICTIONARY_WORD_SAVE_FAILED'), data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::save_word] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/get_dictionary_usage', (req, res, next) => {
        logger.info('[DictionaryService::get_dictionary] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let dictionaryId = req.body.dictionaryId || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetDictionaryUsage(vcc, dictionaryId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::delete_word] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/dictionary/word/delete_word', (req, res, next) => {
        logger.info('[DictionaryService::delete_word] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let dictionaryWord = (req.body || {});
        logger.info(`[DictionaryService::delete_word] Deleting word ${dictionaryWord} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!dictionaryWord)
                throw 'SERVER_ERROR_DICTIONARY_WORD_WAS_NOT_RECEIVED';
            return mssql_service_1.MSSql.DeleteDictionaryWord(vcc, dictionaryWord.id);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_DICTIONARY_WORD_DELETE_FAILED'), data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[DictionaryService::delete_word] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    function SaveDictionaryWord(vcc, dictionaryWord, saveUser) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.SaveDictionaryWord(vcc, dictionaryWord.dictionaryId, dictionaryWord.id, dictionaryWord.word, saveUser);
        })
            .then(result => {
            return mssql_service_1.MSSql.GetDictionaryWordSoundLikes(vcc, dictionaryWord.id);
        })
            .then(result => {
            if (dictionaryWord.soundLikes) {
                let classifiedItems = arrays_shared_1.GetCompareItemsFromObjectArray(result, dictionaryWord.soundLikes, 'id');
                let promiseArray = [];
                classifiedItems.new.map((ni) => {
                    promiseArray.push(mssql_service_1.MSSql.SaveDictionaryWordSoundLike(vcc, dictionaryWord.id, ni.id, ni.soundLike, saveUser));
                });
                classifiedItems.removed.map((ri) => {
                    promiseArray.push(mssql_service_1.MSSql.DeleteDictionaryWordSoundLike(vcc, dictionaryWord.id, ri.id));
                });
                return Promise.all(promiseArray);
            }
            else {
                return Promise.resolve([]);
            }
        })
            .then(result => {
            return Promise.resolve(true);
        })
            .catch(err => {
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[DictionaryService::SaveDictionaryWord] Error: ' + errorMsg);
            return Promise.reject(errorMsg);
        });
    }
}
exports.DictionaryService = DictionaryService;
