"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EfectivityPlanService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
let logger = log4js.getLogger('ServerScripts');
function EfectivityPlanService(app) {
    app.get('/api/efectivity_plan/get_efectivity_plans/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[EfectivityPlanService::get_efectivityplans] Received ' + req.method + ' post method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetEfectivityPlans(vcc, search_criteria, search_expression, disabled);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/efectivity_plan/save_evaluation_efectivity_sample', (req, res, next) => {
        logger.info('[EfectivityPlanService::save_evaluationplan] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let efectivityPlanId = req.body.efectivityPlanId;
        let evaluationPlanId = req.body.evaluationPlanId;
        let auditorId = req.body.auditorId;
        let sampleSize = req.body.sampleSize;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.SaveEvaluationEfectivitySample(vcc, efectivityPlanId, evaluationPlanId, auditorId, sampleSize);
        })
            .then(resultSave => {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'ERROR_EFECTIVITY_ID_ALREADY_EXISTS'), data: resultSave });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EfectivityPlanService::save_evaluation-efectivity-sample] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/efectivity_plan/get_evaluation_result_efectivity_summary/:efectivityPlanId', (req, res, next) => {
        logger.info('[EfectivityPlanService::get_evaluation_result_efectivity_summary] Received ' + req.method + ' post method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let efectivityPlanId = req.params.efectivityPlanId;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetEvaluationResultEfectivitySummary(vcc, efectivityPlanId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.EfectivityPlanService = EfectivityPlanService;
