"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvaluationFormatService = void 0;
const moment = require("moment");
const jwt = require("jsonwebtoken");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const template_service_1 = require("./template.service");
const promises_shared_1 = require("../shared/promises.shared");
const utilities_1 = require("../shared/utilities");
const template_model_1 = require("../models/template.model");
const custom_field_service_1 = require("./custom-field.service");
const ability_service_1 = require("./ability.service");
const process_implementation_1 = require("../implementation/process.implementation");
const jwt_config_1 = require("../config/jwt.config");
let logger = log4js.getLogger('ServerScripts');
function EvaluationFormatService(app) {
    app.get('/api/evaluationformat/get_evaluationformats/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[EvaluationFormatService::get_evaluationformats] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let disabled = req.query.disabled === 'true' || false;
        let historic = req.query.historic === 'true' || false;
        logger.info('[EvaluationFormatService::get_evaluationformats] Instance received: ' + instance);
        logger.info('[EvaluationFormatService::get_evaluationformats] Search_criteria received: ' + search_criteria);
        logger.info('[EvaluationFormatService::get_evaluationformats] Search_expression received: ' + search_expression);
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (historic) {
                return mssql_service_1.MSSql.GetEvaluationFormatsFromHistoricEvaluationPlans(instance);
            }
            if (search_criteria === 'idGroup' || search_criteria === 'idSubGroup' || search_criteria === 'idAbility') {
                logger.info('[EvaluationFormatService::get_evaluationformats] Calling SP GetEvaluationFormatDescriptionsBySearchCriteria');
                return mssql_service_1.MSSql.GetEvaluationFormatDescriptionsBySearchCriteria(instance, search_criteria, search_expression);
            }
            else {
                logger.info('[EvaluationFormatService::get_evaluationformats] Calling SP GetEvaluationFormats');
                return mssql_service_1.MSSql.GetEvaluationFormats(instance, search_criteria, search_expression, disabled);
            }
        }).then(evaluationformats => {
            if (res.headersSent)
                return;
            logger.info('[EvaluationFormatService::get_evaluationformats] Received restult from SP GetEvaluationFormats: ' + evaluationformats.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationformats });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::get_evaluationformats] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationformat/create_evaluationformat', (req, res, next) => {
        logger.info('[EvaluationFormatService::create_evaluationformat] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newEvaluationFormat = (req.body || {});
        newEvaluationFormat.createdDate = nowUtc;
        newEvaluationFormat.createdByUserId = userId;
        newEvaluationFormat.lastModifiedDate = nowUtc;
        newEvaluationFormat.lastModifiedByUserId = userId;
        logger.info('[EvaluationFormatService::create_evaluationformat] Received vcc: ' + vcc);
        promises_shared_1.DummyPromise()
            .then(_ => {
            logger.info('[EvaluationFormatService::create_evaluationformat] Calling GetEvaluationFormatDescription SP');
            return mssql_service_1.MSSql.GetEvaluationFormatDescription(vcc, newEvaluationFormat.description);
        })
            .then(result => {
            logger.info('[EvaluationFormatService::create_evaluationformat] Received result from GetEvaluationFormatDescription SP: ' + JSON.stringify(result));
            if (result)
                throw 'EVALUATION_FORMAT_SERVER_ERROR_EXISTS';
            newEvaluationFormat.id = newEvaluationFormat.description;
            newEvaluationFormat.templateId = ['MANUAL_AUTOMATIC', 'MIXED'].includes(newEvaluationFormat.type) ? `qty-tmpl-${newEvaluationFormat.description}` : null;
            logger.info('[EvaluationFormatService::create_evaluationformat] Saving ev format ' + JSON.stringify(newEvaluationFormat));
            return mssql_service_1.MSSql.SaveEvaluationFormat(vcc, newEvaluationFormat.id, newEvaluationFormat.description, newEvaluationFormat.type, newEvaluationFormat.environmentType, newEvaluationFormat.executionType, newEvaluationFormat.minQualificationAcceptable, newEvaluationFormat.minAutomaticQualification, newEvaluationFormat.managementFormURL, newEvaluationFormat.createdDate, newEvaluationFormat.createdByUserId, newEvaluationFormat.lastModifiedDate, newEvaluationFormat.lastModifiedByUserId, newEvaluationFormat.enabled, newEvaluationFormat.templateId);
        })
            .then(result => {
            if (!result)
                throw 'EVALUATION_FORMAT_ERROR_ON_SAVE';
            return SaveEvaluationFormatContent(vcc, newEvaluationFormat, true, userId, null);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_EVALUATION_FORMAT_SAVE_CONTENT_FAILED';
            if (res.headersSent)
                return;
            logger.info('[EvaluationFormatService::create_evaluationformat] Sending result');
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_FORMAT_ERROR_ON_SAVE'), data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::create_evaluationformat] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/get_evaluationformat/:evaluationformat_id', (req, res) => {
        logger.info('[EvaluationFormatService::get_evaluationformat] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let evaluationFormat = {};
        let evaluationFormatAbilities = [];
        let evaluationFormatId = req.params.evaluationformat_id;
        let findQueries = req.query.find_queries ? true : false;
        let findEntities = req.query.find_entities ? true : false;
        let evaluationPlanId = (typeof req.query.evaluation_plan_id == 'string'
            && req.query.evaluation_plan_id != 'null') ? req.query.evaluation_plan_id : null;
        logger.info('[EvaluationFormatService::get_evaluationformat] Looking for EV ' + evaluationFormatId);
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!evaluationFormatId)
                throw 'ERROR_EVALUATION_FORMAT_ID_MISSING';
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetEvaluationFormat SP');
            return mssql_service_1.MSSql.GetEvaluationFormat(instance, evaluationFormatId, evaluationPlanId);
        }).then(result => {
            evaluationFormat = result || evaluationFormat;
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetAbilityEvaluationFormats SP');
            if (evaluationPlanId) {
                let processImplementation = new process_implementation_1.ProcessImplementation();
                return processImplementation.GetEvaluationPlanAbilities(instance, evaluationPlanId);
            }
            else
                return mssql_service_1.MSSql.GetAbilityEvaluationFormats(instance, evaluationFormatId, evaluationPlanId);
        }).then(result => {
            evaluationFormatAbilities = result || evaluationFormatAbilities;
            evaluationFormatAbilities.forEach(element => {
                if (element.idSubGroup === null) {
                    element.idSubGroup = '';
                    element.descriptionSubGroup = '';
                }
            });
            let promiseArray = [];
            if (findQueries) {
                evaluationFormatAbilities.map(efa => {
                    promiseArray.push(ability_service_1.GetAbilityQueries(instance, efa.idAbility, findEntities, evaluationPlanId));
                });
            }
            return Promise.all(promiseArray);
        })
            .then(result => {
            result.map((r, i) => {
                evaluationFormatAbilities[i].queries = r;
            });
            evaluationFormat.abilityEvaluationFormats = evaluationFormatAbilities;
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetEvaluationFormatCustomFields SP');
            return mssql_service_1.MSSql.GetEvaluationFormatCustomFields(instance, evaluationFormatId, evaluationPlanId);
        })
            .then(result => {
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetEvaluationFormatAttachedSignatures SP');
            evaluationFormat.evaluationFormatCustomFields = result;
            if (evaluationFormat.evaluationFormatCustomFields && evaluationFormat.evaluationFormatCustomFields.length) {
                let promiseArray = [];
                evaluationFormat.evaluationFormatCustomFields.map(efcf => promiseArray.push(custom_field_service_1.GetCustomField(instance, efcf.customFieldId, evaluationPlanId)));
                return Promise.all(promiseArray);
            }
            else {
                return Promise.resolve([null]);
            }
        })
            .then(result => {
            if (evaluationFormat.evaluationFormatCustomFields && evaluationFormat.evaluationFormatCustomFields.length) {
                evaluationFormat.evaluationFormatCustomFields.map((efcf, index) => efcf.customField = result[index]);
            }
            return mssql_service_1.MSSql.GetEvaluationFormatAttachedSignatures(instance, evaluationFormatId, evaluationPlanId);
        })
            .then(result => {
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetCampainEvaluationFormats SP');
            evaluationFormat.evaluationFormatAttachedSignatures = result;
            return mssql_service_1.MSSql.GetCampainEvaluationFormats(instance, evaluationFormatId, evaluationPlanId);
        }).then(result => {
            if (res.headersSent)
                return;
            evaluationFormat.campainEvaluationFormats = result || evaluationFormat.campainEvaluationFormats;
            logger.info('[EvaluationFormatService::get_evaluationformat] Sending response');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationFormat });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::get_evaluationformat] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationformat/save_evaluationformat', (req, res, next) => {
        logger.info('[EvaluationFormatService::save_evaluationformat] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newEvaluationFormat = (req.body || {});
        newEvaluationFormat.createdDate = null;
        newEvaluationFormat.createdByUserId = null;
        newEvaluationFormat.lastModifiedDate = nowUtc;
        newEvaluationFormat.lastModifiedByUserId = userId;
        let oldTemplateId = newEvaluationFormat.templateId ? newEvaluationFormat.templateId : '';
        promises_shared_1.DummyPromise()
            .then(_ => {
            logger.info('[EvaluationFormatService::save_evaluationformat] Looking for a Evaluation format: ' + newEvaluationFormat.description);
            return mssql_service_1.MSSql.GetEvaluationFormatDescription(vcc, newEvaluationFormat.description);
        })
            .then(result => {
            let evaluationFormatDesc = {};
            evaluationFormatDesc = result || evaluationFormatDesc;
            if (result && evaluationFormatDesc.id !== newEvaluationFormat.id)
                throw 'EVALUATION_FORMAT_SERVER_ERROR_EXISTS';
            logger.info('[EvaluationFormatService::save_evaluationformat] Saving evaluation format');
            newEvaluationFormat.templateId = ['MANUAL_AUTOMATIC', 'MIXED'].includes(newEvaluationFormat.type) ? `qty-tmpl-${newEvaluationFormat.id}` : null;
            return mssql_service_1.MSSql.SaveEvaluationFormat(vcc, newEvaluationFormat.id, newEvaluationFormat.description, newEvaluationFormat.type, newEvaluationFormat.environmentType, newEvaluationFormat.executionType, newEvaluationFormat.minQualificationAcceptable, newEvaluationFormat.minAutomaticQualification, newEvaluationFormat.managementFormURL, newEvaluationFormat.createdDate, newEvaluationFormat.createdByUserId, newEvaluationFormat.lastModifiedDate, newEvaluationFormat.lastModifiedByUserId, newEvaluationFormat.enabled, newEvaluationFormat.templateId);
        })
            .then((result) => {
            logger.info('[EvaluationFormatService::save_evaluationformat] Result: ', result);
            if (result?.length) {
                let status;
                let idEvaluationFormat = result[0].idEvaluationFormat;
                logger.info('[EvaluationFormatService::save_evaluationformat] idEvaluationFormat: ', idEvaluationFormat);
                if (idEvaluationFormat === -1) {
                    status = 'EVALUATION_FORMAT_BUSY';
                    logger.info('[EvaluationFormatService::save_evaluationformat] result - status -1:', status);
                    return Promise.resolve(status);
                }
                else {
                    status = idEvaluationFormat;
                    logger.info('[EvaluationFormatService::save_evaluationformat] result - status: ', status);
                    return SaveEvaluationFormatContent(vcc, newEvaluationFormat, false, userId, oldTemplateId);
                }
            }
            else {
                logger.error('[EvaluationFormatService::save_evaluationformat] result error: ', result);
                return Promise.reject(result);
            }
        })
            .then(resultSave => {
            logger.info('[EvaluationFormatService::save_evaluationformat] resultSave: ', resultSave);
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? resultSave === 'EVALUATION_FORMAT_BUSY' ? 'EVALUATION_FORMAT_BUSY' : 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_FORMAT_ERROR_ON_SAVE'), data: resultSave });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::save_evaluationformat] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationformat/delete_evaluationformat', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let evaluationFormat = (req.body || {});
        let evaluationFormatId = evaluationFormat.id;
        logger.info(`[EvaluationFormatService::delete_evaluationformat] Deleting evaluationFormat ${evaluationFormatId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.DeleteEvaluationFormat(instance, evaluationFormatId);
        }).then(result => {
            if (!result)
                throw 'EVALUATION_FORMAT_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_FORMAT_SERVER_ERROR_ON_DELETE'), data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationformat/delete_evaluationformats', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let evaluationFormats = req.body.evaluationFormats || [];
        logger.info(`[EvaluationFormatService::delete_evaluationformats] Deleting evaluationFormats ${evaluationFormats.map(x => x.id).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!evaluationFormats)
                throw 'EVALUATION_FORMAT_SERVER_ERROR_MISSING_IDS';
            let promiseArray = [];
            evaluationFormats.forEach(evaluationFormat => {
                promiseArray.push(mssql_service_1.MSSql.DeleteEvaluationFormat(instance, evaluationFormat.id));
            });
            return Promise.all(promiseArray);
        }).then(resultDeletes => {
            let result;
            if (resultDeletes.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Evaluation formats eliminados correctamente');
            }
            return result;
        }).then(resultDeletes => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeletes, description: (resultDeletes ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_FORMAT_SERVER_ERROR_ON_DELETE'), data: resultDeletes });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/get_evaluationformat_qualification_average/:evaluationFormatId', (req, res, next) => {
        logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let evaluationFormatStr = req.params.evaluationFormatId || '';
        logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Received instance: ' + instance);
        logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Received evaluationFormatId: ' + evaluationFormatStr);
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Calling GetEvaluationFormatQualificationAverage SP');
            return mssql_service_1.MSSql.GetEvaluationFormatQualificationAverage(instance, evaluationFormatStr);
        }).then(result => {
            logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Received result from GetEvaluationFormatQualificationAverage SP ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::get_evaluationformat_qualification_average] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/isQueryBeingUsedByQuality/:queryName', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let queryName = req.params.queryName;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!queryName)
                throw 'EVALUATION_FORMAT_SERVER_ERROR_MISSING_QUERY';
            return mssql_service_1.MSSql.IsQueryBeingUsedByQuality(instance, queryName);
        }).then(result => {
            if (res.headersSent)
                return;
            let data = result > 0 ? true : false;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/isTemplateBeingUsedByQuality/:TemplateId', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let templateId = req.params.TemplateId;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!templateId)
                throw 'EVALUATION_FORMAT_SERVER_ERROR_MISSING_QUERY';
            return mssql_service_1.MSSql.IsTemplateBeingUsedByQuality(instance, templateId);
        }).then(result => {
            if (res.headersSent)
                return;
            let data = result > 0 ? true : false;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/get_evaluationformat_attachedsignatures/:evaluationformat_id', (req, res, next) => {
        logger.info('[EvaluationFormatService::get_evaluationformat_attachedsignatures] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let evaluationFormatAttachedSignatures = [];
        let evaluationFormatId = req.params.evaluationformat_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!evaluationFormatId)
                throw 'SERVER_ERROR_EVALUATION_FORMAT_ID_MISSING';
            logger.info('[EvaluationFormatService::get_evaluationformat_attachedsignatures] Calling GetEvaluationFormatAttachedSignatues SP');
            return mssql_service_1.MSSql.GetEvaluationFormatAttachedSignatures(vcc, evaluationFormatId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.info('[EvaluationFormatService::get_evaluationformat_attachedsignatures] Received restult from SP GetEvaluationFormatAttachedSignatues: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::get_evaluationformat_attachedsignatures] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/load_real_time_formats', (req, res, next) => {
        let decodedToken = null;
        try {
            decodedToken = jwt.verify((req.headers['authorization'] || '').replace('Bearer ', ''), jwt_config_1.JWT_SECRET);
        }
        catch (e) {
            decodedToken = null;
            logger.warn(`[EvaluationFormatService::load_real_time_formats] Received invalid token: ${req.headers['authorization']}`);
            throw 'SERVER_ERROR_CANNOT_DECODE_ACCESS_TOKEN';
        }
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let rtEvaluationFormats = [];
            rtEvaluationFormats.push({ id: 'abc1', name: 'rtFormat01' });
            rtEvaluationFormats.push({ id: 'abc1', name: 'rtFormat02' });
            rtEvaluationFormats.push({ id: 'abc1', name: 'rtFormat03' });
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: rtEvaluationFormats });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::load_evaluation_formats] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/public/load_real_time_formats/:vcc', (req, res, next) => {
        let vcc = req.params.vcc;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetVCC(vcc);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_VCC_NOT_FOUND';
            let rtEvaluationFormats = [];
            rtEvaluationFormats.push({ vcc, id: 'abc1', name: 'rtFormat01' });
            rtEvaluationFormats.push({ vcc, id: 'abc2', name: 'rtFormat02' });
            rtEvaluationFormats.push({ vcc, id: 'abc3', name: 'rtFormat03' });
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: rtEvaluationFormats });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::load_real_time_formats] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.EvaluationFormatService = EvaluationFormatService;
function SaveEvaluationFormatContent(vcc, evaluationFormat, isNewEvaluationFormat = false, userId, oldTemplateId = null) {
    let currentBlockPromise = null;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (isNewEvaluationFormat)
            return Promise.resolve(true);
        else {
            currentBlockPromise = 'BLOCK_TO_DELETE_EVALUATION_FORMAT_CONTENT';
            return mssql_service_1.MSSql.DeleteEvaluationFormatContent(vcc, evaluationFormat.id, oldTemplateId);
        }
    })
        .then(_ => {
        currentBlockPromise = 'BLOCK_TO_SAVE_EVALUATION_FORMAT_CAMPAIGNS';
        return SaveAbilityAndCAmpainEF(vcc, evaluationFormat);
    })
        .then(_ => {
        if (['MANUAL_AUTOMATIC', 'MIXED'].includes(evaluationFormat.type)) {
            currentBlockPromise = 'BLOCK_TO_SET_EVALUATION_FORMAT_ABILTY_QUERIES';
            return SetAbilityQueriesIntoAbilityEvaluationFormat(vcc, evaluationFormat);
        }
        return Promise.resolve(evaluationFormat);
    })
        .then(result => {
        currentBlockPromise = 'BLOCK_TO_FIND_EVALUATION_FORMAT_TEMPLATE';
        evaluationFormat = result;
        return mssql_service_1.MSSql.GetTemplate(vcc, evaluationFormat.templateId);
    })
        .then(result => {
        if (!result) {
            if (['MANUAL_AUTOMATIC', 'MIXED'].includes(evaluationFormat.type)) {
                currentBlockPromise = 'BLOCK_TO_GENERATE_EVALUATION_FORMAT_TEMPLATE';
                return GenerateTemplate(vcc, evaluationFormat);
            }
        }
        return Promise.resolve(true);
    })
        .then(_ => {
        currentBlockPromise = 'BLOCK_TO_SAVE_EVALUATION_FORMAT_TIPIFICATIONS';
        return SaveEvaluationFormatAbilityScoreTipifications(vcc, evaluationFormat, userId);
    })
        .then(result => {
        if (['MANUAL', 'MIXED'].includes(evaluationFormat.type)) {
            currentBlockPromise = 'BLOCK_TO_SAVE_EVALUATION_FORMAT_CUSTOM_FIELDS';
            return SaveEvaluationFormatCustomFields(vcc, evaluationFormat, userId);
        }
        return Promise.resolve(true);
    })
        .then(result => {
        currentBlockPromise = 'BLOCK_TO_SAVE_EVALUATION_FORMAT_ATTACHED_SIGNATURES';
        return SaveEvaluationFormatAttachedSignatures(vcc, evaluationFormat, userId);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error(`[CustomFieldService::SaveEvaluationFormatContent] Block: ${currentBlockPromise}, getting error: ${errorMsg}`);
        return Promise.reject(false);
    });
}
function SaveAbilityAndCAmpainEF(instance, evaluationFormat) {
    let promiseArray = [];
    logger.info('[EvaluationFormatService::SaveAbilityAndCAmpainEF] Saving ' + JSON.stringify(evaluationFormat.abilityEvaluationFormats));
    if (evaluationFormat.abilityEvaluationFormats) {
        evaluationFormat.abilityEvaluationFormats.forEach(abilityEvaluationFormat => {
            promiseArray.push(mssql_service_1.MSSql.SaveAbilityEvaluationFormat(instance, evaluationFormat.id, abilityEvaluationFormat.idAbility, abilityEvaluationFormat.idGroup, abilityEvaluationFormat.idSubGroup, abilityEvaluationFormat.penalizes, abilityEvaluationFormat.score, abilityEvaluationFormat.order, abilityEvaluationFormat.conditional, abilityEvaluationFormat.helpComment));
        });
    }
    if (evaluationFormat.campainEvaluationFormats) {
        evaluationFormat.campainEvaluationFormats.forEach(campainEvaluationFormat => {
            promiseArray.push(mssql_service_1.MSSql.SaveCampainEvaluationFormat(instance, evaluationFormat.id, campainEvaluationFormat.idCampain));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function SaveEvaluationFormatAbilityScoreTipifications(vcc, evaluationFormat, userId) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        let promiseArray = [];
        evaluationFormat.abilityEvaluationFormats.forEach(aef => {
            if (aef.abilityScores?.length) {
                aef.abilityScores.forEach(as => {
                    if (as.tipifications?.length) {
                        as.tipifications.forEach(t => {
                            promiseArray.push(mssql_service_1.MSSql.SaveEvaluationFormatAbilityScoreTipification(vcc, evaluationFormat.id, as.id, t.id, t.description, t.descriptionDetail, t.order, userId));
                            if (t.tipificationQuery?.queryId?.length) {
                                promiseArray.push(mssql_service_1.MSSql.SaveEvaluationFormatAbilityScoreTipificationQuery(vcc, evaluationFormat.id, as.id, t.id, t.tipificationQuery.queryId, t.tipificationQuery.queryContent, t.tipificationQuery.acceptancePercentage, userId));
                            }
                            t.items.forEach(i => {
                                promiseArray.push(mssql_service_1.MSSql.SaveEvaluationFormatAbilityScoreTipificationItem(vcc, evaluationFormat.id, as.id, t.id, i.id, i.secondLevelItemId, i.description, i.order, userId));
                            });
                        });
                    }
                });
            }
        });
        return Promise.all(promiseArray);
    })
        .then(result => {
        return Promise.resolve(true);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[CustomFieldService::SaveEvaluationFormatAbilityScoreTipifications] Error: ' + errorMsg);
        return Promise.reject(false);
    });
}
function SaveEvaluationFormatCustomFields(vcc, evaluationFormat, userId) {
    return promises_shared_1.DummyPromise()
        .then(result => {
        let promiseArray = [];
        evaluationFormat.evaluationFormatCustomFields.map(efcf => {
            promiseArray.push(mssql_service_1.MSSql.SaveEvaluationFormatCustomField(vcc, efcf.id, evaluationFormat.id, efcf.customField.id, efcf.order, userId, userId));
        });
        return Promise.all(promiseArray);
    })
        .then(result => {
        if (result.filter(r => { return (r !== true); }).length > 0) {
            return Promise.reject(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[CustomFieldService::SaveEvaluationFormatCustomFields] Error: ' + errorMsg);
        return Promise.reject(false);
    });
}
function SaveEvaluationFormatAttachedSignatures(vcc, evaluationFormat, userId) {
    return promises_shared_1.DummyPromise()
        .then(result => {
        let promiseArray = [];
        evaluationFormat.evaluationFormatAttachedSignatures.map(efas => {
            if (efas && efas.description && efas.description !== '')
                promiseArray.push(mssql_service_1.MSSql.SaveEvaluationFormatAttachedSignature(vcc, evaluationFormat.id, efas.description, efas.order, userId, userId));
        });
        return Promise.all(promiseArray);
    })
        .then(result => {
        if (result.filter(r => { return (r !== true); }).length > 0) {
            return Promise.reject(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[CustomFieldService::SaveEvaluationFormatAttachedSignatures] Error: ' + errorMsg);
        return Promise.reject(false);
    });
}
function SetAbilityQueriesIntoAbilityEvaluationFormat(vcc, evaluationFormat) {
    return promises_shared_1.DummyPromise()
        .then(result => {
        let promiseArray = [];
        evaluationFormat.abilityEvaluationFormats.map(aef => {
            promiseArray.push(ability_service_1.GetAbilityQueries(vcc, aef.idAbility, false));
        });
        return Promise.all(promiseArray);
    })
        .then(result => {
        result.map((r, i) => {
            evaluationFormat.abilityEvaluationFormats[i].queries = r;
        });
        return Promise.resolve(evaluationFormat);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error('[EvaluationFormatService::SetAbilityQueriesIntoAbilityEvaluationFormat] ', errorMsg);
        return Promise.reject(err);
    });
}
function GenerateTemplate(instance, evaluationFormat) {
    return promises_shared_1.DummyPromise()
        .then(result => {
        logger.error('[EvaluationFormatService::GenerateTemplate] Creating Template');
        let template = new template_model_1.Template('custom', evaluationFormat.templateId, evaluationFormat.description, null, null, moment().utc().toDate(), evaluationFormat.createdByUserId, moment().utc().toDate(), evaluationFormat.lastModifiedByUserId);
        let i = 0;
        let category = '';
        for (let aef of evaluationFormat.abilityEvaluationFormats) {
            i++;
            if (category !== aef.idGroup) {
                i = 1;
                category = aef.idGroup;
            }
            let tempTheme = new template_model_1.TemplateTheme(template.name, utilities_1.guid(), i, aef.titleAbility, aef.descriptionGroup, aef.descriptionSubGroup, moment().utc().toDate(), template.createdByUserId, moment().utc().toDate(), template.lastModifiedByUserId);
            tempTheme.isNegative = aef.isNegative;
            let tempThemeItem = null;
            if (aef.queries && aef.queries.length) {
                aef.queries.map(q => {
                    tempThemeItem = new template_model_1.TemplateThemeItem(template.name, tempTheme.id, q.queryId, q.queryId, moment().utc().toDate(), template.createdByUserId, moment().utc().toDate(), template.lastModifiedByUserId);
                    tempThemeItem.acceptancePercentage = q.acceptancePercentage;
                    template.templateThemeItems.push(tempThemeItem);
                });
            }
            else {
                tempThemeItem = new template_model_1.TemplateThemeItem(template.name, tempTheme.id, aef.idAbility, aef.idAbility, moment().utc().toDate(), template.createdByUserId, moment().utc().toDate(), template.lastModifiedByUserId);
                template.templateThemeItems.push(tempThemeItem);
            }
            template.templateThemes.push(tempTheme);
        }
        return template_service_1.SaveTemplate(instance, template);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error('[EvaluationFormatService::GenerateTemplate] ', errorMsg);
        return false;
    });
}
