"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CancelFailedBillingBatch = exports.CleanFailedEvaluationPlanData = exports.RemoveEvaluationPlanProcess = exports.CreateQueueL = exports.SetAuditorInteractionsNumbersFromPercentage = exports.StartEvaluationPlan = exports.CreateEvaluationPlan = exports.EvaluationPlanService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const evaluation_plan_model_1 = require("../models/evaluation-plan.model");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const linguistic_service_1 = require("./linguistic.service");
const billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
const billing_batch_model_1 = require("../models/billing-batch.model");
const billing_implementation_1 = require("../implementation/billing.implementation");
const interaction_model_1 = require("../models/interaction.model");
const interaction_service_1 = require("./interaction.service");
const billing_model_1 = require("../models/billing.model");
const platform_service_1 = require("./platform.service");
const cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
const process_service_1 = require("./process.service");
const batch_schedule_service_1 = require("./batch-schedule.service");
const utilities_1 = require("../shared/utilities");
const html_to_pdf = require('html-pdf');
let logger = log4js.getLogger('ServerScripts');
function EvaluationPlanService(app) {
    app.post('/api/evaluationplan/pdf/', (req, res) => {
        let htmlBuffer = Buffer.from(req.body.html, 'base64');
        let uriBuffer = Buffer.from(req.body.uri, 'base64');
        let html = htmlBuffer.toString("ascii");
        let uri = uriBuffer.toString("ascii");
        let content = `<!doctype html>
		<html>
		   <head>
				<meta charset="utf-8">
				<title>${req.body.id}</title>
				<style>
				body{
					box-sizing: border-box;
					font-family:arial;
					padding:0 10px;
					font-size:10px;
				  }
				div.in {
					background-color: #f0f8fb;
					color: #0f4979;
					border: 1px solid #8ec8e0;
					padding: 15px;
					margin-bottom: 15px;
				}
				
				div.out{
					background-color: #f8faf5;
					color: #3f5c30;
					border: 1px solid #aeca8d;
					padding: 15px;
					margin-bottom: 15px;
				}
				li.in[_ngcontent-c6] {
					background-color: #F0F8FB;
					color: #fff;
				}
				
				li.out {
					background-color: #EDF3E5;
					color: #fff;
				}
				.page-break {
					position: relative;
					height: 100%;
					width: 100%;
					display: block;
					page-break-after: always;
					overflow: hidden;
				  }
				</style>
			</head>
			<body>
			${html}
			</body>
		</html>`;
        html_to_pdf.create(content, {
            format: 'A4',
            "header": {
                "height": "20mm",
                "contents": `<div style="text-align: center;display:flex">${uri.length > 60 ? uri.substring(0, 58) : uri}...</div>`
            }, "footer": {
                "height": "20mm",
                "contents": '<span style="color: #444;">{{page}}</span>/<span>{{pages}}</span>'
            }
        }).toBuffer((err, buffer) => {
            if (err) {
                res.json({ status: false });
            }
            else {
                res.json({
                    status: true,
                    pdf: buffer.toString("base64")
                });
            }
        });
    });
    app.get('/api/evaluationplan/get_evaluationplan_by_name/:search_name', (req, res, next) => {
        logger.info('[EvaluationPlanService::get_evaluationplan_by_name] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_name = req.params.search_name || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[EvaluationPlanService::get_evaluationplan_by_name] Requesting evaluation plan by name.');
            return mssql_service_1.MSSql.GetEvaluationPlanByName(vcc, search_name);
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplan_by_name] Received evaluation plan data.');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error executing request.');
            logger.error('[EvaluationPlanService::get_evaluationplan_by_name] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationplan/get_evaluationplans/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[EvaluationPlanService::get_evaluationplans] Received ' + req.method + ' post method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetEvaluationPlans(vcc, search_criteria, search_expression, disabled);
        }).then(evaluationplans => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationplans });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/get_evaluationplans_summary/', (req, res, next) => {
        logger.info('[EvaluationPlanService::get_evaluationplans_summary] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let params = new evaluation_plan_model_1.EvaluationPlanFilters;
        let isAdmin = req.body.idAgent || false;
        let idAgent = req.body.idAgent || null;
        let idAuditor = req.body.idAuditor || null;
        params.evaluationFormats = req.body.evaluationFormats || [];
        params.campaigns = req.body.campaigns || [];
        params.limitDate = req.body.limitDate || '2050-01-01';
        params.modalities = req.body.modalities || [];
        params.completed = req.body.completed || false;
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[EvaluationPlanService::get_evaluationplans_summary] Calling SP GetEvaluationPlansAgent from local DB');
            return mssql_service_1.MSSql.GetEvaluationPlansSummary(vcc, isAdmin, idAgent, idAuditor, params.campaigns.toString(), params.limitDate, params.evaluationFormats.toString(), params.modalities.toString(), params.completed);
        }).then(evaluationplans => {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplans_summary] Received result from SP GetEvaluationPlansAgent from local DB');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationplans });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::get_evaluationplans_summary] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/get_evaluationplans_auditor/:idAuditor', (req, res, next) => {
        logger.info('[EvaluationPlanService::get_evaluationplans_auditor] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let currentUserRole = res.locals.user.role;
        let currentUserId = res.locals.userId;
        let userFuntion = null;
        let params = new evaluation_plan_model_1.EvaluationPlanFilters;
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.evaluationFormats = req.body.evaluationFormats || [];
        params.campaigns = req.body.campaigns || [];
        params.limitDate = req.body.limitDate || '2050-01-01';
        params.modalities = req.body.modalities || [];
        params.completed = req.body.completed || false;
        params.view = req.body.view || null;
        params.efectivityMode = req.body.efectivityMode;
        params.efectivityPlans = req.body.efectivityPlans || [];
        promises_shared_1.DummyPromise()
            .then(_ => {
            logger.info('[EvaluationService::get_evaluationplans_auditor] Calling GetEvaluations SP for VCC|IdAuditor: ' + vcc + '|' + currentUserId);
            switch (params.view) {
                case 'AUDITOR_INBOX':
                    userFuntion = 'auditor';
                    break;
                case 'BILLING_DETAIL_MODAL':
                    userFuntion = 'admin';
                    break;
                case 'EVALUATION':
                    userFuntion = 'admin';
                    break;
                case 'AUDITOR_ADMIN':
                    userFuntion = 'admin';
                    break;
                default:
                    userFuntion = 'auditor';
                    break;
            }
            logger.info('[EvaluationPlanService::get_evaluationplans_auditor] Calling SP GetEvaluationPlansAuditor from local DB');
            if (params.efectivityMode) {
                return mssql_service_1.MSSql.GetEfectivityPlansAuditor(vcc, currentUserId, params.campaigns.toString(), params.evaluationPlans.toString(), params.completed, userFuntion, params.efectivityPlans);
            }
            else {
                return mssql_service_1.MSSql.GetEvaluationPlansAuditor(vcc, currentUserId, params.campaigns.toString(), params.limitDate, params.evaluationPlans.toString(), params.evaluationFormats.toString(), params.modalities.toString(), params.completed, userFuntion);
            }
        })
            .then(evaluationplans => {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplans_auditor] Received result from SP GetEvaluationPlansAuditor from local DB');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationplans });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::get_evaluationplans_auditor] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/get_evaluationplans_agent/:idAgent', (req, res, next) => {
        logger.info('[EvaluationPlanService::get_evaluationplans_agent] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let idAgent = res.locals.userId;
        let params = new evaluation_plan_model_1.EvaluationPlanFilters;
        params.evaluationFormats = req.body.evaluationFormats || [];
        params.campaigns = req.body.campaigns || [];
        params.limitDate = req.body.limitDate || '2050-01-01';
        params.modalities = req.body.modalities || [];
        params.completed = req.body.completed || false;
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[EvaluationPlanService::get_evaluationplans_agent] Calling SP GetEvaluationPlansAgent from local DB');
            return mssql_service_1.MSSql.GetEvaluationPlansAgent(vcc, idAgent, params.campaigns.toString(), params.limitDate, params.evaluationFormats.toString(), params.modalities.toString(), params.completed);
        }).then(evaluationplans => {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplans_agent] Received result from SP GetEvaluationPlansAgent from local DB');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationplans });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::get_evaluationplans_agent] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/create_evaluationplan', (req, res, next) => {
        logger.info(`[EvaluationPlanService::create] Getting ${req.method} method on '${req.path}'`);
        let vcc = res.locals.vcc;
        let vccType = res.locals.type;
        let userId = res.locals.userId;
        let newEvaluationPlan = (req.body.evaluationPlan || {});
        let newProcess = (req.body.process || null);
        newEvaluationPlan.id = newEvaluationPlan.description.toLowerCase();
        newEvaluationPlan.percentage = newEvaluationPlan.evaluatePercentage / 100;
        newEvaluationPlan.filterByContentPercentage = newEvaluationPlan.evaluatePercentage;
        if (newProcess)
            newEvaluationPlan.processId = newProcess.name;
        logger.info(`[EvaluationPlanService::create(${newEvaluationPlan.id})] Getting vcc::userId::evaluationplan::process => ${vcc}::${userId}::${newProcess?.name || null}::${newEvaluationPlan.id}`);
        let params = new interaction_model_1.InteractionSearch;
        params = interaction_service_1.GetInteractionSearchFromRequest(newEvaluationPlan.idEvaluationFormat, newEvaluationPlan.interactionSearch, false);
        params.searchType = 'TO_OPERATE';
        let hasAutomaticPart = (['A', 'X'].includes(newEvaluationPlan.evaluation_mode)
            ||
                (newEvaluationPlan.evaluation_mode === 'M'
                    &&
                        newEvaluationPlan.filterByContentQuery
                    &&
                        newEvaluationPlan.filterByContentQueryName)) ? true : false;
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (newEvaluationPlan.scheduleData)
                return mssql_service_1.MSSql.GetBatchScheduleById(vcc, newEvaluationPlan.id);
            else
                return mssql_service_1.MSSql.GetEvaluationPlanByName(vcc, newEvaluationPlan.id);
        })
            .then(result => {
            if (result) {
                if (newEvaluationPlan.scheduleData)
                    throw 'SERVER_ERROR_SCHEDULE_ALREADY_EXISTS';
                else
                    throw 'SERVER_ERROR_EVALUATION_PLAN_ALREADY_EXISTS';
            }
            if (hasAutomaticPart) {
                newProcess.start = false;
                newProcess.type = 'QUALITY';
            }
            if (newEvaluationPlan.scheduleData)
                return CreateScheduleForEvaluationPlan(vcc, vccType, newEvaluationPlan, newProcess, hasAutomaticPart, params, userId);
            else
                return CreateEvaluationPlan(vcc, vccType, newEvaluationPlan, newProcess, hasAutomaticPart, params, userId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CREATE_FAILED';
            if (res.headersSent)
                return;
            logger.info(`[EvaluationPlanService::create(${newEvaluationPlan.id})] The EvaluationPlan batch has been finished.`);
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_SAVE'), data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error(`[EvaluationPlanService::create(${newEvaluationPlan.id})] Error on '${req.path}': ${errorMsg}`);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
            CancelFailedBillingBatch(vcc, newEvaluationPlan);
            CleanFailedEvaluationPlanData(vcc, newEvaluationPlan);
            if (!newEvaluationPlan.isExistingTranscription)
                RemoveEvaluationPlanProcess(vcc, newEvaluationPlan);
        });
    });
    app.post('/api/evaluationplan/save_evaluationplan', (req, res, next) => {
        logger.info('[EvaluationPlanService::save_evaluationplan] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let evaluationPlan = (req.body || {});
        evaluationPlan.createdDate = null;
        evaluationPlan.createdByUserId = null;
        evaluationPlan.lastModifiedDate = nowUtc;
        evaluationPlan.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            SaveEvaluationPlan(vcc, evaluationPlan);
        }).then(result => {
            return mssql_service_1.MSSql.DeleteAuditorEvaluationPlan(vcc, evaluationPlan.id);
        })
            .then(result => {
            evaluationPlan.evaluatePercentage = evaluationPlan.evaluatePercentage % 100;
            if (evaluationPlan.evaluatePercentage > 1)
                evaluationPlan.evaluatePercentage = evaluationPlan.evaluatePercentage / 100;
            if (evaluationPlan.evaluatePercentage < 1 && evaluationPlan.evaluatePercentage > 0) {
                return mssql_service_1.MSSql.SaveInteractionEvaluationPlanSample(vcc, evaluationPlan.id, evaluationPlan.evaluatePercentage);
            }
            else {
                return true;
            }
        }).then(result => {
            return mssql_service_1.MSSql.GetInteractionsEvaluationPlan(vcc, evaluationPlan.id);
        })
            .then(result => {
            if (!result)
                throw 'EVALUATION_PLAN_SERVER_ERROR_ON_SAVE';
            evaluationPlan.interactions = result;
            evaluationPlan.uniqueInteractions = [];
            evaluationPlan.uniqueInteractions = result.reduce((acc, current) => {
                const x = acc.find(item => item.id === current.id);
                if (!x) {
                    return acc.concat([current]);
                }
                else {
                    return acc;
                }
            }, []);
            return SaveAuditorEP(vcc, evaluationPlan);
        }).then(resultSave => {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationPlanService::save_evaluationplan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationplan/get_evaluationplan/:evaluationplan_id', (req, res, next) => {
        logger.info('[EvaluationPlanService::get_evaluationplan] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let evaluationPlan = {};
        let auditorEvaluationPlan = [];
        let evaluationPlanId = req.params.evaluationplan_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!evaluationPlanId) {
                logger.error('[EvaluationPlanService::get_evaluationplan] Evaluation plan ID is null');
                throw 'EVALUATION_PLAN_SERVER_ERROR_MISSING_ID';
            }
            logger.info('[EvaluationPlanService::get_evaluationplan] Sending request to SP GetEvaluationPlan for EP ID: ' + evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationPlan(vcc, evaluationPlanId);
        }).then(result => {
            if (!result) {
                logger.error('[EvaluationPlanService::get_evaluationplan] Evaluation plan doesn\'t exists on DB');
                throw 'EVALUATION_PLAN_SERVER_ERROR_CANNOT_GET';
            }
            logger.info('[EvaluationPlanService::get_evaluationplan] Received evaluation plan from DB');
            evaluationPlan = result || evaluationPlan;
            return mssql_service_1.MSSql.GetAuditorsEvaluationPlan(vcc, evaluationPlanId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplan] Received auditors from EP: ' + result.length);
            evaluationPlan.auditors = result || evaluationPlan.auditors;
            return mssql_service_1.MSSql.GetInteractionsEvaluationPlan(vcc, evaluationPlanId);
        })
            .then((result) => {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplan] Received interaction details(audios/asyncinteractions): ' + result.length);
            evaluationPlan.interactions = result.reduce((acc, current) => {
                const x = acc.find(item => item.id === current.id);
                if (!x) {
                    return acc.concat([current]);
                }
                else {
                    return acc;
                }
            }, []);
            evaluationPlan.interactions = evaluationPlan.interactions || [];
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationPlan });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::get_evaluationplan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/delete_evaluationplan', (req, res, next) => {
        logger.info('[EvaluationPlanService::delete_evaluationplan] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let evaluationPlan = (req.body || {});
        let evaluationPlanId = evaluationPlan.id;
        logger.info(`[EvaluationPlanService::delete_evaluationplan] Deleting plan ${evaluationPlanId} by the user ${userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!evaluationPlan)
                throw 'EVALUATION_PLAN_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteEvaluationPlan(vcc, evaluationPlanId);
        })
            .then(result => {
            if (!result)
                throw 'EVALUATION_PLAN_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_DELETE'), data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::delete_evaluationplan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/delete_evaluationplans', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let evaluationPlans = req.body.evaluationPlans || [];
        logger.info(`[EvaluationPlanService::delete_evaluationplans] Deleting plans ${evaluationPlans.map(x => x.id).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!evaluationPlans)
                throw 'EVALUATION_PLAN_SERVER_ERROR_MISSING_IDS';
            let promiseArray = [];
            evaluationPlans.forEach(evaluationPlan => {
                promiseArray.push(mssql_service_1.MSSql.DeleteAuditorEvaluationPlan(vcc, evaluationPlan.id));
            });
            return Promise.all(promiseArray);
        }).then(result => {
            if (!evaluationPlans)
                throw 'EVALUATION_PLAN_SERVER_ERROR_MISSING_IDS';
            let promiseArray = [];
            evaluationPlans.forEach(evaluationPlan => {
                promiseArray.push(mssql_service_1.MSSql.DeleteEvaluationPlan(vcc, evaluationPlan.id));
            });
            return Promise.all(promiseArray);
        }).then(resultDeletes => {
            let result;
            if (resultDeletes.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Evaluation plans eliminados correctamente');
            }
            return result;
        }).then(resultDeletes => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeletes, description: (resultDeletes ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_DELETE'), data: resultDeletes });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationplan/get_evaluation_items/:evaluationplan_id', (req, res, next) => {
        logger.info(`[EvaluationPlanService::get_groups] Received ${req.method} on ${req.path} for vcc ${res.locals.vcc}`);
        let vcc = res.locals.vcc;
        let evaluationPlanId = req.params.evaluationplan_id || null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            let promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanGroups(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanSubGroups(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanAbilities(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanScores(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanAbilitiesQueries(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanAbilitiesQueriesEntities(vcc, evaluationPlanId));
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (res.headersSent)
                return;
            let resultObject = {
                groups: result[0],
                subgroups: result[1],
                abilities: result[2]
            };
            if (resultObject.abilities?.length) {
                if (result[4]?.length) {
                    result[4].forEach(aq => {
                        aq.entities = result[5]?.length ? result[5].filter(aqe => aqe.abilityQueryId === aq.id) : [];
                    });
                }
                resultObject.abilities.forEach(a => {
                    a.scores = result[3]?.length ? result[3].filter(s => s.idAbility === a.id) : [];
                    a.queries = result[4]?.length ? result[4].filter(aq => aq.abilityId === a.id) : [];
                });
            }
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultObject });
        })
            .catch(err => {
            let groupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.info(`[EvaluationPlanService::get_groups] Error on ${req.path}: ${errorMsg}`);
            if (res.headersSent)
                return;
            res.json({ status: false, description: groupMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/get_filters_from_evaluation', (req, res, next) => {
        logger.info('[EvaluationPlanService::evaluation] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let type = res.locals.type;
        let userId = res.locals.userId;
        let referrer = res.locals.pageReferrer.split('/').pop();
        let searchType = null;
        let campaigns = req.body.campaigns ? req.body.campaigns : [];
        let efectivityMode = req.body.efectivityMode ? true : false;
        switch (referrer) {
            case 'auditorinbox':
                searchType = 'PENDING';
                break;
            case 'evaluation':
                searchType = 'HISTORY';
                break;
            case 'evaluation-agent':
                searchType = 'HISTORY';
                break;
            case 'evaluation-admin':
                searchType = 'HISTORY';
                break;
        }
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetFiltersFromEvaluation(vcc, userId, campaigns, searchType, efectivityMode);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_AGENTS_SEARCH_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AgentsService::evaluation] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.EvaluationPlanService = EvaluationPlanService;
function CreateEvaluationPlan(vcc, vccType, evaluationPlan, process, hasAutomaticPart, params, userId) {
    let hasCampaigns = params?.campaigns?.length ? params.campaigns.length : 0;
    let interactionsSummary = new evaluation_plan_model_1.EvaluationPlanInteractionsSummary();
    let wasEvaluationPlanCreated = false;
    evaluationPlan.status = 1;
    evaluationPlan.createdByUserId = userId;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (!hasCampaigns) {
            logger.info(`[EvaluationPlanService::CreateEvaluationPlan(${evaluationPlan.id})] Searching evaluation format's campaigns`);
            return mssql_service_1.MSSql.GetCampaignsEvaluationFormat(vcc, params.evaluationFormatId);
        }
        else
            return Promise.resolve(true);
    })
        .then(result => {
        if (hasAutomaticPart) {
            if (evaluationPlan.isExistingTranscription) {
                params.campaigns = result.map(r => r.id);
            }
        }
        else {
            if (['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(params.environmentType)) {
                if (params.evaluationFormatId && !hasCampaigns) {
                    if (!result || !result.length)
                        throw 'SERVER_ERROR_EVALUATION_FORMAT_CAMPAIGNS_NOT_FOUND';
                    logger.info(`[EvaluationPlanService::CreateEvaluationPlan(${evaluationPlan.id})] Campaigns on evaluationFormat: ${result.length}`);
                    params.campaigns = result.map(r => r.id);
                }
            }
        }
        return Promise.resolve(true);
    })
        .then(_ => {
        if (hasAutomaticPart && !evaluationPlan.isExistingTranscription) {
            return process_service_1.CreateProcess(vcc, vccType, process, params, userId);
        }
        else
            return Promise.resolve({ status: true });
    })
        .then(result => {
        if (!result || (result && !result.status))
            throw 'SERVER_ERROR_CREATE_PROCESS_FOR_EVALUATION_PLAN_FAILED';
        return GetInteractionsDataForEvaluationPlan(vcc, vccType, evaluationPlan, hasAutomaticPart, params, userId);
    })
        .then(result => {
        interactionsSummary = result;
        if (!hasAutomaticPart
            &&
                !['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(params.environmentType))
            return linguistic_service_1.ImportAgents(vcc, vccType, interactionsSummary.agentIds);
        return Promise.resolve(true);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_CAN_NOT_SAVE_BATCH';
        logger.info(`[EvaluationPlanService::CreateEvaluationPlan(${evaluationPlan.id})] Batch was saved.`);
        if (evaluationPlan.isExistingTranscription) {
            if (evaluationPlan.evaluation_mode === 'M') {
                evaluationPlan.status = 1;
                evaluationPlan.processId = null;
            }
            else {
                evaluationPlan.status = 5;
                evaluationPlan.statusDescription = 'COMPLETED';
            }
        }
        return SaveEvaluationPlan(vcc, evaluationPlan);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_CREATE_EVALUATIONPLAN_FAILED';
        wasEvaluationPlanCreated = true;
        logger.info(`[EvaluationPlanService::CreateEvaluationPlan(${evaluationPlan.id})] The EvaluationPlan was saved.`);
        return mssql_service_1.MSSql.BulkInteractionEvaluationPlan(vcc, evaluationPlan);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_SAVE_INTERACTION_EVALUATIONPLAN_FAILED';
        logger.info(`[EvaluationPlanService::CreateEvaluationPlan(${evaluationPlan.id})] getting result: ` + result + ' after BulkInteractionEvaluationPlan');
        return mssql_service_1.MSSql.GetInteractionsEvaluationPlan(vcc, evaluationPlan.id);
    })
        .then(result => {
        logger.info(`[EvaluationPlanService::CreateEvaluationPlan(${evaluationPlan.id})] Calling function to save interactions per auditor.`, result);
        evaluationPlan.uniqueInteractions = [];
        evaluationPlan.uniqueInteractions = result.reduce((acc, current) => {
            const x = acc.find(item => item.id === current.id);
            if (!x) {
                return acc.concat([current]);
            }
            else {
                if (current.dispositionCode) {
                    let arrDispositionCode = [];
                    arrDispositionCode = x.dispositionCode ? x.dispositionCode.split(',') : [];
                    if (!arrDispositionCode.includes(current.dispositionCode))
                        arrDispositionCode.push(current.dispositionCode);
                    acc.find(item => item.id === current.id)['dispositionCode'] = arrDispositionCode.join(',');
                }
                return acc;
            }
        }, []);
        logger.info(`[EvaluationPlanService::CreateEvaluationPlan] UniqueInteractionIds: ${evaluationPlan.uniqueInteractions.length} (CALL: ${interactionsSummary.totalSyncInteraction}, TEXT: ${interactionsSummary.totalAsyncInteraction})`);
        logger.info(`[EvaluationPlanService::CreateEvaluationPlan] evaluation_mode::filterByContentQuery::filterByContentQueryName => ${evaluationPlan.evaluation_mode}::${evaluationPlan.filterByContentQuery}::${evaluationPlan.filterByContentQueryName}}`);
        if (['A', 'X'].includes(evaluationPlan.evaluation_mode)
            &&
                evaluationPlan.filterByContentQuery === null
            &&
                evaluationPlan.filterByContentQueryName === null) {
            logger.info('[EvaluationPlanService::CreateEvaluationPlan] Saving evaluations for SPEECH auditor');
            let auditor = {
                idAuditor: 'SPEECH',
                numberInteractions: evaluationPlan.uniqueInteractions.length
            };
            if (evaluationPlan.evaluation_mode === 'A')
                evaluationPlan.auditors = [];
            evaluationPlan.auditors.push(auditor);
        }
        return SaveAuditorEP(vcc, evaluationPlan);
    })
        .then(result => {
        if (result === null)
            throw 'SERVER_ERROR_SAVE_AUDITOREP_FAILED';
        logger.info(`[EvaluationPlanService::CreateEvaluationPlan(${evaluationPlan.id})] The EV save proccess has been finished.`);
        if (hasAutomaticPart && evaluationPlan.evaluationStarted)
            return SaveEvaluationPlanBatch(vcc, evaluationPlan, interactionsSummary, userId);
        else
            return Promise.resolve(true);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_COULD_NOT_SAVE_BATCH';
        return Promise.resolve(result);
    })
        .catch(err => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error(`[EvaluationPlanService::CreateEvaluationPlan(${evaluationPlan.id})] Error: ` + errorMsg);
        logger.error(`[EvaluationPlanService::CreateEvaluationPlan(${evaluationPlan.id})] Error: ` + evaluationPlan.isExistingTranscription);
        if (wasEvaluationPlanCreated)
            CleanFailedEvaluationPlanData(vcc, evaluationPlan);
        if (!evaluationPlan.isExistingTranscription)
            RemoveEvaluationPlanProcess(vcc, evaluationPlan);
        return Promise.reject(err);
    });
}
exports.CreateEvaluationPlan = CreateEvaluationPlan;
function GetInteractionsDataForEvaluationPlan(vcc, vccType, evaluationPlan, hasAutomaticPart, params, userId) {
    let cognitiveService = null;
    let summary = new evaluation_plan_model_1.EvaluationPlanInteractionsSummary();
    evaluationPlan.channelInteractions ?? (evaluationPlan.channelInteractions = []);
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
    })
        .then(result => {
        cognitiveService = result;
        if (hasAutomaticPart || (evaluationPlan.isExistingTranscription && evaluationPlan.evaluation_mode === 'M')) {
            return mssql_service_1.MSSql.GetProcessInteractionsByProcessId(vcc, evaluationPlan.processId, 'RAW');
        }
        else {
            if (['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(params.environmentType))
                return Promise.resolve(null);
            else {
                let ftpCSVRedisKey;
                ftpCSVRedisKey = `FTP_EVALUATION_PLAN:${userId}`;
                return redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey);
            }
        }
    })
        .then(result => {
        if (evaluationPlan.isExistingTranscription && ['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(params.environmentType && result)) {
            result = result.filter(interaction => params.campaigns.find(pc => pc === interaction.Campaign));
        }
        let promiseArray = [];
        if (hasAutomaticPart || (evaluationPlan.isExistingTranscription && evaluationPlan.evaluation_mode === 'M')) {
            promiseArray.push(Promise.resolve(result));
        }
        else {
            if (['ftp', 'sftp', 's3'].includes(params.environmentType) && !result)
                'SERVER_ERROR_CSV_DATA_EXPIRED';
            logger.info('[EvaluationPlanService::GetInteractionsDataForEvaluationPlan] Requesting not evaluated interactions from HistoricalData');
            promiseArray = interaction_service_1.GetChannelNewInteractionSummaryPromises(vcc, vccType, false, 'DETAILED', params, result?.data ? result.data : null);
        }
        return Promise.all(promiseArray);
    })
        .then(result => {
        logger.info(`[EvaluationPlanService::GetInteractionsDataForEvaluationPlan] Getting result for ${result.length} channels`);
        if (!result.length)
            logger.error(`[EvaluationPlanService::GetInteractionsDataForEvaluationPlan] Getting failed result:`, result);
        if (!result || result.length === 0)
            throw 'SERVER_ERROR_NO_INTEGRATED_DATA_FOUND';
        if (hasAutomaticPart || (evaluationPlan.isExistingTranscription && evaluationPlan.evaluation_mode === 'M')) {
            evaluationPlan.channelInteractions = result[0];
        }
        else {
            let channelInteractions = result.filter(r => r);
            let totalInteractionsToSave = 0;
            channelInteractions.map(ci => totalInteractionsToSave += ci && ci.totalInteractions ? ci.totalInteractions : 0);
            totalInteractionsToSave = Math.round(totalInteractionsToSave * evaluationPlan.evaluatePercentage);
            channelInteractions.map(ci => {
                if (ci?.totalInteractions) {
                    let idTitle = null;
                    let sampleResult = null;
                    if (ci.interactionType === 'CALL') {
                        let iv = {};
                        iv = ci;
                        idTitle = 'InteractionId';
                        sampleResult = linguistic_service_1.GetInteractionsSample(iv.interactions, evaluationPlan.evaluatePercentage, idTitle, totalInteractionsToSave);
                        summary.audios = sampleResult.sampleInteractionDetails;
                        logger.info(`[EvaluationPlanService::GetInteractionsDataForEvaluationPlan] Adding ${sampleResult.sampleInteractionDetails.length} audios for plan ${evaluationPlan.id}}`);
                        evaluationPlan.channelInteractions = evaluationPlan.channelInteractions.concat(summary.audios);
                        summary.audios.map(a => {
                            if (a.Agent)
                                summary.agentIds.push(a.Agent);
                        });
                    }
                    else {
                        let aiv = {};
                        aiv = ci;
                        idTitle = 'id';
                        sampleResult = linguistic_service_1.GetInteractionsSample(aiv.interactions, evaluationPlan.evaluatePercentage, idTitle, totalInteractionsToSave);
                        summary.asyncInteractions = sampleResult.sampleInteractionDetails;
                        logger.info(`[EvaluationPlanService::GetInteractionsDataForEvaluationPlan] Adding ${sampleResult.sampleInteractionDetails.length} texts for plan ${evaluationPlan.id}}`);
                        evaluationPlan.channelInteractions = evaluationPlan.channelInteractions.concat(summary.asyncInteractions);
                        aiv.interactions.map(a => {
                            if (a.firstTakedAgent)
                                summary.agentIds.push(a.firstTakedAgent);
                            if (a.lastTakedAgent)
                                summary.agentIds.push(a.lastTakedAgent);
                        });
                    }
                    totalInteractionsToSave -= sampleResult.sampleInteractionIds.length;
                }
                else
                    logger.info(`[EvaluationPlanService::GetInteractionsDataForEvaluationPlan] Found ${ci.totalInteractions} interactions for channel ${ci.interactionType}`);
            });
        }
        return Promise.resolve(true);
    })
        .then(_ => {
        if (evaluationPlan.channelInteractions.filter(ci => ci.Channel === 'CALL')) {
            summary.audios = evaluationPlan.channelInteractions.filter(ci => ci?.Channel === 'CALL');
            summary.totalSyncInteraction = summary.audios.map(item => item['InteractionId']).filter((value, index, self) => self.indexOf(value) === index).length;
            summary.totalAudio = summary.audios.length;
            summary.totalSyncDetailPhrases = evaluationPlan.channelInteractions.filter(ci => ci?.Channel === 'CALL').reduce((a, b) => ((isNaN(a) ? a.totalPhrases : a) + b.totalPhrases), 0) || 0;
        }
        else {
            summary.audios = [];
            summary.totalSyncInteraction = 0;
            summary.totalAudio = 0;
            summary.totalSyncDetailPhrases = 0;
        }
        if (evaluationPlan.channelInteractions.filter(ci => ci.channel !== 'CALL')) {
            summary.asyncInteractions = evaluationPlan.channelInteractions.filter(ci => ci.channel && ci.channel !== 'CALL');
            summary.totalAsyncInteraction = summary.asyncInteractions.map(item => item['id']).filter((value, index, self) => self.indexOf(value) === index).length;
            summary.totalAsyncDetail = evaluationPlan.channelInteractions.filter(r => r.channel && r.channel !== 'CALL').length;
            summary.totalAsyncDetailRawChar = evaluationPlan.channelInteractions.filter(ci => ci.channel && ci.channel !== 'CALL').reduce((a, b) => ((isNaN(a) ? a.rawChar : a) + b.rawChar), 0);
            summary.totalAsyncDetailRawPhrases = evaluationPlan.channelInteractions.filter(ci => ci.channel && ci.channel !== 'CALL').reduce((a, b) => ((isNaN(a) ? Math.ceil(a.rawChar / cognitiveService.GetCognitiveEngineCharsPerPhrase()) : a) + Math.ceil(b.rawChar / cognitiveService.GetCognitiveEngineCharsPerPhrase())), 0);
            summary.totalAsyncDetailPhrases = evaluationPlan.channelInteractions.filter(ci => ci.channel && ci.channel !== 'CALL').reduce((a, b) => ((isNaN(a) ? a.totalPhrases : a) + b.totalPhrases), 0);
        }
        else {
            summary.asyncInteractions = [];
            summary.totalAsyncInteraction = 0;
            summary.totalAsyncDetail = 0;
            summary.totalAsyncDetailRawChar = 0;
            summary.totalAsyncDetailRawPhrases = 0;
            summary.totalAsyncDetailPhrases = 0;
        }
        return Promise.resolve(true);
    })
        .then(_ => {
        return Promise.resolve(summary);
    })
        .catch(err => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error('[EvaluationPlanService::GetInteractionsDataForEvaluationPlan] Error: ' + errorMsg);
        return Promise.reject(err);
    });
}
function SaveEvaluationPlanBatch(vcc, evaluationPlan, summary, userId) {
    let batch = {};
    let billingVccRates = new billing_model_1.BillingVCCRates();
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (!summary)
            return GetInteractionsDataForEvaluationPlan(vcc, null, evaluationPlan, true, null, userId);
        else
            return Promise.resolve(summary);
    })
        .then(result => {
        summary ?? (summary = result);
        let billing = new billing_implementation_1.BillingImplementation();
        return billing.GetVCCRates(vcc, evaluationPlan.processId);
    })
        .then(result => {
        logger.info('[EvaluationPlanService::SaveEvaluationPlanBatch] Receiving rates data');
        if (result === null)
            throw 'SERVER_ERROR_RATES_NOT_FOUND';
        let promiseArray = [];
        billingVccRates.SetBillingVCCRates(result);
        batch.vcc = vcc;
        batch.totalInteraction = summary.GetTotalInteractions();
        batch.totalAsync = summary.totalAsyncInteraction;
        batch.totalAsyncDetail = summary.totalAsyncDetail;
        batch.totalSync = summary.totalSyncInteraction;
        batch.totalAudio = summary.totalAudio;
        batch.durationTime = 0;
        batch.roundedTime = 0;
        summary.audios.forEach(a => {
            batch.durationTime += a.DurationTime;
            batch.roundedTime += Math.ceil(a.DurationTime / 60) * 60;
        });
        batch.totalQueries = 0;
        batch.totalModels = 0;
        batch.batchTypeName = 'EVALUATION_PLAN';
        batch.batchTypeId = evaluationPlan.id;
        batch.templateId = evaluationPlan.templateId;
        batch.processId = evaluationPlan.processId;
        batch.batchScheduleId = evaluationPlan.batchScheduleId;
        batch.userApi = userId;
        batch.startDate = moment().utc().toDate();
        promiseArray.push(platform_service_1.GetPlatformVersion());
        if (batch.templateId) {
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItems(batch.vcc, batch.templateId, null));
            promiseArray.push(GetEvaluationPlanModels(batch.vcc, batch.templateId));
        }
        else {
            promiseArray.push(Promise.resolve([]));
            promiseArray.push(Promise.resolve([]));
        }
        promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(batch.vcc, evaluationPlan.processId, billingVccRates.cognitiveAnalysisAverageSecondsPerRequest));
        promiseArray.push(mssql_service_1.MSSql.GetAllegroBillingInteractionsForEvaluationPlan(batch.vcc, evaluationPlan.processId, billingVccRates.cognitiveAnalysisAverageSecondsPerRequest));
        return Promise.all(promiseArray);
    })
        .then(result => {
        logger.info('[EvaluationPlanService::SaveEvaluationPlanBatch] Receiving extra data for batch');
        let alreadyTranscribedAudio = result[3].find(r => r.interactionType === 'CALL').alreadyTranscribedAudio || 0;
        let alreadyDurationTime = result[3].find(r => r.interactionType === 'CALL').alreadyTranscribedDurationTime || 0;
        let alreadyRoundedDurationTime = result[3].find(r => r.interactionType === 'CALL').alreadyTranscribedRoundedDurationTime || 0;
        let alreadyTranscribedDurationTime = result[3].find(r => r.interactionType === 'CALL').alreadyTranscribedValidDurationTime || 0;
        let alreadyTranscribedRoundedDurationTime = result[3].find(r => r.interactionType === 'CALL').alreadyTranscribedValidRoundedDurationTime || 0;
        summary.totalSyncDetailRawPhrases = (batch.durationTime - alreadyDurationTime) / billingVccRates.cognitiveAnalysisAverageSecondsPerRequest;
        let estimatedRequests = {
            modelRequestsFromCallTime: summary.totalSyncDetailRawPhrases,
            modelRequestFromCallAlreadyPhrases: summary.totalSyncDetailPhrases,
            modelRequestsFromTextRawChar: summary.totalAsyncDetailRawPhrases,
            modelRequestsFromTextAlreadyPhrases: summary.totalAsyncDetailPhrases
        };
        if (result[0] === null)
            throw 'INSTALLATIONID_MISSING';
        batch.totalCallPhrases = summary.totalSyncDetailRawPhrases || 0;
        batch.alreadyCallPhrases = summary.totalSyncDetailPhrases || 0;
        batch.totalTextPhrases = summary.totalAsyncDetailRawPhrases || 0;
        batch.alreadyTextPhrases = summary.totalAsyncDetailPhrases || 0;
        batch.startDate = moment().utc().toDate();
        batch.installationId = result[0].installationId;
        batch.totalQueries = ['A', 'X'].includes(evaluationPlan.evaluation_mode) ? result[1].length : 0;
        batch.totalModels = result[2].length;
        batch.modelRequests = billingVccRates.GetCognitiveAnalysisTotalAveragePhrasesFromNone(estimatedRequests);
        batch.alreadyTranscribedAudio = alreadyTranscribedAudio ? alreadyTranscribedAudio : 0;
        batch.alreadyDurationTime = alreadyDurationTime ? alreadyDurationTime : 0;
        batch.alreadyRoundedDurationTime = alreadyRoundedDurationTime ? alreadyRoundedDurationTime : 0;
        batch.alreadyTranscribedDurationTime = alreadyTranscribedDurationTime ? alreadyTranscribedDurationTime : 0;
        batch.alreadyTranscribedRoundedDurationTime = alreadyTranscribedRoundedDurationTime ? alreadyTranscribedRoundedDurationTime : 0;
        batch.inProcessAudio = result[4].inProcessAudio ? result[4].inProcessAudio : 0;
        batch.inProcessDurationTime = result[4].inProcessDurationTime ? result[4].inProcessDurationTime : 0;
        if (evaluationPlan.filterByContent
            &&
                evaluationPlan.filterByContentQuery
            &&
                evaluationPlan.filterByContentQueryName)
            batch.totalQueries++;
        let billing = new billing_batch_implementation_1.BillingBatchImplementation();
        return billing.SaveBatchSummary(batch);
    })
        .then(result => {
        logger.info('[EvaluationPlanService::SaveEvaluationPlanBatch] Billing was saved.');
        if (result?.status === false)
            throw result.error ? result.error : 'SERVER_ERROR_CAN_NOT_SAVE_BATCH';
        if (evaluationPlan.evaluationStarted)
            return StartEvaluationPlan(vcc, evaluationPlan);
        else
            return Promise.resolve(true);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_COULD_NOT_START_EVALUATION_PLAN';
        logger.info('[EvaluationPlanService::SaveEvaluationPlanBatch] Batch was saved.');
        return Promise.resolve(true);
    })
        .catch(err => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error('[EvaluationPlanService::SaveEvaluationPlanBatch] Error: ' + errorMsg);
        CancelFailedBillingBatch(vcc, evaluationPlan);
        return Promise.resolve(false);
    });
}
function CreateScheduleForEvaluationPlan(vcc, vccType, evaluationPlan, process, hasAutomaticPart, params, userId) {
    let hasCampaigns = params?.campaigns?.length ? params.campaigns.length : 0;
    let newId = utilities_1.guid();
    evaluationPlan.scheduleData.batchType = 'EVALUATION_PLAN';
    evaluationPlan.scheduleData.id = evaluationPlan.description;
    evaluationPlan.scheduleData.processFiltersId = newId;
    process ?? (process = {});
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (!hasCampaigns) {
            logger.info(`[EvaluationPlanService::CreateScheduleForEvaluationPlan(${evaluationPlan.id})] Searching evaluation format's campaigns`);
            return mssql_service_1.MSSql.GetCampaignsEvaluationFormat(vcc, params.evaluationFormatId);
        }
        else
            return Promise.resolve(true);
    })
        .then(result => {
        if (['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(params.environmentType)) {
            if (!hasCampaigns) {
                if (!result || !result.length)
                    throw 'SERVER_ERROR_EVALUATION_FORMAT_CAMPAIGNS_NOT_FOUND';
                logger.info(`[EvaluationPlanService::CreateScheduleForEvaluationPlan(${evaluationPlan.id})] Campaigns on evaluationFormat: ${result.length}`);
                params.campaigns = result.map(r => r.id);
            }
            process.name = newId;
            process.processFilters = interaction_service_1.GetProcessFilterFromInteractionSearch(process.name, params, userId);
        }
        return Promise.resolve(true);
    })
        .then(_ => {
        return batch_schedule_service_1.SaveEvaluationPlanBatchSchedule(vcc, vccType, evaluationPlan, process, userId);
    })
        .catch(err => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error(`[EvaluationPlanService::CreateScheduleForEvaluationPlan(${evaluationPlan.id})] Error on:`, err);
        return Promise.resolve(false);
    });
}
function StartEvaluationPlan(vcc, evaluationPlan) {
    logger.info(`[EvaluationPlanService::StartEvaluationPlan(${evaluationPlan.id})] Starting evaluation plan.`);
    let createdEvaluationPlanQueue = false;
    let createdAbilityQueue = false;
    let createdTipificationQueue = false;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (evaluationPlan.filterByContentQuery
            &&
                evaluationPlan.filterByContentQueryName) {
            logger.info(`[EvaluationPlanService::StartEvaluationPlan(${evaluationPlan.id})] Adding into evaluationPlan queue.`);
            createdEvaluationPlanQueue = true;
            return CreateQueueL(vcc, evaluationPlan.processId, 'evaluationPlan', evaluationPlan.id);
        }
        else
            return Promise.resolve(true);
    })
        .then(_ => {
        let promiseArray = [];
        if (['A', 'X'].includes(evaluationPlan.evaluation_mode)) {
            logger.info(`[EvaluationPlanService::StartEvaluationPlan(${evaluationPlan.id})] Adding into process queue`);
            promiseArray.push(CreateQueueL(vcc, evaluationPlan.processId, 'ability', evaluationPlan.id));
            promiseArray.push(CreateQueueL(vcc, evaluationPlan.processId, 'tipification', evaluationPlan.id));
        }
        return Promise.all(promiseArray);
    })
        .then(_ => {
        createdAbilityQueue = true;
        createdTipificationQueue = true;
        logger.info(`[EvaluationPlanService::StartEvaluationPlan(${evaluationPlan.id})] Starting process`);
        if (!evaluationPlan.isExistingTranscription)
            return linguistic_service_1.BeginProcess(vcc, evaluationPlan.processId);
        return true;
    })
        .then(result => {
        return Promise.resolve(true);
    })
        .catch(err => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error(`[EvaluationPlanService::StartEvaluationPlan(${evaluationPlan.id})] Error on:`, err);
        if (createdEvaluationPlanQueue)
            redis_service_1.Redis.DeleteQueue('Process', `${evaluationPlan.processId}:evaluationPlan:${evaluationPlan.id}`);
        if (createdAbilityQueue)
            redis_service_1.Redis.DeleteQueue('Process', `${evaluationPlan.processId}:ability:${evaluationPlan.id}`);
        if (createdTipificationQueue)
            redis_service_1.Redis.DeleteQueue('Process', `${evaluationPlan.processId}:tipification:${evaluationPlan.id}`);
        return Promise.resolve(false);
    });
}
exports.StartEvaluationPlan = StartEvaluationPlan;
function GetEvaluationPlanModels(vcc, templateId) {
    return promises_shared_1.DummyPromise()
        .then(result => {
        return mssql_service_1.MSSql.GetTemplateThemeModels(vcc, templateId, null);
    }).then(result => {
        if (!result)
            throw 'SERVER_ERROR_TEMPLATE_NLU_ITEMS_NOT_FOUND';
        let modelIntentIds = result.map(mi => { return mi.id; });
        return mssql_service_1.MSSql.GetUniqueNLUByIntents(vcc, templateId, modelIntentIds.join(','));
    }).then(result => {
        if (!result)
            throw 'SERVER_ERROR_NLUS_NOT_FOUND';
        return Promise.resolve(result);
    }).catch(err => {
        return Promise.reject(err);
    });
}
function SaveEvaluationPlan(vcc, evaluationPlan) {
    return mssql_service_1.MSSql.SaveEvaluationPlan(vcc, evaluationPlan.id, evaluationPlan.modality, moment(evaluationPlan.dateFin).utc().toDate(), evaluationPlan.createdDate, evaluationPlan.createdByUserId, evaluationPlan.lastModifiedDate, evaluationPlan.lastModifiedByUserId, evaluationPlan.idEvaluationFormat, evaluationPlan.transcriptionLanguage, evaluationPlan.environmentId, evaluationPlan.description, evaluationPlan.enabled, evaluationPlan.filterByContent, evaluationPlan.filterByContentPercentage, evaluationPlan.percentage, evaluationPlan.processAndEvaluate, evaluationPlan.filterByContentQueryName, evaluationPlan.filterByContentQuery, evaluationPlan.evaluationStarted, evaluationPlan.processId, evaluationPlan.status, evaluationPlan.evaluation_mode, evaluationPlan.batchScheduleId);
}
function SetAuditorInteractionsNumbersFromPercentage(auditors, modality, totalInteractions) {
    let initQuantity = totalInteractions;
    let toFix = totalInteractions;
    if (toFix > 0) {
        switch (modality) {
            case 'FREE':
                auditors.forEach(a => a.numberInteractions = totalInteractions);
                toFix = 0;
                break;
            case 'PROPORTIONAL':
                auditors.forEach(a => {
                    let cantidad = Math.round(a.percentage * initQuantity / 100);
                    toFix -= cantidad;
                    a.numberInteractions = cantidad;
                });
                break;
            case 'CALIBRATION':
                auditors.forEach(a => a.numberInteractions = totalInteractions);
                toFix = 0;
                break;
        }
    }
    if (toFix !== 0) {
        if (toFix > 0) {
            for (let i = auditors.length - 1; i >= 0; i--) {
                if (auditors[i].selected) {
                    auditors[i].numberInteractions += toFix;
                    toFix = 0;
                    break;
                }
            }
        }
        else {
            for (let i = auditors.length - 1; i >= 0; i--) {
                if (toFix !== 0) {
                    for (let j = toFix; j < 0; j++) {
                        if (auditors[i].numberInteractions >= 1) {
                            auditors[i].numberInteractions--;
                            toFix++;
                        }
                        else
                            break;
                    }
                }
                else
                    break;
            }
        }
    }
    logger.info(`[EvaluationPlanService::SetAuditorInteractionsNumbersFromPercentage] Assigned:`, auditors);
    return auditors;
}
exports.SetAuditorInteractionsNumbersFromPercentage = SetAuditorInteractionsNumbersFromPercentage;
function SaveAuditorEP(vcc, evaluationPlan) {
    let promiseResult = true;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (evaluationPlan.auditors)
            return mssql_service_1.MSSql.BulkAuditorEvaluationPlan(vcc, evaluationPlan);
        else
            return Promise.resolve([]);
    })
        .then(result => {
        logger.info(`[EvaluationPlanService::SaveAuditorEP] Save evaluations: ${result}`);
        return Promise.resolve(result);
    })
        .catch(err => {
        promiseResult = false;
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[EvaluationPlanService::SaveAuditorEP] Error: ' + errorMsg);
        return Promise.reject(promiseResult);
    });
}
function CreateQueueL(vcc, processId, typeName, typeId) {
    let currentQueueId = '';
    currentQueueId = vcc;
    currentQueueId += ':';
    currentQueueId += processId;
    if (typeName && typeId) {
        currentQueueId += ':';
        currentQueueId += typeName;
        currentQueueId += ':';
        currentQueueId += typeId;
    }
    return redis_service_1.Redis.CreateQueueL('Process', currentQueueId)
        .then(result => {
        if (result)
            return true;
        else
            return false;
    })
        .catch(err => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_CREATE_QUEUE');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[ProcessService::CreateQueue] getting errorMsg:', errorMsg);
    });
}
exports.CreateQueueL = CreateQueueL;
function RemoveEvaluationPlanProcess(vcc, evaluationPlan) {
    let myProcessId = evaluationPlan.processId;
    let promiseResult = true;
    return Promise.resolve(promiseResult)
        .then(result => {
        return mssql_service_1.MSSql.DeleteProcess(vcc, myProcessId);
    })
        .then(result => {
        if (!result)
            throw 'ERROR_DELETING_PROCESS';
        return Promise.resolve(promiseResult);
    })
        .catch(err => {
        promiseResult = false;
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[EvaluationPlanService::SaveAuditorEP] Error: ' + errorMsg);
        return Promise.reject(promiseResult);
    });
}
exports.RemoveEvaluationPlanProcess = RemoveEvaluationPlanProcess;
function CleanFailedEvaluationPlanData(vcc, evaluationPlan) {
    let myEvaluationPlanId = evaluationPlan.id;
    let promiseResult = true;
    return Promise.resolve(promiseResult)
        .then(result => {
        return mssql_service_1.MSSql.CleanFailedEvaluationPlanData(vcc, myEvaluationPlanId);
    })
        .then(result => {
        if (!result)
            throw 'ERROR_CLEANING_FAILED_EVALUATION_PLAN_DATA';
        return Promise.resolve(promiseResult);
    })
        .catch(err => {
        promiseResult = false;
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[EvaluationPlanService::CleanFailedEvaluationPlanData] Error: ' + errorMsg);
        return Promise.reject(promiseResult);
    });
}
exports.CleanFailedEvaluationPlanData = CleanFailedEvaluationPlanData;
function CancelFailedBillingBatch(vcc, evaluationPlan) {
    let promiseResult = true;
    let batchManage = new billing_batch_model_1.BatchManage;
    batchManage.vcc = vcc;
    batchManage.batchTypeName = 'EVALUATION_PLAN';
    batchManage.batchTypeId = evaluationPlan.id;
    batchManage.systemService = 'ALL';
    batchManage.user = evaluationPlan.lastModifiedByUserId;
    return Promise.resolve(promiseResult)
        .then(result => {
        let billingBatch = new billing_batch_implementation_1.BillingBatchImplementation();
        return billingBatch.CancelBatchSummary(batchManage);
    })
        .catch(err => {
        promiseResult = false;
        let batchRunningStatus = null;
        mssql_service_1.MSSql.SaveBatchStatus(vcc, batchManage.batchTypeId, batchManage.systemService, batchRunningStatus);
        logger.error('[EvaluationPlanService::create_evaluationplan_error] Error on \'' + err.toString());
        return Promise.reject(promiseResult);
    });
}
exports.CancelFailedBillingBatch = CancelFailedBillingBatch;
