"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenericTemplateService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
let logger = log4js.getLogger('ServerScripts');
function GenericTemplateService(app) {
    app.get('/api/generic_template/get_generic_templates/:type/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let type = req.params.type || '';
        type = type === 'null' ? null : type;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetGenericTemplates(vcc, type, search_criteria, search_expression);
        }).then(GenericTemplates => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: GenericTemplates });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/generic_template/get_generic_template/:generic_template_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let genericTemplateId = req.params.generic_template_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!genericTemplateId)
                throw 'TEMPLATE_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetGenericTemplate(vcc, genericTemplateId);
        }).then(resultGenericTemplate => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultGenericTemplate });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/generic_template/get_generic_template_queries/:generic_template_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let genericTemplateId = req.params.generic_template_id;
        genericTemplateId = genericTemplateId === 'null' ? '' : genericTemplateId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetGenericTemplateQueries(vcc, genericTemplateId);
        }).then(resultGenericTemplate => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultGenericTemplate });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/generic_template/save_generic_template', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newGenericTemplate = (req.body || {});
        newGenericTemplate.createdDate = null;
        newGenericTemplate.createdByUserId = null;
        newGenericTemplate.lastModifiedDate = nowUtc;
        newGenericTemplate.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.DeleteGenericTemplate(vcc, newGenericTemplate.name);
        }).then(resultDeleteGenericTemplate => {
            if (!resultDeleteGenericTemplate)
                throw 'TEMPLATE_SERVER_ERROR_ON_SAVE';
            return SaveGenericTemplate(vcc, newGenericTemplate);
        }).then(resultSaveGenericTemplate => {
            if (!resultSaveGenericTemplate)
                throw 'TEMPLATE_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(resultSaveGenericTemplate => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveGenericTemplate, description: (resultSaveGenericTemplate ? 'SERVER_SUCCESS_MESSAGE' : 'TEMPLATE_SERVER_ERROR_ON_SAVE'), data: resultSaveGenericTemplate });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.GenericTemplateService = GenericTemplateService;
function SaveGenericTemplate(vcc, genericTemplate) {
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveGenericTemplate(vcc, genericTemplate.name, genericTemplate.type, genericTemplate.description, genericTemplate.template, genericTemplate.category, genericTemplate.createdDate, genericTemplate.createdByUserId, genericTemplate.lastModifiedDate, genericTemplate.lastModifiedByUserId));
    if (genericTemplate.genericTemplateQueries) {
        genericTemplate.genericTemplateQueries.forEach(genericTemplateQuery => {
            promiseArray.push(mssql_service_1.MSSql.SaveGenericTemplateQuery(vcc, genericTemplateQuery.templateId, genericTemplateQuery.id, genericTemplateQuery.query, moment().utc().toDate(), genericTemplateQuery.createdByUserId, moment().utc().toDate(), genericTemplateQuery.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
