"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GlobalSearchService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const arrays_shared_1 = require("../shared/arrays.shared");
let logger = log4js.getLogger('ServerScripts');
function GlobalSearchService(app) {
    app.post('/api/global_search/get_global_search_filters', (req, res, next) => {
        logger.info('[GlobalSearchService::globalSerach] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetGlobalSearchFilters(vcc);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_GLOBAL_SEARCH_FAILED';
            if (result?.agents?.length) {
                result.agents = arrays_shared_1.GetUniqueFromObjectArray(result.agents, 'agentId');
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('GlobalSearchService::globalSearch] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/global_search/get_global_search_interactions', (req, res, next) => {
        logger.info('[GlobalSearchService::globalSerachInteractions] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let campaigns = req.body.campaigns ? req.body.campaigns : [];
        let agents = req.body.agents ? req.body.agents : [];
        let dateFrom = req.body.dateFrom ? req.body.dateFrom : [];
        let dateTo = req.body.dateTo ? req.body.dateTo : [];
        let origins = req.body.origins ? req.body.origins : [];
        let interactionTypes = req.body.interactionTypes ? req.body.interactionTypes : [];
        let interactionIds = req.body.interactionIds ? req.body.interactionIds : '';
        let contactAddress = req.body.contactAddress ? req.body.contactAddress : '';
        let dispositionCodes = req.body.dispositionCodes ? req.body.dispositionCodes : [];
        logger.debug("dataArturoVelaEnviando", vcc, agents, campaigns, dateFrom, dateTo, origins, interactionTypes, interactionIds, contactAddress, dispositionCodes);
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetGlobalSearchInteractions(vcc, agents, campaigns, dateFrom, dateTo, origins, interactionTypes, interactionIds, contactAddress, dispositionCodes);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_GLOBAL_SEARCH_INTERACTIONS_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('GlobalSearchService::globalSearchInteractions] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.GlobalSearchService = GlobalSearchService;
