"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloseI6TextInteraction = exports.CloseI6CallInteraction = exports.GetEmlDataFromI6Data = exports.GetHtmlDataFromI6Data = exports.GetJsonDataFromI6Data = exports.UpdateI6Interaction = exports.GetI6InteractionsFromRedis = exports.GetI6InteractionsFromApi = exports.GetChannelOldInteractionSummaryPromises = exports.GetChannelNewInteractionSummaryPromises = exports.GetInteractionSearchFromProcessFilter = exports.GetProcessFilterFromInteractionSearch = exports.GetInteractionSearchFromRequest = exports.CreateLocalInteractionFileContent = exports.GetLocalInteractionFileContent = exports.GetLocalInteractionFile = exports.InteractionService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const fs = require("fs");
const ExternalDB = require("../db/external");
const interaction_model_1 = require("../models/interaction.model");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const arrays_shared_1 = require("../shared/arrays.shared");
const http_request_occ_1 = require("../shared/http-request-occ");
const promises_shared_1 = require("../shared/promises.shared");
const config_1 = require("../config/config");
const bluebird_1 = require("bluebird");
const integration_config_1 = require("../config/integration.config");
const mssql_config_1 = require("../config/mssql.config");
const process_model_1 = require("../models/process.model");
const audio_transfer_implementation_1 = require("../implementation/audio-transfer.implementation");
const file_manager_1 = require("../shared/file-manager");
const semantic_analysis_implementation_1 = require("../implementation/semantic-analysis.implementation");
const role_service_1 = require("./role.service");
let logger = log4js.getLogger('ServerScripts');
let httpRequestOccCompatible = true;
function InteractionService(app) {
    app.get('/api/interaction/get_interaction/:evaluationPlanId/:type/:id?', (req, res, next) => {
        logger.info('[InteractionService::get_interaction] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let evaluationPlanId = req.params.evaluationPlanId || '';
        let type = req.params.type || '';
        let id = req.params.id || '';
        let environmentId = null;
        let environmentType = null;
        let interaction = null;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!evaluationPlanId) {
                logger.error('[EvaluationPlanService::get_evaluationplan] Received evaluation plan ID is null');
                throw 'SERVER_ERROR_EVALUATION_PLAN_ID_MISSING';
            }
            logger.info('[EvaluationPlanService::get_evaluationplan] Sending request to SP GetEvaluationPlan for EP ID: ' + evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationPlan(vcc, evaluationPlanId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_EVALUATION_PLAN_NOT_FOUND';
            environmentId = result.environmentId;
            environmentType = result.environmentType;
            if (['ftp', 'sftp', 's3'].includes(environmentType)) {
                logger.info('[InteractionService::get_interaction] Requesting environment data');
                return mssql_service_1.MSSql.GetEnvironment(vcc, environmentId);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(result => {
            if (['ftp', 'sftp', 's3'].includes(environmentType)) {
                if (!result)
                    throw 'SERVER_ERROR_ENVIRONMENT_NOT_FOUND';
                environmentId = result.name;
                environmentType = result.type;
            }
            return GetEvaluationPlanInteraction(vcc, type, environmentType, evaluationPlanId, id);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_INTERACTION_EVALUATION_PLAN_NOT_FOUND';
            interaction = result;
            if (type === 'CALL' && environmentType && environmentType === 'allegro') {
                fillInteractionContentURL(interaction, vcc);
            }
            return interaction;
        })
            .then(result => {
            if (type === 'CALL') {
                if (environmentType && environmentType === 'allegro') {
                    return ExternalDB.inconcert.getCampaignWebdesignerURL(vcc, result.idCampaign, type).then(url => {
                        result.managementFormURL = (url && url.length > 0) ? config_1.INCONCERT_APPS_BASE_URL + 'webdesigner/' + url + '?INTERACTIONID=' + result.id + '&AGENTID=' + result.idAgent + '&VIRTUALCC=' + vcc + '&OUTBOUNDPROCESS=&OUTBOUNDBATCH=&SOURCE=QUALITY' : '';
                        return Promise.resolve(result);
                    });
                }
                else {
                    return Promise.resolve(result);
                }
            }
            else {
                return result;
            }
        })
            .then(result => {
            if (type === 'CALL') {
                let i = result;
                return mssql_service_1.MSSql.GetAllTagsInteraction(vcc, id).then(result => {
                    i.tags = result;
                    return Promise.resolve(i);
                }).catch(err => {
                    i.tags = [];
                    return Promise.resolve(i);
                });
            }
            else {
                return result;
            }
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            if (typeof err === 'string'
                &&
                    ['SERVER_ERROR_INTERACTION_EVALUATION_PLAN_NOT_FOUND'].includes(err)) { }
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[InteractionService::get_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_not_evaluated_interactions_auditor/:idAuditor', (req, res, next) => {
        logger.info('[InteractionService::get_not_evaluated_interactions_auditor] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let userRole = res.locals.user.role;
        let userId = res.locals.userId;
        let params = new interaction_model_1.InteractionSearch;
        let evaluationsToUpdate = [];
        params.campaigns = req.body.campaigns || [];
        params.agents = req.body.agents || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.types = req.body.types || [];
        params.auditors = req.body.auditors || [];
        params.results = req.body.results ? req.body.results.map(r => {
            let dispositionCode = null;
            if (typeof r === 'string') {
                dispositionCode = r;
            }
            else {
                dispositionCode = r.campaignId;
                if (r.description)
                    dispositionCode += '_' + r.description;
                if (r.description2)
                    dispositionCode += '_' + r.description2;
                if (r.description3)
                    dispositionCode += '_' + r.description3;
                if (r.description4)
                    dispositionCode += '_' + r.description4;
            }
            return dispositionCode;
        }) : [];
        params.dateEvaluationPlanFin = req.body.dateLimit || '';
        params.evaluationStatus = req.body.evaluationStatus || [true];
        params.interactionIds = req.body.interactionIds || [];
        params.contactAddress = req.body.contactAddress || '';
        params.efectivityMode = req.body.efectivityMode;
        params.efectivityPlans = req.body.efectivityPlans || [];
        params.dateIni = req.body.dateIni;
        params.dateFin = req.body.dateFin;
        logger.debug('[InteractionService::get_not_evaluated_interactions_auditor] Received params: :' + JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(_ => {
            return role_service_1.GetRoleFunctions(vcc, userId, userRole);
        })
            .then(result => {
            if (result && result.data && result.data.hasAdminFunctions)
                params.evaluationStatus.push(false);
            return redis_service_1.Redis.GetEvaluationsStatus(vcc);
        })
            .then(result => {
            evaluationsToUpdate = result ? result : [];
            logger.info('[InteractionService::get_not_evaluated_interactions_auditor] Requesting interactions(SP GetNotEvaluatedInteractionsAuditor) for VCC|idAuditor: ' + vcc + '|' + userId);
            if (params.efectivityMode) {
                return mssql_service_1.MSSql.GetNotEvaluatedInteractionsEfectivityAuditor(vcc, userId, params.campaigns, params.agents, params.evaluationPlans, params.types, params.results, params.dateEvaluationPlanFin, params.evaluationStatus.length && params.evaluationStatus.length > 1 ? null : params.evaluationStatus, params.interactionIds.toString(), params.contactAddress.toString(), params.efectivityPlans, params.dateIni, params.dateFin);
            }
            else {
                return mssql_service_1.MSSql.GetNotEvaluatedInteractionsAuditor(vcc, userId, params.campaigns, params.agents, params.evaluationPlans, params.types, params.results, params.dateEvaluationPlanFin, params.evaluationStatus.length && params.evaluationStatus.length > 1 ? null : params.evaluationStatus, params.interactionIds.toString(), params.contactAddress.toString(), params.dateIni, params.dateFin, params.auditors);
            }
        })
            .then((result) => {
            if (result) {
                result.map(r => {
                    r.isEvaluationBeingEvaluated = evaluationsToUpdate.find(etu => etu.evaluationPlanId === r.idEvaluationPlan && etu.interactionId === r.id && etu.auditorId === r.idAuditor) ? true : false;
                    r.isInteractionBeingEvaluated = evaluationsToUpdate.find(etu => etu.evaluationPlanId === r.idEvaluationPlan && etu.interactionId === r.id) ? true : false;
                });
            }
            if (res.headersSent)
                return;
            logger.debug('[InteractionService::get_not_evaluated_interactions_auditor] Received response from SP GetNotEvaluatedInteractionsAuditor: ' + JSON.stringify(result ? result.length : 0));
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result || [] });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[InteractionService::get_not_evaluated_interactions_auditor] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_evaluated_interactions_auditor/', (req, res, next) => {
        logger.info('[InteractionService::get_evaluated_interactions_auditor] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let idAuditor = res.locals.userId;
        logger.info('[InteractionService::get_evaluated_interactions_auditor] Filling the filter\'s object');
        let params = new interaction_model_1.InteractionSearch;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.agents = req.body.agents || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results || [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav', 'ogg'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(o => {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        params.contactAddress = req.body.contactAddress || '';
        logger.debug('[InteractionService::get_evaluated_interactions_auditor] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[InteractionService::get_evaluated_interactions_auditor] Calling GetEvaluations SP for VCC|IdAuditor: ' + instance + '|' + idAuditor);
            return mssql_service_1.MSSql.GetEvaluatedInteractionsByAuditor(instance, idAuditor, params.agents.toString(), params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.interactionIds.toString());
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.debug('[InteractionService::get_evaluated_interactions_auditor] Received result: ' + JSON.stringify(result));
            logger.info('[InteractionService::get_evaluated_interactions_auditor] Sending results: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            err => {
                let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
                let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
                logger.error('[InteractionService::get_evaluated_interactions_auditor] Error on \'' + req.path + '\': ' + errorMsg);
                if (res.headersSent)
                    return;
                res.json({ status: false, description: msg, error: errorMsg });
            };
        });
    });
    app.get('/api/interaction/get_evaluated_interactions/:idAgent?', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let evaluationDateFrom = req.query.evaluationDateFrom ? req.query.evaluationDateFrom.toString() : '1980-01-01';
        let evaluationDateTo = req.query.evaluationDateTo ? req.query.evaluationDateTo.toString() : '2050-01-01';
        let idAgent = req.params.idAgent || '';
        let campaigns = req.query.campaigns ? req.query.campaigns.toString() : 'GetEvaluatedInteractions';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetEvaluatedInteractions(instance, idAgent, campaigns, evaluationDateFrom, evaluationDateTo);
        }).then(result => {
            let interactions = result;
            let p = [];
            interactions.forEach(i => {
                p.push(ExternalDB.interactions.getInteractionByTypeAndId(instance, i.interactionType, i.id)
                    .then(r => {
                    i.address = r.address;
                    i.dispositionCode = r.dispositionCode;
                    i.origin = r.origin;
                    i.hasVideo = r.hasVideo;
                    return i;
                }).catch(err => {
                    let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                    logger.error('Error obteniendo info de las interacciones: ' + errorMsg);
                    throw err;
                }));
            });
            return Promise.all(p).then(values => {
                return [].concat.apply([], values);
            });
        }).then(result => {
            let interactions = result || [];
            return fillInteractionTags(instance, interactions);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_evaluated_interactions_by_campaigns_auditors_and_agents?', (req, res, next) => {
        logger.info('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        logger.info('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Filling the filter\'s object');
        let params = new interaction_model_1.InteractionSearch;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.auditors = req.body.auditors || [];
        params.agents = req.body.agents || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results || [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.contactAddress = req.body.contactAddress || '';
        params.multimediaFormats = ['mp3', 'wav', 'ogg'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(o => {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        logger.debug('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.debug('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Calling GetEvaluatedInteractionsByCampaignsAuditorsAndAgents SP');
            return mssql_service_1.MSSql.GetEvaluatedInteractionsByCampaignsAuditorsAndAgents(instance, params.auditors.toString(), params.agents.toString(), params.campaigns.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.contactAddress.toString());
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.debug('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Results from GetEvaluatedInteractionsByCampaignsAuditorsAndAgents SP: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[InteractionService::get_evaluated_interactions_by_campaigns_auditors_and_agents] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_evaluated_interactions_agent?', (req, res, next) => {
        logger.info('[InteractionService::get_evaluated_interactions_agent] Received ' + req.method + ' on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let idAgent = res.locals.userId;
        logger.info('[InteractionService::get_evaluated_interactions_agent] Filling the filter\'s object');
        let params = new interaction_model_1.InteractionSearch;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results || [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav', 'ogg'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(o => {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        params.contactAddress = req.body.contactAddress || '';
        logger.debug('[InteractionService::get_evaluated_interactions_agent] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[InteractionService::get_evaluated_interactions_agent] Calling GetEvaluatedInteractions SP for VCC|idAgent: ' + instance + '|' + idAgent);
            return mssql_service_1.MSSql.GetEvaluatedInteractionsByAuditor(instance, null, idAgent, params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.interactionIds.toString());
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[InteractionService::get_evaluated_interactions_agent] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.get('/api/interaction/get_interactions_evaluationplan/:idEvaluationPlan/:idAuditor?', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let idEvaluationPlan = req.params.idEvaluationPlan || '';
        let idAuditor = req.params.idAuditor || '';
        let interactions;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetInteractionsEvaluationPlan(instance, idEvaluationPlan, idAuditor);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_interactions', (req, res, next) => {
        logger.info('[InteractionService::get_interactions] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let type = res.locals.type;
        let userId = res.locals.userId;
        let limit = 10;
        let params = new interaction_model_1.InteractionSearch;
        params = GetInteractionSearchFromRequest(req.body.evaluationFormatId, req.body, false);
        params.searchType = 'TO_SEARCH';
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (params.evaluationFormatId && (params.campaigns && params.campaigns.length === 0)) {
                logger.info('[InteractionService::get_interactions] Searching evaluation format\'s campaigns');
                return mssql_service_1.MSSql.GetCampaignsEvaluationFormat(vcc, params.evaluationFormatId);
            }
            else {
                return bluebird_1.resolve(true);
            }
        })
            .then(result => {
            if (params.environmentType && ['allegro', 'i6'].includes(params.environmentType)) {
                if (params.evaluationFormatId && params?.campaigns?.length === 0) {
                    if (!result || result.length === 0)
                        throw 'SERVER_ERROR_EVALUATION_FORMAT_CAMPAIGNS_NOT_FOUND';
                    params.campaigns = result.map(r => r.id);
                }
            }
            return Promise.resolve(true);
        })
            .then(result => {
            if (params.environmentType && ['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(params.environmentType)) {
                return Promise.resolve(true);
            }
            else {
                let ftpCSVRedisKey;
                if (params.evaluationFormatId)
                    ftpCSVRedisKey = 'FTP_EVALUATION_PLAN:';
                else
                    ftpCSVRedisKey = 'FTP_PROCESS:';
                return redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey + userId);
            }
        })
            .then(result => {
            if (['ftp', 'sftp', 's3'].includes(params.environmentType))
                if (result === null)
                    throw 'SERVER_ERROR_CSV_DATA_EXPIRED';
            let promiseArray = [];
            logger.info('[InteractionService::get_interactions] Requesting not evaluated interactions from HistoricalData');
            promiseArray = GetChannelNewInteractionSummaryPromises(vcc, type, true, 'SUMMARIZED', params, result.data);
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.info('[InteractionService::get_interactions] Received interactions from database');
            res.json({
                status: true,
                description: 'SERVER_SUCCESS_MESSAGE',
                data: result.filter(r => r)
            });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::get_interactions] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_interaction_view', (req, res, next) => {
        logger.info('[InteractionService::get_interaction_view] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let evaluationPlanId = null;
        let processId = null;
        let interactionType = null;
        let folder = null;
        let campaign = null;
        let interactionId = null;
        let fileType = interactionType === 'CALL' ? 'json' : 'view';
        let interactionItems = [];
        let viewResults = [];
        let transcriptionResults = [];
        let sentimentResults = [];
        let transcriptionQualification = null;
        let audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (req.body) {
                evaluationPlanId = req.body.evaluationPlanId || null;
                processId = req.body.processId || null;
                interactionType = req.body.interactionType || null;
                folder = req.body.folder || null;
                campaign = req.body.campaign || null;
                interactionId = req.body.interactionId || null;
            }
            else {
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            }
            logger.info('[InteractionService::get_interaction_view] Searching inetraction instances');
            return mssql_service_1.MSSql.GetInteractionTranscription(vcc, evaluationPlanId, processId, interactionType, interactionId);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_INTERACTION_NOT_FOUND';
            interactionItems = folder ? result.filter(r => r.folder === folder) : result;
            let promiseArray = [];
            interactionItems.forEach(i => {
                let filePath = '';
                if (i.interactionType === 'MAIL') {
                    filePath = audioTransfer.GetFileNamePathFromInteraction(i, fileType, 1);
                    promiseArray.push(GetLocalInteractionFile(i, filePath, i.folder));
                }
                else {
                    filePath = audioTransfer.GetFileNamePathFromInteraction(i, fileType);
                    promiseArray.push(GetLocalInteractionFile(i, filePath, i.folder));
                }
            });
            return Promise.all(promiseArray);
        })
            .then(result => {
            viewResults = result;
            let promiseArray = [];
            interactionItems.forEach(i => {
                let filePath = '';
                if (i.interactionType === 'MAIL') {
                    if (i.totalMessages >= 2) {
                        for (let n = 2; n <= i.totalMessages; n++) {
                            filePath = audioTransfer.GetFileNamePathFromInteraction(i, fileType, n);
                            promiseArray.push(GetLocalInteractionFile(i, filePath, i.folder));
                        }
                    }
                }
            });
            return Promise.all(promiseArray);
        }).then(result => {
            viewResults = viewResults.concat(result);
            let semanticAnalysis = new semantic_analysis_implementation_1.SemanticAnalysisImplementation();
            let promiseArray = [];
            interactionItems.map(i => {
                promiseArray.push(semanticAnalysis.SemanticAnalysisViewFileGenerate(i));
            });
            return Promise.all(promiseArray);
        }).then(result => {
            transcriptionResults = result;
            if (transcriptionResults[0] && transcriptionResults[0].fileName) {
                return Promise.resolve(false);
            }
            else {
                return Promise.resolve(false);
            }
        }).then(result => {
            if (typeof (result) !== 'boolean') {
                sentimentResults = result;
                transcriptionResults[0].transcriptionResult.sentimentResults = sentimentResults;
            }
            let transcriptionDTO = {
                vcc: vcc,
                processId: processId,
                id: interactionId,
                section: null,
                part: null
            };
            transcriptionDTO.interactionType = interactionType;
            transcriptionDTO.folder = folder;
            transcriptionDTO.campaign = campaign;
            return mssql_service_1.MSSql.GetTranscription(transcriptionDTO);
        }).then(result => {
            transcriptionQualification = (result && result.StatusCodeError) ? null : result;
            if (res.headersSent)
                return;
            logger.info('[InteractionService::get_interaction_view] Received results');
            res.json({
                status: true,
                description: 'SERVER_SUCCESS_MESSAGE',
                data: {
                    interactionView: viewResults,
                    transcriptionResult: transcriptionResults,
                    transcriptionQualification: transcriptionQualification && transcriptionQualification.value ? transcriptionQualification.value : null
                }
            });
        }).catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::get_interaction_view] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/get_interaction_query_results', (req, res, next) => {
        logger.info('[InteractionService::get_interaction_query_results] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let interactionType = null;
        let interactionId = null;
        let processId = null;
        let folder = null;
        let campaign = null;
        let transactionType = null;
        let transactionId = null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (req.body) {
                interactionType = req.body.interactionType || null;
                interactionId = req.body.interactionId || null;
                folder = req.body.folder || null;
                campaign = req.body.campaign || null;
                transactionId = req.body.transactionId || null;
                transactionType = req.body.transactionType || null;
            }
            else {
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            }
            logger.info('[InteractionService::get_interaction_query_results] Searching inetraction instances');
            return mssql_service_1.MSSql.GetInteractionQueryResults(vcc, transactionType, transactionId, interactionType, interactionId, campaign, folder);
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.info('[InteractionService::get_interaction_query_results] Received results');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::get_interaction_query_results] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/discard_interaction', (req, res, next) => {
        logger.info('[InteractionService::discard_interaction] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let userRole = res.locals.user.role;
        let userId = res.locals.userId;
        let evaluationToDiscard = (req.body.evaluation || {});
        let multiple = req.body.multiple || false;
        let saveResult = null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return role_service_1.GetRoleFunctions(vcc, userId, userRole);
        })
            .then(result => {
            if (result && result.data && !result.data.hasAdminFunctions)
                throw 'SERVER_ERROR_USER_HAS_NOT_PERMISSION';
            return redis_service_1.Redis.GetEvaluationStatus(vcc, evaluationToDiscard);
        })
            .then(result => {
            if (result)
                throw 'SERVER_ERROR_EVALUATION_IS_LOCKED';
            return mssql_service_1.MSSql.SaveEvaluationStatus(vcc, evaluationToDiscard.evaluationPlanId, evaluationToDiscard.interactionId, evaluationToDiscard.auditorId, false, multiple, userId);
        })
            .then(result => {
            if (result && result.status === false)
                throw result.error;
            saveResult = result;
            return mssql_service_1.MSSql.SaveEvaluationPlanCompleted(vcc, evaluationToDiscard.evaluationPlanId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.info('[InteractionService::discard_interaction] The evaluation update has been finished.');
            res.json({ status: true, description: (saveResult ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::discard_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/interaction/recover_interaction', (req, res, next) => {
        logger.info('[InteractionService::recover_interaction] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let userRole = res.locals.user.role;
        let userId = res.locals.userId;
        let evaluationToDiscard = (req.body.evaluation || {});
        let multiple = req.body.multiple || false;
        let saveResult = null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return role_service_1.GetRoleFunctions(vcc, userId, userRole);
        })
            .then(result => {
            if (result && result.data && result.data.hasAdminFunctions) { }
            else
                throw 'SERVER_ERROR_USER_HAS_NOT_PERMISSION';
            return mssql_service_1.MSSql.SaveEvaluationStatus(vcc, evaluationToDiscard.evaluationPlanId, evaluationToDiscard.interactionId, evaluationToDiscard.auditorId, true, multiple, userId);
        })
            .then(result => {
            if (result && result.status === false)
                throw result.error;
            saveResult = result;
            return mssql_service_1.MSSql.SaveEvaluationPlanCompleted(vcc, evaluationToDiscard.evaluationPlanId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.info('[InteractionService::recover_interaction] The evaluation update has been finished.');
            res.json({ status: true, description: (saveResult ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[InteractionService::recover_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
}
exports.InteractionService = InteractionService;
function fillInteractionContentURL(i, vcc) {
    if (i.interactionType === 'CALL') {
        i.interactionPartGroups.forEach(group => {
            group.parts.forEach(p => {
                p.partURL = config_1.QUALITY_AUDIO_BASE_URL + i.id + '/' + p.section + '/' + p.part;
            });
        });
    }
}
function fillInteractionTags(instance, interactions) {
    let p = [];
    interactions.forEach(i => {
        if (i.interactionType === 'CALL')
            p.push(mssql_service_1.MSSql.GetAllTagsInteraction(instance, i.id)
                .then(tags => {
                i.tags = tags || [];
                return i;
            }).catch(err => {
                let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                logger.error('Error obteniendo info de los tags: ' + errorMsg);
                throw err;
            }));
        else
            p.push(Promise.resolve(i));
    });
    return Promise.all(p).then(values => {
        return [].concat.apply([], values);
    });
}
function GetLocalInteractionFile(interaction, filePath, folder, searchedDate) {
    if (!searchedDate)
        searchedDate = 'CURRENT';
    let genericResult = {};
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return GetLocalInteractionFileContent(interaction, filePath);
    })
        .then((gr) => {
        if (gr) {
            genericResult = gr;
            genericResult.result = folder;
            return Promise.resolve(genericResult);
        }
        else {
            genericResult.status = false;
            genericResult.description = 'SERVER_ERROR_COULDNOT_GET_FILE_CONTENT';
            genericResult.result = folder;
            return Promise.resolve(genericResult);
        }
    })
        .catch((err) => {
        genericResult.status = false;
        genericResult.description = 'ERROR';
        genericResult.data = err.toString();
        return Promise.resolve(genericResult);
    });
}
exports.GetLocalInteractionFile = GetLocalInteractionFile;
function GetLocalInteractionFileContent(interaction, filePath) {
    let genericResult = {};
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return CreateLocalInteractionFileContent(interaction, filePath);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_CONTENT_FROM_I6_NOT_FOUND';
        genericResult.status = true;
        genericResult.description = filePath.split('/').pop();
        genericResult.data = '';
        if (interaction.interactionType === 'MAIL') {
            const EmlParser = require('eml-parser');
            return new EmlParser(fs.createReadStream(filePath))
                .parseEml()
                .then(data => {
                genericResult.data = data.html ?
                    data.html
                    :
                        data.text ?
                            data.text
                            :
                                'MESSAGE_CONTENT_NOT_FOUND';
                if (data?.attachments?.length) {
                    genericResult.extraContent = [];
                    data.attachments.forEach(da => {
                        let cid = 'cid:';
                        let buff = Buffer.from(da.content);
                        let base64 = buff.toString('base64');
                        let format = 'data:' + da.contentType.split(';')[0] + ';base64,';
                        if (da.cid) {
                            cid += da.cid.replace('<', '').replace('>', '');
                        }
                        else {
                            cid += 'vmime';
                        }
                        if (genericResult.data.includes(cid)) {
                            let attachmentSrc = format + base64;
                            if (cid === 'cid:vmime') {
                                let imgSrcPattern = /src\s*=\s*"cid:vmime(.+?)"/;
                                genericResult.data = genericResult.data.replace(imgSrcPattern, 'src="' + attachmentSrc + '"');
                            }
                            else {
                                genericResult.data = genericResult.data.replace(cid, attachmentSrc);
                            }
                        }
                        if (da.type && da.type === 'attachment' && !da.related) {
                            let aFile = {};
                            aFile.id = da.id;
                            aFile.contentType = da.contentType.split(';')[0];
                            aFile.base64 = base64;
                            if (da.filename) {
                                aFile.name = da.filename;
                            }
                            else {
                                let counter = genericResult.extraContent.filter(ec => ec.name.includes('untitled')).length;
                                counter = counter + 1;
                                aFile.name = 'untitled_' + counter + '.' + aFile.contentType.split('/')[1];
                            }
                            genericResult.extraContent.push(aFile);
                        }
                    });
                }
                return Promise.resolve(genericResult);
            });
        }
        else {
            return file_manager_1.ReadFileContent(filePath, 'utf-8', false)
                .then(res => {
                genericResult.data = res;
                return Promise.resolve(genericResult);
            });
        }
    })
        .then(result => {
        return Promise.resolve(genericResult);
    })
        .catch(err => {
        logger.error('[InteractionService::GetLocalInteractionFileContent] Getting errorMessage:', err);
        genericResult.status = false;
        genericResult.description = 'SERVER_ERROR_CANNOT_GET_FILE_CONTENT';
        return Promise.reject(err);
    });
}
exports.GetLocalInteractionFileContent = GetLocalInteractionFileContent;
function CreateLocalInteractionFileContent(interaction, filePath) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (!filePath) {
            let audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
            if (interaction.interactionType === 'MAIL')
                filePath = audioTransfer.GetFileNamePathFromInteraction(interaction, 'view', 1);
            else
                filePath = audioTransfer.GetFileNamePathFromInteraction(interaction, 'view');
        }
        return file_manager_1.ExistInFileSystem(filePath);
    })
        .then(result => {
        if (!result) {
            if (interaction.interactionType === 'MAIL')
                return GetEmlDataFromI6Data(interaction);
            else
                return GetHtmlDataFromI6Data(interaction);
        }
        return Promise.resolve(true);
    })
        .catch(err => {
        logger.error('[InteractionService::CreateLocalInteractionFileContent] Getting errorMessage:', err);
        return Promise.reject(err);
    });
}
exports.CreateLocalInteractionFileContent = CreateLocalInteractionFileContent;
function GetEvaluationPlanInteraction(vcc, interactionType, environmentType, evaluationPlanId, interactionId) {
    let interaction = {};
    let interactionParts = [];
    let hadI6Update = false;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetEvaluationPlanInteractionByTypeAndId(vcc, evaluationPlanId, interactionType, interactionId);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_INTERACTION_NOT_FOUND';
        interaction = result;
        if ((interactionType !== 'CALL'
            ||
                (interactionType === 'CALL'
                    &&
                        environmentType === 'i6'))
            &&
                interaction?.needI6Update) {
            hadI6Update = true;
            return UpdateI6Interaction({ vcc: vcc, id: interactionId, interactionType });
        }
        else {
            return Promise.resolve(null);
        }
    })
        .then(result => {
        if (hadI6Update)
            return mssql_service_1.MSSql.GetEvaluationPlanInteractionByTypeAndId(vcc, evaluationPlanId, interactionType, interactionId);
        else
            return Promise.resolve(result);
    })
        .then(result => {
        if (hadI6Update)
            interaction = result;
        return mssql_service_1.MSSql.GetI6InteractionAudios(vcc, interactionId);
    })
        .then(result => {
        interactionParts = result;
        if (interactionType === 'CALL' && environmentType === 'i6') {
            if (interactionParts && !interactionParts.length) {
                return http_request_occ_1.ApiI6GetInteractionData(vcc, interactionId);
            }
        }
        return Promise.resolve(result);
    })
        .then(result => {
        if (interactionType === 'CALL' && environmentType === 'i6' && interactionParts.length === 0) {
            let i6Data = null;
            let i6Audios = [];
            i6Data = result;
            i6Data.body.interactionData.records = i6Data.body.interactionData.records.filter(r => r.Format !== 'mp4');
            i6Data.body.interactionData.records = arrays_shared_1.SortObjectArrayByValue(i6Data.body.interactionData.records, 'Part', 'A');
            i6Data.body.interactionData.records.forEach(r => {
                i6Audios.push({
                    id: interactionId,
                    section: r.Section,
                    part: r.Part,
                    agentId: r.agent.includes('@') ? r.agent.split('@')[0] : r.agent,
                    format: r.Format
                });
            });
            return Promise.resolve(i6Audios);
        }
        return Promise.resolve(result);
    })
        .then(result => {
        if (interactionParts.length === 0)
            interactionParts = result;
        interaction.interactionPartGroups = [];
        let uniqueAgentItems = arrays_shared_1.GetUniqueFromObjectArray(interactionParts, 'agentId');
        uniqueAgentItems.map(ua => {
            let item = {};
            item.agentId = ua.agentId;
            item.parts = interactionParts.filter(ip => ip.agentId === item.agentId);
            interaction.interactionPartGroups.push(item);
        });
        return Promise.resolve(true);
    })
        .then(_ => {
        return Promise.resolve(interaction);
    })
        .catch(err => {
        logger.error('[InteractionService::GetEvaluationPlanInteraction] Error: ', err);
        return Promise.resolve(null);
    });
}
function DeleteEvaluationPlanInteraction(vcc, interactionType, evaluationPlanId, interactionId) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.DeleteEvaluationPlanInteractionByTypeAndId(vcc, evaluationPlanId, interactionType, interactionId);
    })
        .catch(err => {
        logger.info('[InteractionService::DeleteEvaluationPlanInteraction] Error: ', err);
        return Promise.resolve(false);
    });
}
function GetInteractionSearchFromRequest(evaluationFormatId, filtersObject, withSectionPart) {
    let params = new interaction_model_1.InteractionSearch;
    params.evaluationFormatId = evaluationFormatId || null;
    params.withSectionPart = withSectionPart;
    params.storageId = filtersObject.storageId || null;
    params.environmentId = filtersObject.environmentId || null;
    params.environmentType = filtersObject.environmentType || null;
    params.types = filtersObject.types || [];
    params.dateIni = filtersObject.dateIni ? moment(filtersObject.dateIni).toDate() : null;
    params.dateFin = filtersObject.dateFin ? moment(filtersObject.dateFin).toDate() : null;
    params.hourIni = filtersObject.hourIni || null;
    params.hourFin = filtersObject.hourFin || null;
    params.minDuration = filtersObject.minDuration === 0 ? 0 : filtersObject.minDuration || null;
    params.maxDuration = filtersObject.maxDuration || null;
    params.campaigns = filtersObject.campaigns || [];
    params.agents = filtersObject.agents || [];
    params.results = filtersObject.results || [];
    params.origins = filtersObject.origins || [];
    params.channelsWithLimit = filtersObject.channelsWithLimit || [];
    params.multimediaFormats = JSON.parse(JSON.stringify(config_1.DEFAULT_INTERACTION_FORMAT));
    params.evaluated = filtersObject.evaluated || [];
    if (params.evaluationFormatId) {
        params.evaluationTypes = filtersObject.evaluationTypes || [];
        params.tags = filtersObject.tags || [];
        params.hasVideoOptions = filtersObject.hasVideoOptions || [];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
    }
    params.interactionIds = filtersObject.interactionIds ? filtersObject.interactionIds : [];
    params.contactAddress = filtersObject.contactAddress ? filtersObject.contactAddress : '';
    return params;
}
exports.GetInteractionSearchFromRequest = GetInteractionSearchFromRequest;
function GetProcessFilterFromInteractionSearch(processId, interactionSearch, userId) {
    let processFilterArray = [];
    let interactionSearchKeys = Object.keys(interactionSearch);
    let uniqueItemArray = ['dateIni', 'dateFin', 'hourIni', 'hourFin', 'minDuration', 'maxDuration', 'interactionType'];
    let multipleItemArray = ['campaigns', 'types', 'origins', 'results', 'interactionIds', 'agents', 'evaluated'];
    interactionSearchKeys.map(isk => {
        if (interactionSearch[isk]) {
            let keyName = '';
            switch (isk) {
                case 'dateIni':
                    keyName = 'startDate';
                    break;
                case 'dateFin':
                    keyName = 'endDate';
                    break;
                case 'hourIni':
                    keyName = 'startTime';
                    break;
                case 'hourFin':
                    keyName = 'endTime';
                    break;
                case 'results':
                    keyName = 'disposition';
                    break;
                default:
                    keyName = isk;
                    break;
            }
            if (uniqueItemArray.includes(isk)) {
                let uniqueValue = '';
                if (isk === 'agents')
                    uniqueValue = interactionSearch[isk].join('-');
                else if (isk === 'dateIni')
                    uniqueValue = interactionSearch[isk].toISOString();
                else if (isk === 'dateFin')
                    uniqueValue = interactionSearch[isk].toISOString();
                else
                    uniqueValue = interactionSearch[isk];
                if (uniqueValue !== null && uniqueValue !== '') {
                    processFilterArray.push(new process_model_1.ProcessFilter(processId, keyName, uniqueValue, moment().utc().toDate(), userId, moment().utc().toDate(), userId));
                }
            }
            else if (multipleItemArray.includes(isk)) {
                let multipleValue = [];
                if (isk === 'origins') {
                    if (interactionSearch[isk].length === 1)
                        multipleValue = interactionSearch[isk];
                }
                else if (isk === 'results') {
                    multipleValue = interactionSearch[isk].map(r => JSON.stringify(r));
                }
                else
                    multipleValue = interactionSearch[isk];
                multipleValue.map(mv => {
                    if (mv !== null && mv !== '') {
                        processFilterArray.push(new process_model_1.ProcessFilter(processId, keyName, mv, moment().utc().toDate(), userId, moment().utc().toDate(), userId));
                    }
                });
            }
        }
    });
    return processFilterArray;
}
exports.GetProcessFilterFromInteractionSearch = GetProcessFilterFromInteractionSearch;
function GetInteractionSearchFromProcessFilter(processFilter) {
    let interactionSearch = new interaction_model_1.InteractionSearch();
    let uniqueItemArray = ['startDate', 'startDate', 'startTime', 'endTime', 'minDuration', 'maxDuration', 'interactionType'];
    let multipleItemArray = ['campaigns', 'types', 'origins', 'disposition', 'interactionIds', 'agents', 'evaluated'];
    processFilter.map(pf => {
        let keyName = '';
        switch (pf.key) {
            case 'startDate':
                keyName = 'dateIni';
                break;
            case 'endDate':
                keyName = 'dateFin';
                break;
            case 'startTime':
                keyName = 'hourIni';
                break;
            case 'endTime':
                keyName = 'hourFin';
                break;
            case 'disposition':
                keyName = 'results';
                break;
            default:
                keyName = pf.key;
                break;
        }
        if (uniqueItemArray.includes(pf.key)) {
            if (['dateIni', 'dateFin'].includes(keyName)) {
                interactionSearch[keyName] = moment().toDate();
            }
            else {
                interactionSearch[keyName] = pf.value;
            }
        }
        else if (multipleItemArray.includes(pf.key)) {
            if (!interactionSearch[keyName])
                interactionSearch[keyName] = [];
            if (keyName === 'results')
                interactionSearch[keyName].push(JSON.parse(pf.value));
            else
                interactionSearch[keyName].push(pf.value);
        }
    });
    if (!interactionSearch.campaigns)
        interactionSearch.campaigns = [];
    if (!interactionSearch.agents)
        interactionSearch.agents = [];
    if (!interactionSearch.origins)
        interactionSearch.origins = [];
    if (!interactionSearch.multimediaFormats)
        interactionSearch.multimediaFormats = [];
    if (!interactionSearch.interactionIds)
        interactionSearch.interactionIds = [];
    if (!interactionSearch.contactAddress)
        interactionSearch.contactAddress = '';
    if (!interactionSearch.evaluated)
        interactionSearch.evaluated = [];
    return interactionSearch;
}
exports.GetInteractionSearchFromProcessFilter = GetInteractionSearchFromProcessFilter;
function GetChannelNewInteractionSummaryPromises(vcc, vccType, isSummary, resultType, filters, csvData) {
    if (filters.types
        &&
            ((filters.types.includes('CALL') && filters.types.length > 1)
                ||
                    (!filters.types.includes('CALL') && filters.types.length > 0))) {
        if (!integration_config_1.USER_API_I6 || integration_config_1.USER_API_I6 === '') {
            filters.types = ['CALL'];
        }
    }
    let promiseArray = [];
    let results = filters.results || [];
    let result_numbers = [];
    let result_texts = [];
    if (results.length > 0) {
        results.forEach(r => {
            if (typeof r === 'string') {
                result_texts.push(r);
                result_numbers.push(r);
            }
            else {
                if (r.code) {
                    let rcf = JSON.parse(JSON.stringify(r));
                    result_numbers.push(rcf.campaignId + '_' + rcf.code);
                }
                else {
                    let dc = null;
                    if (r.description)
                        dc = r.description;
                    if (r.description2)
                        dc = r.description2;
                    if (r.description3)
                        dc = r.description3;
                    if (r.description4)
                        dc = r.description4;
                    if (!result_texts.includes(dc))
                        result_texts.push(dc);
                }
            }
        });
        if (result_texts.length > 0)
            filters.results = arrays_shared_1.GetUniqueFromArray(result_texts);
        if (result_numbers.length === 0)
            result_numbers.push('NULL');
        if (result_texts.length === 0)
            result_texts.push('NULL');
    }
    if (!filters.multimediaFormats)
        filters.multimediaFormats = null;
    if (filters.environmentType
        &&
            ['ftp', 'sftp', 's3'].includes(filters.environmentType))
        vccType = 'OTHER';
    let flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
    switch (vccType) {
        case 'ALLEGRO':
            if (filters.types.includes('CALL'))
                promiseArray.push(mssql_service_1.MSSql.GetAllegroInteractions(integration_config_1.IP_BD, vcc, filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_numbers, flagAllegro, filters.origins.toString(), filters.minDuration, filters.maxDuration, filters.multimediaFormats ? filters.multimediaFormats.toString() : null, filters.interactionIds.toString(), filters.evaluated.toString(), resultType, filters.contactAddress.toString(), filters.channelsWithLimit && filters.channelsWithLimit.length > 0 ? filters.channelsWithLimit[0].value : 0));
            break;
        case 'I6':
            if (filters.searchType === 'TO_SEARCH') {
                if (filters.types.includes('APPSTORE'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'APPSTORE', filters));
                if (filters.types.includes('CALL'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'CALL', filters));
                if (filters.types.includes('CHAT'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'CHAT', filters));
                if (filters.types.includes('FACEBOOK'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'FACEBOOK', filters));
                if (filters.types.includes('GOOGLEPLAY'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'GOOGLEPLAY', filters));
                if (filters.types.includes('INSTAGRAM'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'INSTAGRAM', filters));
                if (filters.types.includes('MAIL'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'MAIL', filters));
                if (filters.types.includes('TWITTER'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'TWITTER', filters));
                if (filters.types.includes('WHATSAPP'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'WHATSAPP', filters));
                if (filters.types.includes('YOUTUBE'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'YOUTUBE', filters));
                if (filters.types.includes('SMS'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'SMS', filters));
            }
            else if (filters.searchType === 'TO_OPERATE') {
                if (filters.types.includes('APPSTORE'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'APPSTORE'));
                if (filters.types.includes('CALL'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'CALL'));
                if (filters.types.includes('CHAT'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'CHAT'));
                if (filters.types.includes('FACEBOOK'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'FACEBOOK'));
                if (filters.types.includes('GOOGLEPLAY'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'GOOGLEPLAY'));
                if (filters.types.includes('INSTAGRAM'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'INSTAGRAM'));
                if (filters.types.includes('MAIL'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'MAIL'));
                if (filters.types.includes('TWITTER'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'TWITTER'));
                if (filters.types.includes('WHATSAPP'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'WHATSAPP'));
                if (filters.types.includes('YOUTUBE'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'YOUTUBE'));
                if (filters.types.includes('SMS'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'SMS'));
            }
            break;
        case 'OCC':
            if (filters.types.includes('CALL'))
                promiseArray.push(mssql_service_1.MSSql.GetAllegroInteractions(integration_config_1.IP_BD, vcc, filters.dateIni, filters.dateFin, filters.hourIni, filters.hourFin, filters.campaigns.toString(), filters.agents.toString(), result_numbers, flagAllegro, filters.origins.toString(), filters.minDuration, filters.maxDuration, filters.multimediaFormats ? filters.multimediaFormats.toString() : null, filters.interactionIds.toString(), filters.evaluated.toString(), resultType, filters.contactAddress, filters.channelsWithLimit && filters.channelsWithLimit.length > 0 ? filters.channelsWithLimit[0].value : 0));
            if (filters.searchType === 'TO_SEARCH') {
                if (filters.types.includes('APPSTORE'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'APPSTORE', filters));
                if (filters.types.includes('CHAT'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'CHAT', filters));
                if (filters.types.includes('FACEBOOK'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'FACEBOOK', filters));
                if (filters.types.includes('GOOGLEPLAY'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'GOOGLEPLAY', filters));
                if (filters.types.includes('INSTAGRAM'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'INSTAGRAM', filters));
                if (filters.types.includes('MAIL'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'MAIL', filters));
                if (filters.types.includes('TWITTER'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'TWITTER', filters));
                if (filters.types.includes('WHATSAPP'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'WHATSAPP', filters));
                if (filters.types.includes('YOUTUBE'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'YOUTUBE', filters));
                if (filters.types.includes('SMS'))
                    promiseArray.push(GetI6InteractionsFromApi(vcc, 'SMS', filters));
            }
            else if (filters.searchType === 'TO_OPERATE') {
                if (filters.types.includes('APPSTORE'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'APPSTORE'));
                if (filters.types.includes('CHAT'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'CHAT'));
                if (filters.types.includes('FACEBOOK'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'FACEBOOK'));
                if (filters.types.includes('GOOGLEPLAY'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'GOOGLEPLAY'));
                if (filters.types.includes('INSTAGRAM'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'INSTAGRAM'));
                if (filters.types.includes('MAIL'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'MAIL'));
                if (filters.types.includes('TWITTER'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'TWITTER'));
                if (filters.types.includes('WHATSAPP'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'WHATSAPP'));
                if (filters.types.includes('YOUTUBE'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'YOUTUBE'));
                if (filters.types.includes('SMS'))
                    promiseArray.push(GetI6InteractionsFromRedis(vcc, filters.storageId, 'SMS'));
            }
            break;
        case 'OTHER':
            promiseArray.push(mssql_service_1.MSSql.BulkCSVSummaryInteractions(csvData, vcc, filters.dateIni, filters.dateFin, filters.campaigns.toString(), filters.agents.toString(), filters.origins.toString(), filters.hourIni, filters.hourFin, filters.minDuration, filters.maxDuration, result_numbers, isSummary, filters.interactionIds.toString(), filters.channelsWithLimit && filters.channelsWithLimit.length > 0 ? filters.channelsWithLimit[0].value : 0));
            break;
    }
    return promiseArray;
}
exports.GetChannelNewInteractionSummaryPromises = GetChannelNewInteractionSummaryPromises;
function GetChannelOldInteractionSummaryPromises(vcc, vccType, modelSecondsPerRequest, modelCharsPerPhrase, filters, csvData, retranscription) {
    return promises_shared_1.DummyPromise()
        .then(result => {
        let promiseArray = [];
        promiseArray = GetChannelNewInteractionSummaryPromises(vcc, vccType, false, 'DETAILED', filters, csvData);
        return Promise.all(promiseArray);
    })
        .then(result => {
        let promiseArray = [];
        result.filter(r => r).forEach(r => {
            if (r.totalInteractions)
                promiseArray.push(mssql_service_1.MSSql.BulkBillingInteractions(vcc, modelSecondsPerRequest, modelCharsPerPhrase, r.interactionType, r.interactions, retranscription));
        });
        return Promise.all(promiseArray);
    })
        .catch(err => {
        logger.info('[InteractionService::GetChannelOldInteractionSummaryPromises] Error: ', err);
        return Promise.resolve([]);
    });
}
exports.GetChannelOldInteractionSummaryPromises = GetChannelOldInteractionSummaryPromises;
function GetI6InteractionsFromApi(vcc, channel, filters) {
    let defaultRowsNumber = 500;
    let totalInteractions = 0;
    let apiInteractions = [];
    let bdInteractions = [];
    let formattedInteractions = [];
    let interactionsResult = {};
    let filtersAgent = filters.agents;
    let limitByChannel = filters.channelsWithLimit.length > 0 ? filters.channelsWithLimit.find(cl => cl.name === channel).value : 0;
    if (!httpRequestOccCompatible)
        return Promise.resolve(null);
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return http_request_occ_1.ApiI6GetInteractions(vcc, channel, filters, defaultRowsNumber);
    })
        .then(result => {
        if (!result.status)
            throw 'SERVER_ERROR_GET_I6_INTERACTIONS_FAILED';
        totalInteractions = result.body.total;
        apiInteractions = result.body.interactions;
        if (defaultRowsNumber >= totalInteractions) {
            return Promise.resolve([]);
        }
        else {
            let promiseArray = [];
            let totalPages = Math.ceil(totalInteractions / defaultRowsNumber);
            for (let page = 2; page <= totalPages; page++)
                promiseArray.push(http_request_occ_1.ApiI6GetInteractions(vcc, channel, filters, defaultRowsNumber, page));
            return Promise.all(promiseArray);
        }
    })
        .then(results => {
        if (results.filter(r => !r.status).length)
            throw 'SERVER_ERROR_AT_LEAST_ONE_OF_GET_I6_INTERACTIONS_REQUEST_FAILED';
        results.forEach(r => apiInteractions = apiInteractions.concat(r.body.interactions));
        formattedInteractions = apiInteractions.map(i => {
            let formattedInteraction = {};
            let campaign = i.campaign.includes('@') ? i.campaign.split('@')[0] : i.campaign;
            let agent = i.agent ? (i.agent.includes('@') ? i.agent.split('@')[0] : i.agent) : null;
            let durationTime = i.total_records_time_sec ?
                i.total_records_time_sec
                :
                    moment(i.state_timestamp).diff(moment(i.start_timestamp), 'seconds');
            if (channel === 'CALL')
                formattedInteraction = {
                    vcc,
                    ID: `${i.id}_0_1`,
                    InteractionId: i.id,
                    Section: 0,
                    Part: 1,
                    Channel: channel,
                    Number: null,
                    ContactAddress: null,
                    Gender: null,
                    SilenceTime: 0,
                    DurationTime: durationTime,
                    AttentionTime: durationTime,
                    TotalParts: 1,
                    Category1: null,
                    Category2: null,
                    Category3: null,
                    Description: i.disposition_code ? i.disposition_code : null,
                    DispositionCode: null,
                    Campaign: campaign,
                    Agent: agent,
                    agentName: null,
                    Type: null,
                    StartDate: i.start_timestamp,
                    EndDate: i.state_timestamp,
                    Format: null,
                    needI6Update: true
                };
            else
                formattedInteraction = {
                    vcc,
                    id: i.id,
                    InteractionId: i.id,
                    folder: moment.utc(i.state_timestamp).format('YYYY-MM-DD'),
                    exportedContent: `data/${i.id}.html`,
                    channel,
                    campaign: campaign,
                    startDate: i.start_timestamp,
                    endDate: i.state_timestamp,
                    account: null,
                    initiative: null,
                    contactName: null,
                    contactAddress: null,
                    contactId: null,
                    firstTakedAgent: null,
                    lastTakedAgent: agent,
                    totalMessages: null,
                    durationTime: durationTime,
                    attentionTime: durationTime,
                    dispositionCode: i.disposition_code ? i.disposition_code : null,
                    dispositionTreePath: null,
                    rawChar: i.total_characters,
                    needI6Update: true
                };
            return formattedInteraction;
        });
        return Promise.resolve(true);
    })
        .then(_ => {
        if (limitByChannel > 0) {
            let distinctAgents = [];
            if (filtersAgent.length >= 0) {
                if (channel === 'CALL') {
                    distinctAgents = arrays_shared_1.GetUniqueFromObjectArray(formattedInteractions, 'Agent');
                    filtersAgent = distinctAgents.map(a => a.Agent);
                }
                else {
                    distinctAgents = arrays_shared_1.GetUniqueFromObjectArray(formattedInteractions, 'lastTakedAgent');
                    filtersAgent = distinctAgents.map(a => a.lastTakedAgent);
                }
            }
            let formattedInteractionWithLimit = [];
            filtersAgent.forEach(fa => {
                let agentx = [];
                if (channel === 'CALL') {
                    agentx = formattedInteractions.filter(fi => fi.Agent === fa).slice(0, limitByChannel);
                }
                else {
                    agentx = formattedInteractions.filter(fi => fi.lastTakedAgent === fa).slice(0, limitByChannel);
                }
                if (agentx.length > 0)
                    formattedInteractionWithLimit = formattedInteractionWithLimit.concat(agentx);
            });
            formattedInteractions = [];
            formattedInteractions = formattedInteractionWithLimit;
        }
    })
        .then(_ => {
        if (channel === 'CALL')
            return mssql_service_1.MSSql.GetI6AlreadyRegisteredCallInteractions(vcc, formattedInteractions);
        else
            return mssql_service_1.MSSql.GetI6AlreadyRegisteredTextInteractions(vcc, formattedInteractions);
    })
        .then(result => {
        if (!filters.interactionIds.length) {
            bdInteractions = result;
            formattedInteractions = formattedInteractions.filter(fi => !bdInteractions.map(bdi => bdi.InteractionId).includes(fi.InteractionId));
            formattedInteractions = formattedInteractions.concat(bdInteractions);
        }
        logger.info(`[InteractionService::GetI6InteractionsFromApi] Getting ${bdInteractions.length} already registered interactions for channel ${channel}`);
        logger.info(`[InteractionService::GetI6InteractionsFromApi] Getting ${formattedInteractions.length} total interactions for channel ${channel}`);
        return Promise.resolve(true);
    })
        .then(_ => {
        interactionsResult.interactionType = channel;
        interactionsResult.totalInteractions = arrays_shared_1.GetUniqueFromObjectArray(formattedInteractions, 'InteractionId').length;
        interactionsResult.totalInteractionDetails = formattedInteractions.length;
        interactionsResult.totalInteractionTime = formattedInteractions.reduce((acc, current) => { return acc + (current.DurationTime ? current.DurationTime : 0); }, 0);
        interactionsResult.interactions = formattedInteractions;
        interactionsResult.filtersAgent = filtersAgent;
        return redis_service_1.Redis.SaveI6Interactions(vcc, filters.storageId, channel, interactionsResult);
    })
        .then(_ => {
        delete interactionsResult.interactions;
        return Promise.resolve(interactionsResult);
    })
        .catch(err => {
        logger.info('[InteractionService::GetI6InteractionsFromApi] Getting error:', err, `for channel ${channel}`);
        return Promise.resolve(null);
    });
}
exports.GetI6InteractionsFromApi = GetI6InteractionsFromApi;
function GetI6InteractionsFromRedis(vcc, storageId, channel) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return redis_service_1.Redis.GetI6Interactions(vcc, storageId, channel);
    })
        .then(result => {
        return Promise.resolve(result);
    })
        .catch(err => {
        logger.info('[InteractionService::GetI6InteractionsFromRedis] Getting error:', err, `for vcc::channel::storageId => "${vcc}::${channel}::${storageId}"`);
        return Promise.resolve([]);
    });
}
exports.GetI6InteractionsFromRedis = GetI6InteractionsFromRedis;
function UpdateI6Interaction(interaction, processId) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (interaction.interactionType === 'CALL')
            return UpdateI6CallInteraction(interaction, processId);
        else
            return UpdateI6TextInteraction(interaction, processId);
    })
        .then(_ => {
        return Promise.resolve(interaction);
    })
        .catch(err => {
        return Promise.reject(err);
    });
}
exports.UpdateI6Interaction = UpdateI6Interaction;
function UpdateI6CallInteraction(interaction, processId) {
    let i6Data = null;
    let i6Parts = [];
    let i6Audios = [];
    let bdAudios = [];
    let classifiedData = null;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        let promiseArray = [];
        promiseArray.push(http_request_occ_1.ApiI6GetInteractionData(interaction.vcc, interaction.id));
        promiseArray.push(http_request_occ_1.ApiI6GetInteractionParts(interaction.vcc, interaction.id));
        return Promise.all(promiseArray);
    })
        .then(result => {
        if (!result || (result?.length && !result[0].status))
            throw 'SERVER_ERROR_I6_INTERACTION_DATA_NOT_FOUND';
        if (!result || (result?.length > 1 && !result[1].status))
            throw 'SERVER_ERROR_I6_INTERACTION_PARTS_NOT_FOUND';
        i6Data = result[0].body;
        i6Parts = result[1].body;
        return mssql_service_1.MSSql.GetI6InteractionAudios(interaction.vcc, interaction.id);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_BD_INTERACTION_DATA_NOT_FOUND';
        let promiseArray = [];
        bdAudios = result;
        i6Parts = i6Parts.filter(r => r.format !== 'mp4');
        i6Parts.forEach(i6p => i6p.section_part = `${i6p.section}_${i6p.part}`);
        i6Parts = arrays_shared_1.SortObjectArrayByValue(i6Parts, 'section_part', 'A');
        i6Audios = i6Parts;
        bdAudios.forEach(bda => bda.section_part = `${bda.section}_${bda.part}`);
        classifiedData = arrays_shared_1.GetCompareItemsFromObjectArray(bdAudios, i6Audios, 'section_part', 'section_part');
        classifiedData.old.forEach(o => {
            let i6Audio = i6Audios.find(r => r.section === o.section && r.part === o.part);
            if (!i6Audio)
                i6Audio = i6Audios.shift();
            let idAgent = i6Audio?.agent?.includes('@') ? i6Audio.agent.split('@')[0] : (i6Audio.agent || null);
            if (!idAgent)
                idAgent = i6Data?.__last_step?.agent?.includes('@') ? i6Data.__last_step.agent.split('@')[0] : (i6Data.__last_step.agent || null);
            if (!idAgent)
                idAgent = i6Data?.interactionData?.enderAgent?.includes('@') ? i6Data.interactionData.enderAgent.split('@')[0] : (i6Data.interactionData.enderAgent || '');
            promiseArray.push(mssql_service_1.MSSql.SaveI6InteractionAudio(interaction.vcc, interaction.id, o.section, o.part, i6Audio?.section || 0, i6Audio?.part, i6Data.startTimestampReadable ? moment.utc(i6Data.startTimestampReadable).toDate() : null, i6Data.timestamp ? moment.utc(i6Data.timestamp).toDate() : null, moment(i6Data.timestamp).diff(moment(i6Data.startTimestampReadable), 'seconds'), idAgent, i6Audio?.format || null, i6Data?.campaign?.includes('@') ? i6Data.campaign.split('@')[0] : i6Data.campaign, i6Data?.initiative === 'INBOUND' ? 'E' : 'S', null, null, null, i6Data?.dispositionCode ? i6Data.dispositionCode : null, `${interaction.id}_${i6Audio?.section || 0}_${i6Audio?.part}.${i6Audio.format}`, i6Data?.contact?.includes('@') ? i6Data.contact.includes('@')[0] : i6Data.contact, i6Data?.interactionData?.contactName ? i6Data.interactionData.contactName : null, i6Data?.contactAddress ? i6Data.contactAddress : null, processId || null, i6Data?.interactionData?.ender || null, i6Data?.interactionData?.ticket || null));
            if (interaction?.section === o.section
                &&
                    interaction?.part === o.part) {
                interaction.agent = i6Audio?.agent.includes('@') ? i6Audio.agent.split('@')[0] : null;
                interaction.format = i6Audio.format || null;
            }
        });
        classifiedData.new.forEach(n => {
            let currentAudio = classifiedData.removed.shift();
            let idAgent = n?.agent.includes('@') ? n.agent.split('@')[0] : (n.agent || null);
            if (!idAgent)
                idAgent = i6Data?.__last_step?.agent?.includes('@') ? i6Data.__last_step.agent.split('@')[0] : (i6Data.__last_step.agent || null);
            if (!idAgent)
                idAgent = i6Data?.interactionData?.enderAgent?.includes('@') ? i6Data.interactionData.enderAgent.split('@')[0] : (i6Data.interactionData.enderAgent || '');
            promiseArray.push(mssql_service_1.MSSql.SaveI6InteractionAudio(interaction.vcc, interaction.id, currentAudio ? currentAudio.section : null, currentAudio ? currentAudio.part : null, n.section, n.part, i6Data.startTimestampReadable ? moment.utc(i6Data.startTimestampReadable).toDate() : null, i6Data.timestamp ? moment.utc(i6Data.timestamp).toDate() : null, moment(i6Data.timestamp).diff(moment(i6Data.startTimestampReadable), 'seconds'), idAgent, n.format || null, i6Data?.campaign?.includes('@') ? i6Data.campaign.split('@')[0] : i6Data.campaign, i6Data?.initiative === 'INBOUND' ? 'E' : 'S', null, null, null, i6Data?.dispositionCode ? i6Data.dispositionCode : null, `${interaction.id}_${n.section}_${n.part}.${n.format}`, i6Data?.contact?.includes('@') ? i6Data.contact.includes('@')[0] : i6Data.contact, i6Data?.interactionData?.contactName ? i6Data.interactionData.contactName : null, i6Data?.contactAddress ? i6Data.contactAddress : null, processId || null, i6Data?.interactionData?.ender || null, i6Data?.interactionData?.ticket || null));
            if (currentAudio
                &&
                    interaction?.section === currentAudio.section
                &&
                    interaction?.part === currentAudio.part) {
                interaction.agent = n?.agent.includes('@') ? n.agent.split('@')[0] : null;
                interaction.format = n.format || null;
            }
        });
        return Promise.all(promiseArray);
    })
        .then(_ => {
        let promiseArray = [];
        classifiedData.removed.forEach(r => {
            logger.info('[InteractionService::UpdateI6CallInteraction] Removing:', r);
            let bdAudio = bdAudios.find(a => a.section === r.section && a.part === r.part);
            if (bdAudio) {
                promiseArray.push(mssql_service_1.MSSql.DeleteI6InteractionAudio(interaction.vcc, interaction.id, bdAudio.section, bdAudio.part));
            }
        });
        return Promise.all(promiseArray);
    })
        .then(_ => {
        return Promise.resolve(interaction);
    })
        .catch(err => {
        logger.error('[InteractionService::UpdateI6CallInteraction] Getting error:', err);
        return Promise.reject(err);
    });
}
function UpdateI6TextInteraction(interaction, processId) {
    let i6Data = null;
    let i6Messages = [];
    let i6Texts = [];
    let bdTexts = [];
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return http_request_occ_1.ApiI6GetInteractionData(interaction.vcc, interaction.id);
    })
        .then(result => {
        if (!result || !result.status)
            throw 'SERVER_ERROR_I6_INTERACTION_DATA_NOT_FOUND';
        i6Data = result.body;
        i6Texts = GetI6InteractionMessages(interaction, i6Data);
        return mssql_service_1.MSSql.GetI6InteractionMessages(interaction.vcc, interaction.id);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_BD_INTERACTION_DATA_NOT_FOUND';
        let promiseArray = [];
        bdTexts = result;
        let classifiedData = arrays_shared_1.GetCompareItemsFromObjectArray(bdTexts, i6Texts, 'folder', 'folder');
        let idCampaign = i6Data?.campaign?.includes('@') ? i6Data.campaign.split('@')[0] : i6Data.campaign;
        interaction.campaign = idCampaign;
        classifiedData.old.forEach(o => {
            let i6Text = i6Texts.find(r => r.folder === o.folder);
            if (i6Text) {
                if (!interaction.folder)
                    interaction.folder = i6Text.folder;
                let idAgent = i6Text?.agent?.includes('@') ? i6Text.agent.split('@')[0] : (i6Text.agent || null);
                if (!idAgent)
                    idAgent = i6Data?.__last_step?.agent?.includes('@') ? i6Data.__last_step.agent.split('@')[0] : (i6Data.__last_step.agent || null);
                if (!idAgent)
                    idAgent = i6Data?.interactionData?.enderAgent?.includes('@') ? i6Data.interactionData.enderAgent.split('@')[0] : (i6Data.interactionData.enderAgent || '');
                promiseArray.push(mssql_service_1.MSSql.SaveI6InteractionAsyncInteraction(interaction.vcc, interaction.id, interaction.interactionType, o.folder, i6Text.startDate, i6Text.endDate, moment(i6Data.timestamp).diff(moment(i6Data.startTimestampReadable), 'seconds'), idAgent, interaction.campaign, i6Data?.account?.includes('@') ? i6Data.account.split('@')[0] : i6Data.account, i6Data?.initiative === 'INBOUND' ? 'IN' : 'OUT', null, i6Data?.dispositionCode ? i6Data.dispositionCode : null, i6Text?.exportedContent ? i6Text.exportedContent : null, i6Data?.contact?.includes('@') ? i6Data.contact.includes('@')[0] : i6Data.contact, i6Data?.interactionData?.contactName ? i6Data.interactionData.contactName : null, i6Data?.contactAddress ? i6Data.contactAddress : null, i6Text.totalMessages, processId || null, i6Data?.interactionData?.ender || null, i6Data?.interactionData?.ticket || null));
            }
        });
        classifiedData.new.forEach(n => {
            let idAgent = n?.agent?.includes('@') ? n.agent.split('@')[0] : (n.agent || null);
            if (!idAgent)
                idAgent = i6Data?.__last_step?.agent?.includes('@') ? i6Data.__last_step.agent.split('@')[0] : (i6Data.__last_step.agent || null);
            if (!idAgent)
                idAgent = i6Data?.interactionData?.enderAgent?.includes('@') ? i6Data.interactionData.enderAgent.split('@')[0] : (i6Data.interactionData.enderAgent || '');
            promiseArray.push(mssql_service_1.MSSql.SaveI6InteractionAsyncInteraction(interaction.vcc, interaction.id, interaction.interactionType, n.folder, n.startDate, n.endDate, moment(i6Data.timestamp).diff(moment(i6Data.startTimestampReadable), 'seconds'), idAgent, interaction.campaign, i6Data?.account?.includes('@') ? i6Data.account.split('@')[0] : i6Data.account, i6Data?.initiative === 'INBOUND' ? 'IN' : 'OUT', null, i6Data?.dispositionCode ? i6Data.dispositionCode : null, n?.exportedContent ? n.exportedContent : null, i6Data?.contact?.includes('@') ? i6Data.contact.includes('@')[0] : i6Data.contact, i6Data?.interactionData?.contactName ? i6Data.interactionData.contactName : null, i6Data?.contactAddress ? i6Data.contactAddress : null, n.totalMessages, processId || null, i6Data?.interactionData?.ender || null, i6Data?.interactionData?.ticket || null));
        });
        return Promise.all(promiseArray);
    })
        .then(_ => {
        if (processId) {
            return GetJsonDataFromI6Data(interaction, i6Data);
        }
        return Promise.resolve(true);
    })
        .then(_ => {
        if (processId)
            return CreateLocalInteractionFileContent(interaction);
        return Promise.resolve(true);
    })
        .then(_ => {
        return Promise.resolve(interaction);
    })
        .catch(err => {
        logger.error('[InteractionService::UpdateI6TextInteraction] Getting errorMessage:', err);
        return Promise.reject(err);
    });
}
function GetI6InteractionMessages(interaction, i6Data) {
    let messages = [];
    let texts = [];
    switch (interaction.interactionType) {
        case 'CHAT':
            messages = [{
                    id: interaction.id,
                    startDate: i6Data.startTimestampReadable ? moment.utc(i6Data.startTimestampReadable).toDate() : null,
                    endDate: i6Data.timestamp ? moment.utc(i6Data.timestamp).toDate() : null,
                    folder: moment.utc(i6Data.timestamp).format('YYYY-MM-DD'),
                    totalMessages: i6Data?.interactionData?.chatMessagesDetail?.length ? i6Data.interactionData.chatMessagesDetail.length : 1,
                    exportedContent: interaction.id
                }];
            texts = messages;
            break;
        default:
            messages = i6Data.analysisTimes?.openLapses?.length ? i6Data.analysisTimes.openLapses.map(ol => {
                let folder = moment.unix(ol.end).utc().format('YYYY-MM-DD');
                return {
                    id: interaction.id,
                    startDate: moment.unix(ol.start).toDate(),
                    endDate: moment.unix(ol.end).toDate(),
                    folder: folder,
                    totalMessages: i6Data.events.filter(e => moment.utc(e.timestamp).isSameOrBefore(moment.utc(`${folder} 23:59:59`))).length,
                    unixStart: ol.start,
                    unixEnd: ol.end
                };
            }) : [];
            let lastMessage = messages.length > 1 ? messages.pop() : messages[0];
            texts = [{
                    id: interaction.id,
                    startDate: messages[0].startDate,
                    endDate: lastMessage.endDate,
                    folder: lastMessage.folder,
                    totalMessages: lastMessage.totalMessages,
                    exportedContent: i6Data?.events?.length ? i6Data.events.pop()?.id : null
                }];
            break;
    }
    return texts;
}
function GetJsonDataFromI6Data(interaction, i6Data) {
    let audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
    let fileNamePath = audioTransfer.GetFileNamePathFromInteraction(interaction, 'json');
    let messages = [];
    let format = '2.1';
    let results = [];
    let createdDate = moment.utc().format('YYYY-MM-DD HH:mm:ss');
    let metadata = {
        type: 'transcription',
        occ_data: {
            messages: [],
            interactionId: interaction.id
        },
        transcription_config: {
            diarization: 'channel',
            channel_diarization_labels: ['Agent', 'Caller'],
            language: interaction.language
        },
        created_at: createdDate
    };
    let job = {
        created_at: createdDate
    };
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (!i6Data)
            return http_request_occ_1.ApiI6GetInteractionData(interaction.vcc, interaction.id);
        return Promise.resolve(i6Data);
    })
        .then(result => {
        i6Data ?? (i6Data = result.body);
        switch (interaction.interactionType) {
            case 'CHAT': return http_request_occ_1.ApiI6GetAsyncInteractionContent(interaction.vcc, interaction.interactionType, interaction.id);
            default: return Promise.resolve(i6Data);
        }
    })
        .then(result => {
        if (!i6Data && (!result || !result.status))
            throw 'SERVER_ERROR_I6_DATA_NOT_FOUND';
        logger.info(`[InteractionService::GetJsonDataFromI6Data] Getting messages from ${interaction.interactionType} interaction`);
        messages = GetTextMessagesFromI6Data(interaction, result, i6Data);
        return Promise.resolve(true);
    })
        .then(_ => {
        if (!messages.length)
            throw 'SERVER_ERROR_MESAGGES_NOT_FOUND';
        let firstMessageStartDate = null;
        messages.forEach((m, mi) => {
            let startDate = moment.unix(m.msgTime);
            if (!firstMessageStartDate)
                firstMessageStartDate = startDate.set('milliseconds', 0);
            metadata.occ_data.messages.push(m);
            let tokenizedText = m.fullMsg.split(/(.+?)([\s,.!?;:)([\]]+)/);
            tokenizedText.forEach(tt => tt = tt.trim());
            tokenizedText = tokenizedText.filter(tt => tt.trim().length);
            tokenizedText.forEach(tt => {
                let wordType = 'word';
                if (tt.length === 1 && tt.match(/[`!@#$%^&*()_+\-=\[\]{};':"\\|,.<>\/?~]/))
                    wordType = 'punctuation';
                results.push({
                    end_time: startDate.diff(firstMessageStartDate, 'milliseconds') / 1000,
                    channel: m.channel === 'Agent' ? 'Agent' : 'Caller',
                    occ_data: {
                        msgTime: m.msgTime,
                        msgDate: startDate.utc().format('YYYY-MM-DD HH:mm:ss.000'),
                        word_number: 0,
                        msgNumber: mi + 1
                    },
                    start_time: startDate.diff(firstMessageStartDate, 'milliseconds') / 1000,
                    type: wordType,
                    alternatives: [{
                            confidence: 1,
                            content: tt,
                            language: interaction.language
                        }]
                });
            });
            results.forEach((r, i) => r.occ_data.word_number = (i + 1));
        });
        return Promise.resolve(true);
    })
        .then(_ => {
        let objectResult = {
            format,
            metadata,
            results,
            job
        };
        return file_manager_1.WriteFileContent(fileNamePath, JSON.stringify(objectResult), 'utf-8', false);
    })
        .then(_ => {
        interaction.fileName = fileNamePath.split('/').pop();
        return Promise.resolve(true);
    })
        .catch(err => {
        logger.error('[InteractionService::GetJsonDataFromI6Data] Getting errorMessage:', err);
        return Promise.reject(err);
    });
}
exports.GetJsonDataFromI6Data = GetJsonDataFromI6Data;
function GetHtmlDataFromI6Data(interaction, i6Data) {
    let messages = null;
    let audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
    let jsonFileNamePath = audioTransfer.GetFileNamePathFromInteraction(interaction, 'json');
    let viewFileNamePath = audioTransfer.GetFileNamePathFromInteraction(interaction, 'view');
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (!i6Data)
            return http_request_occ_1.ApiI6GetInteractionData(interaction.vcc, interaction.id);
        else
            return Promise.resolve(i6Data);
    })
        .then(result => {
        i6Data ?? (i6Data = result.body);
        return file_manager_1.ReadFileContent(jsonFileNamePath, 'utf-8', true);
    })
        .then(result => {
        let jsonResult = null;
        try {
            jsonResult = JSON.parse(result);
        }
        catch (err) {
            logger.error('[InteractionService::GetHtmlDataFromI6Data] Error: ', err);
        }
        return Promise.resolve(jsonResult);
    })
        .then(result => {
        if (result?.metadata?.occ_data?.messages) {
            messages = result.metadata.occ_data.messages;
            return Promise.resolve(null);
        }
        else {
            switch (interaction.interactionType) {
                case 'CHAT': return http_request_occ_1.ApiI6GetAsyncInteractionContent(interaction.vcc, interaction.interactionType, interaction.id);
                default: return Promise.resolve(i6Data);
            }
        }
    })
        .then(result => {
        if (!messages) {
            messages = GetTextMessagesFromI6Data(interaction, result, i6Data);
        }
        return Promise.resolve(messages);
    })
        .then(result => {
        let htmlContent = '<!DOCTYPE html><html><head><meta charset="utf-8">';
        htmlContent += '<link rel="stylesheet" type="text/css" href="../resources/webchat.css"></link>';
        htmlContent += '</head><body><table border="0" cellspacing="0" cellpadding="0">';
        result.forEach(m => {
            htmlContent += `<tr class="${m.channel === 'Agent' ? 'participantTwo' : 'participantOne'}">`;
            htmlContent += `<td>${moment.unix(m.msgTime).utc().format('YYYY-MM-DD HH:mm:ss')}</td>`;
            htmlContent += `<td><span>${m.channel}:</span><span>${m.fullMsg}</span></td></tr>`;
        });
        htmlContent += '</table></body></html>';
        return file_manager_1.WriteFileContent(viewFileNamePath, htmlContent, 'utf-8', false);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch(err => {
        logger.error('[InteractionService::GetHtmlDataFromI6Data] Getting errorMessage:', err);
        return Promise.reject(err);
    });
}
exports.GetHtmlDataFromI6Data = GetHtmlDataFromI6Data;
function GetEmlDataFromI6Data(interaction, i6Data) {
    let audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (!i6Data)
            return http_request_occ_1.ApiI6GetInteractionData(interaction.vcc, interaction.id);
        else
            return Promise.resolve(i6Data);
    })
        .then(result => {
        if (!i6Data)
            i6Data = result.body;
        let promiseArray = [];
        i6Data.events.forEach(e => promiseArray.push(http_request_occ_1.ApiI6GetAsyncInteractionContent(interaction.vcc, interaction.interactionType, e.id)));
        return Promise.all(promiseArray);
    })
        .then(result => {
        let promiseArray = [];
        result.forEach((e, i) => {
            let viewFileNamePath = audioTransfer.GetFileNamePathFromInteraction(interaction, 'view', (i + 1));
            logger.debug('[InteractionService::GetEmlDataFromI6Data] Saving:', viewFileNamePath);
            if (e.status)
                promiseArray.push(file_manager_1.WriteFileContent(viewFileNamePath, e.body, 'utf-8', false));
            else if (i6Data.events[i].body?.length) {
                let event = i6Data.events[i];
                let toList = Object.keys(event.to);
                let toObject = toList?.length ?
                    { name: event.to[toList[0]], to: toList[0] }
                    :
                        { name: 'Unknown', to: 'Unknown' };
                let messageList = event.body;
                let messageObject = messageList.find(ml => ml?.type === 'text/html') ? messageList.find(ml => ml.type === 'text/html') : messageList[0];
                let mailContent = '';
                mailContent += 'Subject: ' + event.subject + '\n';
                mailContent += 'From: "' + event.name + '" <' + event.from + '>' + '\n';
                mailContent += 'To: "' + toObject.name + '" <' + toObject.to + '>' + '\n';
                mailContent += 'Date: ' + event.date + '\n';
                mailContent += 'Content-Type: text/html; charset=utf-8\n';
                mailContent += 'Content-Transfer-Encoding: quoted-printable\n';
                mailContent += 'Message-Id: <' + event.id + '>\n';
                mailContent += '\n';
                mailContent += messageObject.content + '\n';
                promiseArray.push(file_manager_1.WriteFileContent(viewFileNamePath, mailContent, 'utf-8', false));
            }
            else
                promiseArray.push(file_manager_1.WriteFileContent(viewFileNamePath, 'SERVER_ERROR_NO_CONTENT_FOUND', 'utf-8', false));
        });
        return Promise.all(promiseArray);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch(err => {
        logger.error('[InteractionService::GetEmlDataFromI6Data] Getting errorMessage:', err);
        return Promise.reject(err);
    });
}
exports.GetEmlDataFromI6Data = GetEmlDataFromI6Data;
function GetTextMessagesFromI6Data(interaction, i6Response, i6Data) {
    i6Data ?? (i6Data = i6Response);
    let messages = [];
    switch (interaction.interactionType) {
        case 'CHAT':
            let agentHistory = arrays_shared_1.GetUniqueFromArray(Object.keys(i6Data.agentHistory)).map(ah => ah.split('@')[0]);
            i6Response.body.messages.forEach((m, i) => {
                messages.push({
                    channel: agentHistory.includes((m.user_id.split('@')[0])) ? 'Agent' : m.display_name,
                    fullMsg: m.text,
                    msgTime: m.msgtime,
                    msgDate: moment.unix(m.msgtime).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                    msgNumber: (i + 1)
                });
            });
            break;
        case 'MAIL':
            i6Data.events.forEach((e, i) => {
                let message = e.body.find(b => b.type === 'text/plain');
                if (message?.content?.length) {
                    messages.push({
                        channel: e.creator === 'contact' ? 'Web Client' : 'Agent',
                        fullMsg: message.content,
                        msgTime: e.timestampDouble,
                        msgDate: moment.unix(e.timestampDouble).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                        msgNumber: (i + 1)
                    });
                }
            });
            messages.forEach(m => m.fullMsg = m.fullMsg.replace(/(?:\\[rn]|[\r\n]+)+/g, ' '));
            break;
        case 'FACEBOOK':
            i6Data.events.forEach((e, i) => {
                if (e.message?.message?.length || e.message?.text?.length) {
                    messages.push({
                        channel: e.creator === 'contact' ? 'Web Client' : 'Agent',
                        fullMsg: e.message.message || e.message.text,
                        msgTime: e.timestampDouble,
                        msgDate: moment.unix(e.timestampDouble).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                        msgNumber: (i + 1)
                    });
                }
                ;
            });
            break;
        case 'TWITTER':
            i6Data.events.forEach((e, i) => {
                if (e.text?.length) {
                    messages.push({
                        channel: e.creator === 'contact' ? 'Web Client' : 'Agent',
                        fullMsg: e.text,
                        msgTime: e.timestampDouble,
                        msgDate: moment.unix(e.timestampDouble).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                        msgNumber: (i + 1)
                    });
                }
                ;
            });
            break;
        case 'WEBCONTACT':
            if (i6Data.data?.reportingData?.contactForm?.fields?.length) {
                i6Data.data.reportingData.contactForm.fields.forEach((f, i) => {
                    if (f.displayName?.length) {
                        messages.push({
                            channel: 'Agent',
                            fullMsg: f.displayName,
                            msgTime: i6Data.timestampDouble,
                            msgDate: moment.unix(i6Data.timestampDouble).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                            msgNumber: (i + 1)
                        });
                    }
                    if (f.value?.length) {
                        messages.push({
                            channel: 'Web Client',
                            fullMsg: f.value,
                            msgTime: i6Data.timestampDouble,
                            msgDate: moment.unix(i6Data.timestampDouble).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                            msgNumber: (i + 1)
                        });
                    }
                });
            }
            break;
        default:
            i6Data.events.forEach((e, i) => {
                if (e.message?.text?.length) {
                    messages.push({
                        channel: e.creator === 'contact' ? 'Web Client' : 'Agent',
                        fullMsg: e.message.text,
                        msgTime: e.timestampDouble,
                        msgDate: moment.unix(e.timestampDouble).utc().format('YYYY-MM-DD HH:mm:ss.000'),
                        msgNumber: (i + 1)
                    });
                }
                ;
            });
            break;
    }
    messages.forEach((m, i) => m.msgNumber = (i + 1));
    return messages;
}
function CloseI6CallInteraction(interaction) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.CloseI6InteractionAudio(interaction.vcc, interaction.id, false, interaction.errorMessage);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch(err => {
        return Promise.reject(err);
    });
}
exports.CloseI6CallInteraction = CloseI6CallInteraction;
function CloseI6TextInteraction(interaction) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.CloseI6InteractionAsyncInteraction(interaction.vcc, interaction.id, interaction.folder, interaction.interactionType, false, interaction.errorMessage);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch(err => {
        return Promise.reject(err);
    });
}
exports.CloseI6TextInteraction = CloseI6TextInteraction;
