"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveWord = exports.SaveNeuroLinguisticProgrammings = exports.SaveNeuroLinguisticProgramming = exports.GetNeuroLinguisticProgramming = exports.NeuroLinguisticProgrammingService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
let logger = log4js.getLogger('ServerScripts');
function NeuroLinguisticProgrammingService(app) {
    app.get('/api/neuro_linguistic_programming/get_neuro_linguistic_programmings/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetNeuroLinguisticProgrammings(vcc, search_criteria, search_expression);
        }).then(neuroLinguisticProgrammings => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: neuroLinguisticProgrammings });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/neuro_linguistic_programming/get_neuro_linguistic_programming/:neuro_linguistic_programming_id/:processId/:language?', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let neuroLinguisticProgrammingId = req.params.neuro_linguistic_programming_id;
        let processId = req.params.processId;
        let language = req.params.language;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!neuroLinguisticProgrammingId)
                throw 'NEURO_LINGUISTIC_PROGRAMING_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetNeuroLinguisticProgramming(vcc, neuroLinguisticProgrammingId, processId, language);
        }).then(resultNeuroLinguisticProgramming => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultNeuroLinguisticProgramming });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/neuro_linguistic_programming/get_neuro_linguistic_programming_words/:search_criteria/:search_expression/:processId/:language?', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let processId = req.params.processId;
        let language = req.params.language;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetWords(vcc, search_criteria, search_expression, processId, language);
        }).then(resultWords => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultWords });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/neuro_linguistic_programming/get_system_words/:search_criteria/:processId/:language?', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let processId = req.params.processId;
        let language = req.params.language;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetSystemWords(vcc, search_criteria, '', processId, language);
        }).then(resultSystemWords => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultSystemWords });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/neuro_linguistic_programming/save_neuro_linguistic_programming', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newNeuroLinguisticProgramming = (req.body || {});
        newNeuroLinguisticProgramming.id = (newNeuroLinguisticProgramming.id || '').toLowerCase();
        newNeuroLinguisticProgramming.createdDate = null;
        newNeuroLinguisticProgramming.createdByUserId = null;
        newNeuroLinguisticProgramming.lastModifiedDate = nowUtc;
        newNeuroLinguisticProgramming.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(resultDeleteNeuroLinguisticProgramming => {
            if (!resultDeleteNeuroLinguisticProgramming)
                throw 'NEURO_LINGUISTIC_PROGRAMING_SERVER_ERROR_ON_SAVE';
            return SaveNeuroLinguisticProgramming(vcc, newNeuroLinguisticProgramming);
        }).then(resultSaveNeuroLinguisticProgramming => {
            if (!resultSaveNeuroLinguisticProgramming)
                throw 'NEURO_LINGUISTIC_PROGRAMING_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(resultSaveNeuroLinguisticProgramming => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveNeuroLinguisticProgramming, description: (resultSaveNeuroLinguisticProgramming ? 'SERVER_SUCCESS_MESSAGE' : 'NEURO_LINGUISTIC_PROGRAMING_SERVER_ERROR_ON_SAVE'), data: resultSaveNeuroLinguisticProgramming });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.NeuroLinguisticProgrammingService = NeuroLinguisticProgrammingService;
function GetNeuroLinguisticProgramming(vcc, id) {
    id = (id || '').toLowerCase();
    let cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(() => {
        return mssql_service_1.MSSql.GetNeuroLinguisticProgramming(vcc, id);
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetNeuroLinguisticProgramming: ' + errMsg);
    });
}
exports.GetNeuroLinguisticProgramming = GetNeuroLinguisticProgramming;
function SaveNeuroLinguisticProgramming(vcc, neuroLinguisticProgramming) {
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveNeuroLinguisticProgramming(vcc, neuroLinguisticProgramming.name, neuroLinguisticProgramming.description, neuroLinguisticProgramming.createdDate, neuroLinguisticProgramming.createdByUserId, neuroLinguisticProgramming.lastModifiedDate, neuroLinguisticProgramming.lastModifiedByUserId));
    if (neuroLinguisticProgramming.words) {
        neuroLinguisticProgramming.words.forEach(word => {
            promiseArray.push(mssql_service_1.MSSql.SaveWord(vcc, word.neuroLinguisticProgrammingId, word.name, word.processId, word.language, word.type, (word.createdDate === null ? neuroLinguisticProgramming.lastModifiedDate : null), (word.createdByUserId === null ? neuroLinguisticProgramming.lastModifiedByUserId : null), neuroLinguisticProgramming.lastModifiedDate, neuroLinguisticProgramming.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveNeuroLinguisticProgramming = SaveNeuroLinguisticProgramming;
function SaveNeuroLinguisticProgrammings(vcc, neuroLinguisticProgramming) {
    let promiseArray = [];
    if (neuroLinguisticProgramming) {
        neuroLinguisticProgramming.forEach(neuroLinguisticProgrammings => {
            promiseArray.push(mssql_service_1.MSSql.SaveNeuroLinguisticProgramming(vcc, neuroLinguisticProgrammings.name, neuroLinguisticProgrammings.description, neuroLinguisticProgrammings.createdDate, neuroLinguisticProgrammings.createdByUserId, neuroLinguisticProgrammings.lastModifiedDate, neuroLinguisticProgrammings.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveNeuroLinguisticProgrammings = SaveNeuroLinguisticProgrammings;
function SaveWord(vcc, word) {
    let promiseArray = [];
    if (word) {
        word.forEach(words => {
            promiseArray.push(mssql_service_1.MSSql.SaveWord(vcc, words.neuroLinguisticProgrammingId, words.name, words.processId, words.language, words.type, words.createdDate, words.createdByUserId, words.lastModifiedDate, words.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveWord = SaveWord;
