"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JSONToCSV = exports.BeginStudy = exports.CreateStudy = exports.GetStudy = exports.StudyService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const redis_service_1 = require("../db/redis.service");
const billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
const billing_implementation_1 = require("../implementation/billing.implementation");
const billing_model_1 = require("../models/billing.model");
const platform_service_1 = require("./platform.service");
let logger = log4js.getLogger('ServerScripts');
function StudyService(app) {
    app.post('/api/study/begin_study', (req, res, next) => {
        logger.info('[StudyService::begin_study] Received ' + req.method + ' on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let studyData = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(_ => {
            return BeginStudy(vcc, studyData, userId);
        })
            .then((result) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (result.status ? 'SERVER_SUCCESS_MESSAGE' : result.saveResult ? result.saveResult : 'STUDY_START_FAILED'), data: result.status });
        })
            .catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[StudyService::begin_study] with param:', studyData, 'an error has occured in:', req.path, 'with error meesage:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/get_studies', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let study = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetStudies(vcc, study.searchCriteria, study.searchExpression, study.startDate, study.endDate, study.type);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/study/get_study_result_summary/:study_id/?', (req, res, next) => {
        logger.info('[StudyService::get_study_result_summary] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let studyId = req.params.study_id || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetStudyResultSummary(vcc, studyId, null);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/study/get_study/:study_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let studyId = req.params.study_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!studyId)
                throw 'STUDY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetStudy(vcc, studyId);
        }).then(resultStudy => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultStudy });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/create_study', (req, res, next) => {
        logger.info('[StudyService::create_study] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let vccType = res.locals.type;
        let userId = res.locals.userId;
        let newStudy = (req.body || {});
        newStudy.name = (newStudy.name || '').toLowerCase();
        promises_shared_1.DummyPromise()
            .then(_ => {
            return CreateStudy(vcc, vccType, newStudy, userId);
        })
            .then(resultSaveStudy => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveStudy ? 'SERVER_SUCCESS_MESSAGE' : 'STUDY_SERVER_ERROR_ON_SAVE'), data: resultSaveStudy });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[StudyService::create_study] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/save_study', (req, res, next) => {
        logger.info('[StudyService::save_study] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newStudy = (req.body || {});
        newStudy.name = (newStudy.name || '').toLowerCase();
        newStudy.createdDate = null;
        newStudy.createdByUserId = null;
        newStudy.lastModifiedDate = nowUtc;
        newStudy.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return SaveStudy(vcc, newStudy);
        }).then(resultSaveStudy => {
            if (!resultSaveStudy)
                throw 'STUDY_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(resultSaveStudy => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveStudy, description: (resultSaveStudy ? 'SERVER_SUCCESS_MESSAGE' : 'STUDY_SERVER_ERROR_ON_SAVE'), data: resultSaveStudy });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/delete_study', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let study = req.body.study;
        let studyId = study.name;
        logger.info(`[StudyService::delete_study] Deleting study ${studyId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!study)
                throw 'STUDY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteStudy(vcc, studyId);
        }).then(resultDeleteStudy => {
            if (!resultDeleteStudy)
                throw 'STUDY_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(resultDeleteStudy => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteStudy, description: (resultDeleteStudy ? 'SERVER_SUCCESS_MESSAGE' : 'STUDY_SERVER_ERROR_ON_DELETE'), data: resultDeleteStudy });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/delete_studies', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let studies = req.body.studies || [];
        logger.info(`[StudyService::delete_studies] Deleting studies ${studies.map(x => x.name).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!studies)
                throw 'STUDY_SERVER_ERROR_MISSING_IDS';
            let promiseArray = [];
            studies.forEach(study => {
                promiseArray.push(mssql_service_1.MSSql.DeleteStudy(vcc, study.name));
            });
            return Promise.all(promiseArray);
        }).then(resultDeleteStudies => {
            let result;
            if (resultDeleteStudies.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Estudios eliminados correctamente');
            }
            return result;
        }).then(resultDeleteStudies => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteStudies, description: (resultDeleteStudies ? 'SERVER_SUCCESS_MESSAGE' : 'STUDY_SERVER_ERROR_ON_DELETE'), data: resultDeleteStudies });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/public/study/get_study_report/:start_date/:end_date/:processes', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let startDate = req.params.start_date || '';
        let endDate = req.params.end_date || '';
        let processes = req.params.processes || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetStudyReport(startDate, endDate, processes);
        })
            .then((result) => {
            return Promise.resolve(JSONToCSV(result));
        }).then(result => {
            if (res.headersSent)
                return;
            let fileName = [startDate.replace('-', ''), endDate.replace('-', ''), processes.replace(',', '_')].join('_');
            res.setHeader('Content-disposition', 'attachment; filename=' + fileName + '.csv');
            res.set('Content-Type', 'text/csv');
            res.status(200).send(result);
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
}
exports.StudyService = StudyService;
function GetStudy(vcc, studyId) {
    studyId = (studyId || '').toLowerCase();
    let cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(() => {
        return mssql_service_1.MSSql.GetStudy(vcc, studyId);
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetStudy: ' + errMsg);
    });
}
exports.GetStudy = GetStudy;
function CreateStudy(vcc, vccType, newStudy, userId) {
    let nowUtc = moment().utc().toDate();
    newStudy.createdDate = nowUtc;
    newStudy.createdByUserId = userId;
    newStudy.lastModifiedDate = nowUtc;
    newStudy.lastModifiedByUserId = userId;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetStudyByName(vcc, newStudy.name);
    })
        .then(resultStudy => {
        if (resultStudy)
            throw 'SERVER_ERROR_STUDY_ALREADY_EXISTS';
        return platform_service_1.GetPlatformVersion();
    })
        .then(result => {
        if (result === null)
            throw 'SERVER_ERROR_INSTALLATIONID_MISSING';
        let batch = {};
        batch.installationId = result.installationId;
        batch.vcc = vcc;
        batch.batchTypeName = newStudy.type === 'SPEECH' ? 'STUDY' : 'TEMPLATE';
        batch.batchTypeId = newStudy.name;
        let billing = new billing_batch_implementation_1.BillingBatchImplementation();
        return billing.FindBatchId(batch);
    })
        .then(result => {
        if (result && result.result && result.result.length > 0)
            throw 'SERVER_ERROR_STUDY_ID_INVALID';
        return SaveStudy(vcc, newStudy);
    })
        .then((resultSaveStudy) => {
        if (!resultSaveStudy)
            throw 'SERVER_ERROR_SAVE_STUDY_FAILED';
        logger.info('[StudyService::CreateStudy] Finishing function');
        return Promise.resolve({ resultSaveStudy });
    })
        .catch(err => {
        logger.error('[StudyService::CreateStudy] Function failed');
        return Promise.reject(err);
    });
}
exports.CreateStudy = CreateStudy;
function BeginStudy(vcc, studyData, userId) {
    let currentQueueId = '';
    let study = {};
    let channelSummaries = [];
    let totalSyncInteraction = 0;
    let totalAudios = 0;
    let totalAsyncInteraction = 0;
    let totalAsyncDetail = 0;
    let averageModelRequest = 0;
    let nowUtc = moment().utc().toDate();
    let batch = {};
    let billingVccRates = new billing_model_1.BillingVCCRates();
    let status = false;
    let saveResult = null;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetStudy(vcc, studyData.studyId);
    })
        .then((result) => {
        if (!result)
            throw 'SERVER_ERROR_STUDY_NOT_FOUND';
        study = result;
        let billing = new billing_implementation_1.BillingImplementation();
        return billing.GetVCCRates(vcc, study.processId);
    })
        .then(result => {
        if (result === null)
            throw 'SERVER_ERROR_RATES_NOT_FOUND';
        billingVccRates.SetBillingVCCRates(result);
        let promiseArray = [];
        promiseArray.push(platform_service_1.GetPlatformVersion());
        promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItems(vcc, study.templateId, null));
        promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeModels(vcc, study.templateId, null));
        promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(vcc, study.processId, billingVccRates.cognitiveAnalysisAverageSecondsPerRequest));
        return Promise.all(promiseArray);
    })
        .then(result => {
        if (result[0] === null)
            throw 'INSTALLATIONID_MISSING';
        if (result[1] === null)
            throw 'QUERIES_INFO_MISSING';
        if (result[2] === null)
            throw 'MODELS_INFO_MISSING';
        if (result[3] === null)
            throw 'INTERACTIONS_INFO_MISSING';
        let modelIntents = result[2].map(mi => { return mi.id; });
        let models = modelIntents.map(i => i.split('|')[0]);
        let nluModels = models.filter((item, i, ar) => ar.indexOf(item) === i);
        channelSummaries = result[3];
        channelSummaries.map(cs => {
            if (cs) {
                if (cs.interactionType === 'CALL') {
                    totalSyncInteraction = cs.totalRecords;
                    totalAudios = cs.audios;
                    averageModelRequest += cs.totalCallPhrases || cs.averageModelRequest || 0;
                }
                else {
                    totalAsyncInteraction += cs.totalInteractions;
                    totalAsyncDetail += cs.totalInteractionDetails;
                    averageModelRequest += cs.totalTextPhrases || cs.averageModelRequest || 0;
                    ;
                }
            }
        });
        batch.installationId = result[0].installationId;
        batch.vcc = vcc;
        batch.totalAsync = 0;
        batch.totalAsyncDetail = 0;
        batch.totalSync = 0;
        batch.totalAudio = 0;
        batch.durationTime = 0;
        batch.roundedTime = 0;
        batch.totalQueries = result[1].length;
        batch.totalModels = nluModels.length;
        batch.modelRequests = billingVccRates.GetCognitiveAnalysisTotalAveragePhrasesFromProcess();
        batch.batchTypeName = study.type === 'SPEECH' ? 'STUDY' : 'TEMPLATE';
        batch.batchTypeId = study.name;
        batch.templateId = study.templateId;
        batch.processId = study.processId;
        batch.userApi = userId;
        batch.startDate = nowUtc;
        let billing = new billing_batch_implementation_1.BillingBatchImplementation();
        return billing.SaveBatchSummary(batch);
    })
        .then(result => {
        if (typeof result !== 'boolean') {
            if (result) {
                if (result.status === true) {
                    status = true;
                    study.status = 0.1;
                    let lastModifiedDate = moment().utc().toDate();
                    return mssql_service_1.MSSql.SaveStudyStatus(vcc, studyData.processId, study.templateId, studyData.studyId, study.status, null, null, lastModifiedDate, userId);
                }
                else {
                    status = false;
                    saveResult = result.error ? result.error : 'SERVER_ERROR_CAN_NOT_SAVE_BATCH';
                }
            }
            else {
                return Promise.resolve(false);
            }
        }
        else {
            return Promise.resolve(false);
        }
    })
        .then(_ => {
        if (status === true) {
            currentQueueId = vcc;
            currentQueueId += ':';
            currentQueueId += study.processId;
            currentQueueId += study.type === 'SPEECH' ? ':study:' : ':template:';
            currentQueueId += study.name;
            return redis_service_1.Redis.CreateQueueL('Process', currentQueueId);
        }
        return Promise.resolve(true);
    })
        .then(_ => {
        return Promise.resolve({ status, saveResult });
    })
        .catch(err => {
        logger.error('[StudyService::BeginStudy] Function failed');
        return Promise.reject(err);
    });
}
exports.BeginStudy = BeginStudy;
function SaveStudy(vcc, study) {
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveStudy(vcc, study.processId, study.templateId, study.name, study.description, study.status, study.type, study.batchScheduleId, study.createdDate, study.createdByUserId, study.lastModifiedDate, study.lastModifiedByUserId));
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return Promise.resolve(false);
        }
        else {
            return Promise.resolve(true);
        }
    });
}
function JSONToCSV(data) {
    let items = data, replacer = (key, value) => value === null ? '' : value, header = Object.keys(items[0]), csv = items.map(row => header.map(fieldName => JSON.stringify(row[fieldName], replacer)).join(','));
    csv.unshift(header.join(','));
    csv = csv.join('\r\n');
    return csv;
}
exports.JSONToCSV = JSONToCSV;
