"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TagService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
let logger = log4js.getLogger('ServerScripts');
function TagService(app) {
    app.get('/api/tag/get_tags/:audioId/:section/:part?', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let audioId = req.params.audioId || '';
        let section = Number(req.params.section);
        let part = Number(req.params.part);
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetTags(vcc, audioId, section, part);
        }).then(resultRoles => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultRoles });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: "ERROR", datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/tag/get_unique_tags', (req, res, next) => {
        logger.info('[TagService] Received ' + req.method + ' method on  \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[TagService] Requesting all unique tags from MSSql.');
            return mssql_service_1.MSSql.GetUniqueTags(vcc);
        }).then(resultGetUniqueTags => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultGetUniqueTags });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error on request');
            logger.error('[TagService] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: "ERROR", datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/tag/save_tag', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newTag = (req.body || {});
        newTag.createdDate = nowUtc;
        newTag.createdByUserId = userId;
        newTag.lastModifiedDate = nowUtc;
        newTag.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!newTag.tag)
                throw 'ERROR_TAG_NAME_MISSING';
            return mssql_service_1.MSSql.SaveTag(vcc, newTag.audioId, newTag.section, newTag.part, newTag.tag, newTag.currentTime, newTag.comment, newTag.createdDate, newTag.createdByUserId, newTag.lastModifiedDate, newTag.lastModifiedByUserId);
        }).then(resultSaveTag => {
            if (res.headersSent)
                return;
            logger.info('Comentario \'' + newTag.id + '\' guardado correctamente');
            res.json({ status: resultSaveTag, description: (resultSaveTag ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: resultSaveTag });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: "ERROR", datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/tag/delete_tag', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let newTag = (req.body || {});
        logger.info(`[TagService::delete_tag] Deleting tagAudioId ${newTag.audioId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.DeleteTag(vcc, newTag.audioId, newTag.section, newTag.part, newTag.tag);
        }).then(resultSaveTag => {
            if (res.headersSent)
                return;
            logger.info('Comentario \'' + newTag.id + '\' eliminado correctamente');
            res.json({ status: resultSaveTag, description: (resultSaveTag ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: resultSaveTag });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: "ERROR", datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/tag/get_all_tags?', (req, res, next) => {
        logger.info('[TagService::get_all_tags] Received ' + req.method + ' method on  \'' + req.path + '\'');
        let instance = res.locals.instance;
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[TagService::get_all_tags] Requesting all tags from MSSql SP GetAllTags');
            return mssql_service_1.MSSql.GetAllTags(instance);
        }).then(result => {
            if (res.headersSent)
                return;
            logger.info('[TagService::get_all_tags] Received resultro from MSSql');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error on request');
            logger.error('[TagService::get_all_tags] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TagService = TagService;
