"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TeamPerformanceService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const team_performance_model_1 = require("../models/team-performance.model");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
let logger = log4js.getLogger('ServerScripts');
function TeamPerformanceService(app) {
    app.get('/api/team_performance/get_filters/:batchType/:evaluationFormatId/:templateId', (req, res, next) => {
        logger.info(`[TeamPerformanceService::get_filters] Received ${req.method} method on '${req.path}'`);
        let vcc = res.locals.vcc;
        let batchType = req.params.batchType || '';
        let evaluationFormatId = req.params.evaluationFormatId || '';
        let templateId = req.params.templateId || '';
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetTeamPerformanceFilters(vcc, batchType, evaluationFormatId, templateId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`[TeamPerformanceService::get_filters] Error ${req.path}: '${errorMsg}'`);
            if (res.headersSent)
                return;
            res.json({
                status: false,
                description: clientMsg,
                error: errorMsg,
                detail: {
                    type: 'ERROR',
                    datetime: moment().utc().toDate(),
                    pid: process.pid,
                    source: __filename,
                    stack: err.stack
                }
            });
        });
    });
    app.post('/api/team_performance/get_team_performance_from_evaluation', (req, res, next) => {
        logger.info('[TeamPerformanceService::get_team_performance_from_evaluation] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let currentUserRole = res.locals.user.role;
        let currentUserId = res.locals.userId;
        let userFuntion = null;
        let params = new team_performance_model_1.TeamPerformanceFilters;
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.evaluationFormatId = req.body.evaluationFormatId || null;
        params.agents = req.body.agents || [];
        params.abilities = req.body.abilities || [];
        params.startDate = req.body.startDate;
        params.endDate = req.body.endDate || [];
        params.drilldownTarget = req.body.drilldownTarget || null;
        params.drilldownId = req.body.drilldownId || null;
        params.supervisors = req.body.supervisors || [];
        params.resultBySupervisor = req.body.resultBySupervisor || null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            logger.info('[TeamPerformanceService::get_team_performance_from_evaluation] Calling GetTeamPerformanceFromEvaluationPlan SP for VCC ' + vcc);
            return mssql_service_1.MSSql.GetTeamPerformanceFromEvaluationPlan(vcc, params.evaluationFormatId, params.evaluationPlans, params.agents, params.abilities, params.startDate, params.endDate, params.drilldownTarget, params.drilldownId, params.supervisors, params.resultBySupervisor);
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.info('[TeamPerformanceService::get_team_performance_from_evaluation] Received result from SP GetTeamPerformanceFromEvaluationPlan from local DB');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[TeamPerformanceService::get_team_performance_from_evaluation] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TeamPerformanceService = TeamPerformanceService;
