"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetTemplateItems = exports.SaveTemplate = exports.CreateTemplate = exports.GetTemplate = exports.TemplateService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const batch_schedule_service_1 = require("./batch-schedule.service");
let logger = log4js.getLogger('ServerScripts');
function TemplateService(app) {
    app.get('/api/template/get_templates/:type/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[TemplateService::get_templates] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let type = req.params.type || '';
        type = type === 'null' ? null : type;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let language = req.query.language ? req.query.language.toString() : 'en';
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetTemplates(vcc, type, search_criteria, search_expression, language);
        })
            .then(templates => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: templates });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TemplateService::get_templates] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/template/get_template/:type/:template_id', (req, res, next) => {
        logger.info('[TemplateService::get_template] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let templateId = req.params.template_id || '';
        let type = req.params.type || '';
        let vccTemp = type === 'system' ? 'system' : vcc;
        let template = null;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!templateId)
                throw 'TEMPLATE_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetTemplate(vccTemp, templateId);
        })
            .then(result => {
            if (result)
                return result;
            return mssql_service_1.MSSql.GetTemplate(vcc, templateId);
        })
            .then(resultTemplate => {
            template = resultTemplate;
            let promiseArray = [];
            if (template.templateThemes && template.templateThemes.length) {
                template.templateThemes.map(tt => {
                    promiseArray.push(GetTemplateItems(vcc, tt.id, true));
                });
            }
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (result && result.length) {
                template.templateThemes.map((tt, i) => {
                    tt.items = result[i];
                });
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: template });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/template/get_template_theme_items/:type/:template_id?', (req, res, next) => {
        logger.info('[TemplateService::get_template_theme_items] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let templateId = req.params.template_id || '';
        let type = req.params.type || '';
        vcc = type === 'system' ? 'system' : vcc;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetTemplateThemeItems(vcc, templateId, null);
        })
            .then(resultTemplateThemeItems => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultTemplateThemeItems });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/template/get_template_theme_models/:type/:template_id?', (req, res, next) => {
        logger.info('[TemplateService::get_template_theme_models] Received ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let templateId = req.params.template_id || '';
        let type = req.params.type || '';
        vcc = type === 'system' ? 'system' : vcc;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetTemplateThemeModels(vcc, templateId, null);
        }).then(resultTemplateThemeItems => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultTemplateThemeItems });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TemplateService::get_template_theme_models] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/template/create_template', (req, res, next) => {
        logger.info('[TemplateService::create_template] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let newTemplate = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (newTemplate.scheduleData) {
                return CreateScheduleForTemplate(vcc, newTemplate, userId);
            }
            else {
                return CreateTemplate(vcc, newTemplate, userId);
            }
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'TEMPLATE_SERVER_ERROR_ON_SAVE'), data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TemplateService::create_template] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/template/save_template', (req, res, next) => {
        logger.info('[TemplateService::save_template] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newTemplate = (req.body || {});
        newTemplate.createdDate = null;
        newTemplate.createdByUserId = null;
        newTemplate.lastModifiedDate = nowUtc;
        newTemplate.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.DeleteTemplateTheme(vcc, newTemplate.name);
        })
            .then(resultDeleteTemplate => {
            if (!resultDeleteTemplate)
                throw 'TEMPLATE_SERVER_ERROR_ON_SAVE';
            return SaveTemplate(vcc, newTemplate);
        })
            .then(resultSaveTemplate => {
            if (!resultSaveTemplate)
                throw 'TEMPLATE_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        })
            .then(resultSaveTemplate => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveTemplate, description: (resultSaveTemplate ? 'SERVER_SUCCESS_MESSAGE' : 'TEMPLATE_SERVER_ERROR_ON_SAVE'), data: resultSaveTemplate });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/template/create_template_by_api', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newTemplateByAPI = (req.body || {});
        logger.info('newTemplateByAPI parameter: ', newTemplateByAPI);
        let newTemplate = createTemplate.convertToTemplateThemes(newTemplateByAPI, nowUtc, userId);
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetTemplate('system', newTemplate.name);
        }).then(resultTemplate => {
            if (resultTemplate)
                throw 'TEMPLATE_SERVER_ERROR_EXISTS';
            return mssql_service_1.MSSql.GetTemplate(vcc, newTemplate.name);
        }).then(resultTemplateByAPI => {
            if (resultTemplateByAPI)
                throw 'TEMPLATE_SERVER_ERROR_EXISTS';
            return SaveTemplate(vcc, newTemplate);
        }).then((resultSaveTemplateByAPI) => {
            if (!resultSaveTemplateByAPI)
                throw 'TEMPLATE_SERVER_ERROR_ON_SAVE';
            return resultSaveTemplateByAPI;
        }).then(resultSaveTemplateByAPI => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveTemplateByAPI ? 'SERVER_SUCCESS_MESSAGE' : 'TEMPLATE_SERVER_ERROR_ON_SAVE'), data: resultSaveTemplateByAPI });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/template/delete_template', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let template = req.body.template;
        let templateId = template.name;
        logger.info(`[TemplateService::delete_template] Deleting template ${templateId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!template)
                throw 'TEMPLATE_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteTemplate(vcc, templateId);
        }).then(resultDeleteTemplate => {
            if (!resultDeleteTemplate)
                throw 'TEMPLATE_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(resultDeleteTemplate => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteTemplate, description: (resultDeleteTemplate ? 'SERVER_SUCCESS_MESSAGE' : 'TEMPLATE_SERVER_ERROR_ON_DELETE'), data: resultDeleteTemplate });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/template/delete_templates', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let templates = req.body.templates || [];
        logger.info(`[TemplateService::delete_templates] Deleting templates ${templates.map(x => x.name).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!templates)
                throw 'TEMPLATE_SERVER_ERROR_ON_DELETE';
            let promiseArray = [];
            templates.forEach(template => {
                promiseArray.push(mssql_service_1.MSSql.DeleteTemplate(vcc, template.name));
            });
            return Promise.all(promiseArray);
        }).then(resultDeleteTemplates => {
            let result;
            if (resultDeleteTemplates.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Plantillas eliminadas correctamente');
            }
            return result;
        }).then(resultDeleteTemplates => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteTemplates, description: (resultDeleteTemplates ? 'SERVER_SUCCESS_MESSAGE' : 'TEMPLATE_SERVER_ERROR_ON_DELETE'), data: resultDeleteTemplates });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TemplateService = TemplateService;
function GetTemplate(vcc, name) {
    let cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(() => {
        return mssql_service_1.MSSql.GetTemplate(vcc, name);
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetTemplate: ' + errMsg);
    });
}
exports.GetTemplate = GetTemplate;
function CreateTemplate(vcc, newTemplate, userId) {
    let nowUtc = moment().utc().toDate();
    let newTemplatecreated = false;
    newTemplate.createdDate = nowUtc;
    newTemplate.createdByUserId = userId;
    newTemplate.type = vcc === 'system' ? 'system' : newTemplate.type;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetTemplate('system', newTemplate.name);
    })
        .then(resultTemplate => {
        if (resultTemplate)
            throw 'TEMPLATE_SERVER_ERROR_EXISTS';
        return mssql_service_1.MSSql.GetTemplate(vcc, newTemplate.name);
    })
        .then((resultTemplate) => {
        if (resultTemplate)
            throw 'TEMPLATE_SERVER_ERROR_EXISTS';
        return SaveTemplate(vcc, newTemplate);
    })
        .then(resultSaveTemplate => {
        if (!resultSaveTemplate)
            throw 'TEMPLATE_SERVER_ERROR_ON_SAVE';
        newTemplatecreated = true;
        if (newTemplate && newTemplate.template) {
            return CloneTemplate(vcc, newTemplate);
        }
        return Promise.resolve(true);
    })
        .then((resultBulkTemplateThemeItems) => {
        if (!resultBulkTemplateThemeItems)
            throw 'SERVER_ERROR_TEMPLATE_SAVE_FAILED';
        return Promise.resolve(true);
    })
        .catch(err => {
        logger.error('[TemplateService::CreateTemplate] Function failed');
        if (newTemplatecreated)
            mssql_service_1.MSSql.DeleteTemplate(vcc, newTemplate.name);
        return Promise.reject(err);
    });
}
exports.CreateTemplate = CreateTemplate;
function CloneTemplate(vcc, newTemplate) {
    let baseTemplate = null;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        let promiseArray = [];
        promiseArray.push(mssql_service_1.MSSql.GetTemplate('system', newTemplate.template));
        switch (newTemplate.type) {
            case 'global':
                promiseArray.push(mssql_service_1.MSSql.GetTemplate(vcc, newTemplate.template));
                break;
        }
        return Promise.all(promiseArray);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_GET_TEMPLAE_FAILED';
        switch (newTemplate.type) {
            case 'custom':
                if (!result[0])
                    throw 'SERVER_ERROR_TEMPLATE_NOT_FOUND';
                baseTemplate = result[0];
                break;
            case 'global':
                if (!result[0] && !result[1])
                    throw 'SERVER_ERROR_TEMPLATE_NOT_FOUND';
                baseTemplate = result[0] || result[1];
                break;
        }
        return mssql_service_1.MSSql.GetTemplateThemes(baseTemplate.vcc, baseTemplate.name);
    })
        .then(resultTemplateThemes => {
        if (resultTemplateThemes && resultTemplateThemes.length > 0) {
            return mssql_service_1.MSSql.BulkTemplateThemeTemp(vcc, newTemplate, newTemplate.name, resultTemplateThemes);
        }
        else {
            logger.warn(`[TemplateService::CloneTemplate] Not found themes for template ${baseTemplate.name}`);
            return Promise.resolve(false);
        }
    })
        .then(resultBulkTemplateThemes => {
        if (resultBulkTemplateThemes) {
            let promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItems(baseTemplate.vcc, baseTemplate.name, null));
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeModels(baseTemplate.vcc, baseTemplate.name, null));
            return Promise.all(promiseArray);
        }
        else {
            logger.warn(`[TemplateService::CloneTemplate] Not found theme-items for template ${baseTemplate.name}`);
            return Promise.resolve([]);
        }
    })
        .then(resultTemplateThemeItems => {
        if (resultTemplateThemeItems?.length) {
            let templateItems = [];
            templateItems = templateItems.concat(resultTemplateThemeItems[0]?.length ? resultTemplateThemeItems[0] : []);
            templateItems = templateItems.concat(resultTemplateThemeItems[1]?.length ? resultTemplateThemeItems[1] : []);
            return mssql_service_1.MSSql.BulkTemplateThemeItemTemp(vcc, newTemplate, newTemplate.name, templateItems);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then((resultBulkTemplateThemeItems) => {
        if (!resultBulkTemplateThemeItems)
            throw 'TEMPLATE_SERVER_ERROR_ON_SAVE';
        return Promise.resolve(true);
    })
        .catch(err => {
        logger.error('[TemplateService::CloneTemplate] Function failed');
        return Promise.reject(err);
    });
}
function CreateScheduleForTemplate(vcc, newTemplate, userId) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return batch_schedule_service_1.SaveTemplateBatchSchedule(vcc, newTemplate, userId);
    })
        .then(result => {
        logger.info('[TemplateService::CreateScheduleForTemplate] Result of SaveProcessBatchSchedule:', result);
        return Promise.resolve({ status: result, saveResult: null });
    })
        .catch(err => {
        logger.error('[TemplateService::CreateScheduleForTemplate] Function failed');
        return Promise.reject(err);
    });
}
function SaveTemplate(vcc, template) {
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveTemplate(vcc, template.name, template.type, template.description, template.template, template.category, template.createdDate, template.createdByUserId, template.lastModifiedDate, template.lastModifiedByUserId));
    if (template.templateThemes) {
        template.templateThemes.forEach(templateTheme => {
            if (templateTheme.category1 === '')
                templateTheme.category1 = null;
            if (templateTheme.category2 === '')
                templateTheme.category2 = null;
            promiseArray.push(mssql_service_1.MSSql.SaveTemplateTheme(vcc, templateTheme.templateId, templateTheme.id, templateTheme.order, templateTheme.description, templateTheme.category1, templateTheme.category2, moment().utc().toDate(), templateTheme.createdByUserId, moment().utc().toDate(), templateTheme.lastModifiedByUserId, templateTheme.analysisType, templateTheme.isNegative));
        });
    }
    if (template.templateThemeItems) {
        template.templateThemeItems.forEach(templateThemeItem => {
            promiseArray.push(mssql_service_1.MSSql.SaveTemplateThemeItem(vcc, templateThemeItem.templateId, templateThemeItem.templateThemeId, templateThemeItem.id, moment().utc().toDate(), templateThemeItem.createdByUserId, moment().utc().toDate(), templateThemeItem.lastModifiedByUserId, templateThemeItem.acceptancePercentage));
            if (templateThemeItem.entities && templateThemeItem.entities.length) {
                templateThemeItem.entities.map(e => {
                    promiseArray.push(mssql_service_1.MSSql.SaveTemplateThemeItemEntity(vcc, templateThemeItem.templateId, templateThemeItem.templateThemeId, templateThemeItem.id, e.entityId, e.entityName, e.acceptancePercentage, e.order, templateThemeItem.createdByUserId, templateThemeItem.lastModifiedByUserId));
                });
            }
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveTemplate = SaveTemplate;
var createTemplate = (function (o) {
    o.convertToTemplateThemes = function (data, dateTime, userId) {
        var result = null;
        if (data) {
            result = {
                type: data.type,
                name: data.name,
                description: data.description,
                template: "",
                category: "",
                createdDate: dateTime,
                createdByUserId: userId,
                lastModifiedDate: dateTime,
                lastModifiedByUserId: userId,
                templateThemes: [],
                templateThemeItems: []
            };
        }
        if (data.items && data.items.length > 0) {
            data.items = o.setReference(null, data.items);
            var data = o.getAllThemes(data.name, data.items, dateTime, userId, { themes: [], themeItems: [] });
            if (data && data.themes && data.themes.length > 0) {
                result.templateThemes = data.themes;
                result.templateThemeItems = data.themeItems;
            }
        }
        return result;
    };
    o.i = 0;
    o.setReference = function (id, items) {
        if (items && items.length > 0) {
            var c, len = items.length, obj;
            for (c = 0; c < len; c++) {
                o.i++;
                obj = items[c];
                obj.id = obj.type;
                obj.id += "_";
                obj.id += o.i;
                obj.parentId = id;
                if (obj.items && obj.items.length > 0) {
                    o.setReference(obj.id, obj.items);
                }
            }
        }
        return items;
    };
    o.getAllThemes = function (name, items, dateTime, userId, result) {
        if (items && items.length > 0) {
            var c, len = items.length, obj, guid = "";
            for (c = 0; c < len; c++) {
                obj = items[c];
                guid = [o.generateGuid(), obj.id, obj.parentId].join("#");
                result.themes.push({
                    templateId: name,
                    id: guid,
                    order: (c + 1),
                    description: (obj.type === "description" ? obj.item : null),
                    category1: (obj.type === "category1" ? obj.item : null),
                    category2: (obj.type === "category2" ? obj.item : null),
                    createdDate: dateTime,
                    createdByUserId: userId,
                    lastModifiedDate: dateTime,
                    lastModifiedByUserId: userId
                });
                if (obj.query) {
                    result.themeItems.push({
                        templateId: name,
                        templateThemeId: guid,
                        id: obj.query,
                        createdDate: dateTime,
                        createdByUserId: userId,
                        lastModifiedDate: dateTime,
                        lastModifiedByUserId: userId,
                        selected: false
                    });
                }
                if (obj.items && obj.items.length > 0) {
                    o.getAllThemes(name, obj.items, dateTime, userId, result);
                }
            }
        }
        return result;
    };
    o.generateGuid = function () {
        return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (c) {
            var r = Math.random() * 16 | 0, v = c === "x" ? r : (r & 0x3 | 0x8);
            return v.toString(16).toUpperCase();
        });
    };
    return o;
}(createTemplate || {}));
function GetTemplateItems(vcc, themeId, getEntities = false) {
    let themeItems = [];
    return promises_shared_1.DummyPromise()
        .then(result => {
        return mssql_service_1.MSSql.GetTemplateThemeModels(vcc, null, themeId);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_THEME_ITEMS_NOT_FOUND';
        result.forEach(r => r.entities = []);
        if (getEntities)
            return GetTemplateThemeItemEntities(vcc, result);
        else
            return Promise.resolve(result);
    })
        .then(result => {
        themeItems = result;
        return Promise.resolve(themeItems);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[TemplateService::GetTemplateItems] Error: ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetTemplateItems = GetTemplateItems;
function GetTemplateThemeItemEntities(vcc, templateThemeItems) {
    return promises_shared_1.DummyPromise()
        .then(result => {
        let promiseArray = [];
        templateThemeItems.map(tti => {
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItemEntities(vcc, tti.id));
        });
        return Promise.all(promiseArray);
    })
        .then(result => {
        result.map((r, i) => {
            templateThemeItems[i].entities = r;
        });
        return Promise.resolve(templateThemeItems);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[TemplateService::GetTemplateThemeItemEntities] Error: ' + errorMsg);
        return Promise.reject(err);
    });
}
