"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TopBestAgentsService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
let logger = log4js.getLogger('ServerScripts');
function TopBestAgentsService(app) {
    app.post('/api/top_best_agents/get_top_best_agents/', (req, res, next) => {
        logger.info('[TopBestAgentsService::get_top_best_agents] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let initialDate = req.body.initialDate ? moment(req.body.initialDate).toDate() : null;
        let endDate = req.body.endDate ? moment(req.body.endDate).toDate() : null;
        let campaigns = (req.body.campaigns && req.body.campaigns !== undefined) ? req.body.campaigns : null;
        let evaluationFormats = (req.body.evaluationFormats && req.body.evaluationFormats !== undefined) ? req.body.evaluationFormats : null;
        let evaluationPlans = (req.body.evaluationPlans && req.body.evaluationPlans !== undefined) ? req.body.evaluationPlans : null;
        let agents = (req.body.agents && req.body.agents !== undefined) ? req.body.agents : null;
        let idGroup = (req.body.idGroup && req.body.idGroup !== 0) ? req.body.idGroup : null;
        let idSubGroup = (req.body.idSubGroup && req.body.idSubGroup !== 0) ? req.body.idSubGroup : null;
        let idAbility = (req.body.idAbility && req.body.idAbility !== 0) ? req.body.idAbility : null;
        let auditors = (req.body.auditors && req.body.auditors !== undefined) ? req.body.auditors : null;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (initialDate && endDate) {
                return mssql_service_1.MSSql.GetTopBestAgents(instance, initialDate, endDate, 10, campaigns, evaluationFormats, evaluationPlans, agents, idGroup, idSubGroup, idAbility, auditors);
            }
            else {
                return [];
            }
        }).then(resultTopMejoresAgentes => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: (resultTopMejoresAgentes ? resultTopMejoresAgentes : []) });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TopBestAgentsService::get_top_best_agents] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TopBestAgentsService = TopBestAgentsService;
