"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PublishRasaApiApp = exports.TrainApp = exports.TrainService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const nlu_service_1 = require("./nlu.service");
const cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
const nlu_status_const_1 = require("../const/nlu-status.const");
const nlu_mssql_1 = require("../db/mssql/nlu.mssql");
const io_model_1 = require("../models/io.model");
const io_request_1 = require("../io/io.request");
const events_1 = require("../io/events");
const utterance_service_1 = require("./utterance.service");
const utterance_service_2 = require("./utterance.service");
let logger = log4js.getLogger('ServerScripts');
function TrainService(app) {
    let cognitiveService;
    app.post('/api/train/train_app', (req, res, next) => {
        req.setTimeout(300000, () => { });
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let param = (req.body || {});
        let nlu = {};
        let trainResult = null;
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(result => {
            cognitiveService = result;
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, param.appId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_NLU_MODEL_NOT_FOUND';
            nlu = result;
            if (nlu.cognitiveEngine === 'RASA') {
                return mssql_service_1.MSSql.GetNLUs(vcc, null, null, null, null, true);
            }
            return true;
        })
            .then(result => {
            if (typeof result === 'boolean') {
                return TrainApp(cognitiveService, azure, nlu);
            }
            let count = result.reduce((n, x) => n + (x.statusCode === 'TRAINING'), 0);
            if (count < 10) {
                TrainApp(cognitiveService, azure, nlu).then(_ => { });
                return true;
            }
            else {
                return Promise.reject('SERVER_CONCURRENT_TRAINING_MODELS_LIMIT_REACHED');
            }
        })
            .then(result => {
            trainResult = result;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: trainResult });
        }).catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TrainService::train_app] Getting error:', errorMsg, 'subscriptionRegion:', azure.subscriptionRegion, 'subscriptionKey:', azure.subscriptionKey, 'appId:', param.appId);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/train/get_train_app/:app_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let appId = req.params.app_id || '';
        let trainData = [];
        let intents = [];
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(result => {
            cognitiveService = result;
            return cognitiveService.GetTrainApp(azure.subscriptionRegion, azure.subscriptionKey, appId);
        }).then((result) => {
            trainData = result;
            trainData = trainData.map((x) => {
                if (x.details.failureReason) {
                    x.details['failureReasonMessage'] = x.details.failureReason.toUpperCase();
                }
                return x;
            });
            let arrayId = result.map(a => a.modelId);
            let stringId = arrayId.join('\',\'');
            return mssql_service_1.MSSql.GetIntents(vcc, appId, 'name', '', stringId);
        }).then((result) => {
            intents = result;
            logger.info('[IntentService::/api/intent/get_intents/] Getting result:', result.length, 'after MSSql.GetIntents');
            return GetVersionTrainingStatus(intents, trainData);
        }).then((result) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TrainService::/api/train/get_train_app/] Getting error:', errorMsg, 'subscriptionRegion:', azure.subscriptionRegion, 'subscriptionKey:', azure.subscriptionKey, 'appId:', appId);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/train/publish_app', (req, res, next) => {
        logger.info('[TrainService::publish_app] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let param = (req.body || {});
        let nlu = {};
        let azureResult = null;
        let azureSubscription = {};
        let azureSubscriptionKey = {};
        let azureSubscriptionDemo = {};
        let tenantVCC = {};
        let authObject;
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(result => {
            cognitiveService = result;
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, param.appId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_NLU_MODEL_NOT_FOUND';
            nlu = result;
            if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                let promiseArray = [];
                promiseArray.push(mssql_service_1.MSSql.GetAzureSubscriptionKeyByName(vcc, 'DEMO'));
                promiseArray.push(mssql_service_1.MSSql.GetAzureSubscriptionKeyByKey(vcc, param.subscriptionKey));
                promiseArray.push(mssql_service_1.MSSql.GetTenantVCCByVCC(vcc));
                return Promise.all(promiseArray);
            }
            else {
                return null;
            }
        })
            .then(result => {
            if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                if (!result[0])
                    throw 'SERVER_ERROR_NLU_SUBSCRIPTIONDEMO_NOT_FOUND';
                if (!result[1])
                    throw 'SERVER_ERROR_NLU_SUBSCRIPTIONKEY_NOT_FOUND';
                if (!result[2])
                    throw 'SERVER_ERROR_TENANT_NOT_FOUND';
                tenantVCC = result[2];
                azureSubscriptionKey = result[1];
                azureSubscriptionDemo = result[0];
                let info = {
                    tenant_id: tenantVCC.tenantId,
                    client_id: tenantVCC.clientId,
                    client_secret: tenantVCC.clientSecret,
                    resource: tenantVCC.resource
                };
                if (nlu.subscriptionKey !== param.subscriptionKey) {
                    return cognitiveService.GetManagerToken(info);
                }
                else {
                    return true;
                }
            }
            else {
                return true;
            }
        })
            .then(result => {
            if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                if (!result)
                    throw 'SERVER_ERROR_NLU_TOKEN_GENERATOR_FAILED';
                if (nlu.subscriptionKey !== param.subscriptionKey) {
                    authObject = result;
                    return mssql_service_1.MSSql.GetAzureSubscription(vcc);
                }
                else {
                    return true;
                }
            }
            else {
                return true;
            }
        })
            .then(result => {
            if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                if (!result)
                    throw 'SERVER_ERROR_NLU_AZURE_SUBSCRIPTION_NOT_FOUND';
                if (nlu.subscriptionKey !== param.subscriptionKey) {
                    azureSubscription = result;
                    let unAssignData = {
                        azureSubscriptionId: azureSubscription.azureSubscription,
                        resourceGroup: azureSubscription.resourceGroup,
                        accountName: azureSubscription.accountName,
                        token_type: authObject.token_type,
                        access_token: authObject.access_token
                    };
                    return cognitiveService.UnAssignSubscriptionKeyOfVCC(azureSubscriptionKey.subscriptionRegion, azureSubscriptionDemo.subscriptionKey, param.appId, unAssignData);
                }
                else {
                    return true;
                }
            }
            else {
                return true;
            }
        })
            .then(result => {
            if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                if (!result)
                    throw 'SERVER_ERROR_NLU_KEY_UNASSIGNING_FAILED';
                if (nlu.subscriptionKey !== param.subscriptionKey) {
                    let assignData = {
                        azureSubscriptionId: azureSubscription.azureSubscription,
                        resourceGroup: azureSubscription.resourceGroup,
                        accountName: azureSubscription.accountName,
                        token_type: authObject.token_type,
                        access_token: authObject.access_token
                    };
                    return cognitiveService.AssignSubscriptionKeyOfVCC(azureSubscriptionKey.subscriptionRegion, azureSubscriptionDemo.subscriptionKey, param.appId, assignData);
                }
                else {
                    return true;
                }
            }
            else {
                return true;
            }
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_NLU_KEY_ASSIGNING_FAILED';
            nlu_mssql_1.UpdateNluStatus(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.publishing).then(_ => { });
            TriggerUpdateNLU(nlu.vcc);
            return cognitiveService.PublishApp(azureSubscriptionDemo.subscriptionRegion, azureSubscriptionDemo.subscriptionKey, param.appId, !azureSubscriptionDemo.isDemo, nlu.trainVersion);
        })
            .then(result => {
            if (result) {
                nlu.isPublished = true;
                nlu.needPublish = false;
                nlu.subscriptionKey = azureSubscriptionKey.subscriptionKey;
                nlu.lastPublishedDate = moment().utc().toDate();
                azureResult = result;
                nlu_mssql_1.UpdateNluStatus(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.available).then(_ => { });
                TriggerUpdateNLU(nlu.vcc);
                return nlu_service_1.SaveNLU(vcc, nlu);
            }
            else {
                return Promise.resolve(result);
            }
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: true });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TrainService::publish_app] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TrainService = TrainService;
function TriggerUpdateNLU(vcc) {
    let events = [];
    events.push(new io_model_1.IOEvent(io_model_1.VCC_EVENT, events_1.UPDATE_NLU_STATUS, null, vcc));
    io_request_1.SendIOEvents(events).then(_ => { });
}
function TrainApp(cognitiveService, azure, nlu) {
    let appIdRasaApi = '';
    let rasaApiTrainId = '';
    let rasaApiTrainVersion = '';
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (cognitiveService.GetCognitiveEngineName() === 'RASA') {
            if (nlu.isSystemApp)
                return utterance_service_1.CreateSystemAppIfNotExists(cognitiveService, nlu.vcc, nlu);
            else
                return Promise.resolve(true);
        }
        else if (cognitiveService.GetCognitiveEngineName() === 'RASAAPI') {
            if (nlu.isSystemApp)
                return utterance_service_2.CreateRasaApiSystemAppIfNotExists(cognitiveService, nlu.vcc, nlu);
            else
                return Promise.resolve(nlu.appId);
        }
        else
            return Promise.resolve(true);
    })
        .then(result => {
        if (!nlu.isSystemApp && !nlu.trainMode) {
            nlu_mssql_1.UpdateNluStatus(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.training).then(_ => { });
            TriggerUpdateNLU(nlu.vcc);
        }
        if (cognitiveService.GetCognitiveEngineName() === 'RASAAPI') {
            appIdRasaApi = result;
            if (nlu.trainId && nlu.trainVersion)
                return Promise.resolve({ id: nlu.trainId, versionId: nlu.trainVersion });
            return cognitiveService.TrainApp(azure ? azure.subscriptionRegion : null, azure ? azure.subscriptionKey : null, appIdRasaApi, nlu.vcc);
        }
        else {
            return cognitiveService.TrainApp(azure ? azure.subscriptionRegion : null, azure ? azure.subscriptionKey : null, nlu.appId);
        }
    })
        .then(result => {
        if (result) {
            rasaApiTrainId = result.id;
            rasaApiTrainVersion = result.versionId;
            if (nlu.cognitiveEngine === 'LUIS' || (nlu.cognitiveEngine === 'RASAAPI' && !nlu.isSystemApp && !nlu.trainMode)) {
                setTimeout(() => GetTrainingStatus(cognitiveService, azure, nlu, 0, rasaApiTrainId), 5000);
            }
            if (cognitiveService.GetCognitiveEngineName() === 'RASA') {
                ProcessSuccessfulTrainingResult(cognitiveService, nlu);
            }
        }
        return Promise.resolve(true);
    }).then(result => {
        if (cognitiveService.GetCognitiveEngineName() === 'RASAAPI' && nlu.isSystemApp || (nlu.trainMode === 'A' && !nlu.isSystemApp)) {
            return cognitiveService.GetTrainApp(null, null, nlu.appId, rasaApiTrainId);
        }
        else {
            return Promise.resolve(false);
        }
    })
        .then(result => {
        if (result) {
            nlu.trainId = result.id;
            nlu.trainVersion = result.versionId;
            if (result.status === 'trained') {
                ProcessSuccessfulTrainingResult(cognitiveService, nlu);
            }
            else {
                nlu_service_1.SaveNLU(nlu.vcc, nlu).then(_ => { });
            }
        }
        return Promise.resolve(true);
    })
        .catch(err => {
        logger.error('[TrainService::TrainApp] Getting nlu: ', nlu.appId, ' error:', err);
        if (!nlu.isSystemApp) {
            nlu_mssql_1.UpdateNluStatus(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.available).then(_ => { });
            TriggerUpdateNLU(nlu.vcc);
        }
        return Promise.reject(err);
    });
}
exports.TrainApp = TrainApp;
function PublishRasaApiApp(cognitiveService, nlu) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return cognitiveService.GetTrainApp(null, null, nlu.appId, nlu.trainId);
    })
        .then(result => {
        if (result && result.status === 'trained') {
            return cognitiveService.PublishApp(null, null, nlu.appId, null, result.versionId);
        }
        else
            return Promise.resolve(result);
    })
        .then(result => {
        if (result.active) {
            nlu.isPublished = true;
            nlu.needPublish = false;
            nlu.lastPublishedDate = moment().utc().toDate();
            nlu_mssql_1.UpdateNluStatus(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.available).then(_ => { });
            TriggerUpdateNLU(nlu.vcc);
            return nlu_service_1.SaveNLU(nlu.vcc, nlu);
        }
        else
            return Promise.resolve(result);
    })
        .then(result => {
        return Promise.resolve(result);
    })
        .catch(err => {
        logger.error('[TrainService::PublishRasaApiApp] Getting nlu: ', nlu.appId, ' error:', err);
        if (!nlu.isSystemApp) {
            nlu_mssql_1.UpdateNluStatus(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.available).then(_ => { });
            TriggerUpdateNLU(nlu.vcc);
        }
        return Promise.reject(err);
    });
}
exports.PublishRasaApiApp = PublishRasaApiApp;
function ProcessSuccessfulTrainingResult(cognitiveService, nlu) {
    nlu.isTrained = true;
    nlu.needTrain = false;
    nlu.needPublish = true;
    nlu.isPublished = false;
    nlu.lastTrainedDate = moment().utc().toDate();
    nlu.createdDate = typeof nlu.createdDate === 'string' ? moment(nlu.createdDate).toDate() : nlu.createdDate;
    nlu.lastModifiedByUserId = ['dev-1', 'dev-2'].includes(nlu.lastModifiedByUserId) ? 'system' : nlu.lastModifiedByUserId;
    nlu.lastModifiedDate = moment().utc().toDate();
    if (cognitiveService.GetCognitiveEngineName() === 'RASA') {
        nlu.isPublished = true;
        nlu.needPublish = false;
        nlu.lastPublishedDate = nlu.lastTrainedDate;
    }
    else {
        nlu.needPublish = true;
    }
    nlu_service_1.SaveNLU(nlu.vcc, nlu).then(_ => { });
    nlu_mssql_1.UpdateNluStatus(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.available).then(_ => { });
    TriggerUpdateNLU(nlu.vcc);
}
function GetTrainingStatus(cognitiveService, azure, nlu, retryCount = 0, rasaApiTrainId) {
    cognitiveService.GetTrainApp(azure ? azure.subscriptionRegion : null, azure ? azure.subscriptionKey : null, nlu.appId, rasaApiTrainId)
        .then(result => {
        if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
            if (result && result.every(model => ['UpToDate', 'Success'].includes(model.details.status))) {
                ProcessSuccessfulTrainingResult(cognitiveService, nlu);
            }
            else {
                if (retryCount < 10) {
                    setTimeout(() => GetTrainingStatus(cognitiveService, azure, nlu, retryCount + 1, rasaApiTrainId), 5000);
                }
                else {
                    nlu_mssql_1.UpdateNluStatus(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.trainingFailed).then(_ => { });
                    TriggerUpdateNLU(nlu.vcc);
                }
            }
        }
        else if (cognitiveService.GetCognitiveEngineName() === 'RASAAPI') {
            if (result && result.status === 'trained') {
                nlu.trainId = result.id;
                nlu.trainVersion = result.versionId;
                ProcessSuccessfulTrainingResult(cognitiveService, nlu);
            }
            else {
                if (retryCount < 10) {
                    setTimeout(() => GetTrainingStatus(cognitiveService, azure, nlu, retryCount + 1, rasaApiTrainId), 15000);
                }
                else {
                    nlu.trainId = rasaApiTrainId;
                    nlu_service_1.SaveNLU(nlu.vcc, nlu).then(_ => { });
                    nlu_mssql_1.UpdateNluStatus(nlu.vcc, nlu.appId, nlu_status_const_1.NluStatus.trainingFailed).then(_ => { });
                    TriggerUpdateNLU(nlu.vcc);
                }
            }
        }
    });
}
function GetVersionTrainingStatus(intents, trainData) {
    let intent;
    function find(x) {
        return x.modelId === intent.intentId;
    }
    let result = [];
    let i, lenIntents = intents.length, found;
    for (i = 0; i < lenIntents; i++) {
        intent = intents[i];
        found = trainData.find(find);
        if (found) {
            intent.trainData = found;
        }
    }
    result = intents;
    return result;
}
