"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveUser = exports.GetUser = exports.UserService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_config_1 = require("../config/mssql.config");
const mssql_service_1 = require("../db/mssql.service");
const http_request_occ_1 = require("../shared/http-request-occ");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const integration_config_1 = require("../config/integration.config");
const bluebird_1 = require("bluebird");
const api_user_implementation_1 = require("../implementation/api-user.implementation");
const role_service_1 = require("./role.service");
let logger = log4js.getLogger('ServerScripts');
function UserService(app) {
    app.get('/api/user/get_users/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let isExcludeUsers = req.query.isExcludeUsers === 'true' || false;
        let usersApi = [];
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!isExcludeUsers)
                return Promise.resolve([]);
            return mssql_service_1.MSSql.GetApiUsers(vcc, null, null);
        })
            .then(result => {
            usersApi = result;
            return mssql_service_1.MSSql.GetUsers(vcc, search_criteria, search_expression);
        }).then(users => {
            let data = users;
            if (usersApi.length) {
                data = users.filter(el => usersApi.every(f => f.user !== el.id));
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/user/get_user/:user_id', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = req.params.user_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!userId)
                throw 'USER_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetUser(vcc, userId);
        }).then(resultUser => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultUser });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/user/create_user', (req, res, next) => {
        logger.info('[UserService::create_user] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let vccType = res.locals.type;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newUser = (req.body || {});
        newUser.id = (newUser.id || '').toLowerCase();
        newUser.isSystemUser = false;
        newUser.createdDate = nowUtc;
        newUser.createdByUserId = userId;
        newUser.lastModifiedDate = nowUtc;
        newUser.lastModifiedByUserId = userId;
        let flag = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return SaveUserIntegration(vcc, vccType, newUser, integration_config_1.IP_BD, flag, false);
        })
            .then(resultSaveUser => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveUser, description: (resultSaveUser ? 'SERVER_SUCCESS_MESSAGE' : resultSaveUser), data: resultSaveUser });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UserService::create_user] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/user/save_user', (req, res, next) => {
        logger.info('[UserService::save_user] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let vccType = res.locals.type;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newUser = (req.body || {});
        let oldUser = {};
        let oldRoleFunctions = {};
        let newRoleFunctions = {};
        newUser.id = (newUser.id || '').toLowerCase();
        newUser.createdDate = null;
        newUser.createdByUserId = null;
        newUser.lastModifiedDate = nowUtc;
        newUser.lastModifiedByUserId = userId;
        let flag = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
        logger.info('[UserService::save_user] External database: ' + flag);
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!userId)
                throw 'USER_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetUser(vcc, newUser.id);
        })
            .then(result => {
            if (!result)
                throw 'ERROR_USER_NOT_FOUND';
            oldUser = result;
            newUser.isSystemUser = oldUser.isSystemUser;
            let promiseArray = [];
            promiseArray.push(role_service_1.GetRoleFunctions(vcc, newUser.id, oldUser.role));
            promiseArray.push(role_service_1.GetRoleFunctions(vcc, newUser.id, newUser.role));
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_ROLE_DATA_FAILED';
            oldRoleFunctions = result[0].data;
            newRoleFunctions = result[1].data;
            if (oldRoleFunctions.hasAuditorFunctions && !newRoleFunctions.hasAuditorFunctions) {
                logger.info('[UserService::save_user] Checking interactions from Auditor');
                return mssql_service_1.MSSql.GetAuditorsInteractions(vcc, 'id', newUser.id);
            }
            else {
                return bluebird_1.resolve(true);
            }
        })
            .then(result => {
            if (oldRoleFunctions.hasAuditorFunctions && !newRoleFunctions.hasAuditorFunctions
                && result && result[0] && result[0].uncompleteInteractions && result[0].uncompleteInteractions > 0) {
                throw 'ERROR_USER_PENDING_EVALUATIONS';
            }
            return SaveUserIntegration(vcc, vccType, newUser, integration_config_1.IP_BD, flag, true);
        })
            .then(resultSaveUser => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveUser, description: (resultSaveUser ? 'SERVER_SUCCESS_MESSAGE' : resultSaveUser), data: resultSaveUser });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[UserService::save_user] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/user/delete_user', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let user = req.body.user;
        let userId = user.id;
        logger.info(`[UserService::delete_user] Deleting user ${userId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!userId)
                throw 'USER_SERVER_ERROR_MISSING_ID';
            return DeleteUser(vcc, userId);
        })
            .then(resultDeleteUser => {
            if (typeof resultDeleteUser === 'boolean' && resultDeleteUser === true) {
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: '', data: resultDeleteUser });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/user/delete_users', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let users = req.body.users || [];
        logger.info(`[UserService::delete_users] Deleting users ${users.map(x => x.id).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!users)
                throw 'MISSING_USERS_INPUT_PARAMETER';
            let promiseArray = [];
            users.forEach((user) => {
                promiseArray.push(DeleteUser(vcc, user.id));
            });
            return Promise.all(promiseArray);
        }).then(resultDeleteUsers => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: '', data: resultDeleteUsers });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/current_user/get_user', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!userId)
                throw 'USER_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetUser(vcc, userId);
        }).then(resultUser => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultUser });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/current_user/save_user', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newUser = (req.body || {});
        newUser.id = (newUser.id || '').toLowerCase();
        newUser.createdDate = null;
        newUser.createdByUserId = null;
        newUser.lastModifiedDate = nowUtc;
        newUser.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetUser(vcc, userId);
        })
            .then(resultCurrentUser => {
            if (!resultCurrentUser)
                throw 'USER_SERVER_ERROR_CANNOT_GET';
            if (!newUser || resultCurrentUser.id !== newUser.id)
                throw 'USER_SERVER_ERROR_ON_MATCH';
            newUser.isSystemUser = resultCurrentUser.isSystemUser;
            newUser.role = resultCurrentUser.role;
            newUser.limitClients = resultCurrentUser.limitClients;
            return Promise.resolve(true);
        })
            .then(resultDeleteUser => {
            if (!resultDeleteUser)
                throw 'USER_SERVER_ERROR_ON_SAVE';
            return SaveUser(vcc, newUser);
        })
            .then(resultSaveUser => {
            if (!resultSaveUser)
                throw 'USER_SERVER_ERROR_ON_SAVE';
            return redis_service_1.Redis.SaveUser(vcc, newUser);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'USER_SERVER_ERROR_ON_SAVE'), data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/user/validate_user_pin', (req, res, next) => {
        logger.info('[UserService::validate_user_pin] Getting req.method:', req.method, 'req.path:', req.path);
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let pin = req.body.pin;
        promises_shared_1.DummyPromise()
            .then((result) => {
            let apiUser = new api_user_implementation_1.ApiUserImplementation();
            return apiUser.ValidateUserPin(vcc, userId, pin);
        }).then((result) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result ? result.data : null });
        }).catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UserService::/api/user/validate_user_pin] Getting error:', errorMsg, 'req.path:', req.path);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.UserService = UserService;
function GetUser(vcc, id) {
    id = (id || '').toLowerCase();
    let cacheUpdateNeeded = false;
    return redis_service_1.Redis.GetUser(vcc, id)
        .then((redisUser) => {
        if (redisUser) {
            return Promise.resolve(redisUser);
        }
        else {
            logger.info('El dato de GetUser para \'' + id + '\' no se encuentra en cache. se consultara a base de datos.');
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetUser(vcc, id);
        }
    }).then((user) => {
        if (user) {
            user.id = (user.id || '').toLowerCase();
            if (cacheUpdateNeeded)
                redis_service_1.Redis.SaveUser(vcc, user);
            return Promise.resolve(user);
        }
        else {
            throw 'USER_SERVER_ERROR_NOT_FOUND';
        }
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetUser: ' + errMsg);
    });
}
exports.GetUser = GetUser;
function SaveUser(vcc, user) {
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveUser(vcc, user.id, user.email, user.firstname, user.lastname, user.password, user.role, user.country, user.language, user.timeZone, user.preferences, user.isSystemUser, user.isLocalSupervisor, user.isExternalSupervisor, user.isExternalAgent, user.limitClients, user.createdDate, user.createdByUserId, user.lastModifiedDate, user.lastModifiedByUserId));
    if (user.userClients) {
        user.userClients.forEach(userClient => {
            promiseArray.push(mssql_service_1.MSSql.SaveUserClient(vcc, userClient.userId, userClient.clientId));
        });
    }
    return Promise.all(promiseArray)
        .then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return Promise.resolve(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(_ => {
        return role_service_1.GetRoleFunctions(vcc, user.id, user.role);
    })
        .then(result => {
        if (result.data.hasAuditorFunctions) {
            logger.info('[UserService::SaveUser] Saving user as auditor: ', user);
            return mssql_service_1.MSSql.SaveAuditor(vcc, user.id, '', user.createdDate, user.createdByUserId, user.lastModifiedDate, user.lastModifiedByUserId);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .catch((err) => {
        return Promise.resolve(false);
    });
}
exports.SaveUser = SaveUser;
function SaveUserIntegration(vcc, vccType, newUser, ip, flag, isEdit) {
    let roleFunctions = {};
    newUser.isLocalSupervisor = newUser.isLocalSupervisor ? true : false;
    newUser.isExternalSupervisor = newUser.isExternalSupervisor ? true : false;
    newUser.isExternalAgent = newUser.isExternalAgent ? true : false;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetUser(vcc, newUser.id);
    })
        .then(resultUser => {
        if (!isEdit) {
            if (resultUser)
                throw 'USER_ID_ALREADY_EXISTS';
        }
        if (isEdit) {
            newUser.password = newUser.password === null ? resultUser.password !== undefined ? resultUser.password : '' : newUser.password;
            if (!newUser || resultUser.id !== newUser.id)
                throw 'USER_NOT_MATCH';
        }
        return role_service_1.GetRoleFunctions(vcc, newUser.id, newUser.role);
    })
        .then(result => {
        roleFunctions = result.data;
        newUser.isLocalSupervisor = roleFunctions.hasSupervisorFunctions;
        logger.info('[UserService::SaveUserIntegration] Not password arrived, using: ' + newUser.password);
        if (['I6', 'OCC'].includes(vccType)
            &&
                (roleFunctions.hasAgentFunctions
                    ||
                        roleFunctions.hasAuditorFunctions
                    ||
                        roleFunctions.hasSupervisorFunctions)) {
            return ExecuteSaveUserByAPIi6(vcc, newUser, roleFunctions);
        }
        else if (vccType === 'ALLEGRO'
            &&
                (roleFunctions.hasAgentFunctions
                    ||
                        roleFunctions.hasAuditorFunctions
                    ||
                        roleFunctions.hasSupervisorFunctions)) {
            return ExecuteSaveUserByAllegro(vcc, newUser, flag, roleFunctions);
        }
        else {
            return Promise.resolve(newUser);
        }
    })
        .then((result) => {
        if (result) {
            newUser = result;
            return SaveUser(vcc, newUser);
        }
        return Promise.resolve(false);
    })
        .then(resultSaveUser => {
        if (vccType === 'ALLEGRO'
            &&
                (roleFunctions.hasAgentFunctions
                    ||
                        roleFunctions.hasAuditorFunctions
                    ||
                        roleFunctions.hasSupervisorFunctions)) {
            if (!resultSaveUser)
                throw 'USER_SAVE_FAILED_QUALITY';
        }
        if (!resultSaveUser)
            throw 'USER_SAVE_FAILED';
        return redis_service_1.Redis.SaveUser(vcc, newUser);
    })
        .catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[UserService::SaveUserIntegration] Getting error:', err, 'IP:', ip, 'flag:', flag, 'newUser:', newUser);
        return Promise.reject(errMsg);
    });
}
function ExecuteSaveUserByAllegro(vcc, user, flag, roleFunctions) {
    let userExists = false;
    let roleExists = false;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        let promiseArray = [];
        if (user.role !== 'administrator' && roleFunctions.hasSupervisorFunctions)
            promiseArray.push(mssql_service_1.MSSql.GetAllegroSupervisor(integration_config_1.IP_BD, vcc, user.id, flag));
        else
            promiseArray.push(Promise.resolve(true));
        if (user.role !== 'administrator' && roleFunctions.hasAgentFunctions)
            promiseArray.push(mssql_service_1.MSSql.GetAllegroAgent(integration_config_1.IP_BD, vcc, user.id, flag));
        else
            promiseArray.push(Promise.resolve(true));
        return Promise.all(promiseArray);
    })
        .then(result => {
        let promiseArray = [];
        if (result[0]) {
            user.isExternalSupervisor = true;
            if (user.role !== 'administrator' && roleFunctions.hasSupervisorFunctions)
                promiseArray.push(mssql_service_1.MSSql.GetAllegroRole(integration_config_1.IP_BD, vcc, 'supervisor', flag));
            else
                promiseArray.push(Promise.resolve(true));
        }
        else
            throw 'SERVER_ERROR_USER_HAS_NOT_SUPERVISOR_FUNCTIONS_ON_ALLEGRO';
        if (result[1]) {
            user.isExternalAgent = true;
            if (user.role !== 'administrator' && roleFunctions.hasAgentFunctions)
                promiseArray.push(mssql_service_1.MSSql.GetAllegroRole(integration_config_1.IP_BD, vcc, 'quality_agent', flag));
            else
                promiseArray.push(Promise.resolve(true));
        }
        else
            throw 'SERVER_ERROR_USER_HAS_NOT_AGENT_FUNCTIONS_ON_ALLEGRO';
        return Promise.all(promiseArray);
    })
        .then(result => {
        roleExists = result[0] && result[1] ? true : false;
        if (roleExists) {
            let name = user.firstname + (user.lastname ? ' ' + user.lastname : '');
            let allegroRole = null;
            if (roleFunctions.hasAuditorFunctions)
                allegroRole = 'qualityauditor';
            if (roleFunctions.hasSupervisorFunctions && !allegroRole)
                allegroRole = 'quality_supervisor';
            if (roleFunctions.hasAgentFunctions && !allegroRole)
                allegroRole = 'quality_agent';
            return Promise.resolve(true);
        }
        else {
            throw 'SERVER_ERROR_ALLEGRO_USER_NOT_FOUND';
        }
    })
        .then(result => {
        if (roleFunctions.hasSupervisorFunctions || roleFunctions.hasAgentFunctions) {
            if (roleExists) {
                return Promise.resolve(true);
            }
            return Promise.resolve(userExists);
        }
        else {
            return Promise.resolve(result);
        }
    })
        .then(result => {
        return Promise.resolve(user);
    })
        .catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[UserService::ExecuteSaveUserByAllegro] ExecuteTask (inConcert): Error: ' + errMsg);
        return Promise.reject(errMsg);
    });
}
function ExecuteSaveUserByAPIi6(vcc, user, roleFunctions) {
    let needUserCreate = false;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return http_request_occ_1.ApiI6GetUser(vcc, user.id);
    })
        .then(result => {
        if (result.status) {
            let promiseArray = [];
            if (user.role !== 'administrator' && roleFunctions.hasSupervisorFunctions)
                promiseArray.push(http_request_occ_1.ApiI6SearchUsers(vcc, [], 'supervisor'));
            else
                promiseArray.push(Promise.resolve(true));
            if (user.role !== 'administrator' && roleFunctions.hasAgentFunctions)
                promiseArray.push(http_request_occ_1.ApiI6SearchUsers(vcc, [], 'interaction'));
            else
                promiseArray.push(Promise.resolve(true));
            if (roleFunctions.hasAuditorFunctions)
                promiseArray.push(Promise.resolve(true));
            else
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        }
        else {
            needUserCreate = true;
            return Promise.resolve([true, true, true]);
        }
    })
        .then(result => {
        if (!needUserCreate) {
            if (user.role !== 'administrator' && roleFunctions.hasSupervisorFunctions) {
                if (result[0].body.filter(u => u.userId === user.id).length === 0)
                    throw 'SERVER_ERROR_USER_HAS_NOT_SUPERVISOR_FUNCTIONS_ON_OCC';
                else
                    user.isExternalSupervisor = true;
            }
            else
                user.isExternalSupervisor = false;
            if (user.role !== 'administrator' && roleFunctions.hasAgentFunctions) {
                if (result[1].body.filter(u => u.userId === user.id).length === 0)
                    throw 'SERVER_ERROR_USER_HAS_NOT_AGENT_FUNCTIONS_ON_OCC';
                else
                    user.isExternalAgent = true;
            }
            else
                user.isExternalAgent = false;
            return Promise.resolve(true);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(_ => {
        return Promise.resolve(user);
    })
        .catch((err) => {
        logger.error('[UserService::ExecuteSaveUserByAPIi6] ExecuteTask (inConcert): Error: ' + err.message);
        return Promise.reject(err);
    });
}
function SetError(userId, reason) {
    let details = null;
    if ((userId && userId.length) && (reason && reason.length)) {
        details = {};
        details.error = {
            userId: userId,
            reason: reason
        };
    }
    return Promise.resolve(details);
}
function DeleteUser(vcc, userId) {
    let user, reason = '';
    let roleFunctions = {};
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (!userId) {
            reason = 'MISSING_USER_ID_PARAMETER';
            throw reason;
        }
        return mssql_service_1.MSSql.GetApiUser(vcc, userId);
    })
        .then(result => {
        if (result)
            throw 'ERROR_IS_API_USER';
        return mssql_service_1.MSSql.GetUser(vcc, userId);
    })
        .then(result => {
        user = result;
        if (!result)
            throw 'ERROR_USER_NOT_FOUND';
        return role_service_1.GetRoleFunctions(vcc, user.id, user.role);
    })
        .then(result => {
        roleFunctions = result.data;
        if (roleFunctions.hasAuditorFunctions)
            return mssql_service_1.MSSql.GetAuditorsInteractions(vcc, 'id', userId);
        else
            return Promise.resolve(true);
    })
        .then(result => {
        if (roleFunctions.hasAuditorFunctions && result && result[0] && result[0].uncompleteInteractions && result[0].uncompleteInteractions > 0)
            throw 'ERROR_USER_PENDING_EVALUATIONS_MSG';
        if (result)
            return mssql_service_1.MSSql.DeleteUser(vcc, userId);
        return Promise.resolve(true);
    })
        .then((result) => {
        if (!result)
            throw 'ERROR_DELETING_USER';
        return redis_service_1.Redis.DeleteUser(vcc, user);
    })
        .then((result) => {
        if (!result)
            throw 'ERROR_DELETING_USER_REDIS';
        return Promise.resolve(result);
    })
        .catch((error) => {
        return SetError(userId, error);
    });
}
