"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetSaveUtterances = exports.GetUtterances = exports.GetIntent = exports.CreateRasaApiSystemAppIfNotExists = exports.CreateSystemAppIfNotExists = exports.UtteranceService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
const nlu_service_1 = require("./nlu.service");
const config_1 = require("../config/config");
let logger = log4js.getLogger('ServerScripts');
let useRasaApi = config_1.USE_RASA_API;
function UtteranceService(app) {
    let cognitiveService;
    app.get('/api/utterance/get_utterances/:app_id/:search_criteria/:search_expression?', (req, res) => {
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let search_criteria = req.params.search_criteria || null;
        let search_expression = req.params.search_expression || null;
        let appId = req.params.app_id || '';
        let intent = req.query.intent ? req.query.intent.toString() : null;
        let implUtterances = [];
        let dbUtterances = [];
        let filters = {
            name: 'U.[Text]'
        };
        let hasSearch = false;
        if (search_criteria && search_expression) {
            search_criteria = filters[search_criteria];
            hasSearch = true;
        }
        let nluName = '';
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, appId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            nluName = result.name;
            vcc = result.vcc;
            if (result.isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        })
            .then((cognitiveService) => {
            return cognitiveService.GetUtterancesList(azure.subscriptionRegion, azure.subscriptionKey, appId, vcc);
        })
            .then(utterances => {
            implUtterances = utterances;
            return GetUtterances(vcc, appId, intent, search_criteria, search_expression, false);
        })
            .then(utterances => {
            dbUtterances = utterances;
            let promiseArray = [];
            if (hasSearch) {
                let filteredUtterances = [];
                for (let du of dbUtterances) {
                    for (let iu of implUtterances) {
                        if ([du.id].includes(`${iu.id}`)) {
                            filteredUtterances.push(iu);
                        }
                    }
                }
                implUtterances = filteredUtterances;
                return Promise.all(promiseArray);
            }
            implUtterances.map(iu => {
                if (!dbUtterances.find(du => du.id === iu.id) && iu.intentLabel !== 'None') {
                    promiseArray.push(mssql_service_1.MSSql.SaveUtterance(vcc, appId, null, iu.intentLabel, iu.id, iu.text, 'system'));
                }
            });
            dbUtterances.map(du => {
                if (!implUtterances.find(iu => iu.id === du.id)) {
                    promiseArray.push(mssql_service_1.MSSql.DeleteUtterance(vcc, appId, du.intentId, du.Id));
                }
            });
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: implUtterances });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/utterance/get_utterance/:utterance_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let utteranceId = req.params.utterance_id;
        let utterance;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!utteranceId)
                throw 'UTTERANCE_SERVER_ERROR_MISSING_ID';
            return Promise.resolve(true);
        }).then(resultUtterance => {
            utterance = resultUtterance;
            return Promise.resolve(true);
        }).then(result => {
            return utterance;
        }).then(utterance => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: utterance });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/utterance/get_utterances_by_interaction/:audio_id/:section/:part', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let audioId = req.params.audio_id ? req.params.audio_id.toString() : '';
        let section = req.params.section?.length ? Number(req.params.section) : null;
        let part = req.params.part?.length ? Number(req.params.part) : null;
        logger.info('Request parameters: ', { audioId: audioId, section: section, part: part });
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!audioId)
                throw 'UTTERANCE_SERVER_ERROR_MISSING_AUDIO_ID';
            if (section === null)
                throw 'UTTERANCE_SERVER_ERROR_MISSING_AUDIO_SECTION';
            if (part === null)
                throw 'UTTERANCE_SERVER_ERROR_MISSING_AUDIO_PART';
            return mssql_service_1.MSSql.GetUtterancesByInteraction(vcc, audioId, section, part);
        }).then((resultUtterances) => {
            logger.info('resultUtterances: ', resultUtterances);
            return Promise.resolve(resultUtterances);
        }).then((resultUtterances) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultUtterances });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/create_utterance', (req, res, next) => {
        logger.info('[UtteranceService::create_utterance] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let nlu = null;
        let vcc_new = '';
        let newUtterance = (req.body || {});
        newUtterance.name = newUtterance.name.trim();
        newUtterance.name = newUtterance.name.replace(/\s+/g, ' ');
        let createUtteranceResult = null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!newUtterance.appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, newUtterance.appId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            nlu = result;
            if (result.vcc === 'system')
                nlu.appId = vcc + '::' + nlu.name;
            return mssql_service_1.MSSql.GetUtterance(vcc, newUtterance.appId, newUtterance.intent, newUtterance.name);
        })
            .then(result => {
            if (result)
                throw 'SERVER_ERROR_UTTERANCE_ALREADY_EXISTS';
            if (nlu.isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        })
            .then(result => {
            cognitiveService = result;
            if (nlu.isSystemApp) {
                if (nlu.cognitiveEngine === 'RASA')
                    return CreateSystemAppIfNotExists(cognitiveService, vcc, nlu);
                return CreateRasaApiSystemAppIfNotExists(cognitiveService, vcc, nlu);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(result => {
            if (typeof result === "string") {
                newUtterance.appId = result;
                nlu.appId = result;
            }
            nlu.needTrain = true;
            nlu.needPublish = false;
            nlu.isPublished = false;
            nlu.isTrained = false;
            nlu.trainId = null;
            nlu.trainVersion = null;
            nlu.lastModifiedDate = nowUtc;
            nlu.lastModifiedByUserId = userId;
            return nlu_service_1.SaveNLU(vcc, nlu);
        })
            .then(result => {
            return cognitiveService.AddUtterance(azure.subscriptionRegion, azure.subscriptionKey, newUtterance.appId, newUtterance, vcc);
        })
            .then((resultSaveUtterance) => {
            if (!resultSaveUtterance)
                throw 'UTTERANCE_SERVER_ERROR_ON_SAVE';
            createUtteranceResult = resultSaveUtterance;
            if (nlu.cognitiveEngine === 'RASAAPI' && nlu.isSystemApp)
                return mssql_service_1.MSSql.SaveUtterance(vcc, newUtterance.appId, newUtterance.intent, null, resultSaveUtterance.ExampleId, resultSaveUtterance.UtteranceText, userId);
            return mssql_service_1.MSSql.SaveUtterance(vcc, newUtterance.appId, null, newUtterance.intent, resultSaveUtterance.ExampleId, resultSaveUtterance.UtteranceText, userId);
        })
            .then(_ => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (createUtteranceResult ? 'SERVER_SUCCESS_MESSAGE' : 'UTTERANCE_SERVER_ERROR_ON_SAVE'), data: createUtteranceResult });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UtteranceService::create_utterance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/create_utterance_underlined', (req, res, next) => {
        logger.info('[UtteranceService::create_utterance_underlined] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newUtteranceData = (req.body || {});
        newUtteranceData.createdDate = nowUtc;
        newUtteranceData.createdByUserId = userId;
        newUtteranceData.lastModifiedDate = nowUtc;
        newUtteranceData.lastModifiedByUserId = userId;
        logger.info('newUtteranceDTO: ', newUtteranceData);
        promises_shared_1.DummyPromise()
            .then((resultSaveUnderlined) => {
            if (!resultSaveUnderlined)
                throw 'UTTERANCE_SERVER_ERROR_ON_SAVE';
            logger.info('[UtteranceService::create_utterance_underlined] resultSaveUnderlined: ', resultSaveUnderlined);
            return mssql_service_1.MSSql.SaveUnderlined(vcc, newUtteranceData.appId, newUtteranceData.intentId, newUtteranceData.audioId, newUtteranceData.section, newUtteranceData.part, newUtteranceData.speaker, newUtteranceData.anchorOffset, newUtteranceData.focusOffset, newUtteranceData.currentTime, newUtteranceData.endCurrentTime, newUtteranceData.utteranceId, newUtteranceData.textContent, newUtteranceData.createdDate, newUtteranceData.createdByUserId, newUtteranceData.lastModifiedDate, newUtteranceData.lastModifiedByUserId);
        }).then(resultSaveUtteranceUnderlined => {
            logger.info('[UtteranceService::create_utterance_underlined] resultSaveUtteranceUnderlined: ', resultSaveUtteranceUnderlined);
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveUtteranceUnderlined ? 'SERVER_SUCCESS_MESSAGE' : 'UTTERANCE_SERVER_ERROR_ON_SAVE'), data: resultSaveUtteranceUnderlined });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UtteranceService::create_utterance_underlined] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/save_utterance', (req, res, next) => {
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let nlu = null;
        let utterance = (req.body || {});
        utterance.name = utterance.name.trim();
        utterance.name = utterance.name.replace(/\s+/g, ' ');
        let createUtteranceResult = null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!utterance.appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, utterance.appId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            nlu = result;
            return mssql_service_1.MSSql.GetUtterance(vcc, utterance.appId, utterance.intent, utterance.name);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_UTTERANCE_DOES_NOT_EXISTS';
            utterance.id = result.id;
            if (nlu.isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        })
            .then(result => {
            cognitiveService = result;
            if (!cognitiveService.GetCognitiveEngineName().includes('RASA'))
                return cognitiveService.AddUtterance(azure.subscriptionRegion, azure.subscriptionKey, utterance.appId, utterance);
            return true;
        })
            .then((resultSaveUtterance) => {
            if (!resultSaveUtterance)
                throw 'UTTERANCE_SERVER_ERROR_ON_SAVE';
            createUtteranceResult = resultSaveUtterance;
            nlu.needTrain = true;
            nlu.needPublish = false;
            nlu.isPublished = false;
            nlu.isTrained = false;
            nlu.trainId = null;
            nlu.trainVersion = null;
            nlu.lastModifiedDate = nowUtc;
            nlu.lastModifiedByUserId = userId;
            return nlu_service_1.SaveNLU(vcc, nlu);
        })
            .then(result => {
            if (utterance.entityLabels && utterance.entityLabels.length > 0) {
                if (utterance) {
                    let promises = [];
                    for (let e of utterance.entityLabels) {
                        promises.push(mssql_service_1.MSSql.SaveUtteranceEntity(vcc, utterance.id, e.entityName, e.start, e.end, e.startTokenIndex, e.endTokenIndex));
                    }
                    return Promise.all(promises);
                }
            }
            return [result];
        })
            .then(_ => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (createUtteranceResult ? 'SERVER_SUCCESS_MESSAGE' : 'UTTERANCE_SERVER_ERROR_ON_SAVE'), data: createUtteranceResult });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UtteranceService::save_utterance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/delete_utterances', (req, res) => {
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let utterances = req.body.utterances || [];
        let mode = Number(req.query.mode) || 0;
        let deleteByIntent = mode == 0;
        let ids = [];
        logger.info(`[UtteranceService::delete_utterances] Deleting utterances ${utterances.map(x => x.id).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise().then(() => {
            if (utterances.length == 0) {
                throw new Error("EMPTY_UTTERANCES");
            }
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then((cognitiveService) => {
            let promises = [];
            let isLuis = ['LUIS'].includes(cognitiveService.GetCognitiveEngineName());
            for (let u of utterances) {
                if (deleteByIntent) {
                    ids.push(u.intentId);
                }
                else {
                    ids.push(u.utteranceId);
                }
                if (isLuis) {
                    promises.push(cognitiveService.DeleteAppUtterance(azure.subscriptionRegion, azure.subscriptionKey, u.appId, u.intentId, u.utteranceId));
                }
            }
            if (!isLuis) {
                promises.push(Promise.resolve({
                    code: 'Success'
                }));
            }
            return Promise.all(promises);
        })
            .then((resultDeleteUtterance) => {
            let success = resultDeleteUtterance.filter(r => ["Success"].includes(r.code));
            if (success.length == 0)
                throw 'UTTERANCES_SERVER_ERROR_CANNOT_DELETE';
            return mssql_service_1.MSSql.DeleteMultipleUtterances(vcc, ids.toString(), deleteByIntent);
        }).then((result) => {
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'UTTERANCE_SERVER_ERROR_CANNOT_DELETE'),
                data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UtteranceService::delete_utterances] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/delete_utterance', (req, res) => {
        logger.info('[UtteranceService::delete_utterance] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let utterance = req.body.utterance;
        let utteranceId = utterance.id;
        let intentId = utterance.intentLabel;
        let appId = utterance.appId;
        let rasaapi = false;
        let nlu = null;
        let deleteUtteranceResult = null;
        logger.info(`[UtteranceService::delete_utterance] Deleting utterance ${utteranceId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, appId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            nlu = result;
            if (nlu.isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        })
            .then((cognitiveService) => {
            let id = utteranceId;
            if (cognitiveService.GetCognitiveEngineName() === 'RASA' || cognitiveService.GetCognitiveEngineName() === 'RASAAPI') {
                id = utterance.text;
            }
            if (!utterance)
                throw 'UTTERANCE_SERVER_ERROR_MISSING_ID';
            return cognitiveService.DeleteAppUtterance(azure.subscriptionRegion, azure.subscriptionKey, appId, intentId, id, vcc);
        }).then(resultDeleteUtterance => {
            if (!resultDeleteUtterance)
                throw 'UTTERANCE_SERVER_ERROR_CANNOT_DELETE';
            let promises = [mssql_service_1.MSSql.DeleteUtterance(vcc, appId, intentId, utteranceId)];
            deleteUtteranceResult = resultDeleteUtterance;
            nlu.needTrain = true;
            nlu.needPublish = false;
            nlu.isPublished = false;
            nlu.isTrained = false;
            nlu.trainId = null;
            nlu.trainVersion = null;
            nlu.lastPublishedDate = null;
            nlu.lastTrainedDate = null;
            nlu.lastModifiedDate = nowUtc;
            nlu.lastModifiedByUserId = userId;
            promises.push(nlu_service_1.SaveNLU(vcc, nlu));
            return Promise.all(promises);
        }).then(resultDeleteUtterance => {
            if (res.headersSent)
                return;
            let status = !resultDeleteUtterance.includes(false);
            res.json({ status, description: (status ? 'SERVER_SUCCESS_MESSAGE' : 'UTTERANCE_SERVER_ERROR_CANNOT_DELETE'), data: resultDeleteUtterance });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[UtteranceService::delete_utterance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.UtteranceService = UtteranceService;
function CreateSystemAppIfNotExists(cognitiveService, vcc, nlu) {
    let isCreated = false;
    return promises_shared_1.DummyPromise()
        .then(result => {
        return cognitiveService.GetAppsList(null, null);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_GET_INTENTS_FAILED';
        if (!result.find(n => n.id === nlu.appId)) {
            return cognitiveService.AddApp(null, null, nlu);
        }
        else {
            isCreated = true;
            return Promise.resolve(true);
        }
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_MIGRATE_DATABASE_RASAAPI_TO_RASA_FAILED';
        if (!isCreated) {
            return cognitiveService.AddIntent(null, null, nlu.appId, 'Agent');
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(result => {
        if (result)
            throw 'SERVER_ERROR_CREATE_SYSTEM_MODEL_INTENT_FAILED';
        return Promise.resolve(true);
    })
        .catch(err => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[UtteranceService::CreateDiarizationAppIfNotExists] Error: ' + errorMsg);
        return Promise.resolve(false);
    });
}
exports.CreateSystemAppIfNotExists = CreateSystemAppIfNotExists;
function CreateRasaApiSystemAppIfNotExists(cognitiveService, vcc, nlu) {
    let alreadyCreated = false;
    let newAppId = '';
    return promises_shared_1.DummyPromise()
        .then(result => {
        return cognitiveService.GetAppById(nlu.appId, null, null);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_GET_APPBYID_RASAAPI_FAILED';
        logger.debug("GetAppIdResultado", result);
        if (result.id === nlu.appId) {
            alreadyCreated = true;
            return true;
        }
        else {
            return cognitiveService.AddApp(null, null, nlu);
        }
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_UPDATENLUMIGRATION_RASAAPI_FAILED';
        newAppId = alreadyCreated ? nlu.appId : result;
        return Promise.resolve(newAppId);
    })
        .catch(err => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[UtteranceService::CreateRasaApiSystemAppIfNotExists] Error: ' + errorMsg);
        return Promise.resolve(false);
    });
}
exports.CreateRasaApiSystemAppIfNotExists = CreateRasaApiSystemAppIfNotExists;
function GetIntent(vcc, name) {
    let cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(() => {
        return mssql_service_1.MSSql.GetIntent(vcc, name);
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetIntent: ' + errMsg);
    });
}
exports.GetIntent = GetIntent;
function GetUtterances(vcc, appId, intentId, searchCriteria, searchExpression, findUtteranceEntities) {
    let utterances = [];
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetUtterances(vcc, appId, intentId, searchCriteria, searchExpression);
    })
        .then(result => {
        utterances = result;
        let promiseArray = [];
        if (findUtteranceEntities) {
            result.map(r => {
                promiseArray.push(mssql_service_1.MSSql.GetUtteranceEntities(vcc, r.id));
            });
        }
        else {
            promiseArray.push(Promise.resolve([]));
        }
        return Promise.all(promiseArray);
    })
        .then(result => {
        let utterancesFiltered = [];
        utterances.filter(u => u && u.text).map((u, i) => {
            let tokenizedText = u.text.split(/(.+?)([\s,.!?;:)([\]]+)/);
            tokenizedText = tokenizedText.filter(tt => tt.trim().length > 0);
            u.entityLabels = result[i] ? result[i] : [];
            u.entityPredictions = [];
            u.intentLabel = u.intentName;
            u.intentPredictions = [
                {
                    name: u.intentName.trim(),
                    score: null
                }
            ],
                u.tokenizedText = tokenizedText;
        });
        utterancesFiltered = utterances.filter(u => u && u.text);
        return Promise.resolve(utterancesFiltered);
    })
        .catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error on:', errMsg);
        return Promise.resolve(null);
    });
}
exports.GetUtterances = GetUtterances;
function GetSaveUtterances(nlu) {
    let cognitiveService;
    let implUtterances = [];
    let dbUtterances = [];
    return promises_shared_1.DummyPromise()
        .then(result => {
        if (nlu.cognitiveEngine) {
            return cognitive_analysis_implementation_1.GetCognitiveService(null, nlu.cognitiveEngine);
        }
        else {
            return cognitive_analysis_implementation_1.GetCognitiveService(nlu.vcc, null);
        }
    })
        .then(result => {
        cognitiveService = result;
        if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
            return mssql_service_1.MSSql.GetAzureSubscriptionKeys(nlu.vcc);
        }
        else {
            return Promise.resolve(null);
        }
    })
        .then(result => {
        if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
            if (result && result.find(r => r.isDemo)) {
                return cognitiveService.GetUtterancesList(result.find(r => r.isDemo) ? result.find(r => r.isDemo).subscriptionRegion : null, result.find(r => r.isDemo) ? result.find(r => r.isDemo).subscriptionKey : null, nlu.appId);
            }
            else {
                throw 'SERVER_ERROR_NOT_FOUND_AZURE_CREDENTIALS';
            }
        }
        else {
            return cognitiveService.GetUtterancesFromMD(nlu.appId);
        }
    })
        .then(utterances => {
        implUtterances = utterances;
        return GetUtterances(nlu.vcc, nlu.appId, null, null, null, true);
    })
        .then(utterances => {
        dbUtterances = utterances;
        let promiseArray = [];
        implUtterances.map((iu) => {
            if (!dbUtterances.find(du => du.id === iu.id) && iu.intentLabel !== 'None') {
                logger.debug('[UtteranceService::GetSaveUtterances] Saving:', iu.text);
                promiseArray.push(mssql_service_1.MSSql.SaveUtterance(nlu.vcc, nlu.appId, null, iu.intentLabel, iu.id, iu.text, 'system'));
                if (iu.entityLabels) {
                    iu.entityLabels.map(el => {
                        if (!el.start) {
                            let utteranceText = '';
                            iu.tokenizedText.map((tt, i) => {
                                if (i > 0
                                    &&
                                        iu.text.startsWith(utteranceText + ' '))
                                    utteranceText += ' ';
                                if (i === el.startTokenIndex)
                                    el.start = utteranceText.length;
                                utteranceText += tt;
                                if (i === el.endTokenIndex)
                                    el.end = utteranceText.length - 1;
                            });
                        }
                        promiseArray.push(mssql_service_1.MSSql.SaveUtteranceEntity(nlu.vcc, iu.id, el.entityName, el.start ? el.start : null, el.end ? el.end : null, el.startTokenIndex, el.endTokenIndex));
                    });
                }
            }
        });
        dbUtterances.map(du => {
            if (!implUtterances.find(iu => iu.id === du.id)) {
                promiseArray.push(mssql_service_1.MSSql.DeleteUtterance(nlu.vcc, nlu.appId, du.intentId, du.Id));
            }
        });
        return Promise.resolve(promiseArray);
    })
        .then(result => {
    })
        .then(result => {
        nlu.lastModifiedByUserId = 'system';
        nlu.lastModifiedDate = moment().utc().toDate();
        return nlu_service_1.SaveNLU(nlu.vcc, nlu);
    })
        .then(result => {
        return Promise.resolve(true);
    })
        .catch(error => {
        logger.error('[UtteranceService::GetSaveUtterances] Error on:', error);
        if (typeof error === 'string' && error === 'SERVER_ERROR_NOT_FOUND_AZURE_CREDENTIALS') {
            nlu.lastModifiedByUserId = 'system';
            nlu.lastModifiedDate = moment().utc().toDate();
            return nlu_service_1.SaveNLU(nlu.vcc, nlu);
        }
        else {
            if (error.message) {
                if (error.message.includes(`Cannot find an application with the ID ${nlu.appId}.`)) {
                    return mssql_service_1.MSSql.DeleteNLU(nlu.vcc, nlu.name);
                }
                else if (error.message.includes(`The user is not the owner`)) {
                    return mssql_service_1.MSSql.DeleteNLU(nlu.vcc, nlu.name);
                }
                else if (error.message.includes(`wrong API`)) {
                    return mssql_service_1.MSSql.DeleteNLU(nlu.vcc, nlu.name);
                }
                else {
                    return Promise.resolve(false);
                }
            }
            else {
                return Promise.resolve(false);
            }
        }
    });
}
exports.GetSaveUtterances = GetSaveUtterances;
