"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InitTask = void 0;
const express = require("express");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const mssql_config_1 = require("../config/mssql.config");
const mssql_init_1 = require("../db/mssql.init");
const promises_shared_1 = require("../shared/promises.shared");
const redis_init_1 = require("../db/redis.init");
const process_implementation_1 = require("../implementation/process.implementation");
const audio_transfer_implementation_1 = require("../implementation/audio-transfer.implementation");
const logger_config_1 = require("../config/logger.config");
const transcriptions_download_implementation_1 = require("../implementation/transcriptions-download.implementation");
const transcriptions_upload_implementation_1 = require("../implementation/transcriptions-upload.implementation");
const cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
const redis_service_1 = require("../db/redis.service");
const moment = require("moment");
const supervisor_implementation_1 = require("../implementation/supervisor.implementation");
const cleaner_implementation_1 = require("../implementation/cleaner.implementation");
const file_manager_1 = require("../shared/file-manager");
const semantic_query_implementation_1 = require("../implementation/semantic-query.implementation");
const config_1 = require("../config/config");
const billing_batch_status_implementation_1 = require("../implementation/billing-batch-status.implementation");
const diarization_convert_implementation_1 = require("../implementation/diarization-convert.implementation");
const diarization_analysis_implementation_1 = require("../implementation/diarization-analysis.implementation");
const train_service_1 = require("../services/train.service");
const utterance_service_1 = require("../services/utterance.service");
const cognitive_engine_migrate_implementation_1 = require("../implementation/cognitive-engine-migrate.implementation");
const nlu_service_1 = require("../services/nlu.service");
const arrays_shared_1 = require("../shared/arrays.shared");
const i6_update_implementation_1 = require("../implementation/i6-update.implementation");
const batch_schedule_implementation_1 = require("../implementation/batch-schedule.implementation");
const tipification_analysis_implementation_1 = require("../implementation/tipification-analysis.implementation");
const sentiment_analysis_implementation_1 = require("../implementation/sentiment-analysis.implementation");
const evaluationPlans_implementations_1 = require("../implementation/evaluationPlans.implementations");
const cognitive_engine_migration_model_1 = require("../models/cognitive-engine-migration.model");
const transcriptions_standby_implementation_1 = require("../implementation/transcriptions-standby.implementation");
const semantic_analysis_upload_implementation_1 = require("../implementation/semantic-analysis-upload.implementation");
const semantic_analysis_download_implementation_1 = require("../implementation/semantic-analysis-download.implementation");
const semantic_analysis_convert_audio_implementation_1 = require("../implementation/semantic-analysis-convert-audio.implementation");
log4js.configure(logger_config_1.LOGGER_CONFIG);
let logger = log4js.getLogger('Task');
function InitTask() {
    mssql_config_1.MSSQL_CONFIG.connectionTimeout = 300 * 1000;
    mssql_config_1.MSSQL_CONFIG.requestTimeout = 300 * 1000;
    let sqlConn;
    let redisClient;
    let executionCode = Date.now().toString();
    let app = express();
    logger.info('[schedule.task.ts::InitTask] begin Task executionCode:', executionCode);
    let useRasaApi = config_1.USE_RASA_API;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return redis_init_1.RedisInit(app);
    })
        .then(_ => {
        redisClient = app.get('redisClient');
        return mssql_init_1.MsSqlInit(app);
    })
        .then(_ => {
        sqlConn = app.get('sqlConn');
        let supervisor = new supervisor_implementation_1.SupervisorImplementation();
        return supervisor.Execute(executionCode);
    })
        .then(_ => {
        if (!useRasaApi)
            return InitCognitiveModel();
        return InitRasaApiCognitiveModel();
    })
        .then(_ => {
        if (!useRasaApi)
            return StopAllRunningRasaModelFromTest();
        return true;
    })
        .then(_ => {
        return MigrateRasaCognitiveEngine();
    })
        .then(_ => {
        return TrainSystemCognitiveModel();
    })
        .then(_ => {
        return InitBatchSchedule(executionCode);
    })
        .then(_ => {
        return InitEvaluationPlansExpired(executionCode);
    })
        .then((result) => {
        if (!result)
            logger.warn('[schedule.task::InitTask] InitCognitiveModel function failed');
        let billingBatchStatus = new billing_batch_status_implementation_1.BillingBatchStatusImplementation();
        return billingBatchStatus.Execute(executionCode);
    })
        .then((result) => {
        if (!(result && result.data && result.data['allowedBatches'])
            &&
                !(result && result.data && result.data['cancelledBatches']))
            logger.debug('[schedule.task.ts::InitTask] Result from billingBatchStatus: SERVER_ERROR_BATCHES_NOT_FOUND');
        let promiseArray = [];
        let process = new process_implementation_1.ProcessImplementation();
        let i6Update = new i6_update_implementation_1.I6UpdateImplementation();
        let audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
        let diarizarionConvert = new diarization_convert_implementation_1.DiarizationConvertImplementation();
        let transcriptionsUpload = new transcriptions_upload_implementation_1.TranscriptionsUploadImplementation();
        let transcriptionsStandby = new transcriptions_standby_implementation_1.TranscriptionsStandbyImplementation();
        let transcriptionsDownload = new transcriptions_download_implementation_1.TranscriptionsDownloadImplementation();
        let diarizarionAnalysis = new diarization_analysis_implementation_1.DiarizationAnalysisImplementation();
        let semanticAnalysisUpload = new semantic_analysis_upload_implementation_1.SemanticAnalysisUploadImplementation();
        let semanticAnalysisDownload = new semantic_analysis_download_implementation_1.SemanticAnalysisDownloadImplementation();
        let semanticAnalysisConvertAudio = new semantic_analysis_convert_audio_implementation_1.SemanticAnalysisConvertAudioImplementation();
        let semanticQuery = new semantic_query_implementation_1.SemanticQueryImplementation();
        let cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
        let cognitiveEngineMigationImplementation = new cognitive_engine_migrate_implementation_1.CognitiveEngineMigrateImplementation();
        let batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
        let tipificationAnalysisImplementation = new tipification_analysis_implementation_1.TipificationAnalysisImplementation();
        let sentimentAnalysis = new sentiment_analysis_implementation_1.SentimentAnalysisImplementation();
        promiseArray.push(batchScheduleImplementation.Execute(executionCode, config_1.RATE_RECURSIVE_BATCH_SCHEDULE));
        promiseArray.push(process.Execute(executionCode, config_1.RATE_RECURSIVE_PROCESS));
        promiseArray.push(i6Update.Execute(executionCode, config_1.RATE_RECURSIVE_I6_UPDATE));
        promiseArray.push(audioTransfer.Execute(executionCode, config_1.RATE_RECURSIVE_AUDIO_TRANSFER));
        promiseArray.push(diarizarionConvert.Execute(executionCode, config_1.RATE_RECURSIVE_DIARIZATION_CONVERT));
        promiseArray.push(transcriptionsUpload.Execute(executionCode, config_1.RATE_RECURSIVE_TRANSCRIPTION, result.data?.allowedBatches.transcription ? result.data.allowedBatches.transcription : [], result.data?.cancelledBatches.transcription ? result.data.cancelledBatches.transcription : []));
        promiseArray.push(transcriptionsStandby.Execute(executionCode, config_1.RATE_RECURSIVE_TRANSCRIPTION));
        promiseArray.push(transcriptionsDownload.Execute(executionCode, config_1.RATE_RECURSIVE_TRANSCRIPTION));
        promiseArray.push(diarizarionAnalysis.Execute(executionCode, config_1.RATE_RECURSIVE_DIARIZATION_ANALYSIS));
        promiseArray.push(sentimentAnalysis.Execute(executionCode, config_1.RATE_RECURSIVE_SENTIMENT_ANALYSIS));
        promiseArray.push(semanticAnalysisUpload.Execute(executionCode, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_UPLOAD));
        promiseArray.push(semanticAnalysisDownload.Execute(executionCode, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_DOWNLOAD));
        promiseArray.push(semanticAnalysisConvertAudio.Execute(executionCode, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS));
        promiseArray.push(semanticQuery.Execute(executionCode, config_1.RATE_RECURSIVE_SEMANTIC_QUERY, result.data?.allowedBatches.semanticAnalysis ? result.data.allowedBatches.semanticAnalysis : [], result.data?.cancelledBatches.semanticAnalysis ? result.data.cancelledBatches.semanticAnalysis : []));
        promiseArray.push(cognitiveAnalysis.Execute(executionCode, config_1.RATE_RECURSIVE_COGNITIVE_ANALYSIS, result.data?.allowedBatches.cognitiveAnalysis ? result.data.allowedBatches.cognitiveAnalysis : [], result.data?.cancelledBatches.cognitiveAnalysis ? result.data.cancelledBatches.cognitiveAnalysis : []));
        promiseArray.push(tipificationAnalysisImplementation.Execute(executionCode, config_1.RATE_RECURSIVE_TIPIFICATION_ANALYSIS, result.data?.allowedBatches.cognitiveAnalysis ? result.data.allowedBatches.cognitiveAnalysis : [], result.data?.cancelledBatches.cognitiveAnalysis ? result.data.cancelledBatches.cognitiveAnalysis : []));
        promiseArray.push(cognitiveEngineMigationImplementation.Execute(executionCode));
        return Promise.all(promiseArray);
    })
        .then((results) => {
        logger.info('[schedule.task::InitTask] executionCode:', executionCode, 'Receiving result from implementations');
        if (results.filter((result) => { return (result !== true); }).length > 0) {
            logger.error('[schedule.task::InitTask] executionCode:', executionCode, 'getting error results');
            return Promise.resolve(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(_ => {
        logger.info('[schedule.task::InitTask] executionCode:', executionCode, 'Cleaning files');
        let promiseArray = [];
        promiseArray.push(file_manager_1.CleanupTempFile(config_1.COGNITIVE_ANALYSIS_FOLDER, config_1.PERSISTENCE));
        promiseArray.push(file_manager_1.CleanupTempFile(config_1.SEMANCTIC_ANALYSIS_FOLDER, config_1.PERSISTENCE));
        promiseArray.push(file_manager_1.CleanupTempFile(config_1.QUERY_FOLDER, config_1.PERSISTENCE));
        return Promise.all(promiseArray);
    })
        .then(_ => {
        let cleaner = new cleaner_implementation_1.CleanerImplementation();
        return cleaner.Execute(executionCode);
    })
        .then(_ => {
        logger.info('[schedule.task::InitTask] executionCode:', executionCode, 'Task has been finished');
        if (sqlConn)
            sqlConn.close();
        if (redisClient)
            redisClient.quit();
        setTimeout(() => {
            process.exit(0);
        }, 200);
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[schedule.task::InitTask] executionCode:', executionCode, 'Task has been finished', executionCode, 'getting error:', err);
        sqlConn.close();
        if (redisClient)
            redisClient.quit();
        setTimeout(() => {
            process.exit(0);
        }, 200);
        return Promise.resolve(false);
    });
}
exports.InitTask = InitTask;
function MigrateRasaCognitiveEngine() {
    let cognitiveService;
    let vccToMigrate;
    let vccNluUsage;
    return promises_shared_1.DummyPromise().then(result => {
        return mssql_service_1.MSSql.GetNluIntentUtteranceToExport('ALL', 2);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_NLUS_NOT_FOUND';
        vccToMigrate = result.quantity;
        vccNluUsage = result.usage.InUse;
        logger.debug("[CognitiveEngineMigrateImplementation::Execute] vcctoMigrate ", vccToMigrate);
        return redis_service_1.Redis.GetCognitiveEngineMigrationData();
    })
        .then(results => {
        let promiseArray = [];
        if (typeof results != "object" || !results) {
            logger.debug("[CognitiveEngineMigrateImplementation::Execute] NOT_FOUND_DATA ", results);
            if (vccToMigrate.totalNlu && vccToMigrate.totalNlu > 0 && !vccNluUsage) {
                logger.info('[schedule.task::MigrateRasaToRasaApi] Migrating:', vccToMigrate.vcc);
                promiseArray.push(redis_service_1.Redis.AddTaskCognitiveEngineMigration(vccToMigrate.vcc, new cognitive_engine_migration_model_1.CognitiveEngineMigrationConfig('OLD_RASA_ENGINE', 'NEW_RASA_ENGINE', null, null, vccToMigrate.totalNlu, vccToMigrate.totalIntent, vccToMigrate.totalUtterance, vccToMigrate.totalEntity)));
            }
        }
        return Promise.all(promiseArray);
    })
        .then(result => {
        return Promise.resolve(true);
    })
        .catch((error) => {
        let errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::TrainSystemCognitiveModel] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function TrainSystemCognitiveModel() {
    let cognitiveService;
    let needUpdateNLUToUtterancesBulk = false;
    let needUpdateNLUToTrainDueToRasaUpdate = false;
    return promises_shared_1.DummyPromise().then(result => {
        return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
    })
        .then(result => {
        cognitiveService = result;
        return mssql_service_1.MSSql.GetNLUs('ALL', null, null, null, true);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_SYSTEM_APPS_NOT_FOUND';
        let promiseArray = [];
        result = arrays_shared_1.SortObjectArrayByShuffle(result);
        let timeOut = 0;
        result.forEach((r, i) => {
            if (r.vcc !== 'system' || (r.vcc === 'system' && r.name.includes('sentiment-'))) {
                if (r.needTrain
                    &&
                        r.lastModifiedDate
                    &&
                        (moment().utc() > moment(r.lastModifiedDate).utc().add(5, 'minutes'))
                    &&
                        !config_1.USE_RASA_API
                    &&
                        r.cognitiveEngine === 'RASA') {
                    logger.info('[schedule.task::TrainSystemCognitiveModel] Training system app:', r.appId);
                    timeOut = i * 1000;
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(() => {
                            return resolve(train_service_1.TrainApp(cognitiveService, null, r));
                        }, timeOut);
                    }));
                }
            }
        });
        result.filter(r => r.lastModifiedByUserId === 'dev-1'
            &&
                r.isSystemApp === true
            &&
                r.lastTrainedDateTime)
            .map((n, i) => {
            if (i === 0) {
                promiseArray.push(utterance_service_1.GetSaveUtterances(n));
            }
        });
        return Promise.all(promiseArray);
    })
        .then(result => {
        return nlu_service_1.GetNLUToUtterancesBulk(false);
    })
        .then(result => {
        let promiseArray = [];
        if (result && result.length) {
            needUpdateNLUToUtterancesBulk = true;
            result = arrays_shared_1.SortObjectArrayByShuffle(result);
            result.map((n, i) => {
                if (n.lastModifiedByUserId === 'dev-1' && i === 0) {
                    promiseArray.push(utterance_service_1.GetSaveUtterances(n));
                }
            });
        }
        return Promise.all(promiseArray);
    })
        .then(result => {
        return nlu_service_1.GetNLUToTrainDueToRasaUpdate(false);
    })
        .then(result => {
        let promiseArray = [];
        if (result && result.length) {
            needUpdateNLUToTrainDueToRasaUpdate = true;
            result = arrays_shared_1.SortObjectArrayByShuffle(result);
            result.map((n, i) => {
                if (i === 0
                    &&
                        n.needTrain
                    &&
                        n.lastModifiedByUserId === 'dev-2') {
                    promiseArray.push(train_service_1.TrainApp(cognitiveService, null, n));
                }
            });
        }
        return Promise.all(promiseArray);
    })
        .then(result => {
        let promiseArray = [];
        if (needUpdateNLUToUtterancesBulk)
            promiseArray.push(nlu_service_1.GetNLUToUtterancesBulk(true));
        if (needUpdateNLUToTrainDueToRasaUpdate)
            promiseArray.push(nlu_service_1.GetNLUToTrainDueToRasaUpdate(true));
        return Promise.all(promiseArray);
    })
        .then(result => {
        return Promise.resolve(true);
    })
        .catch((error) => {
        let errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::TrainSystemCognitiveModel] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function InitCognitiveModel() {
    let cognitiveService;
    return promises_shared_1.DummyPromise().then(result => {
        return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
    }).then(result => {
        cognitiveService = result;
        let promiseArray = [];
        promiseArray.push(redis_service_1.Redis.GetCurrentBatchesRasaModels());
        promiseArray.push(redis_service_1.Redis.GetRunningRasaModels());
        return Promise.all(promiseArray);
    }).then(result => {
        logger.debug('[schedule.task::InitCognitiveModel] uniqueCurrentBatchesNlus: ', result[0]);
        logger.debug('[schedule.task::InitCognitiveModel] allRunningRasaModels: ', result[1]);
        let allCurrentBatchesNlus = [];
        if (result[0]) {
            let attributesArray = Object.getOwnPropertyNames(result[0]);
            attributesArray.map(aa => {
                allCurrentBatchesNlus = allCurrentBatchesNlus.concat(JSON.parse(result[0][aa].toLowerCase()));
            });
        }
        let uniqueCurrentBatchesNlus = allCurrentBatchesNlus.filter((item, i, ar) => ar.indexOf(item) === i);
        logger.debug('[schedule.task::InitCognitiveModel] uniqueCurrentBatchesNlus: ', uniqueCurrentBatchesNlus);
        let allRunningRasaModels = [];
        if (result[1]) {
            let attributesArray = Object.getOwnPropertyNames(result[1]);
            attributesArray.map(aa => {
                let arr = aa.split('::');
                allRunningRasaModels = allRunningRasaModels.concat(arr[1] + '::' + arr[2]);
            });
        }
        let uniqueRunningRasaModels = allRunningRasaModels.filter((item, i, ar) => ar.indexOf(item) === i);
        logger.debug('[schedule.task::InitCognitiveModel] uniqueRunningRasaModels: ', uniqueRunningRasaModels);
        let promiseArray = [];
        let index = 0;
        uniqueCurrentBatchesNlus.map(ucbn => {
            if (!uniqueRunningRasaModels.includes(ucbn)) {
                let timeOut = index * 1000;
                promiseArray.push(new Promise((resolve, reject) => {
                    setTimeout(() => {
                        return resolve(cognitiveService.PublishApp(null, null, ucbn, true, ''));
                    }, timeOut);
                }));
                index++;
            }
        });
        uniqueRunningRasaModels.map(urrm => {
            if (!uniqueCurrentBatchesNlus.includes(urrm)) {
                let timeOut = index * 1000;
                promiseArray.push(new Promise((resolve, reject) => {
                    setTimeout(() => {
                        return resolve(cognitiveService.PublishApp(null, null, urrm, false, ''));
                    }, timeOut);
                }));
                index++;
            }
        });
        return Promise.all(promiseArray);
    }).then(result => {
        logger.info('[schedule.task::InitCognitiveModel] Result: ' + result);
        return Promise.resolve(true);
    }).catch((error) => {
        let errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::InitCognitiveModel] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function InitRasaApiCognitiveModel() {
    let cognitiveService;
    return promises_shared_1.DummyPromise().then(result => {
        return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
    })
        .then(result => {
        cognitiveService = result;
        let promiseArray = [];
        promiseArray.push(redis_service_1.Redis.GetCurrentBatchesRasaModels());
        promiseArray.push(redis_service_1.Redis.GetRunningRasaModels());
        promiseArray.push(mssql_service_1.MSSql.GetNLUs('ALL', null, null, null, null));
        return Promise.all(promiseArray);
    }).then(result => {
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] uniqueCurrentBatchesNlus: ', result[0]);
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] allRunningRasaModels: ', result[1]);
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] NeedTrainRasaModels: ', result[2]);
        let allNluNeedTrain = result[2].filter(nlu => nlu.needTrain);
        let allNluNeedPublish = result[2].filter(nlu => nlu.needPublish);
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] allNluNeedTrain: ', allNluNeedTrain);
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] allNluNeedPublish: ', allNluNeedPublish);
        let allCurrentBatchesNlus = [];
        if (result[0]) {
            let attributesArray = Object.getOwnPropertyNames(result[0]);
            attributesArray.map(aa => {
                allCurrentBatchesNlus = allCurrentBatchesNlus.concat(JSON.parse(result[0][aa].toLowerCase()));
            });
        }
        let uniqueCurrentBatchesNlus = allCurrentBatchesNlus.filter((item, i, ar) => ar.indexOf(item) === i);
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] uniqueCurrentBatchesNlus: ', uniqueCurrentBatchesNlus);
        let allRunningRasaModels = [];
        if (result[1]) {
            let attributesArray = Object.getOwnPropertyNames(result[1]);
            attributesArray.map(aa => {
                let arr = aa.split('::');
                allRunningRasaModels = allRunningRasaModels.concat(arr[1] + '::' + arr[2]);
            });
        }
        let uniqueRunningRasaModels = allRunningRasaModels.filter((item, i, ar) => ar.indexOf(item) === i);
        logger.debug('[schedule.task::InitRasaApiCognitiveModel] uniqueRunningRasaModels: ', uniqueRunningRasaModels);
        let promiseArray = [];
        let index = 0;
        uniqueCurrentBatchesNlus.map(ucbn => {
            if (!uniqueRunningRasaModels.includes(ucbn)) {
                let nluNeedTrain = allNluNeedTrain.find(nnt => nnt.vcc + '::' + nnt.name === ucbn);
                let timeOut = index * 1000;
                if (nluNeedTrain && nluNeedTrain.cognitiveEngine === 'RASAAPI' && config_1.USE_RASA_API) {
                    nluNeedTrain.trainMode = 'A';
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(() => {
                            return resolve(train_service_1.TrainApp(cognitiveService, null, nluNeedTrain));
                        }, timeOut);
                    }));
                    index++;
                }
            }
        });
        uniqueRunningRasaModels.map(urrm => {
            if (!uniqueCurrentBatchesNlus.includes(urrm)) {
                let nluNeedTrain = allNluNeedTrain.find(nnt => nnt.vcc + '::' + nnt.name === urrm);
                let timeOut = index * 1000;
                if (nluNeedTrain && nluNeedTrain.cognitiveEngine === 'RASAAPI' && config_1.USE_RASA_API) {
                    nluNeedTrain.trainMode = 'A';
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(() => {
                            return resolve(train_service_1.TrainApp(cognitiveService, null, nluNeedTrain));
                        }, timeOut);
                    }));
                    index++;
                }
            }
        });
        uniqueCurrentBatchesNlus.map(ucbn => {
            if (!uniqueRunningRasaModels.includes(ucbn)) {
                let nluNeedPublish = allNluNeedPublish.find(nnt => nnt.vcc + '::' + nnt.name === ucbn);
                let timeOut = index * 1000;
                if (nluNeedPublish && nluNeedPublish.cognitiveEngine === 'RASAAPI' && config_1.USE_RASA_API) {
                    promiseArray.push(new Promise((resolve, reject) => {
                        setTimeout(() => {
                            return resolve(train_service_1.PublishRasaApiApp(cognitiveService, nluNeedPublish));
                        }, timeOut);
                    }));
                    index++;
                }
            }
        });
        uniqueRunningRasaModels.map(urrm => {
            if (!uniqueCurrentBatchesNlus.includes(urrm)) {
                let nluNeedPublish = allNluNeedPublish.find(nnt => nnt.vcc + '::' + nnt.name === urrm);
                let timeOut = index * 1000;
                if (nluNeedPublish && nluNeedPublish.cognitiveEngine === 'RASAAPI' && config_1.USE_RASA_API) {
                    promiseArray.push(new Promise((resolve, reject) => {
                        setTimeout(() => {
                            return resolve(train_service_1.PublishRasaApiApp(cognitiveService, nluNeedPublish));
                        }, timeOut);
                    }));
                    index++;
                }
            }
        });
        return Promise.all(promiseArray);
    }).then(result => {
        logger.info('[schedule.task::InitRasaApiCognitiveModel] Result: ' + result);
        return Promise.resolve(true);
    }).catch((error) => {
        let errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::InitRasaApiCognitiveModel] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function StopAllRunningRasaModelFromTest() {
    let cognitiveService;
    return promises_shared_1.DummyPromise().then(result => {
        return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
    }).then(result => {
        cognitiveService = result;
        return redis_service_1.Redis.GetAllRunningRasaModelFromTest();
    })
        .then(result => {
        let promiseArray = [];
        let index = 0;
        if (result !== null) {
            for (let key in result) {
                let value = JSON.parse(result[key]);
                let timeOut = index * 1000;
                if (moment.utc(value.created).add(45, 'minutes').isBefore(moment().utc())) {
                    promiseArray.push(new Promise((resolve, reject) => {
                        setTimeout(() => {
                            return resolve(cognitiveService.StopApp(value.id, value.port, 'TEST'));
                        }, timeOut);
                    }));
                }
                index++;
            }
        }
        return Promise.all(promiseArray);
    }).then(result => {
        return Promise.resolve(true);
    })
        .catch((error) => {
        let errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_STOP_EXPIRED_MODELS_FROM_TEST';
        logger.error('[schedule.task::StopAllRunningModelsFromTest] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function InitBatchSchedule(executionCode, force) {
    let batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
    return promises_shared_1.DummyPromise()
        .then(_ => {
        let promiseArray = [];
        let startRange01 = moment();
        startRange01.set('hour', 0);
        startRange01.set('minute', 0);
        startRange01.set('second', 0);
        let endRange01 = moment();
        endRange01.set('hour', 0);
        endRange01.set('minute', 4);
        endRange01.set('second', 59);
        let startRange02 = moment();
        startRange02.set('hour', 12);
        startRange02.set('minute', 0);
        startRange02.set('second', 0);
        let endRange02 = moment();
        endRange02.set('hour', 12);
        endRange02.set('minute', 4);
        endRange02.set('second', 59);
        let firstRange = moment().isBetween(startRange01, endRange01);
        let secondRange = moment().isBetween(startRange02, endRange02);
        if (firstRange || secondRange || force) {
            logger.info('[schedule.task::InitBatchSchedule] Registering scheduleExecutions on redis at:', moment().format('YYYY-MM-DD HH:mm:ss'));
            promiseArray.push(batchScheduleImplementation.GetNewScheduledBatchExecutions(executionCode));
            if (firstRange && [1, 4].includes(moment().isoWeekday()) || force) {
                logger.info('[schedule.task::InitBatchSchedule] Generating new scheduleExecutions on db at:', moment().format('YYYY-MM-DD HH:mm:ss'));
                promiseArray.push(batchScheduleImplementation.GenerateNewScheduleBatchExecution(executionCode));
            }
        }
        return Promise.all(promiseArray);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((error) => {
        let errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::InitBatchSchedule] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function InitEvaluationPlansExpired(executionCode) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        let evaluationPlansImplementation = new evaluationPlans_implementations_1.EvaluationPlansImplementation();
        let startRange01 = moment();
        startRange01.set('hour', 0);
        startRange01.set('minute', 0);
        startRange01.set('second', 0);
        let endRange01 = moment();
        endRange01.set('hour', 0);
        endRange01.set('minute', 4);
        endRange01.set('second', 59);
        logger.info('[schedule.task::InitEvaluationPlansExpired] currentDate:', moment().format('YYYY-MM-DD HH:mm:ss'));
        logger.info('[schedule.task::InitEvaluationPlansExpired] startRange01:', startRange01.format('YYYY-MM-DD HH:mm:ss'));
        logger.info('[schedule.task::InitEvaluationPlansExpired] endRange01:', endRange01.format('YYYY-MM-DD HH:mm:ss'));
        let firstRange = moment().isBetween(startRange01, endRange01);
        if (firstRange) {
            logger.info('[schedule.task::InitEvaluationPlansExpired] Update EvaluationPlans to Expired on BD at:', moment().format('YYYY-MM-DD HH:mm:ss'));
            return evaluationPlansImplementation.Execute(new Date(startRange01.toDate()));
        }
        return Promise.resolve(true);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((error) => {
        logger.error('[schedule.task::InitEvaluationPlansExpired] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
