"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const moment = require("moment");
const log4js = require("log4js");
const mssql_config_1 = require("../config/mssql.config");
const http_request_occ_1 = require("../shared/http-request-occ");
const mssql_service_1 = require("../db/mssql.service");
const mssql_init_1 = require("../db/mssql.init");
const integration_config_1 = require("../config/integration.config");
const TYPE = '5.5';
let loggerAllegro = log4js.getLogger('Task');
function ExecuteTask() {
    loggerAllegro.info('ExecuteTask (inConcert): Inicia sincronizacion de Usuarios');
    let users = [];
    let usersToSave;
    let nowUtc = moment().utc().toDate();
    let request = {};
    request.columns = ["Id", "Mail", "Password", "UserGroup", "Name", "FirstSurname", "LastName", "LastSurname", "LanguageId", "countryCode", "TimeZoneId"];
    request.limit = 1000;
    if (TYPE === 'OCC' || TYPE === 'INTEGRATED') {
        loggerAllegro.info('ExecuteTask (inConcert): Inicia sincronizacion de Usuarios de i6');
        return mssql_service_1.MSSql.GetAPIUsersOCC()
            .then(apiUsers => {
            if (apiUsers?.length === 0)
                return Promise.resolve(null);
            let promiseArray = [];
            let vcc;
            apiUsers.forEach(apiUser => {
                promiseArray.push(http_request_occ_1.Login(apiUser.user + '@' + apiUser.vcc, apiUser.password)
                    .then(resultLogin => {
                    if (!resultLogin.accessToken)
                        return Promise.resolve(null);
                    return http_request_occ_1.SearchUsers(resultLogin.accessToken, request);
                }).then(resultSearchUsers => {
                    if (!resultSearchUsers.status && resultSearchUsers.rows?.length === 0)
                        return Promise.resolve(null);
                    users = [];
                    usersToSave = [];
                    usersToSave = resultSearchUsers.rows.filter(resultSearchUsers => resultSearchUsers.UserGroup.toLowerCase() === 'agent' || resultSearchUsers.UserGroup.toLowerCase() === 'qualityauditor');
                    for (let row of usersToSave) {
                        let user = {};
                        user.vcc = apiUser.vcc;
                        user.id = row.Id;
                        user.email = row.Mail ? row.Mail : '';
                        user.firstname = row.Name + (row.LastName ? ' ' + row.LastName : '');
                        user.lastname = row.FirstSurname + (row.LastSurname ? ' ' + row.LastSurname : '');
                        user.password = row.Password ? row.Password : '';
                        user.role = row.UserGroup ? row.UserGroup : '';
                        user.country = row.countryCode ? row.countryCode : '';
                        user.language = row.LanguageId ? row.LanguageId : '';
                        user.timeZone = row.TimeZoneId ? row.TimeZoneId : '';
                        user.preferences = null;
                        user.isSystemUser = false;
                        user.limitClients = null;
                        user.createdDate = nowUtc;
                        user.createdByUserId = 'admin';
                        user.lastModifiedDate = nowUtc;
                        user.lastModifiedByUserId = 'admin';
                        users.push(user);
                    }
                    return Promise.resolve(users);
                }).then(usersToSave => {
                    if (usersToSave?.length === 0)
                        return Promise.resolve(null);
                    return ExecuteCreateUser(usersToSave);
                }).catch((err) => {
                    loggerAllegro.error('ExecuteTask (inConcert): Error: ' + err.message);
                    return Promise.resolve(null);
                }));
            });
            return Promise.all(promiseArray).then(results => {
                if (results.filter(result => { return (result !== true); }).length > 0) {
                    return false;
                }
                else {
                    return true;
                }
            });
        }).catch((err) => {
            loggerAllegro.error('ExecuteTask (inConcert): Error: ' + err.message);
            return Promise.resolve(null);
        });
    }
    else if (TYPE === '5.5') {
        loggerAllegro.info('ExecuteTask (inConcert): Inicia sincronizacion de Usuarios de Allegro');
        let flag = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
        return mssql_service_1.MSSql.GetIntegrationUsers(integration_config_1.IP_BD, flag)
            .then(usersToSave => {
            if (usersToSave?.length === 0)
                return Promise.resolve(null);
            return ExecuteCreateUser(usersToSave);
        }).catch((err) => {
            loggerAllegro.error('ExecuteTask (inConcert): Error: ' + err.message);
            return Promise.resolve(null);
        });
    }
    else {
        loggerAllegro.error('ExecuteTask (inConcert): No cumplio con el tipo de ambiente');
        return Promise.resolve(null);
    }
}
function ExecuteCreateUser(users) {
    let promiseArray = [];
    if (users) {
        users.forEach(user => {
            promiseArray.push(mssql_service_1.MSSql.SaveIntegratedUser(user.vcc, user.id, user.email, user.firstname, user.lastname, user.password, user.role.toLowerCase() === 'agent' ? 'quality_agent' : (user.role.toLowerCase() === 'qualityauditor' ? 'quality_auditor' : ''), user.country, user.language, user.timeZone, user.preferences, user.isSystemUser, user.limitClients, user.createdDate, user.createdByUserId, user.lastModifiedDate, user.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
if (require.main === module) {
    let app = express();
    let sqlConn;
    mssql_config_1.MSSQL_CONFIG.connectionTimeout = 300 * 1000;
    mssql_config_1.MSSQL_CONFIG.requestTimeout = 300 * 1000;
    mssql_init_1.MsSqlInit(app)
        .then((result) => {
        sqlConn = app.get('sqlConn');
        return ExecuteTask();
    }).then(results => {
        loggerAllegro.info('Task (allegro): Termino la sincronizacion de usuarios');
        sqlConn.close();
        setTimeout(function () {
            process.exit(0);
        }, 200);
    }).catch(error => {
        loggerAllegro.error('Task (allegro): Termino la sincronizacion de usuarios con errores :', error);
        sqlConn.close();
        setTimeout(function () {
            process.exit(0);
        }, 200);
    });
}
