"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const moment = require("moment");
const log4js = require("log4js");
let ab2str = require('arraybuffer-to-string');
var md5 = require('md5');
const mssql_config_1 = require("../config/mssql.config");
const mssql_service_1 = require("../db/mssql.service");
const mssql_init_1 = require("../db/mssql.init");
const idatha_service_1 = require("../services/idatha.service");
const cassandra_service_1 = require("../db/cassandra.service");
const cassandra_init_1 = require("../db/cassandra.init");
const integration_config_1 = require("../config/integration.config");
const config_1 = require("../config/config");
const user_service_1 = require("../services/user.service");
const category_service_1 = require("../services/category.service");
const role_service_1 = require("../services/role.service");
const vcc_service_1 = require("../services/vcc.service");
const feeling_service_1 = require("../services/feeling.service");
const neuro_linguistic_programming_service_1 = require("../services/neuro-linguistic-programming.service");
const environment_service_1 = require("../services/environment.service");
const api_shared_1 = require("../shared/api.shared");
const TYPE = '5.5';
let loggerAllegro = log4js.getLogger('Task');
function ExecuteTask() {
    loggerAllegro.info('ExecuteTask (inConcert): Inicia sincronizacion de VCCs');
    let VCCs = [];
    let vcc = '';
    let description = '';
    let nowUtc = moment().utc().toDate();
    let obji, objj, leni, lenj;
    if (TYPE === 'OCC') {
        loggerAllegro.info('ExecuteTask (inConcert): Inicia sincronizacion de VCCs de i6');
        return cassandra_service_1.CassandraSql.GetCassandraVCCs()
            .then(result => {
            VCCs = result;
            return Promise.resolve(VCCs);
        }).then(result => {
            if (result?.length === 0)
                return Promise.resolve(null);
            return ExecuteCreateVCC(result);
        }).catch((err) => {
            loggerAllegro.error('ExecuteTask (inConcert): Error: ' + err.message);
            return Promise.resolve(null);
        });
    }
    else if (TYPE === '5.5' || TYPE === 'INTEGRATED') {
        loggerAllegro.info('ExecuteTask (inConcert): Inicia sincronizacion de VCCs con 5.5/Integrado');
        let flag = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
        return mssql_service_1.MSSql.GetIntegrationVCCs(integration_config_1.IP_BD, flag, TYPE)
            .then(result => {
            if (result?.length === 0)
                return Promise.resolve(null);
            return ExecuteCreateVCC(result);
        }).catch((err) => {
            loggerAllegro.error('ExecuteTask (inConcert): Error: ' + err.message);
            return Promise.resolve(null);
        });
    }
    else {
        loggerAllegro.error('ExecuteTask (inConcert): no cumplio con el tipo de ambiente');
        return Promise.resolve(null);
    }
}
function ExecuteCreateVCC(vccs) {
    let promiseArray = [];
    if (vccs) {
        vccs.forEach(vcc => {
            promiseArray.push(CreateVCC(vcc.id, vcc));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function CreateVCC(vcc, objVCC) {
    let nowUtc = moment().utc().toDate();
    let api;
    loggerAllegro.info('[vcc-bulk.task::CreateVCC] Checking if exists vcc ' + vcc);
    return mssql_service_1.MSSql.GetVCC(vcc)
        .then(resultVCC => {
        if (resultVCC)
            throw 'Ya existe un vcc con el identificador \'' + vcc + '\'';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] The vcc name is available');
        return vcc_service_1.SaveVCC(vcc, objVCC);
    }).then(resultSaveVCC => {
        if (!resultSaveVCC)
            throw 'Se produjo un error al guardar el vcc';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting the GetApi for system');
        return api_shared_1.GetAPIDataFromConfig('system');
    }).then((result) => {
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Receiving the system\'s info');
        api = {
            'speechmatics': result.find((x) => {
                return x.apiName === 'SpeechMatics';
            }), 'idatha': result.find((x) => {
                return x.apiName === 'IDATHA';
            })
        };
        return Promise.resolve(api);
    }).then(resultGetAPI => {
        if (!resultGetAPI)
            throw 'Se produjo un error al recuperar datos del api';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Registering on idatha');
        let userName = config_1.REALM_USER_NAME;
        let password = config_1.REALM_PASSWORD;
        let idathaAuth = { vcc: vcc, userName: userName, password: password, realm: vcc };
        return idatha_service_1.IdathaRegister(idathaAuth);
    }).then(resultIdathaRegister => {
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] ExecuteTask (inConcert): accessToken: ' + resultIdathaRegister);
        let user = {};
        user.id = 'analytic';
        user.isSystemUser = false;
        user.role = 'administrator';
        user.firstname = 'user' + objVCC.id;
        user.password = md5('analytic' + 'inc0nc3rt');
        user.createdDate = nowUtc;
        user.createdByUserId = 'admin';
        user.lastModifiedDate = nowUtc;
        user.lastModifiedByUserId = 'admin';
        user.language = null;
        return user_service_1.SaveUser(vcc, user);
    }).then(resultSaveUser => {
        if (!resultSaveUser)
            throw 'USER_SERVER_ERROR_ON_SAVE';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting roles from system');
        return mssql_service_1.MSSql.GetRoles('system', 'name', '');
    }).then(resultGetRoles => {
        if (!resultGetRoles)
            throw 'Se produjo un error al recuperar roles';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Registering roles for new vcc');
        return role_service_1.SaveRoles(vcc, resultGetRoles);
    }).then(resultSaveRoles => {
        if (!resultSaveRoles)
            throw 'Se produjo un error al guardar los roles';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting role for system');
        return mssql_service_1.MSSql.GetRole('system', null);
    }).then(resultGetRole => {
        if (!resultGetRole)
            throw 'Se produjo un error al recuperar permisos y rutas';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving roles routes');
        return role_service_1.SaveRolesRoutes(vcc, resultGetRole);
    }).then(resultSaveRolesRoutes => {
        if (!resultSaveRolesRoutes)
            throw 'Se produjo un error al guardar los permisos y rutas';
        return mssql_service_1.MSSql.GetRolePermissionsGroups('system');
    }).then(resultGetRolePermissionsGroups => {
        if (!resultGetRolePermissionsGroups)
            throw 'Se produjo un error al recuperar grupos de rol';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving role perission groups for new vcc');
        return role_service_1.SaveRolePermissionsGroup(vcc, resultGetRolePermissionsGroups);
    }).then(resultSaveRolePermissionsGroup => {
        if (!resultSaveRolePermissionsGroup)
            throw 'Se produjo un error al guardar permisos de grupo';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting feelings from system');
        return mssql_service_1.MSSql.GetFeelings('system', 'name', '');
    }).then(resultGetFeelings => {
        if (!resultGetFeelings)
            throw 'Se produjo un error al recuperar tipos de sentimientos';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving feelings for new vcc');
        return feeling_service_1.SaveFeelings(vcc, resultGetFeelings);
    }).then(resultSaveFeeling => {
        if (!resultSaveFeeling)
            throw 'Se produjo un error al guardar tipos de sentimientos';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting NeuroLinguisticProgrammings from system');
        return mssql_service_1.MSSql.GetNeuroLinguisticProgrammings('system', 'name', '');
    }).then(resultGetNeuroLinguisticProgrammings => {
        if (!resultGetNeuroLinguisticProgrammings)
            throw 'Se produjo un error al recuperar los tipos de pnl';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving NeuroLinguisticProgrammings for new vcc');
        return neuro_linguistic_programming_service_1.SaveNeuroLinguisticProgrammings(vcc, resultGetNeuroLinguisticProgrammings);
    }).then(resultSaveNeuroLinguisticProgramming => {
        if (!resultSaveNeuroLinguisticProgramming)
            throw 'Se produjo un error al guardar tipos de pnl';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting words from system');
        return mssql_service_1.MSSql.GetWords('system', 'NeuroLinguisticProgrammingId', '', 'init', 'es');
    }).then(resultGetWords => {
        if (!resultGetWords)
            throw 'Se produjo un error al recuperar las palabras de pnl';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving words for new vcc');
        return neuro_linguistic_programming_service_1.SaveWord(vcc, resultGetWords);
    }).then(resultSaveWord => {
        if (!resultSaveWord)
            throw 'Se produjo un error al guardar las palabras de pnl';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting category\'s names from system');
        return mssql_service_1.MSSql.GetCategories('system', 'name', '');
    }).then(resultGetCategories => {
        if (!resultGetCategories)
            throw 'Se produjo un error al recuperar las categorías';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving category\'s names for new vcc');
        return category_service_1.SaveCategories(vcc, resultGetCategories);
    }).then(resultSaveCategories => {
        if (!resultSaveCategories)
            throw 'Se produjo un error al guardar las categorias';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Getting Environments from system');
        return mssql_service_1.MSSql.GetEnvironments('system', 'name', '');
    }).then(resultGetEnvironments => {
        if (!resultGetEnvironments)
            throw 'Se produjo un error al recuperar los ambientes';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving Environments for new vcc');
        return environment_service_1.SaveEnvironments(vcc, resultGetEnvironments);
    }).then(resultSaveEnvironments => {
        if (!resultSaveEnvironments)
            throw 'Se produjo un error al guardar las categorías';
        if (objVCC.type === 'ALLEGRO') {
            let flag = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
            loggerAllegro.info('[vcc-bulk.task::CreateVCC] Saving allegro group for new vcc type ALLEGRO');
            return mssql_service_1.MSSql.SaveAllegroGroup(vcc, 'quality_agent', 0, 'quality_agent', integration_config_1.IP_BD, flag);
        }
        else {
            loggerAllegro.info('[vcc-bulk.task::CreateVCC] vcc type isn\'t ALLEGRO, continuing');
            return Promise.resolve(true);
        }
    }).then(resultSaveAllegroGroup => {
        if (!resultSaveAllegroGroup)
            throw 'Se produjo un error al guardar el rol quality_agent en allegro';
        loggerAllegro.info('[vcc-bulk.task::CreateVCC] Finished create process for vcc ' + vcc);
        return Promise.resolve(true);
    }).catch((err) => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        loggerAllegro.error('[vcc-bulk.task::CreateVCC] Error procesing \'' + err + '\': ' + errorMsg);
        return Promise.resolve(false);
    });
}
if (require.main === module) {
    let app = express();
    let sqlConn;
    let cassandraConn;
    mssql_config_1.MSSQL_CONFIG.connectionTimeout = 300 * 1000;
    mssql_config_1.MSSQL_CONFIG.requestTimeout = 300 * 1000;
    mssql_init_1.MsSqlInit(app)
        .then((result) => {
        sqlConn = app.get('sqlConn');
        if (TYPE === 'OCC')
            return cassandra_init_1.CassandralInit(app);
        return Promise.resolve(true);
    }).then((result) => {
        if (TYPE === 'OCC')
            cassandraConn = app.get('cassandraConn');
        return ExecuteTask();
    }).then(results => {
        loggerAllegro.info('Task (allegro): Termino la sincronizacion de VCCs');
        sqlConn.close();
        if (TYPE === 'OCC')
            cassandraConn.close();
        setTimeout(function () {
            process.exit(0);
        }, 200);
    }).catch(error => {
        loggerAllegro.error('Task (allegro): Termino la sincronizacion de VCCs con errores :', error);
        sqlConn.close();
        if (TYPE === 'OCC')
            cassandraConn.close();
        setTimeout(function () {
            process.exit(0);
        }, 200);
    });
}
